package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.AccreditCourseVO;
import com.yizhi.course.application.vo.AccreditVO;
import com.yizhi.course.application.vo.AuthorizeCourseVO;
import com.yizhi.course.application.vo.PageAuthorizeVO;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * 课程资源授权管理
 *
 * @author Fairyland
 *
 */
@FeignClient(name = "course", contextId = "AccreditClient")
public interface AccreditClient {

	/**
	 * 根据课程关键字（课程名称 name或者标签tags）、授权站点id,站点id 查询未选择课程（分页并显示总条数)
	 *
	 */
	@GetMapping("/accredit/course/noCourseList")
	Page<CourseEntityVo> noCourseList(@RequestBody AccreditCourseVO accreditCourseVO);


	/**
	 * 根据课程关键字（课程名称 name或者标签tags）企业id、 站点id 查询已选择课程（分页并显示总条数)
	 *
	 */
	@GetMapping("/accredit/courseList")
	Page<CourseEntityVo> courseList(@RequestBody AccreditCourseVO accreditCourseVO);


	/**
	 * 根据课程关键字（课程名称 name或者标签tags）授权站点id、 站点id 查询已选择课程（不用分页）
	 * 作用：供 保存、 授权接口调用  与传进来的已选课程对象集合相比较
	 *
	 */
	@GetMapping("/accredit/XYcourseList")
	 List<CourseEntityVo> YXcourseList(@RequestBody AccreditCourseVO accreditCourseVO);

    /**
     * 授权列表接口
     * @param siteName 站点名称
     * @param pageNo 页码
     * @param pageSize 每页条数
     * @return
     */
    @GetMapping(value = "/accredit/get/list")
    public PageAuthorizeVO queryAuthorizeCourse(@RequestParam(name = "type",required = true) String type, @RequestParam(name = "siteName",required = false) String siteName,
												@RequestParam(name = "companyName",required = false) String companyName,
												@RequestParam(name = "pageNo", defaultValue = "0") Integer pageNo,
												@RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize);


    @GetMapping(value = "/accredit/authorize/list")
    Page<AuthorizeCourseVO> queryAuthorizedCourseList(@RequestParam(name = "type", required = true) String type,
													  @RequestParam(name = "siteName", required = false) String siteName,
													  @RequestParam(name = "companyName", required = false) String companyName,
													  @RequestParam(name = "pageNo", defaultValue = "0") Integer pageNo,
													  @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize);
        /**
         * 修改课程授权参数
         * @auhtor lilingye
         * @param accreditVO
         * @return
         */
    @PostMapping(value = "/accredit/update")
    public Boolean updateAuthorize(@RequestBody AccreditVO accreditVO);

    /**
     *  此授权课程当前授权有效课程
     * @param id
     * @return
     */
    @GetMapping(value = "/accredit/courseCount")
     Integer getAccreditByCourseId(@RequestParam(value = "id") Long id);

}
