package com.yizhi.course.application.feign;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.vo.AuthorizeChapterVO;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.vo.domain.ChapterEntityVo;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import com.yizhi.course.application.vo.domain.ProjectCourseEntityVo;
import com.yizhi.course.application.vo.domain.RecordeEntityVo;
import com.yizhi.course.application.vo.*;
import com.yizhi.util.application.domain.BizResponse;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;
import java.util.Set;

@FeignClient(name = "course", contextId = "CourseClient")
public interface CourseClient {

    /**
     * 根据courseIds 查询课程积分
     *
     * @param ids
     * @return
     */
    @PostMapping("/course/score/sum")
    Map<Long, Integer> getCourseScoreByIds(@RequestBody List<Long> ids);

    @GetMapping("/course/check/chapter/test/only")
    Integer countChapterTest(@RequestParam("courseId") Long courseId,
                             @RequestParam(value = "materialType", required = false) Integer materialType);

    /**
     * 检查讲师的关联课程信息有效的信息
     *
     * @param vo
     * @return
     */
    @PostMapping("/course/valid/filter")
    List<LecturerCourseVO> filterValidCourse(@RequestBody LecturerCourseValidReqVO vo);

    /**
     * 查询站点下全部讲师课程
     *
     * @param type   1为线上课程，2为线下课程
     * @param siteId 站点id
     * @param name   名称关键字
     * @return
     */
    @GetMapping("/course/type/and/site/get")
    List<LecturerCourseVO> getCoursesBysiteId(@RequestParam("type") Integer type,
                                              @RequestParam("siteId") Long siteId,
                                              @RequestParam(value = "name", required = false) String name);

    /**
     * 统计课程的学时
     *
     * @param courseId
     * @return
     */
    @GetMapping("/course/totalDuration/dispaly/info/get")
    CourseTotalDurationInfoVO getCourseDurationInfo(@RequestParam("courseId") Long courseId,
                                                    @RequestParam("siteId") Long siteId);

    /**
     * 查询用户学习课程的学习次数和学习时长
     *
     * @param courseId
     * @param accountId
     * @return
     */
    @GetMapping("/course/study/statistic")
    CourseStudyStatisticVO getCourseStudyStatistic(@RequestParam("courseId") Long courseId,
                                                   @RequestParam("accountId") Long accountId);

    /**
     * 查询用户学习课程的学习记录列表
     *
     * @param courseId
     * @param accountId
     * @return
     */
    @GetMapping("/course/study/recorde")
    List<RecordeEntityVo> getCourseStudyRecorde(@RequestParam("courseId") Long courseId,
                                                @RequestParam("accountId") Long accountId);

    @GetMapping("/course/list")
    Page<CourseEntityVo> list(@RequestBody Map<String, Object> map);

    @GetMapping("/course/list/notIds")
    Page<CourseEntityVo> listNotIds(
            @RequestParam(value = "siteId", required = false) Long siteId,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "ids", required = false) List<Long> ids,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize);

    @PostMapping("course/experiencePoolList")
    Page<Map<String, Object>> experiencePoolList(@RequestParam(value = "name", required = false) String name,
                                                 @RequestBody List<Long> ids,
                                                 @RequestParam(value = "pageNo", required = false) Integer pageNo,
                                                 @RequestParam(value = "pageSize", required = false) Integer pageSize);

    @PostMapping("/course/delete")
    Boolean delete(@RequestBody CourseDeleteVo courseDeleteVo);

    @PostMapping("/course/up")
    Integer up(@RequestBody CourseEntityVo course);

    @PostMapping("/course/down")
    Boolean down(@RequestBody CourseEntityVo course);

    @PostMapping("/course/save")
    CourseVo save(@RequestBody CourseVo courseVo);

    /**
     * 保存课程 版本2
     */
    @PostMapping("/course/save/v2")
    BizResponse<CourseVo> saveV2(@RequestBody CourseVo courseVo);

    @GetMapping("/course/code/vail")
    Boolean vailCode(
            @RequestParam(value = "id", required = false) Long id,
            @RequestParam(value = "code") String code,
            @RequestParam(value = "siteId") Long siteId);

    @PostMapping("/course/update")
    Boolean update(@RequestBody CourseVo courseVo);

    @PostMapping("/course/update/v2")
    BizResponse<Boolean> updateV2(@RequestBody CourseVo courseVo);

    @GetMapping("/course/get")
    CourseDetailsVo get(@RequestParam(value = "id") Long id, @RequestParam(value = "accountId") Long accountId);

    @GetMapping("/course/mg/get")
    CourseDetailsVo manageGet(@RequestParam(value = "id") Long id, @RequestParam(value = "accountId") Long accountId);

    @GetMapping("/course/chapters/get")
    List<ChapterDetailsVo> getChapter(@RequestParam(value = "id") Long id, @RequestParam(value = "accountId") Long accountId);


    @GetMapping("/course/chapter/list")
    List<ChapterEntityVo> chapterList(@RequestParam(value = "id") Long id);

    @GetMapping("/course/chapter/get")
    ChapterEntityVo chapterGet(@RequestParam(value = "id") Long id);

    @PostMapping("/course/chapter/save")
    ChapterEntityVo chapterSave(@RequestBody ChapterEntityVo chapter);

    @PostMapping("/course/chapter/update")
    Boolean chapterUpdate(@RequestBody ChapterEntityVo chapter);

    @PostMapping("/course/chapter/delete")
    Boolean chapterDelete(@RequestBody ChapterEntityVo chapter);

    @PostMapping("/course/chapter/sort")
    Boolean chapterSort(@RequestBody List<String> ids);

    @GetMapping("/course/api/list")
    Page<ApiListCourseVo> apiList(@RequestBody ApiSeachCourseVo apiSeachCourseVo);

    @GetMapping("/course/api/classify/sub/list")
    Page<ApiListCourseVo> searchPageSubClassify(@RequestBody ApiSeachCourseVo apiSeachCourseVo);

    @GetMapping("/course/project/list")
    Page<ProjectCourseVo> projectList(@RequestBody ProjectSeachCourseVo projectSeachCourseVo);

    @PostMapping("/projectCourse/save")
    Boolean projectCourseSave(@RequestBody ProjectCourseEntityVo projectCourse);

    @PostMapping("/projectCourse/delete")
    Boolean projectCourseDelete(@RequestBody ProjectCourseEntityVo projectCourse);


    @GetMapping("/homePage/list")
    Page<HomePageCourseVo> getHomePageCourseList(
            @RequestBody ApiSeachCourseVo apiSeachCourseVo
//            @RequestParam(name = "name") String name,
//            @RequestParam(name = "companyId") Long companyId,
//            @RequestParam(name = "siteId") Long siteId,
//            @RequestParam(name = "ids") List<Long> ids,
//            @RequestParam(name = "pageNo") Integer pageNo,
//            @RequestParam(name = "pageSize") Integer pageSize
    );

    @GetMapping("/homePage/hot/list")
    List<HomePageCoursePageVo> hotList(
            @RequestBody ApiSeachCourseVo apiSeachCourseVo
//            @RequestParam(name = "name") String name,
//            @RequestParam(name = "companyId") Long companyId,
//            @RequestParam(name = "siteId") Long siteId,
//            @RequestParam(name = "ids") List<Long> ids,
//            @RequestParam(name = "pageNo") Integer pageNo,
//            @RequestParam(name = "pageSize") Integer pageSize
    );

    //课程收藏的接口
    @PostMapping("/favorite/save")
    Boolean saveFavorite(@RequestBody FavoriteVo favorite);


    @PostMapping("/favorite/delete")
    Boolean deleteFavorite(@RequestBody FavoriteVo favorite);

    @GetMapping("/favorite/total")
    Integer getMyFavoriteCount(@RequestParam(name = "accountId") Long accountId, @RequestParam(name = "companyId") Long companyId,
                               @RequestParam(name = "siteId") Long siteId);

    @GetMapping("/favorite/list")
    List<ApiListCourseVo> getMyFavoriteList(
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize
    );


    @PostMapping("/course/courseAccount/save")
    boolean saveCoursrAccount(@RequestBody List<CourseAccountVO> courseAccounts);

    @PostMapping("/course/courseAccount/insert")
    boolean insertCourseAccount(@RequestBody List<CourseAccountVO> courseAccounts);

    @GetMapping("/course/courseAccount/list")
    List<CourseAccountVO> getCourseAccountList(@RequestParam(name = "courseId") Long courseId);


    @GetMapping("/course/search/list")
    List<CourseEntityVo> getCourseSearchList(
            @RequestParam(name = "name") String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "orgId") Long orgId
    );

    @PostMapping("/course/scorm/save")
    ScormRecordVO saveScorm(@RequestBody ScormRecordVO record);

    @PostMapping("/course/heart/scorm/save")
    void saveScormHeartBeat(@RequestBody StudyLog scormStudyLog);

    /**
     * scorm 进度 获取
     *
     * @param scormId
     * @return
     */
    @GetMapping("/course/scorm/get")
    ScormRecordVO getScorm(@RequestParam(name = "scormId") Long scormId, @RequestParam(name = "chapterId") Long chapterId);

    @GetMapping("/course/offline/list")
    List<ApiListCourseVo> getOfflineList(
            @RequestParam(name = "ids") String ids,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pagesize") Integer pagesize
    );

    /**
     * 查询授权的课程列表
     *
     * @param name
     * @param classifyId
     * @param companyId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/course/get/list")
    public PageCourseAuthorizeVO queryeAuthorizeCoursesList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "companyId", required = false) Long companyId,
            @RequestParam(name = "csiteId", required = false) Long csiteId,
            @RequestParam(name = "classifyId", required = false) Long classifyId,
            @RequestParam(name = "pageNo", defaultValue = "1", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "20", required = false) Integer pageSize

    );

    /**
     * 提交保存授权课程的接口
     *
     * @return
     */
    @PostMapping(value = "/course/insert/authorization")
    public CourseRespVO saveAuthorization(@RequestBody AuthorizeVO authorizeVO);

    /**
     * 查询授权后的可恨列表
     *
     * @param courseKeys
     * @param startTime
     * @param siteId
     * @param endTime
     * @param pageNo
     * @param pageSize
     */
    @GetMapping(value = "/course/get/authorized")
    PageAuthorizedVO getAuthorizedList(
            @RequestParam(name = "courseKeys", required = false) String courseKeys,
            @RequestParam(name = "startTime", required = false) String startTime,
            @RequestParam(name = "endTime", required = false) String endTime,
            @RequestParam(name = "siteId", required = true) Long siteId,
            @RequestParam(name = "activeType", required = false) Integer activeType,
            @RequestParam(name = "type") Integer type,
            @RequestParam(name = "pageNo", defaultValue = "1", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "20", required = false) Integer pageSize);


    /**
     * 查询课程 只包含课程信息无简介
     *
     * @param id 课程id
     * @return
     */
    @GetMapping("/course/view")
    CourseEntityVo view(@RequestParam(value = "id") Long id);

    /**
     * 查询指定范围可见id
     *
     * @param courseId
     * @return
     */
    @GetMapping("/course/getRange")
    List<Long> getRange(@RequestParam("courseId") Long courseId);

    /**
     * 课程指定范围导出
     *
     * @param courseId
     * @return
     */
    @GetMapping("/course/export/visiblRange")
    VisibleRangeExport vsibleRangeExport(@RequestParam("courseId") Long courseId);

    /**
     * 授权过期的课程 失效
     *
     * @param id 授权id
     * @return
     * @since 2018-9-21
     */
    @PostMapping(value = "/course/invalid")
    boolean invalidCourse(@RequestParam(value = "id") Long id);

    /**
     * 根据课程id 获取课程评分和(课程学习人数)课程浏览人次
     *
     * @param courseId
     * @return
     */
    @GetMapping("/course/getManageCourseList")
    public HomePageCoursePageVo getManageCourseList(@RequestParam("courseId") Long courseId,
                                                    @RequestParam("siteId") Long siteId,
                                                    @RequestParam("companyId") Long companyId);


    /**
     * 首页火热课程-- 为了优化sql新写的火热课程接口
     *
     * @param apiSeachCourseVo
     * @return
     */
    @GetMapping("/homePage/hot/course")
    public List<HomePageCoursePageVo> getCourseList(@RequestBody ApiSeachCourseVo apiSeachCourseVo);

    /**
     * 待授权课程列表
     *
     * @param map
     * @return
     */
    @GetMapping("/course/authorizelist")
    public Page<AuthCourseVO> getAuthorizeCourseList(@RequestBody Map<String, Object> map);


    /**
     * 异步任务被授权课章节的新增
     *
     * @param chapterVO
     * @return
     */
    @PostMapping("/course/authorize/chapter/save")
    public Boolean authorizeChapterSave(@RequestBody AuthorizeChapterVO chapterVO);

    /**
     * 异步任务删除被修改的章节
     *
     * @param chapterVO
     * @return
     */
    @PostMapping("/course/authorize/chapter/delete")
    public Boolean authorizeChapterDelete(@RequestBody AuthorizeChapterVO chapterVO);

    /**
     * 异步任务修改被授权课程的章节
     *
     * @param chapterVO 章节实体
     * @return
     */
    @PostMapping("/course/authorize/chapter/update")
    public Boolean authorizeChapterUpdate(@RequestBody AuthorizeChapterVO chapterVO);

    /**
     * 异步任务修改被授权课程的章节
     *
     * @param ids 上下移动的id
     * @return
     */
    @PostMapping("/course/authorize/chapter/sort")
    public Boolean authorizeChapterSort(@RequestBody List<String> ids);

    /**
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/course/home/courseList")
    public List<Long> findCourseList(@RequestParam(value = "pageNo", defaultValue = "0", required = false) Integer pageNo,
                                     @RequestParam(value = "pageSize", defaultValue = "20", required = false) Integer pageSize);

    /**
     * 根据course 查询课程列表
     *
     * @param courseList
     * @return
     */
    @GetMapping("/homePage/home/course")
    public List<HomePageCoursePageVo> findHomeCourseList(@RequestBody List<Long> courseList);

    /**
     * 判断课程是否完成
     *
     * @param courseAccountIds
     * @param isTestSensitive  是否包括测验是否完成
     * @return
     */
    @PostMapping(value = "/course/record/isCourseFinished")
    Map<String, Boolean> isCourseFinished(@RequestBody Set<String> courseAccountIds, @RequestParam(name = "isTestSensitive") Boolean isTestSensitive);


    //*****************************************************  PC端  接口  *********************************************************************//


    /**
     * 课程收藏  接口
     *
     * @param favorite
     * @return
     */
    @PostMapping("/favoritepc/save")
    public Boolean savePCFavorite(@RequestBody FavoriteVo favorite);

    /**
     * 取消课程收藏  接口
     *
     * @param favorite
     * @return
     */
    @PostMapping("/favoritepc/delete")
    public Boolean deletePCFavorite(@RequestBody FavoriteVo favorite);

    /**
     * 课程收藏总数 接口
     *
     * @param accountId
     * @return
     */
    @GetMapping("/favoritepc/total")
    public Integer getPCMyFavoriteCount(
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId
    );


    /**
     * 我的课程收藏列表  接口
     *
     * @param accountId
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/favoritepc/list")
    public Page<ApiListCourseVo> getPCMyFavoriteList(
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize
    );


    @GetMapping("/course/by/new/server")
    public List<Map<String, Object>> getServerByCompanyIdAndIds(@RequestParam("companyId") Long companyId, @RequestParam(name = "ids", required = false) List<Long> ids);


    @GetMapping(value = "/course/get/course/time/message")
    public Map<String, Object> getCourseTimeMessage(@RequestParam("companyId") Long companyId, @RequestParam(value = "ids", required = false) List<Long> ids);

    /**
     * 素材站点转移
     *
     * @param afterSiteId
     * @param companyId
     */
    @PostMapping(value = "/course/backupMaterial")
    void updateMaterialData(@RequestBody List<String> codeList,
                            @RequestParam(name = "afterSiteId", required = true) Long afterSiteId,
                            @RequestParam(name = "beforeSiteId", required = true) Long beforeSiteId,
                            @RequestParam(name = "companyId", required = true) Long companyId);

    /**
     * 获取可见范围的课程
     *
     * @param courseIds 课程list
     * @return
     */
    @GetMapping("/homePage/home/course/range")
    List<HomePageCoursePageVo> findCourseIdList(@RequestBody List<Long> courseIds);

    /**
     * o
     * 自定义项目获取课程log
     *
     * @param courseIds
     * @return
     */
    @GetMapping("/course/getCourseLogos")
    Map<Long, String> getCourseLogos(@RequestParam("courseIds") List<Long> courseIds);

    @GetMapping("/course/list/export")
    public Map<String, Object> listExport(@RequestParam(required = false, value = "type", defaultValue = "0") Integer type,
                                          @ApiParam(name = "name", value = "课程名称或作者", required = false) @RequestParam(name = "name", required = false) String name,
                                          @ApiParam(name = "classifyId", value = "分类id ", required = false) @RequestParam(name = "classifyId", required = false) Long classifyId,
                                          @ApiParam(name = "shelves", value = "0 未上架 1 已上架 2 草稿", required = false) @RequestParam(name = "shelves", required = false) Integer shelves,
                                          @ApiParam(name = "courseType", value = "课程类型", required = false) @RequestParam(required = false, value = "courseType") Integer courseType);

    /**
     * 交银康联中调用的老接口
     *
     * @param siteId
     * @param ids
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Deprecated
    @GetMapping("/course/list/notIdsjykl")
    Page<CourseEntityVo> listNotIds(
            @RequestParam(value = "siteId", required = false) Long siteId,
            @RequestParam(value = "ids", required = false) List<Long> ids,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize);

    @GetMapping("/course/getCourseCodeName")
    Map<Long, String> getCourseCodeName(@RequestParam(value = "courseIds", required = false) List<Long> courseIds);

    @GetMapping("/course/VisibleRange")
    public List<CourseAccountVO> VisibleRange(@RequestParam("courseId") Long courseId);

    @GetMapping("/course/getOne")
    public CourseVo getOne(@RequestParam("courseId") Long courseId);

    @PostMapping("/course/getPageToCalendar")
    public Page<CourseEntityVo> getPageToCalendar(@ApiParam("paramVo") @RequestBody CalendarTaskParamVo paramVo);

    /**
     * 嘉兴银行闯关排行
     *
     * @param courseId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/course/getRankings")
    Page<JaxinRankingVO> getRankings(@RequestParam("courseId") Long courseId,
                                     @RequestParam("pageNo") Integer pageNo,
                                     @RequestParam("pageSize") Integer pageSize);

    /**
     * 获取课程根据 code
     * @param courseNo
     * @return
     */
    @GetMapping(value = "/course/getCourseByCode")
    CourseEntityVo getCourseByCode(@RequestParam(value = "courseNo") String courseNo,
                           @RequestParam(value = "siteId") Long siteId,
                           @RequestParam(value = "companyId") Long companyId);

    @GetMapping("/course/getPageByDrools")
    Page<DroolsVo> getPageByDrools(@RequestParam("field") String field,
                                   @RequestParam(value = "value", required = false) String value,
                                   @RequestParam("pageNo") Integer pageNo,
                                   @RequestParam("pageSize") Integer pageSize);

    @PostMapping("/course/ids/get")
    List<CourseEntityVo> getCourseByIds(@RequestBody List<Long> ids);

    @GetMapping("/material/list/export")
    public Map<String, Object> materialListExport(@RequestParam(required = false, value = "type", defaultValue = "0") Integer type,
                                                  @ApiParam(name = "name", value = "课程名称或作者", required = false) @RequestParam(name = "name", required = false) String name,
                                                  @ApiParam(name = "classifyId", value = "分类id ", required = false) @RequestParam(name = "classifyId", required = false) Long classifyId,
                                                  @ApiParam(name = "shelves", value = "0 未上架 1 已上架 2 草稿", required = false) @RequestParam(name = "shelves", required = false) Integer shelves,
                                                  @ApiParam(name = "courseType", value = "课程类型", required = false) @RequestParam(required = false, value = "courseType") Integer courseType);
}

