package com.yizhi.course.application.feign;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.AudioVideoVO;
import com.yizhi.course.application.vo.MaterialVo;
import com.yizhi.course.application.vo.domain.MaterialEntityVo;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

@FeignClient(name = "course", contextId = "MaterialClient")
public interface MaterialClient {

    @PostMapping("/material/save")
    Boolean insert(@RequestBody Map<String, Object> map);

    /**
     * 新增并返回新增对象
     * @param map
     * @return
     */
    @PostMapping("/material/save/obj")
    MaterialEntityVo insertObj(@RequestBody Map<String,Object> map);

    @PostMapping("/material/delete")
    Boolean delete(@RequestBody List<Long> ids);

    @GetMapping("/material/get")
    MaterialVo get(@RequestParam(value = "id") Long id);

    @GetMapping("/material/view")
    MaterialEntityVo view(@RequestParam(value = "id") Long id);

    @PostMapping("/material/update")
    Boolean update(@RequestBody Map<String, Object> map);

    @GetMapping("/material/list")
    Page<MaterialEntityVo> list(@RequestBody Map<String, Object> map);

    @GetMapping("/material/video/view")
    AudioVideoVO videoView(@RequestParam(value = "videoId") String videoId);

    @PostMapping("/material/video/update")
    Boolean videoUpdate(@RequestBody Map<String,Object> mao);

    /**
     * 查询素材列表  ( Scorm课程 )
     * @return
     */
    @GetMapping("/material/scormList")
    Page<MaterialEntityVo> queryScormCourseList(@RequestParam Map<String,Object> map);

    /**
     * 查询素材列表  ( 一般课程  )
     *
     * @param map
     * @return
     */
    @GetMapping("/material/commonList")
    Page<MaterialEntityVo> queryCommonCourseList(@RequestParam Map<String,Object> map);

    @GetMapping("/material/list/export")
    Map<String, Object> materialListExport(@RequestParam(required = false, value = "type", defaultValue = "0") Integer type,
                                                  @ApiParam(name = "name", value = "课程名称或作者", required = false) @RequestParam(name = "name", required = false) String name,
                                                  @ApiParam(name = "classifyId", value = "分类id ", required = false) @RequestParam(name = "classifyId", required = false) Long classifyId,
                                                  @ApiParam(name = "shelves", value = "0 未上架 1 已上架 2 草稿", required = false) @RequestParam(name = "shelves", required = false) Integer shelves,
                                                  @ApiParam(name = "courseType", value = "课程类型", required = false) @RequestParam(required = false, value = "courseType") Integer courseType);
}
