package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.OfflineCourseDeleteVo;
import com.yizhi.course.application.vo.OfflineCourseStudentVo;
import com.yizhi.course.application.vo.OfflineCourseVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

@FeignClient(name = "course", contextId = "OfflineCourseClient")
public interface OfflineCourseClient {

    /**
     * 线下课程删除
     *
     * @param offlineCourseDeleteVo
     * @return
     */
    @PostMapping("/offlineCourse/delete")
    Boolean delete(@RequestBody OfflineCourseDeleteVo offlineCourseDeleteVo);

    /**
     * 下课程保存
     *
     * @param offlineCourseVo
     * @return
     */
    @PostMapping("/offlineCourse/save")
    OfflineCourseVo save(@RequestBody OfflineCourseVo offlineCourseVo);

    /**
     * 线下课程code 唯一性验证
     *
     * @param id
     * @param code
     * @param siteId
     * @return
     */
    @GetMapping("/offlineCourse/code/verifyCourse")
    Boolean verifyCourseCode(@RequestParam(value = "id", required = false) Long id,
                             @RequestParam(value = "code") String code,
                             @RequestParam(value = "siteId") Long siteId);

    /**
     * 修改线下课程
     *
     * @param offlineCourseVo
     * @return
     */
    @PostMapping("/offlineCourse/update")
    Boolean update(@RequestBody OfflineCourseVo offlineCourseVo);

    /**
     * 线下课程分页查询
     *
     * @param map
     * @return
     */
    @GetMapping("/offlineCourse/list")
    Page<OfflineCourseVo> list(@RequestBody Map<String, Object> map);

    /**
     * 全文检索
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param orgId
     * @return
     */
    @GetMapping("/offlineCourse/search/list")
    List<OfflineCourseVo> getCourseList(
            @RequestParam(name = "name") String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "orgId") Long orgId);

    /**
     * 线下课程_课程上架
     *
     * @param offlineCourse
     * @return
     */
    @PostMapping("/offlineCourse/up")
    Boolean up(@RequestBody OfflineCourseVo offlineCourse);

    /**
     * 线下课程_课程下架
     *
     * @param offlineCourse
     * @return
     */
    @PostMapping("/offlineCourse/down")
    Boolean down(@RequestBody OfflineCourseVo offlineCourse);

    /**
     * 课程查看
     *
     * @param id
     * @return
     */
    @GetMapping("/offlineCourse/get")
    OfflineCourseVo get(@RequestParam(value = "id") Long id);

    /**
     * 上传文件
     *
     * @param offlineCourseVo
     * @return
     */
    @PostMapping(value = "/offlineRecorde/upload")
    String upload(@RequestBody OfflineCourseVo offlineCourseVo);

    /**
     * 下载数据
     *
     * @param param
     * @return
     */
    @PostMapping("/offlineRecorde/download")
    String download(@RequestBody Map<String, Object> param);

    /**
     * 线下课程学员端查询 (培训项目中关联)
     *
     * @param id
     * @return
     */
    @GetMapping("/api/offlineCourse/get")
    OfflineCourseStudentVo getList(@RequestParam(value = "id") Long id);


    /**
     * 通id获取信息
     * @param id
     * @return
     */
    @GetMapping("/offlineCourse/get/offlineCourse")
    public OfflineCourseVo getCourseById(@RequestParam(value = "id") Long id);




    /************************************  PC端  开始 **************************************/

    /**
     * 线下课程 PC 端查询 (培训项目中关联)
     * @param id
     * @return
     */
    @GetMapping("/api/offlineCourse/pc/get")
    OfflineCourseStudentVo getPcList(@RequestParam(value = "id") Long id);

    /************************************  PC端  结束 **************************************/
    /**
     * 获取线下课程
     * @param courseNo
     * @return
     */
    @GetMapping("/offlineCourse/getOfflineCourseByCode")
    OfflineCourseVo getOfflineCourseByCode(@RequestParam(value = "courseNo") String courseNo,
                                         @RequestParam(value = "siteId") Long siteId);
}
