package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.TestInfoListReqVO;
import com.yizhi.course.application.vo.TestInfoVO;
import com.yizhi.course.application.vo.TestInfoViewRespVO;
import com.yizhi.course.application.vo.TestSubjectVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * @Author: XieHaijun
 * @Description: 测验信息接口
 * @Date: Created in 16:44 2018/10/15
 * @Modified By
 */
@FeignClient(name = "course", contextId = "TestInfoClient")
public interface TestInfoClient {

    /**
     * 测验信息列表
     * @param reqVO
     * @return
     */
    @PostMapping("/test/list")
    public Page<TestInfoVO> testInfoList(@RequestBody TestInfoListReqVO reqVO);

    /**
     * 根据测验id查询
     * @param id
     * @return
     */
    @GetMapping("/test/all/view")
    public TestInfoViewRespVO viewAll(@RequestParam(value = "id") Long id );
    /**
     * 查询测验的试题列表
     * @param id
     * @return
     */
    @GetMapping("/test/subject/list")
    public List<TestSubjectVO> listSubject(@RequestParam("id") Long id);

    /**
     * 根据ids删除测验信息
     * @param deleteTestIds
     * @return
     */
    @PostMapping("/test/ids/delete")
    public Boolean testInfoDelete(@RequestBody List<Long> deleteTestIds);

    /**
     * 保存测验信息
     * @param vo
     * @return
     */
    @PostMapping("/test/save")
    public TestInfoVO testInfoSave(@RequestBody TestInfoVO vo);

    /**
     * 测验基本信息查看
     * @param id
     * @return
     */
    @GetMapping("/test/view")
    public TestInfoViewRespVO testInfoView(@RequestParam(value = "id") Long id );

    /**
     * 更新测验被使用中
     * @param testId 测验id
     * @param chapterId 课程章节关联的素材id（删除和更新麻烦传原章节id，新增测验关联传null）
     * @return
     */
    @GetMapping("/test/status/update")
    public Boolean updateStatus(@RequestParam(value = "testId") Long testId,
                                @RequestParam(required = false, value = "chapterId") Long chapterId,
                                @RequestParam(value = "type") Integer type);

    /**
     * 保存测验试题
     * @param vo
     * @return
     */
    @PostMapping("/testSubject/save")
    public TestSubjectVO testSubjectSave(@RequestBody TestSubjectVO vo);

    /**
     * 删除测验试题信息
     * @param id
     * @return
     */
    @GetMapping("/testSubject/delete")
    public boolean testSubjectDelete(@RequestParam("id") Long id);

    /**
     * 根据id查看测验试题信息
     * @param id
     * @return
     */
    @GetMapping("/testSubject/view")
    public TestSubjectVO testSubjectView(@RequestParam("id") Long id);

    /**
     * 测验试题上移，下移排序
     * @param id
     * @param moveType
     * @return
     */
    @GetMapping("/testSubject/sort")
    public TestSubjectVO sort(@RequestParam("id") Long id, @RequestParam("moveType") String moveType);

    /**
     * 更新测验试题分数
     * @param id
     * @param score
     * @return
     */
    @GetMapping("/testSubject/score/update")
    public boolean updateScore(@RequestParam("id") Long id, @RequestParam("score") Integer score);
}
