package com.yizhi.course.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.enums.SqlLike;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.domain.ClassifyGroup;
import com.yizhi.course.application.domain.ClassifyGroupInfo;
import com.yizhi.course.application.service.IClassifyGroupInfoService;
import com.yizhi.course.application.service.IClassifyGroupService;
import com.yizhi.course.application.service.IClassifyService;
import com.yizhi.course.application.vo.ClassifyGroupInfoVO;
import com.yizhi.course.application.vo.ClassifyGroupStudentVO;
import com.yizhi.course.application.vo.GroupClassifyItemVO;
import com.yizhi.course.application.vo.domain.ClassifyEntityVo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 分类组和分类的关联信息表 前端控制器
 * </p>
 *
 * @author xiehaijun
 * @since 2019-12-13
 */
@RestController
@RequestMapping("/classify/group")
public class ClassifyGroupController {

    private final static String CLASSIFY_DATA_KEY = "classifyData";

    private final static String CLASSIFY_GROUP_DATA_KEY = "classifyGroupData";

    private final static String CURR_LIVE_DATA = "currLiveData";

    private final static String PARENT_LIVE_DATA = "parentLiveData";

   /* @Autowired
    private IdGenerator idGenerator;*/

    @Autowired
    private IClassifyGroupService classifyGroupService;

    @Autowired
    private IClassifyGroupInfoService classifyGroupInfoService;

    @Autowired
    private IClassifyService classifyService;

    private Logger LOGGER = LoggerFactory.getLogger(ClassifyGroupController.class);

    @PostMapping("/save")
    public List<ClassifyGroup> save(@RequestBody ClassifyGroupInfoVO classifyGroupInfoVO){
        return classifyGroupInfoService.save(classifyGroupInfoVO);
    }

    @PostMapping("/save/all")
    public List<ClassifyGroup> saveAll(@RequestBody ClassifyGroupInfoVO classifyGroupInfoVO){
        //LOGGER.info("classifyGroupInfoVO=", JSON.toJSONString(classifyGroupInfoVO));
        return classifyGroupInfoService.saveAll(classifyGroupInfoVO);
    }

    @GetMapping("/view")
    public ClassifyGroupStudentVO view(@RequestParam(name = "id") Long id){

        ClassifyGroupInfo classifyGroupInfo =classifyGroupInfoService.selectById(id);

        ClassifyGroupStudentVO classifyGroupStudentVO = new ClassifyGroupStudentVO();
        BeanUtils.copyProperties(classifyGroupInfo, classifyGroupStudentVO);

        EntityWrapper<ClassifyGroup> ewCG = QueryUtil.condition(new ClassifyGroup());
        ewCG.eq("classify_group_info_id", id); // 分类组id
        int countSize = classifyGroupService.selectCount(ewCG);

        if(countSize>0){
            int templateType = classifyGroupInfo.getTemplateType();
            Integer templateSubType = classifyGroupInfo.getTemplateSubType();
            List<Long> data = getSelect(id, templateType, templateSubType);

            // 查询分类信息
            EntityWrapper<Classify> allEw = QueryUtil.condition(new Classify());
            allEw.in("id", data);
            allEw.orderBy("sort",false);
            List<Classify> list = classifyService.selectList(allEw);
            List<ClassifyEntityVo> vos = new ArrayList<>();
            for (Classify classify : list) {
                ClassifyEntityVo vo = new ClassifyEntityVo();
                BeanUtils.copyProperties(classify,vo);
                vos.add(vo);
            }
            classifyGroupStudentVO.setClassifyData(vos);
        }

        return classifyGroupStudentVO;
    }

    @GetMapping("/manage/view")
    public ClassifyGroupStudentVO manageView(@RequestParam(name = "id") Long id,
                                             @RequestParam(name = "keyName",required = false) String keyName){

        ClassifyGroupInfo classifyGroupInfo =classifyGroupInfoService.selectById(id);

        ClassifyGroupStudentVO classifyGroupStudentVO = new ClassifyGroupStudentVO();
        BeanUtils.copyProperties(classifyGroupInfo, classifyGroupStudentVO);

        EntityWrapper<ClassifyGroup> ewCG = QueryUtil.condition(new ClassifyGroup());
        ewCG.eq("classify_group_info_id", id); // 分类组id
        int countSize = classifyGroupService.selectCount(ewCG);

        if(countSize>0){
            int templateType = classifyGroupInfo.getTemplateType();
            Integer templateSubType = classifyGroupInfo.getTemplateSubType();
            List<Long> data = getSelect(id, templateType, templateSubType);

            // 查询分类信息
            EntityWrapper<Classify> allEw = QueryUtil.condition(new Classify());

            allEw.in("id", data);
            if(StringUtils.isNotBlank(keyName)){
                allEw.like("name",keyName, SqlLike.DEFAULT);
            }
            allEw.orderBy("sort",false);

            List<Classify> list = classifyService.selectList(allEw);
            List<ClassifyEntityVo> vos = new ArrayList<>();
            for (Classify classify : list) {
                ClassifyEntityVo vo = new ClassifyEntityVo();
                BeanUtils.copyProperties(classify,vo);
                vos.add(vo);
            }
            classifyGroupStudentVO.setClassifyData(vos);
        }

        return classifyGroupStudentVO;
    }

    @GetMapping("/get/select/classifyIds")
    public List<Long> getSelectIds(@RequestParam("id") Long id){
        EntityWrapper<ClassifyGroup> ewCG = QueryUtil.condition(new ClassifyGroup());
        ewCG.eq("classify_group_info_id", id); // 分类组id
        int countSize = classifyGroupService.selectCount(ewCG);

        if(countSize==0){
            return null;
        }
        ClassifyGroupInfo classifyGroupInfo = classifyGroupInfoService.selectById(id);
        int templateType = classifyGroupInfo.getTemplateType();
        Integer templateSubType = classifyGroupInfo.getTemplateSubType();
        List<Long> data = getSelect(id, templateType, templateSubType);
        return data;
    }

    @GetMapping("/join/status")
    public Boolean joinStatus(@RequestParam("cid") Long cid){
        ClassifyGroup classifyGroup = new ClassifyGroup();
        classifyGroup.setClassifyId(cid);
        int countNum = classifyGroupService.selectCount(QueryUtil.condition(classifyGroup));
        return countNum==0;
    }

    /**
     * 获取分类组各种模板保存的分类id信息
     * @param id
     * @param templateType
     * @param templateSubType
     * @return
     */
    private List<Long> getSelect(Long id, int templateType, Integer templateSubType) {
        List<Long> data =  new ArrayList<>();
        EntityWrapper<ClassifyGroup> ew = QueryUtil.condition(new ClassifyGroup());
        ew.eq("classify_group_info_id",id);
        List<ClassifyGroup> selectData = classifyGroupService.selectList(ew);
        if(!CollectionUtils.isEmpty(selectData)){

            if(templateType==1){
                //data = selectData.parallelStream().map(pid -> pid.getClassifyId()).collect(Collectors.toList());
                data.addAll(selectData.parallelStream().map(pid -> pid.getClassifyId()).collect(Collectors.toList()));
            }else if(templateType==2){ // 如果是二级模板
                Map<Integer,List<ClassifyGroup>> mapData = selectData.parallelStream().collect(Collectors.groupingBy(key->key.getLayer()));

                Set<Long> allClassifyId = new HashSet<>();
                List<ClassifyGroup> twoLive = mapData.get(Integer.valueOf(templateSubType.intValue()+1)); // 获取最大关联的层级的上一级
                if(!CollectionUtils.isEmpty(twoLive)){
                    allClassifyId.addAll(getParentIds(twoLive));
                }
                List<ClassifyGroup> oneLive = mapData.get(templateSubType); // 获取最大关联的层级
                if(!CollectionUtils.isEmpty(oneLive)){
                    allClassifyId.addAll(oneLive.parallelStream().map(ClassifyGroup::getClassifyId).collect(Collectors.toSet()));
                }
                data = new ArrayList<>(allClassifyId);
            }else if(templateType==3){
                Map<Integer,List<ClassifyGroup>> mapData = selectData.parallelStream().collect(Collectors.groupingBy(key->key.getLayer()));
                Set<Long> allClassifyId = new HashSet<>();
                Set<Long> upTowIds = null;

                List<ClassifyGroup> threeLive = mapData.get(Integer.valueOf(3)); // 获取第三级
                if(!CollectionUtils.isEmpty(threeLive)){
                    Set<Long> itemIds = threeLive.parallelStream().map(obj->obj.getClassifyId()).collect(Collectors.toSet());
                    allClassifyId.addAll(itemIds);
                    upTowIds = getParentId(itemIds);
                    //allClassifyId.addAll(upTowIds);
                }

                Set<Long> upOneIds = null;
                Set<Long> itemTowIds = null;
                List<ClassifyGroup> towLive = mapData.get(Integer.valueOf(2)); // 获取第二级
                if(!CollectionUtils.isEmpty(towLive) ){
                    itemTowIds = towLive.parallelStream().map(obj->obj.getClassifyId()).collect(Collectors.toSet());

                }
                // 判断第三级查询出来的第二级和分组出来的第二级（进行合并）
                if(upTowIds != null || itemTowIds != null){
                    if(upTowIds != null && itemTowIds != null){
                        itemTowIds.addAll(upTowIds);
                    }else if(upTowIds != null && itemTowIds == null){
                        itemTowIds = upTowIds;
                    }
                    allClassifyId.addAll(itemTowIds);//添加合并出来的全部第二级
                    upOneIds = getParentId(itemTowIds);
                    allClassifyId.addAll(upOneIds); //把合并出来的全部第二级的分类父id（第一级的）添加到集合
                }

                List<ClassifyGroup> oneLive = mapData.get(Integer.valueOf(1)); // 获取第一级下面的id，如果存在就添加，不存在就不添加
                if(!CollectionUtils.isEmpty(oneLive)){
                    Set<Long> itemIds = oneLive.parallelStream().map(obj->obj.getClassifyId()).collect(Collectors.toSet());
                    allClassifyId.addAll(itemIds);
                }
                data = new ArrayList<>(allClassifyId);
            }
        }
        return data;
    }

    /**
     * 获取当前的分类和分类的上一级分类id
     * @param twoLive
     * @return
     */
    private Set<Long> getParentIds(List<ClassifyGroup> twoLive) {
        Set<Long> allClassifyId;
        allClassifyId = twoLive.parallelStream().map(obj->obj.getClassifyId()).collect(Collectors.toSet());
        List<Classify> upClassify = classifyService.selectBatchIds(allClassifyId);
        Set<Long> itemClassifyId = upClassify.parallelStream().map(Classify::getParentId).collect(Collectors.toSet());
        allClassifyId.addAll(itemClassifyId);
        return allClassifyId;
    }

    /**
     * 获取当前的分类和上级的分类
     * @param currIds
     * @return
     */
    private Set<Long> getParentId(Set<Long> currIds) {

        List<Classify> upClassify = classifyService.selectBatchIds(currIds);
        Set<Long> parentClassifyId = upClassify.parallelStream().map(Classify::getParentId).collect(Collectors.toSet());
        return parentClassifyId;
    }

    @GetMapping("/get")
    public List<GroupClassifyItemVO> get(@RequestParam("id") Long id,
                                         @RequestParam(value = "classifyId" ,required = false) Long classifyId,
                                         @RequestParam(value = "layer",required = false) Integer layer){
        if(layer==null){
            layer=1;
        }
        //Map<String, Object> data = new HashMap<>();

        /*ClassifyGroupInfo classifyGroupInfo =classifyGroupInfoService.selectById(id);
        int templateType = classifyGroupInfo.getTemplateType();
        Integer templateSubType = classifyGroupInfo.getTemplateSubType();*/

        // 上下文下信息
        RequestContext rc = ContextHolder.get();
        EntityWrapper<Classify> ew = QueryUtil.condition(new Classify());
        ew.eq("company_id", rc.getCompanyId()).eq("site_id", rc.getSiteId());
        ew.eq("status",1);
        // 查询分类信息
        if(classifyId == null && layer == null){ //第一层 status = 1
            ew.andNew().eq("parent_id",0).or().eq("layer",layer);
        }else if(classifyId == null && layer != null) {
            ew.eq("layer",layer);
        }else {
            ew.eq("parent_id",classifyId);
        }
        List<Classify> classifyData = classifyService.selectList(ew);
        //data.put(CLASSIFY_DATA_KEY, classifyData);

        // 查询已经关联的数据
        Set<Long> selectData = null;
        // 查询该分类组的全部关联信息
        EntityWrapper<ClassifyGroup> ewCG = QueryUtil.condition(new ClassifyGroup());
        ewCG.eq("classify_group_info_id", id); // 分类组id
        List<ClassifyGroup> classifyGroups = classifyGroupService.selectList(ewCG);
        if(!CollectionUtils.isEmpty(classifyGroups)){

            Map<Integer, List<ClassifyGroup>> dataG = classifyGroups.parallelStream().collect(Collectors.groupingBy(gKey->gKey.getLayer()));
            if(layer.intValue() == 1){
                List<ClassifyGroup> temData = dataG.get(layer);
                if(!CollectionUtils.isEmpty(temData)){
                    selectData = temData.parallelStream().map(valOne->valOne.getClassifyId()).collect(Collectors.toSet());
                }
                // 第二层的全部分类id
                Set<Long> twoIds = new HashSet<>();
                // 首先获取第三层分类Id的父集
                List<ClassifyGroup> temDataThree = dataG.get(Integer.valueOf(3));
                if(!CollectionUtils.isEmpty(temDataThree)){
                    EntityWrapper<Classify> ewThree = QueryUtil.condition(new Classify());
                    ewThree.in("id",temDataThree.parallelStream().map(item->item.getClassifyId()).collect(Collectors.toList()));
                    // 查询第三层的父id(第二层id）
                    List<Classify> temDataThreeData = classifyService.selectList(ewThree);
                    if(!CollectionUtils.isEmpty(temDataThreeData)){
                        // 添加第三层的id到全部的第二层id集合
                        twoIds.addAll(temDataThreeData.parallelStream().map(item->item.getParentId()).collect(Collectors.toSet()));
                    }
                }
                // 获取关联的第二层的分类Id
                List<ClassifyGroup> temDataTwo = dataG.get(Integer.valueOf(2));
                if(!CollectionUtils.isEmpty(temDataTwo)){
                    // 合并全部第二层的id
                    twoIds.addAll(temDataTwo.parallelStream().map(item->item.getClassifyId()).collect(Collectors.toSet()));
                    EntityWrapper<Classify> ewTwo = QueryUtil.condition(new Classify());
                    ewTwo.in("id",twoIds);
                    // 查询全部的第二层的id
                    List<Classify> temDataTowData = classifyService.selectList(ewTwo);
                    if(!CollectionUtils.isEmpty(temDataTowData)){
                        selectData.addAll(temDataTowData.parallelStream().map(item->item.getParentId()).collect(Collectors.toSet()));
                    }
                }

            }else if(layer.intValue() == 2){
                List<ClassifyGroup> temData = dataG.get(layer);
                if(!CollectionUtils.isEmpty(temData)){
                    selectData = temData.parallelStream().map(valOne->valOne.getClassifyId()).collect(Collectors.toSet());
                }
                List<ClassifyGroup> temDataThree = dataG.get(Integer.valueOf(3));
                if(!CollectionUtils.isEmpty(temDataThree)){
                    EntityWrapper<Classify> ewThree = QueryUtil.condition(new Classify());
                    ewThree.in("id",temDataThree.parallelStream().map(item->item.getClassifyId()).collect(Collectors.toList()));
                    // 查询第三层的父id(第二层id）
                    List<Classify> temDataThreeData = classifyService.selectList(ewThree);
                    if(!CollectionUtils.isEmpty(temDataThreeData)){
                        // 添加第三层的id到全部的第二层id集合
                        selectData.addAll(temDataThreeData.parallelStream().map(item->item.getParentId()).collect(Collectors.toSet()));
                    }
                }
            }else if(layer.intValue() == 3){
                List<ClassifyGroup> temData = dataG.get(layer);
                if(!CollectionUtils.isEmpty(temData)){
                    selectData = temData.parallelStream().map(valOne->valOne.getClassifyId()).collect(Collectors.toSet());
                }
            }
            //data.put(CLASSIFY_GROUP_DATA_KEY, selectData);
        }

        List<GroupClassifyItemVO> data = null;
        if( !CollectionUtils.isEmpty(classifyData) ){
            data = new ArrayList<>();
            if(CollectionUtils.isEmpty(selectData)){
                selectData = new HashSet<>();
            }
            for(Classify objItem : classifyData){
                GroupClassifyItemVO itemVO = new GroupClassifyItemVO();
                itemVO.setClassify_id(objItem.getId());
                itemVO.setClassify_name(objItem.getName());
                itemVO.setClassifyParentId(objItem.getParentId());
                itemVO.setLayer(objItem.getLayer());
                itemVO.setIsSelectd(selectData.contains(objItem.getId()));
                data.add(itemVO);
            }

        }

        return data;
    }
}
