package com.yizhi.course.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.domain.ClassifyGroup;
import com.yizhi.course.application.domain.ClassifyGroupInfo;
import com.yizhi.course.application.service.IClassifyGroupInfoService;
import com.yizhi.course.application.service.IClassifyGroupService;
import com.yizhi.course.application.service.IClassifyService;
import com.yizhi.course.application.vo.ClassifyGroupStudentVO;
import com.yizhi.course.application.vo.domain.ClassifyEntityVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程分类组基本信息表 前端控制器
 * </p>
 *
 * @author xiehaijun
 * @since 2019-12-13
 */
@RestController
@RequestMapping("/classify/group/info")
public class ClassifyGroupInfoController {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IClassifyGroupInfoService classifyGroupInfoService;

    @Autowired
    private IClassifyGroupService classifyGroupService;

    @Autowired
    private IClassifyService classifyService;

    @PostMapping("/save")
    public ClassifyGroupInfo save(@RequestBody ClassifyGroupInfo classifyGroupInfo){
        // 上下文下信息
        RequestContext rc = ContextHolder.get();

        if(Objects.isNull(classifyGroupInfo.getId())){
            classifyGroupInfo.setCompanyId(rc.getCompanyId());
            classifyGroupInfo.setSiteId(rc.getSiteId());
            classifyGroupInfo.setOrgId(rc.getOrgId());

            classifyGroupInfo.setId(idGenerator.generate());
            classifyGroupInfo.setCreateTime(new Date());
            classifyGroupInfo.setCreateById(rc.getAccountId());
            classifyGroupInfoService.insert(classifyGroupInfo);
        }else {
            classifyGroupInfo.setUpdateTime(new Date());
            classifyGroupInfo.setUpdateById(rc.getAccountId());


            if(Integer.valueOf(1).equals(classifyGroupInfo.getStatus())){
                EntityWrapper<ClassifyGroup> ew = QueryUtil.condition(new ClassifyGroup());
                ew.eq("classify_group_info_id",classifyGroupInfo.getId());
                //ew.eq("status",1).eq("deleted",0);
                if(classifyGroupService.selectCount(ew)==0){
                    return null;
                }
            }else {
                ClassifyGroupInfo classifyGroupInfoDb = classifyGroupInfoService.selectById(classifyGroupInfo.getId());
                Integer templateType = classifyGroupInfo.getTemplateType();
                Integer templateSubType = classifyGroupInfo.getTemplateSubType();
                boolean changeTemplate = false;
                if(templateType!=null && (!classifyGroupInfoDb.getTemplateType().equals(templateType))){
                    changeTemplate = true;
                }else if(templateType!=null && templateSubType!= null &&
                        templateType.intValue()==2 && !classifyGroupInfoDb.getTemplateSubType().equals(templateSubType)){
                    changeTemplate = true;
                }
                if(changeTemplate){
                    EntityWrapper<ClassifyGroup> delEw = QueryUtil.condition(new ClassifyGroup());
                    delEw.eq("classify_group_info_id",classifyGroupInfo.getId());
                    classifyGroupService.delete(delEw);
                }else {
                    classifyGroupInfo.setStatus(null);
                    // 不能修改分组模板
                    classifyGroupInfo.setTemplateType(null);
                    classifyGroupInfo.setTemplateSubType(null);
                }
            }
            classifyGroupInfoService.updateById(classifyGroupInfo);
        }
        return classifyGroupInfo;
    }

    @GetMapping("/list")
    public Page<ClassifyGroupInfo> list(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize,
                                        @RequestParam(name="status", required = false ) Integer status){
        // 上下文下信息
        RequestContext rc = ContextHolder.get();

        // 分级授权修改
        HQueryUtil.startHQ(ClassifyGroupInfo.class);
        EntityWrapper<ClassifyGroupInfo> ew = QueryUtil.condition(new ClassifyGroupInfo());
        ew.eq("company_id", rc.getCompanyId()).eq("site_id", rc.getSiteId());
        if(Objects.nonNull(status)){
            ew.eq("status", status);
        }
        ew.eq("deleted",false); // 未删除的数据
        ew.orderBy("id",false);
        Page<ClassifyGroupInfo> page = new Page<ClassifyGroupInfo>(pageNo, pageSize);
        return classifyGroupInfoService.selectPage(page, ew);
    }

    @GetMapping("/list/all")
    public List<ClassifyGroupInfo> listAll(@RequestParam("companyId") Long companyId,
                                           @RequestParam("siteId") Long siteId){

        EntityWrapper<ClassifyGroupInfo> ew = QueryUtil.condition(new ClassifyGroupInfo());
        ew.eq("company_id", companyId).eq("site_id", siteId);
        ew.eq("status", 1);
        ew.eq("deleted",false); // 未删除的数据
        ew.orderBy("id",false);
        return classifyGroupInfoService.selectList(ew);
    }

    @GetMapping("/get")
    public ClassifyGroupInfo get(@RequestParam(name = "id") Long id){
        return classifyGroupInfoService.selectById(id);
    }

    @GetMapping("/delete")
    public boolean delete(@RequestParam(name = "id") Long id){
        //EntityWrapper<ClassifyGroup> ew = QueryUtil.condition(new ClassifyGroup());
        //ew.eq("classify_group_info_id",id);
        //classifyGroupService.delete(ew);
        //return classifyGroupInfoService.deleteById(id);

        // 上下文下信息
        RequestContext rc = ContextHolder.get();
        ClassifyGroup classifyGroup = new ClassifyGroup();
        classifyGroup.setClassifyGroupInfoId(id);
        classifyGroupService.delete(QueryUtil.condition(classifyGroup));

        ClassifyGroupInfo cgi = new ClassifyGroupInfo();
        cgi.setId(id);
        cgi.setDeleted(true);
        cgi.setUpdateTime(new Date());
        cgi.setUpdateById(rc.getAccountId());
        return classifyGroupInfoService.updateById(cgi);
    }


    @GetMapping("/view")
    public ClassifyGroupStudentVO view(@RequestParam(name = "id") Long id){

        ClassifyGroupStudentVO classifyGroupStudentVO = new ClassifyGroupStudentVO();
        ClassifyGroupInfo classifyGroupInfo =classifyGroupInfoService.selectById(id);
        BeanUtils.copyProperties(classifyGroupInfo, classifyGroupStudentVO);

        // 查询该分类组的全部关联信息
        EntityWrapper<ClassifyGroup> ewCG = QueryUtil.condition(new ClassifyGroup());
        ewCG.eq("classify_group_info_id", id); // 分类组id
        List<ClassifyGroup> classifyGroups = classifyGroupService.selectList(ewCG);

        int templateType = classifyGroupInfo.getTemplateType();

        Set<Long> selectData = new HashSet<>();

        if(!CollectionUtils.isEmpty(classifyGroups)) {
            if(templateType==1){ // 如果是平铺，直接返回各个分类的id信息
                selectData = classifyGroups.parallelStream().map(item->item.getClassifyId()).collect(Collectors.toSet());
            }else{

                Map<Integer, List<ClassifyGroup>> dataG = classifyGroups.parallelStream()
                        .collect(Collectors.groupingBy(gKey -> gKey.getLayer()));
                for(Map.Entry<Integer, List<ClassifyGroup>> item : dataG.entrySet()){
                    int key = item.getKey();
                    List<ClassifyGroup> dataTem = item.getValue();
                    // 每次的数据
                    Set<Long> tmpDataItem = dataTem.parallelStream().map(item1->item1.getClassifyId()).collect(Collectors.toSet());
                    selectData.addAll(tmpDataItem);
                    if(key==2 || key==3){
                        // 找出第二层的第一层数据
                        EntityWrapper<Classify> ewParent = QueryUtil.condition(new Classify());
                        ewParent.in("id",tmpDataItem);
                        // 查询全部的第二层的id
                        Set<Long> parentData = null;
                        List<Classify> temDataParent = classifyService.selectList(ewParent);
                        if(!CollectionUtils.isEmpty(temDataParent)){
                            parentData = temDataParent.parallelStream().map(item2->item2.getParentId()).collect(Collectors.toSet());
                            selectData.addAll(parentData);
                        }
                        if(key==3){
                            EntityWrapper<Classify> ewParentDouble = QueryUtil.condition(new Classify());
                            ewParentDouble.in("id",parentData);
                            List<Classify> parentDataDouble = classifyService.selectList(ewParentDouble);
                            if(!CollectionUtils.isEmpty(parentDataDouble)){
                                selectData.addAll(parentDataDouble.parallelStream().map(item3->item3.getParentId()).collect(Collectors.toSet()));
                            }
                        }
                    }
                }
            }
        }
        // 查询分类信息
        EntityWrapper<Classify> allEw = QueryUtil.condition(new Classify());
        allEw.in("id", selectData);

        List<Classify> list = classifyService.selectList(allEw);
        List<ClassifyEntityVo> vos = new ArrayList<>();
        for (Classify classify : list) {
            ClassifyEntityVo vo = new ClassifyEntityVo();
            BeanUtils.copyProperties(classify,vo);
            vos.add(vo);
        }
        classifyGroupStudentVO.setClassifyData(vos);

        return classifyGroupStudentVO;
    }


}
