package com.yizhi.course.application.controller;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.event.LogEvent;
import com.yizhi.core.application.log.LogEventPublisher;
import com.yizhi.core.application.log.LogQueue;
import com.yizhi.core.application.log.TaskLogEvent;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.vo.AuthorizationCourseVO;
import com.yizhi.core.application.vo.AuthorizeVO;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.service.IAccreditService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.vo.CourseVo;
import com.yizhi.system.application.system.remote.CompanyClient;
import com.yizhi.system.application.vo.SiteVO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @author  lilingye
 * @date 2018年7月19日10:16:35
 */
@Component
@Log4j2
public class CourseAuthorizeTask extends AbstractTaskHandler {
    @Autowired
    private ICourseService courseService;
    @Autowired
    private IAccreditService accreditService;

    @Autowired
    private CompanyClient companyClient;

    @Autowired
    LogEventPublisher publisher;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void handle() {
        AuthorizeVO authorizeVO = this.taskContext.getAuthorizeVO();
        RequestContext context = this.taskContext.getContext();
        Long taskId = this.taskContext.getTaskId();
//        // 发送消息
        TaskContext taskContextTemp = new TaskContext(taskId,"课程运营", context.getAccountId(), new Date(),context.getSiteId(),context.getCompanyId());
        TaskLogEvent taskLogEvent = TaskLogEvent.working(taskContextTemp);
        LogEvent<TaskLogEvent> event = new LogEvent<>(LogQueue.TASK_LOG_QUEUE, taskLogEvent);
        publisher.publish(event);

        // 公司的顶级站点
        Long siteId = null;
        SiteVO siteVO = null;
        if (authorizeVO.getAccreditVO().getCompanyId() != null && !"".equals(authorizeVO.getAccreditVO().getCompanyId())) {
            siteVO = companyClient.getTopSite(authorizeVO.getAccreditVO().getCompanyId());
//            siteId = siteVO.getId();

            siteId=authorizeVO.getAccreditVO().getSiteId();
            log.info("被授权的站点Id："+siteId);
            if(siteId!=null){
                siteVO.setId(siteId);
            }
        }

        CourseVo courseVO = new CourseVo();

        try {
            for (AuthorizationCourseVO authCourseVO : authorizeVO.getAuthorizationCourseVOs()) {
                // 课程id
                Long id = authCourseVO.getCourseId();
                // 课程有效课程
                Date validDate = authCourseVO.getValidDate();
                log.info("课程id："+ id);
                Date sysDate = new Date();
                Course course = courseService.selectById(id);
                // 复制新生成一条记录
                Course newCourse = courseService.copyCourse(course, authorizeVO.getAccreditVO().getCompanyId(), siteId, authorizeVO.getAccreditVO().getType(),context);

                if (newCourse==null){
                    // 失败任务发送
                    TaskLogEvent taskLogEvent1 = TaskLogEvent.fail(taskContextTemp, "执行失败",context.getSiteId(),context.getCompanyId());
                    LogEvent<TaskLogEvent> event1 = new LogEvent<>(LogQueue.TASK_LOG_QUEUE, taskLogEvent);
                    publisher.publish(event);

                } else {
                    // 设置有效期时间
                    if (validDate!=null){
                        (authorizeVO.getAccreditVO()).setUseType(1);
                        (authorizeVO.getAccreditVO()).setUseDuration(validDate);
                    } else {
                        (authorizeVO.getAccreditVO()).setUseType(0);
                    }
                    accreditService.saveAuthorizeCourse(authorizeVO.getAccreditVO(), newCourse, id);
                }

            }


        } catch (Exception e) {
            log.info("");
        }

        TaskLogEvent taskLogEvent1 = TaskLogEvent.success(taskContextTemp,"执行成功",context.getSiteId(),context.getCompanyId());
        LogEvent<TaskLogEvent> event1 = new LogEvent<>(LogQueue.TASK_LOG_QUEUE, taskLogEvent1);
        publisher.publish(event1);
    }
}
