package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.CourseAccount;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.ICourseAccountService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.util.CourseEvenSendMessage;
import com.yizhi.course.application.vo.BathEditCourseVo;
import com.yizhi.course.application.vo.CourseAccountVO;
import com.yizhi.course.application.vo.CourseShelvesVo;
import com.yizhi.course.application.vo.CourseVo;
import com.yizhi.message.application.enums.EvenType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 课程批量修改
 */
@RestController
@RequestMapping("/bath/edit")
public class CourseBathEditController {

    @Autowired
    private ICourseService courseService;

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private ICourseAccountService courseAccountService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private CourseEvenSendMessage courseEvenSendMessage;

    private static final String[] IGNORE_PROPERTIES=new String[]{"name","code","image","authorUnit","description","courseType",
            "tags","createById","createByName","createTime","releaseTime","shelves","companyId","orgId","siteId","classifyId",
            "source","delFlg","readCount"};

    /**
     * 课程分类批量修改
     * @param vo 请求参数
     * @return
     */
    @PostMapping("/classify")
    public Boolean courseClassifyEdit(@RequestBody BathEditCourseVo vo){
        List<Long> editObject = vo.getEditObject();
        CourseVo courseVo = vo.getEditFields();
        Long classifyId = courseVo.getClassifyId();

        Long accountId = vo.getUpdateByaccountId();
        List<Course> courses = new ArrayList<>();
        for (Long cid : editObject) {
            Course item = new Course();
            item.setId(cid);
            item.setClassifyId(classifyId);
            item.setUpdateById(accountId);
            item.setUpdateTime(new Date());
            courses.add(item);
        }
        courseService.updateBatchById(courses);
        return true;
    }

    /**
     * 课程批量上下架检查
     * @param vo 请求参数
     * @return
     */
    @PostMapping("/shelves/check")
    public List<Map<String,Object>> courseShelvesCheck(@RequestBody CourseShelvesVo vo){
        int shelves = vo.getShelves();

        List<Map<String,Object>> ret = new ArrayList<>();
        if(shelves == 0){
            ret = getCourseInfoMsg(vo.getBindCourseIds());
        }else if(shelves == 1){
            List<Long> ids = vo.getCourseIds();
            List<Long> data = chapterService.dataLack(ids);
            if(CollectionUtils.isEmpty(data)){
                ret = getCourseInfoMsg(ids);
            }else if(data.size()<ids.size()){
                List<Long> noCorss = ids.stream().filter(courseId->!data.contains(courseId)).collect(Collectors.toList());
                ret = getCourseInfoMsg(noCorss);
            }
        }

        return ret;
    }

    private List<Map<String,Object>> getCourseInfoMsg(List<Long> msgCourseIds){
        if(msgCourseIds == null){
            return null;
        }
        List<Map<String,Object>> ret = new ArrayList<>();
        EntityWrapper<Course> ew = QueryUtil.condition(new Course());
        ew.in("id",msgCourseIds);
        List<Course> data = courseService.selectList(ew);
        if(CollectionUtils.isEmpty(data)){
            return null;
        }
        for(Course obj : data){
            Map<String,Object> itemObj = new HashMap<>();
            itemObj.put("code",obj.getCode());
            itemObj.put("id",obj.getId());
            ret.add(itemObj);
        }

        return ret;
    }

    /**
     * 课程批量上下架
     * @param vo 请求参数
     * @return
     */
    @PostMapping("/shelves")
    public boolean courseShelvesEdit(@RequestBody BathEditCourseVo vo){
        List<Long> editObject = vo.getEditObject();
        CourseVo courseVo = vo.getEditFields();
        Integer shelves = courseVo.getShelves();
        EntityWrapper<Course> excludes = QueryUtil.condition(new Course());

        excludes.in("id",editObject);
        if(shelves.intValue()==0){
            excludes.ne("shelves",2);
        }
        List<Course> excludesData = courseService.selectList(excludes);
        //

        if(!CollectionUtils.isEmpty(excludesData)){
            Long accountId = vo.getUpdateByaccountId();
            List<Course> courses = new ArrayList<>();
            for (Course cid : excludesData) {
                Course item = new Course();
                item.setId(cid.getId());
                item.setShelves(shelves);
                item.setUpdateById(accountId);
                item.setUpdateTime(new Date());
                courses.add(item);
            }
            if (courseService.updateBatchById(courses)){
                if (!CollectionUtils.isEmpty(excludesData)){
                    for (Course course:excludesData){
                        taskExecutor.asynExecute(new AbstractTaskHandler() {
                            @Override
                            public void handle() {
                                courseEvenSendMessage.evenSendMessage(course,null, EvenType.COURSE_UP);
                            }
                        });
                    }
                }
            }
            return true;
        }else {
            return false;
        }

    }

    /**
     * 课程策略修改--确认检查(返回已上架的课程)
     * @param ids 请求参数
     * @return
     */
    @PostMapping("/strategy/check")
    public List<Course> courseStrategyCheck(@RequestBody List<Long> ids){
        EntityWrapper<Course> ew = QueryUtil.condition(new Course());
        ew.in("id",ids).eq("shelves",1);//查询上架的
        return courseService.selectList(ew);
    }

    /**
     * 课程策略修改
     * @param vo
     * @return
     */
    @PostMapping("/strategy")
    public Boolean courseStrategyEdit(@RequestBody BathEditCourseVo vo){
        RequestContext rt = ContextHolder.get();
        List<Long> editObject = vo.getEditObject();
        CourseVo courseVo = vo.getEditFields();
        Integer scope = courseVo.getScope();
        List<CourseAccountVO> courseAccountVOS = vo.getCourseAccounts();
        Long accountId = vo.getUpdateByaccountId();
        List<Course> courses = new ArrayList<>();

        List<CourseAccount> courseAccounts = new ArrayList<>();
        EntityWrapper<Course> ewtype = QueryUtil.condition(new Course());
        ewtype.in("id",editObject);
        ewtype.eq("course_type",vo.getCourseType());
        List<Course> redata = courseService.selectList(ewtype);
        if(!CollectionUtils.isEmpty(redata)){
            for (Course cid : redata) {
                Course item = new Course();
                courseVo.setId(cid.getId());
                BeanUtils.copyProperties(courseVo,item,IGNORE_PROPERTIES);
                item.setUpdateById(accountId);
                item.setUpdateTime(new Date());
                courses.add(item);
                // 更新可见范围
                if(Objects.nonNull(scope) && scope.intValue()==2){
                    // 指定范围信息
                    for(CourseAccountVO obj:courseAccountVOS){
                        obj.setId(idGenerator.generate());
                        CourseAccount courseAccountObj = new CourseAccount();
                        BeanUtils.copyProperties(obj,courseAccountObj);
                        courseAccountObj.setCourseId(cid.getId());
                        courseAccountObj.setSiteId(rt.getSiteId());
                        courseAccounts.add(courseAccountObj);
                    }
                }
            }
            if(Objects.nonNull(scope) && !CollectionUtils.isEmpty(courseAccounts)) {
                // 删除存在的指定访问
                //Set<Long> datas = courseAccountVOS.stream().map(item->item.getCourseId()).collect(Collectors.toSet());
                EntityWrapper<CourseAccount> deleteEw = QueryUtil.condition(new CourseAccount());
                deleteEw.in("course_id", courseAccounts.stream().parallel().map(CourseAccount::getCourseId).collect(Collectors.toSet()));
                courseAccountService.delete(deleteEw);
                if(scope.intValue()==2){
                    courseAccountService.insertBatch(courseAccounts);
                }
            }

            courseService.updateBatchById(courses);
        }else {
            return false;
        }

        return true;
    }

}
