package com.yizhi.course.application.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.enums.SqlLike;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.mapper.Wrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.DomainConverter;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.cache.distributedlock.impl.RedisDistributedLock;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.log.LogEventPublisher;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.core.application.vo.AuthorizeChapterVO;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.mapper.MaterialMapper;
import com.yizhi.course.application.mapper.StatisticsCourseMapper;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.service.impl.TestServiceImpl;
import com.yizhi.course.application.task.CourseListExportAsync;
import com.yizhi.course.application.util.CourseEvenSendMessage;
import com.yizhi.course.application.vo.*;
import com.yizhi.message.application.enums.EvenType;
import com.yizhi.site.application.enums.FunctionTypeCode;
import com.yizhi.site.application.feign.api.FunctionDisplayConfigApiClients;
import com.yizhi.site.application.vo.domain.FunctionDisplayConfigVo;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.SiteVO;
import com.yizhi.util.application.date.DateUtil;
import com.yizhi.util.application.domain.BizResponse;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.log4j.Log4j2;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程表 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@RestController
@RequestMapping("/course")
@Log4j2
public class CourseController {

    private final static String SCORM_DISTRIBUTED_LOCK = "distributed.lock:scorm";

    @Autowired
    IdGenerator idGenerator;
    @Autowired
    CourseAuthorizeTask courseAuthorizeTask;
    @Autowired
    TaskExecutor taskExecutor;
    @Autowired
    LogEventPublisher publisher;
    @Autowired
    private ICourseService courseService;
    @Autowired
    private IChapterService chapterService;
    @Autowired
    private IMaterialService materialService;
    @Autowired
    private ICourseGradeService courseGradeService;
    @Autowired
    private IFavoriteService favoriteService;
    @Autowired
    private IRecordeService recordeService;
    @Autowired
    private ICourseAccountService courseAccountService;
    @Autowired
    private IScormService scormService;
    @Autowired
    private IAccreditService accreditService;

    @Autowired
    private StatisticsCourseMapper statisticsCourseMapper;

    @Autowired
    private ICourseCountService iCourseCountService;

    @Autowired
    private ChapterAddTask chapterAddTask;

    @Autowired
    private ChapterDeleteTask chapterDeleteTask;

    @Autowired
    private ChapterUpdateTask chapterUpdateTask;

    @Autowired
    ChapterSortTask chapterSortTask;

    @Autowired
    SiteClient siteClient;

    @Autowired
    ITestService testService;

    @Autowired
    ITestCaseService testCaseService;

    @Autowired
    MaterialMapper materialMapper;

    @Autowired
    DomainConverter converter;

    @Autowired
    IDocPagesService docPagesService;

    @Autowired
    IAudioVideoService iAudioVideoService;

    @Autowired
    IRichMediaService iRichMediaService;

    @Autowired
    IScormRecordService scormRecordService;

    @Autowired
    IOfflineCourseService offlineCourseService;

    @Autowired
    FunctionDisplayConfigApiClients configApiClients;

    @Autowired
    CourseListExportAsync courseListExportAsync;

    @Autowired
    IClassifyService classifyService;

    @Autowired
    private RedisDistributedLock distributedLock;

    @Autowired
    CourseEvenSendMessage courseEvenSendMessage;

    @Autowired
    private ILastViewChapterService lastViewChapterService;

    @Autowired
    private ICourseStudentStatusService studentStatusService;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private ILastViewRecordService lastViewRecordService;

    private Logger LOGGER = LoggerFactory.getLogger(CourseController.class);

    @PostMapping("/score/sum")
    public Map<Long, Integer> getCourseScoreByIds(@RequestBody List<Long> ids) {
        List<ChapterVo> data = chapterService.getCourseScoreByIds(ids);
        Map<Long, Integer> ret = null;
        if (org.apache.commons.collections4.CollectionUtils.isNotEmpty(data)) {
            ret = new HashMap<>();
            for (ChapterVo obj : data) {
                ret.put(obj.getCourseId(), obj.getScore());
            }
        }
        return ret;
    }

    /**
     * 检查讲师的关联课程信息有效的信息
     *
     * @param vo
     * @return
     */
    @PostMapping("/valid/filter")
    public List<LecturerCourseVO> filterValidCourse(@RequestBody LecturerCourseValidReqVO vo) {
        List<LecturerCourseVO> data = getLecturerCourseVOS(vo.getCourseType(),
                null, null, vo.getCourseIds());
        return data;
    }

    @GetMapping("/check/chapter/test/only")
    public Integer countChapterTest(@RequestParam("courseId") Long courseId,
                                    @RequestParam(value = "materialType", required = false) Integer materialType) {
        Chapter chapter = new Chapter();
        chapter.setCourseId(courseId);
        if (materialType != null) {
            chapter.setMaterialType(materialType);
        }
        return this.chapterService.selectCount(QueryUtil.condition(chapter));
    }

    /**
     * 查询站点下全部讲师课程
     *
     * @param type   1为线上课程，2为线下课程
     * @param siteId 站点id
     * @param name   名称关键字
     * @return
     */
    @GetMapping("/type/and/site/get")
    public List<LecturerCourseVO> getCoursesBysiteId(@RequestParam("type") Integer type,
                                                     @RequestParam("siteId") Long siteId,
                                                     @RequestParam(value = "name", required = false) String name) {
        List<LecturerCourseVO> data = getLecturerCourseVOS(type, siteId, name, null);
        return data;
    }

    private List<LecturerCourseVO> getLecturerCourseVOS(Integer type, Long siteId, String name,
                                                        List<Long> courseIds) {
        log.info("参数类型type：" + type);
        List<LecturerCourseVO> data = null;
        if (Integer.valueOf(1).equals(type)) {
            EntityWrapper<Course> ew = QueryUtil.condition(new Course());
            ew.eq("del_flg", 0).eq("shelves", 1)
                    .eq("valid", 1).ne("source", 9);
            if (Objects.nonNull(siteId)) {
                ew.eq("site_id", siteId);
            }
            if (!CollectionUtils.isEmpty(courseIds)) {
                ew.in("id", courseIds);
            }
            if (!StringUtils.isEmpty(name)) {
                ew.andNew().like("name", name, SqlLike.DEFAULT).or().like("tags", name, SqlLike.DEFAULT);
                /*
                ew.andNew();
                ew.like("name", name, SqlLike.DEFAULT);
                ew.or().like("tags", name, SqlLike.DEFAULT);
                */
                LOGGER.info("SqlSegment = " + ew.getSqlSegment());
            }
            List<Course> online = this.courseService.selectList(ew);
            if (!CollectionUtils.isEmpty(online)) {
                data = new ArrayList<>();
                for (Course item : online) {
                    LecturerCourseVO vo = new LecturerCourseVO();
                    vo.setCourseId(item.getId());
                    vo.setCourseName(item.getName());
                    vo.setCourseType(type);
                    vo.setImage(item.getImage());
                    vo.setDes(item.getDescription());
                    data.add(vo);
                }
            }
        } else if (Integer.valueOf(2).equals(type)) {
            EntityWrapper<OfflineCourse> ew = QueryUtil.condition(new OfflineCourse());
            ew.eq("status", 0).eq("deleted", 0);
            if (Objects.nonNull(siteId)) {
                ew.eq("site_id", siteId);
            }
            if (!CollectionUtils.isEmpty(courseIds)) {
                ew.in("id", courseIds);
            }
            if (!StringUtils.isEmpty(name)) {
                ew.andNew().like("name", name, SqlLike.DEFAULT).or().like("tags", name, SqlLike.DEFAULT);
                /*ew.like("name", name, SqlLike.DEFAULT);
                ew.or().like("tags", name, SqlLike.DEFAULT);*/
                LOGGER.info("SqlSegment = " + ew.getSqlSegment());
            }
            List<OfflineCourse> offline = this.offlineCourseService.selectList(ew);
            if (!CollectionUtils.isEmpty(offline)) {
                data = new ArrayList<>();
                for (OfflineCourse item : offline) {
                    LecturerCourseVO vo = new LecturerCourseVO();
                    vo.setCourseId(item.getId());
                    vo.setCourseName(item.getName());
                    vo.setCourseType(type);
                    vo.setImage(item.getImage());
                    vo.setDes(item.getDescription());
                    data.add(vo);
                }
            }
        }
        return data;
    }

    /**
     * 统计课程的学时
     *
     * @param courseId
     * @return
     */
    @GetMapping("/totalDuration/dispaly/info/get")
    public CourseTotalDurationInfoVO getCourseDurationInfo(@RequestParam("courseId") Long courseId,
                                                           @RequestParam("siteId") Long siteId) {
        CourseTotalDurationInfoVO data = new CourseTotalDurationInfoVO();
        data.setCourseDurationShow(false);
        data.setTrainingDurationShow(false);
        data.setDefualtTotalDuration(0f);
        List<FunctionDisplayConfigVo> configList = configApiClients.getBySiteId(siteId);
        if (!CollectionUtils.isEmpty(configList)) {
            for (FunctionDisplayConfigVo item : configList) {
                String functionType = item.getFunctionType();
                if (FunctionTypeCode.COURSE.getCode().equals(functionType)) {
                    data.setCourseDurationShow(item.getShowDisplay());
                } else if (FunctionTypeCode.TRAINING.getCode().equals(functionType)) {
                    data.setTrainingDurationShow(item.getShowDisplay());
                }
            }
        }
        Course courseInfo = courseService.selectById(courseId);
        if (courseInfo != null) {
            data.setCourseType(courseInfo.getCourseType());
            Long totalDuration = courseService.getTotalDuration(courseId);
            if (Objects.nonNull(totalDuration)) {
                long hour = totalDuration / 3600;
                long remainder = totalDuration % 3600;
                float defualtTotalDuration = (float) hour;
                if (remainder > 1800) {
                    defualtTotalDuration += 1;
                } else if (remainder > 0 && remainder <= 1800) {
                    defualtTotalDuration += 0.5;
                }
                data.setDefualtTotalDuration(defualtTotalDuration);
            }

            //回显是否开启日历任务设置
            data.setEnableTask(courseInfo.getEnableTask());
            data.setTaskStartDate(courseInfo.getTaskStartDate());
            data.setTaskEndDate(courseInfo.getTaskEndDate());

        }

        return data;
    }

    @GetMapping("/study/statistic")
    public CourseStudyStatisticVO getCourseStudyStatistic(@RequestParam("courseId") Long courseId,
                                                          @RequestParam("accountId") Long accountId) {
        Long siteId = ContextHolder.get().getSiteId();
        return recordeService.getCourseStudyStatistic(courseId, accountId, siteId);
    }

    @GetMapping("/study/recorde")
    public List<Recorde> getCourseStudyRecorde(@RequestParam("courseId") Long courseId,
                                               @RequestParam("accountId") Long accountId) {
        Long siteId = ContextHolder.get().getSiteId();
        EntityWrapper<Recorde> ew = QueryUtil.condition(new Recorde());
        ew.eq("course_id", courseId).eq("account_id", accountId).eq("site_id", siteId);
//                .gt("duration", 0);
        ew.orderBy("start_time", false);
//        return recordeService.selectList(ew);
        List<Recorde> recordes = recordeService.selectList(ew);
        if (CollectionUtils.isEmpty(recordes)) {
            return recordes;
        }
        Iterator<Recorde> iterator = recordes.iterator();
        while (iterator.hasNext()) {
            Recorde r = iterator.next();
            if (r.getDuration() > 0) {
                continue;
            }
            if (r.getEndTime().getTime() - r.getStartTime().getTime() > 0) {
                int duration = (int) ((r.getEndTime().getTime() - r.getStartTime().getTime())/1000);
                r.setDuration(duration);
                r.updateById();
                continue;
            }
            iterator.remove();
        }
        return recordes;
    }

    /***
     *  课程列表
     * @param map
     * @return
     */
    @GetMapping("/list")
    public Page<Course> list(@RequestBody Map<String, Object> map) {
        Integer pageNo = Integer.valueOf(map.get("pageNo").toString());
        Integer pageSize = Integer.valueOf(map.get("pageSize").toString());
        Long companyId = null;
        Long siteId = null;
        List<Long> orgIds = null;
        if (map.containsKey("companyId")) {
            companyId = Long.valueOf(map.get("companyId").toString());
        }
        if (map.containsKey("siteId")) {
            siteId = Long.valueOf(map.get("siteId").toString());
        }
        if (map.containsKey("orgIds")) {
            orgIds = (List<Long>) map.get("orgIds");
        }
        Long classifyId = null;
        if (map.get("classifyId") != null) {
            classifyId = Long.valueOf(map.get("classifyId").toString());
        }
        Integer shelves = null;
        if (map.get("shelves") != null) {
            shelves = Integer.valueOf(map.get("shelves").toString());
        }
        String name = null;
        if (map.get("name") != null && map.get("name") != "") {
            name = map.get("name").toString();
        }
        Integer type = null;
        if (map.get("type") != null && map.get("type") != "") {
            type = Integer.valueOf(map.get("type").toString());
        }
        Integer courseType = null;
        if (map.get("courseType") != null && map.get("courseType") != "") {
            courseType = Integer.valueOf(map.get("courseType").toString());
        }
        List<Long> courseIds = null;
        if (map.containsKey("courseIds")) {
            courseIds = (List<Long>) map.get("courseIds");
        }
        Page<Course> page = courseService.searchPageByClassifyAllMg(name, classifyId, shelves,
                companyId, siteId, orgIds, pageNo, pageSize, type, courseType, courseIds);
        return page;
    }

    /**
     * 经验池课程列表
     *
     * @param name
     * @param ids
     * @param pageNo
     * @param pageSize
     * @return
     */
    @PostMapping("/experiencePoolList")
    Page<Map<String, Object>> experiencePoolList(@RequestParam(value = "name", required = false) String name,
                                                 @RequestBody List<Long> ids,
                                                 @RequestParam(value = "pageNo", required = false) Integer pageNo,
                                                 @RequestParam(value = "pageSize", required = false) Integer pageSize) {
        return courseService.experiencePoolList(name, ids, pageNo, pageSize);
    }

    /**
     * 课程上架  上架时 先验证课程是否有内容
     * 即课程章节中有没有包含素材内容
     *
     * @param course1 参数中有课程主键id
     * @return 0 上架成功  1 课程没有章节内容 2 没有课程 3 业务处理失败
     */
    @PostMapping("/up")
    public Integer up(@RequestBody Course course1) {
        RequestContext context = ContextHolder.get();

        Course course = courseService.selectById(course1.getId());

        if (StringUtils.isEmpty(course)) {
            return 2;
        }

        //验证课程是否有章节内容
        Chapter chapter = new Chapter();
        chapter.setCourseId(course1.getId());
        EntityWrapper<Chapter> wrapper = new EntityWrapper<Chapter>(chapter);
        List<Chapter> chapters = chapterService.selectList(wrapper);

        if (chapters.size() == 0) {
            return 1;
        } else {
            long countNum = chapters.stream().parallel().filter(obj -> Objects.nonNull(obj.getMaterialId())).count();
            if (countNum == 0) {
                return 1;
            }
        }

        if (chapters != null && chapters.size() > 0) {
            if (chapters.size() == 1) {
                Chapter chapter1 = chapters.get(0);
                if (StringUtils.isEmpty(chapter1.getMaterialId()) || Objects.isNull(chapter1.getMaterialType())) {
                    return 1;
                }
            } else {
                Integer flag = 0;
                for (Chapter chapter1 : chapters) {
                    /*if (!StringUtils.isEmpty(chapter1.getMaterialId())) {
                        flag = 0;
                        break;
                    }*/
                    if (checkChapter(chapter1)) {
                        flag = 1;
                        break;
                    }
                }
                if (flag == 1) {
                    return 1;
                }
            }
        }

        if (course.getShelves() == 0 || course.getShelves() == 2) {
            Integer publishingTerminal = course1.getPublishingTerminal();
            if (Objects.nonNull(publishingTerminal)) {
                course.setPublishingTerminal(publishingTerminal);
            }
            course.setShelves(1);
            course.setReleaseTime(new Date());

            if (courseService.updateById(course)) {

                log.info("开始发信息");
                taskExecutor.asynExecute(new AbstractTaskHandler() {
                    @Override
                    public void handle() {
                        courseEvenSendMessage.evenSendMessage(course, null, EvenType.COURSE_UP);
                    }
                });
            }

            return 0;
        }
        return 3;
    }

    public static boolean checkChapter(final Chapter chapter) {
        if (Objects.nonNull(chapter.getMaterialId())) {
            if (Objects.isNull(chapter.getMaterialType()) || StringUtils.isEmpty(chapter.getMaterialName())) {
                return true;
            }
        }
        return false;
    }

    /**
     * 课程下架  当课程为上架状态 1 时 才下架
     *
     * @param course1 参数中有课程主键id
     * @return
     */
    @PostMapping("/down")
    public Boolean down(@RequestBody Course course1) {
        Course course = courseService.selectById(course1.getId());
        if (StringUtils.isEmpty(course)) {
            return Boolean.FALSE;
        }
        if (course.getShelves() == 1) {
            course.setShelves(0);
            courseService.updateById(course);
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }

    /**
     * 课程的第一步保存  返回课程保存的所有内容
     *
     * @param courseVo
     * @return
     */
    @PostMapping("/save")
    public CourseVo save(@RequestBody CourseVo courseVo) {
        CourseVo f = courseService.save(courseVo);
        return f;
    }

    /**
     * 课程保存 版本2
     * 为了新增多中类型的错误提示，版本v2返回值类型变更为BizResponse;
     * @param courseVo 入参
     */
    @PostMapping("/save/v2")
    public BizResponse<CourseVo> saveV2(@RequestBody CourseVo courseVo) {
        return courseService.saveV2(courseVo);
    }

    /**
     * 课程code 唯一性验证
     *
     * @param id
     * @param code
     * @return
     */
    @GetMapping("/code/vail")
    public Boolean vailCode(
            @RequestParam(value = "id", required = false) Long id,
            @RequestParam(value = "code") String code,
            @RequestParam(value = "siteId") Long siteId
    ) {
        return courseService.vailCode(id, code, siteId);
    }

    /**
     * 课程最后一步 保存
     *
     * @param courseVo
     * @return
     */
    @PostMapping("/update")
    public Boolean update(@RequestBody CourseVo courseVo) {
        Boolean f = courseService.update(courseVo);
        return f;
    }
    /**
     * 修改 版本2
     * 版本1的区别是返回值类型修改了;可以返回不同的异常提示
     */
    @PostMapping("/update/v2")
    public BizResponse<Boolean> updateV2(@RequestBody CourseVo courseVo) {
        return courseService.updateV2(courseVo);
    }

    /**
     * 课程详情
     * 包含  课程分类的回写 章节列表  课程默认的第一个素材内容和类型
     * 课程平均分  我的评分  是否收藏  课程学习时长
     *
     * @param id
     * @param accountId
     * @return
     */
    @GetMapping("/get")
    public CourseDetailsVo get(@RequestParam(value = "id") Long id, @RequestParam(value = "accountId") Long accountId) {
        Date startDate = new Date();
        LOGGER.info("*****************************课程详情处理开始时间{}"
                , DateUtil.format(startDate, "yyyy-MM-dd HH:mm:ss.SSS"));
        CourseDetailsVo courseDetailsVo = new CourseDetailsVo();
        CourseVo courseVo = courseService.getCourseDetails(id);
        if (courseVo == null) {
            log.error("无效的id!!!");
            return courseDetailsVo;
        }

        RequestContext rc = ContextHolder.get();

        Long siteId = rc.getSiteId();

        if (!courseVo.getSiteId().equals(siteId)) {
            courseDetailsVo.setCourseVo(courseVo);
            return courseDetailsVo;
        }

        // 获取课程的完成状态
        EntityWrapper<CourseStudentStatus> csEw = QueryUtil.condition(new CourseStudentStatus());
        csEw.eq("course_id", id).eq("account_id", accountId);
        CourseStudentStatus courseStudentStatus = studentStatusService.selectOne(csEw);
        if (courseStudentStatus != null) {
            int status = courseStudentStatus.getStudentStatus();
            courseDetailsVo.setStudentStatus(status);
            /*Boolean grantCertificate = courseVo.getGrantCertificate();
            if(status == 1 && Boolean.TRUE.equals(grantCertificate)){
                sendCertificate(courseVo.getId(),courseVo.getName(),rc.getAccountId(),rc.getSiteId(),courseVo.getCertificateStr());
                // 发放课程证书
            }*/
        } else {
            courseDetailsVo.setStudentStatus(-1);
        }
        // 设置证书的id列表
        if (courseVo.getGrantCertificate()) {
            String certificateStr = courseVo.getCertificateStr();
            String[] certificateArr = certificateStr.split(",");
            List<CourseCertificateVo> data = new ArrayList<>();
            for (String str : certificateArr) {
                CourseCertificateVo courseCertificateVo = new CourseCertificateVo();
                courseCertificateVo.setId(Long.valueOf(str));
                data.add(courseCertificateVo);
            }
            courseVo.setCertificates(data);
        }
        boolean courseVideoHorseLamp = false;
        // 获取课时显示配置
        boolean totalDurationShow = false;
        List<FunctionDisplayConfigVo> dataFunction = configApiClients.getConfigBySiteId(siteId);
        LOGGER.info("查询的配置信息={}", JSON.toJSONString(dataFunction));
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(dataFunction)) {
            for (FunctionDisplayConfigVo item : dataFunction) {

                if (FunctionTypeCode.VIDEO_HORSE_LAMP.getCode()
                        .equals(item.getFunctionType())) {
                    courseVideoHorseLamp = item.getShowDisplay();
                }
                if (FunctionTypeCode.COURSE.getCode()
                        .equals(item.getFunctionType())) {
                    totalDurationShow = item.getShowDisplay();
                }
            }
        }
        courseVo.setVideoHorseLamp(courseVideoHorseLamp);
        //如果学时没有设置，默认为0
        if (courseVo.getTotalDuration() == null) {
            courseVo.setTotalDuration(0.0f);
        }


        if (courseVo != null) {
            courseDetailsVo.setCourseVo(courseVo);

            //课程平均分
            Double level = courseService.getAvgLevel(id);
            if (level != null) {
                courseDetailsVo.setLevel(level.toString());
            } else {
                courseDetailsVo.setLevel("5.0");
            }

            //我的评分
            String myLevel = "0";
            CourseGrade courseGrade = new CourseGrade();
            courseGrade.setAccountId(accountId);
            courseGrade.setCourseId(id);
            EntityWrapper<CourseGrade> wrapper = new EntityWrapper<CourseGrade>(courseGrade);
            CourseGrade courseGrade1 = courseGradeService.selectOne(wrapper);
            if (!StringUtils.isEmpty(courseGrade1)) {
                myLevel = courseGrade1.getLevel() + "";
            }
            courseDetailsVo.setMyLevel(myLevel);

            //是否收藏
            Favorite favorite = new Favorite();
            favorite.setCourseId(id);
            favorite.setAccountId(accountId);
            EntityWrapper<Favorite> fWrapper = new EntityWrapper<Favorite>(favorite);
            List<Favorite> favoriteList = favoriteService.selectList(fWrapper);
            if (null != favoriteList && favoriteList.size() > 0) {
                courseDetailsVo.setIsFavorite("1");
            } else {
                courseDetailsVo.setIsFavorite("0");
            }

            //课程的学习时长
            Integer d = recordeService.selectDurationByCourseId(id);
            if (StringUtils.isEmpty(d)) {
                courseDetailsVo.setDuration(0);
            } else {
                courseDetailsVo.setDuration(d);
            }

            //点击查看新增课程浏览人次，也就是点击查看课程是两个动作，新增一条浏览记录和查询浏览记录数
            try {
                //点击查看新增课程浏览人次
                ViewCourseCount viewCourseCount = new ViewCourseCount();
                viewCourseCount.setId(idGenerator.generate());
                viewCourseCount.setCourseId(courseVo.getId());
                viewCourseCount.setAccountId(accountId);
                viewCourseCount.setTime(new Date());
                viewCourseCount.setSiteId(siteId);
                viewCourseCount.setCompanyId(rc.getCompanyId());
                iCourseCountService.insert(viewCourseCount);
            } catch (Exception e) {
                log.error("新增课程浏览人次异常;异常信息如下:", e);
            }
            //课程浏览人次
           /* Integer count = courseService.courseViewCount(id, accountId);
            //如果为空设0，没人浏览(如果数据未查到，应为N+1)
            if (StringUtils.isEmpty(count)) {
                courseDetailsVo.setViewCount(1);
            } else {
                courseDetailsVo.setViewCount(count + 1);
            }*/
        }
        courseDetailsVo.getCourseVo().setTotalDurationShow(totalDurationShow);
        Date endDate = new Date();
        LOGGER.info("*****************************课程详情处理结束时间{}"
                , DateUtil.format(endDate, "yyyy-MM-dd HH:mm:ss.SSS"));
        LOGGER.info("课程详情处理总耗时{}毫秒", endDate.getTime() - startDate.getTime());
        return courseDetailsVo;
    }

    /**
     * 课程详情(后台管理)
     * 包含  课程分类的回写 章节列表  课程默认的第一个素材内容和类型
     * 课程平均分  我的评分  是否收藏  课程学习时长
     *
     * @param id
     * @param accountId
     * @return
     */
    @GetMapping("/mg/get")
    public CourseDetailsVo manageGet(@RequestParam(value = "id") Long id, @RequestParam(value = "accountId") Long accountId) {
        Date startDate = new Date();
        LOGGER.info("*****************************课程详情处理开始时间{}"
                , DateUtil.format(startDate, "yyyy-MM-dd HH:mm:ss.SSS"));
        CourseDetailsVo courseDetailsVo = new CourseDetailsVo();
        CourseVo courseVo = courseService.getCourseDetails(id);

        RequestContext rc = ContextHolder.get();
        Long siteId = rc.getSiteId();

        if (!courseVo.getSiteId().equals(siteId)) {
            courseDetailsVo.setCourseVo(courseVo);
            return courseDetailsVo;
        }
        if (Objects.nonNull(siteId) && siteId.longValue() < 1) {
            siteId = courseVo.getSiteId();
        }
        // 获取课时显示配置
        boolean courseVideoHorseLamp = false;
        List<FunctionDisplayConfigVo> dataFunction = configApiClients.getConfigBySiteId(siteId);
        LOGGER.info("查询的配置信息={}", JSON.toJSONString(dataFunction));
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(dataFunction)) {
            for (FunctionDisplayConfigVo item : dataFunction) {

                if (FunctionTypeCode.VIDEO_HORSE_LAMP.getCode()
                        .equals(item.getFunctionType())) {
                    courseVideoHorseLamp = item.getShowDisplay();
                }
            }
            /*FunctionDisplayConfig config = dataFunction.stream().filter(item-> FunctionTypeCode.VIDEO_HORSE_LAMP.getCode()
                    .equals(item.getFunctionType())).findAny().get();
            courseVideoHorseLamp = config.getShowDisplay();*/
        }
        courseVo.setVideoHorseLamp(courseVideoHorseLamp);
        if (courseVo != null) {
            courseDetailsVo.setCourseVo(courseVo);
            //课程章节内容
            List<ChapterDetailsVo> chapters = chapterService.selectChapterListUp(id, accountId);

            //课程平均分
            Double level = courseService.getAvgLevel(id);
            if (level != null) {
                courseDetailsVo.setLevel(level.toString());
            } else {
                courseDetailsVo.setLevel("5.0");
            }

            //课程分类回写
            Classify classify = classifyService.selectById(courseVo.getClassifyId());
            if (classify != null && !StringUtils.isEmpty(classify.getPath())) {

                Set<Long> idsList = new HashSet<>();
                if (classify.getRootId() != 0) {
                    idsList.add(classify.getRootId());
                }
                if (classify.getParentId() != 0) {
                    idsList.add(classify.getParentId());
                }
                idsList.add(classify.getId());

                List<Classify> classifies = classifyService.selectClassifyByPath(new ArrayList(idsList));

                StringBuilder classifyName = new StringBuilder();
                for (Classify classify1 : classifies) {
                    if ("".equals(classifyName.toString())) {
                        classifyName.append(classify1.getId());
                    } else {
                        classifyName.append(",");
                        classifyName.append(classify1.getId());
                    }
                }
                courseVo.setClassifyName(classifyName.toString());

            }

            // 查询最后的测验分数
            List<TestCaseRecordVO> testScore = testCaseService.findMaxTestScore(id, accountId);
            Map<Long, TestCaseRecordVO> testScoreMap = testScore.stream()
                    .collect(Collectors.toMap(key -> key.getTestId(), val -> val));

            // 查询最后学习的章节
            //long lastStudyChapter = recordeService.selectLastStudy(id, accountId);

            // 获取素材的详细信息
            Set<Long> materialIds = chapters.stream().filter
                    (mid -> Objects.nonNull(mid) && Objects.nonNull(mid.getMaterialId())).
                    map(chapterId -> chapterId.getMaterialId()).collect(Collectors.toSet());
            List<MaterialVo> materialDetails = materialService.getMaterialDetails(materialIds);
            Map<Long, MaterialVo> materialDetailMap = new HashMap<>();
            if (!CollectionUtils.isEmpty(materialDetails)) {
                materialDetailMap = materialDetails.stream().
                        collect(Collectors.toMap(key -> Long.valueOf(key.getId()), val -> val));
            }

            /*LOGGER.info("materialIds={},materialDetails={}",JSON.toJSONString(materialIds)
                    ,JSON.toJSONString(materialDetails));*/

            for (ChapterDetailsVo vo : chapters) {

                Long materialId = vo.getMaterialId();
                if (Objects.isNull(materialId)) {
                    continue;
                }
                // 完成状态
                Integer overed = vo.getOvered();
                Integer sourseObj = vo.getSource();
                int sourse = Objects.nonNull(sourseObj) ? sourseObj.intValue() : -1;
                if (sourse == 9) {
                    if (testScoreMap.containsKey(materialId)) {
                        TestCaseRecordVO tVo = testScoreMap.get(materialId);
                        vo.setTestScore(tVo.getTestScore());
                    }

                    if (overed.intValue() == 2) {
                        vo.setOvered(0);
                    } else if (overed.intValue() == 0) {
                        vo.setOvered(2);
                    }
                } else {

                    if (materialDetailMap.containsKey(materialId)) {
                        vo.setMaterialVo(materialDetailMap.get(materialId));
                    }

                    if (overed.intValue() == 2) {
                        vo.setOvered(0);
                    }
                }
                /*if (lastStudyChapter > 0 && lastStudyChapter == materialId.longValue()) {
                    vo.setIsLastStudy(true);
                }*/

            }
            //LOGGER.info("课程章节内容 查询的列表={}", JSON.toJSONString(chapters));

            courseDetailsVo.setChapterVos(chapters);

            //我的评分
            String myLevel = "0";
            CourseGrade courseGrade = new CourseGrade();
            courseGrade.setAccountId(accountId);
            courseGrade.setCourseId(id);
            EntityWrapper<CourseGrade> wrapper = new EntityWrapper<CourseGrade>(courseGrade);
            CourseGrade courseGrade1 = courseGradeService.selectOne(wrapper);
            if (!StringUtils.isEmpty(courseGrade1)) {
                myLevel = courseGrade1.getLevel() + "";
            }
            courseDetailsVo.setMyLevel(myLevel);

            //是否收藏
            Favorite favorite = new Favorite();
            favorite.setCourseId(id);
            favorite.setAccountId(accountId);
            EntityWrapper<Favorite> fWrapper = new EntityWrapper<Favorite>(favorite);
            List<Favorite> favoriteList = favoriteService.selectList(fWrapper);
            if (null != favoriteList && favoriteList.size() > 0) {
                courseDetailsVo.setIsFavorite("1");
            } else {
                courseDetailsVo.setIsFavorite("0");
            }

            //课程的学习时长
            Integer d = recordeService.selectDurationByCourseId(id);
            if (StringUtils.isEmpty(d)) {
                courseDetailsVo.setDuration(0);
            } else {
                courseDetailsVo.setDuration(d);
            }

            //点击查看新增课程浏览人次，也就是点击查看课程是两个动作，新增一条浏览记录和查询浏览记录数
            //课程浏览人次
            Integer count = courseService.courseViewCount(id, accountId);
            try {
                ViewCourseCount viewCourseCountWhere = new ViewCourseCount();
                viewCourseCountWhere.setCourseId(courseVo.getId());
                viewCourseCountWhere.setAccountId(accountId);
                int viewCourseCountDB = iCourseCountService.selectCount(QueryUtil.condition(viewCourseCountWhere));
                if (viewCourseCountDB == 0) {
                    //点击查看新增课程浏览人次
                    ViewCourseCount viewCourseCount = new ViewCourseCount();
                    viewCourseCount.setId(idGenerator.generate());
                    viewCourseCount.setCourseId(courseVo.getId());
                    viewCourseCount.setAccountId(accountId);
                    viewCourseCount.setTime(new Date());
                    viewCourseCount.setCompanyId(rc.getCompanyId());
                    viewCourseCount.setSiteId(rc.getSiteId());
                    viewCourseCount.setOrgId(rc.getOrgId());
                    iCourseCountService.insert(viewCourseCount);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            //如果为空设0，没人浏览(如果数据未查到，应为N+1)
            if (StringUtils.isEmpty(count)) {
                courseDetailsVo.setViewCount(1);
            } else {
                courseDetailsVo.setViewCount(count + 1);
            }
        }
        Date endDate = new Date();
        LOGGER.info("*****************************课程详情处理结束时间{}"
                , DateUtil.format(endDate, "yyyy-MM-dd HH:mm:ss.SSS"));
        LOGGER.info("课程详情处理总耗时{}毫秒", endDate.getTime() - startDate.getTime());
        return courseDetailsVo;
    }

    @GetMapping("/chapters/get")
    public List<ChapterDetailsVo> getChapter(@RequestParam(value = "id") Long id,
                                             @RequestParam(value = "accountId") Long accountId) {
        RequestContext rc = ContextHolder.get();
        Long siteId = rc.getSiteId();
        //课程章节内容
        List<ChapterDetailsVo> chapters = chapterService.selectChapterListUpMg(id, accountId, siteId);

        Optional<ChapterDetailsVo> anyData = chapters.stream().parallel().filter(objItem -> (
                Objects.nonNull(objItem.getMaterialId()) && Objects.isNull(objItem.getMaterialType()))).findAny();
        if (anyData.isPresent()) {
            ChapterDetailsVo retVos = new ChapterDetailsVo();
            retVos.setName(ChapterDetailsVo.NOT_EXACTLY_MSG);
            List<ChapterDetailsVo> list = Arrays.asList(retVos);
            return list;
        }
        // 查询最后的测验分数
        List<TestCaseRecordVO> testScore = testCaseService.findMaxTestScore(id, accountId);
        Map<Long, TestCaseRecordVO> testScoreMap = testScore.stream()
                .collect(Collectors.toMap(key -> key.getTestId(), val -> val));

        // 查询最后学习的章节
        //long lastStudyChapter = recordeService.selectLastStudy(id, accountId);
        long lastStudyChapter = 0;
        LastViewChapter lc = new LastViewChapter();
        lc.setCourseId(id);
        lc.setAccountId(accountId);
        lc = lastViewChapterService.selectOne(QueryUtil.condition(lc));
        if (lc != null) {
            lastStudyChapter = lc.getChapterId();
        }

        // 获取素材的详细信息
            /*Set<Long> materialIds = chapters.stream().filter
                    (mid -> Objects.nonNull(mid) && Objects.nonNull(mid.getMaterialId())).
                    map(chapterId -> chapterId.getMaterialId()).collect(Collectors.toSet());*/
        Set<Long> materialIds = new HashSet<>();
        List<Long> studyChapterIds = new ArrayList<>();
        List<Long> rootChapters = new ArrayList<>();
        for (ChapterDetailsVo vo : chapters) {
            if (Objects.nonNull(vo) && Objects.nonNull(vo.getMaterialId())) {
                materialIds.add(vo.getMaterialId());
                studyChapterIds.add(vo.getId());
            }
            if (Long.valueOf(0).equals(vo.getParentId())) {
                rootChapters.add(vo.getId());
            }
        }


        List<ChapterDetailsVo> chaptersSort = null;
        if (!CollectionUtils.isEmpty(rootChapters)) {
            chaptersSort = new ArrayList<>();
            for (Long idItem : rootChapters) {
                ChapterDetailsVo parent = chapters.stream().parallel().filter(idkey -> idkey.getId().equals(idItem)).findAny().get();
                chaptersSort.add(parent);
                if (Objects.nonNull(parent)) {
                    Long parentIdIn = parent.getId();
                    for (ChapterDetailsVo itemObj : chapters) {
                        if (parentIdIn.equals(itemObj.getParentId())) {
                            chaptersSort.add(itemObj);
                        }
                    }
                }
            }
            chapters = chaptersSort;
        }
        List<MaterialVo> materialDetails = materialService.getMaterialDetails(materialIds);
        Map<Long, MaterialVo> materialDetailMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(materialDetails)) {
            materialDetailMap = materialDetails.stream().
                    collect(Collectors.toMap(key -> Long.valueOf(key.getId()), val -> val));
        }

            /*LOGGER.info("materialIds={},materialDetails={}",JSON.toJSONString(materialIds)
                    ,JSON.toJSONString(materialDetails));*/
        // 章节最后学习点
        EntityWrapper<LastViewRecord> ew = QueryUtil.condition(new LastViewRecord());
        ew.eq("account_id", accountId);
        ew.in("chapter_id", studyChapterIds);
        List<LastViewRecord> lastViewChapterData = lastViewRecordService.selectList(ew);
        Map<Long, Long> progress = null;
        if (!CollectionUtils.isEmpty(lastViewChapterData)) {
            progress = lastViewChapterData.parallelStream().collect(Collectors.toMap(progressKey -> progressKey.getChapterId(), progressVal -> progressVal.getViewPonit()));
        }
        for (ChapterDetailsVo vo : chapters) {
            Long chapterIdTem = vo.getId();
            if (progress != null && progress.containsKey(chapterIdTem)) {
                vo.setProgress(progress.get(chapterIdTem));
            } else {
                vo.setProgress(0L);
            }

            Long materialId = vo.getMaterialId();
            if (Objects.isNull(materialId)) {
                continue;
            }
            // 完成状态
            Integer overed = vo.getOvered();
            Integer sourseObj = vo.getSource();
            int sourse = Objects.nonNull(sourseObj) ? sourseObj.intValue() : -1;
            if (sourse == 9) {
                if (testScoreMap.containsKey(chapterIdTem)) {
                    TestCaseRecordVO tVo = testScoreMap.get(chapterIdTem);
                    vo.setTestScore(tVo.getTestScore());
                }

                if (overed.intValue() == 2) {
                    vo.setOvered(0);
                } else if (overed.intValue() == 0) {
                    vo.setOvered(2);
                }
            } else {

                if (materialDetailMap.containsKey(materialId)) {
                    vo.setMaterialVo(materialDetailMap.get(materialId));
                }
                if (overed.intValue() == 2) {
                    vo.setOvered(0);
                }
            }
            if (lastStudyChapter > 0 && lastStudyChapter == materialId.longValue()) {
                vo.setIsLastStudy(true);
            }
        }
        return chapters;
    }

    private void courseTestStatus(Long accountId, ChapterDetailsVo c) {
        // 测验是否通过
        // 1 获取测验的信息
        TestInfo testInfo = new TestInfo();
        testInfo.setId(c.getMaterialId());
        TestInfo testInfoData = testService.selectOne(QueryUtil.condition(testInfo));
        LOGGER.info("获取测验信息={}", JSON.toJSONString(testInfoData));
        // 过程测验
        if (TestServiceImpl.PROCESS_TYPE.equals(testInfoData.getType())) {
            // 查看是否有测验通过的测验
            TestCase testCase = new TestCase();
            testCase.setTestId(c.getMaterialId());
            testCase.setChapterId(c.getId());
            testCase.setCourseId(c.getCourseId());
            //testCase.setStatus(1);
            testCase.setAccountId(accountId);
            List<TestCase> testCaseList = testCaseService.selectList(QueryUtil.condition(testCase));
            LOGGER.info("过程测验信息={}", JSON.toJSONString(testCaseList));
            if (CollectionUtils.isEmpty(testCaseList)) {
                c.setOvered(0);
            } else {
                c.setOvered(2);
                for (TestCase obj : testCaseList) {
                    Integer status = obj.getStatus();
                    if (Objects.nonNull(status) && status.intValue() == 1) {
                        c.setOvered(1);
                        break;
                    }
                }

            }

        } else { // 总结测验
            // 查询最高分数的测验
            Integer testScore = testCaseService.selectMaxScore(c.getCourseId(), c.getId(),
                    c.getMaterialId(), accountId);
            LOGGER.info("最高分数={}", testScore);
            c.setTestScore(testScore);
            if (testScore == null) {
                c.setOvered(0);
            } else if (testScore.intValue() < testInfoData.getPassingScore().intValue()) {
                c.setOvered(2);
            } else {
                c.setOvered(1);
            }
        }
    }


    /**
     * 课程章节列表   根据课程id查询
     *
     * @param id 课程id
     * @return
     */
    @GetMapping("/chapter/list")
    public List<Chapter> chapterList(@RequestParam(value = "id") Long id) {
        List<Chapter> chapters = chapterService.listByCourseId(id);
        return chapters;
    }


    /**
     * 章节详情
     *
     * @param id
     * @return
     */
    @GetMapping("/chapter/get")
    public Chapter chapterGet(@RequestParam(value = "id") Long id) {
        Chapter chapter = chapterService.selectById(id);
        return chapter;
    }

    /**
     * 课程章节保存
     *
     * @param chapter
     * @return
     */
    @PostMapping("/chapter/save")
    public Chapter chapterSave(@RequestBody Chapter chapter) {
        Integer sort = chapterService.getSort(chapter.getParentId());
        Long chapterId = idGenerator.generate();
        chapter.setId(chapterId);
        if (StringUtils.isEmpty(sort)) {
            chapter.setSort(1);
        } else {
            chapter.setSort(sort + 1);
        }
        if (chapter.getParentId() != null) {
            chapter.setPath(chapter.getParentId() + "," + chapter.getId());
            chapter.setLayer(2);
        } else {
            chapter.setParentId(0L);
            chapter.setPath(chapter.getId() + "");
            chapter.setLayer(1);
        }
        log.info("courseId:" + chapter.getCourseId());
        chapter.setIndexId(chapterId);
        Boolean f = chapterService.insert(chapter);

        if (f) {
            //保存成功
            log.info("保存章节成功！");
            return chapter;
        } else {
            return null;
        }
    }

    /**
     * 章节更新
     *
     * @param chapter
     * @return
     */
    @PostMapping("/chapter/update")
    public Boolean chapterUpdate(@RequestBody Chapter chapter) {
        Boolean f = chapterService.updateById(chapter);
        if (f) {
            Chapter parent = chapterService.selectById(chapter.getParentId());
            if (parent != null) {
                if (chapter.getParentId() != 0 & !StringUtils.isEmpty(chapter.getParentId())) {
                    parent.setScore(chapterService.selectSumScoreByParentId(chapter));
                    Boolean ff = chapterService.updateById(parent);
                    return ff;
                }
            }
        }
        return f;
    }

    /**
     * 章节删除
     * 注释掉的代码原因时涉及多个事务要写在service中,controller中不涉及事务注解，查询没关系
     *
     * @param chapter
     * @return
     */
    @PostMapping("/chapter/delete")
    public Boolean chapterDelete(@RequestBody Chapter chapter) {
//        //同一个事务进行处理，先获取章节对象
//        Chapter chapter1 = chapterService.selectById(chapter.getId());
//        //获取章节的父对象
//        Chapter parent = chapterService.selectById(chapter1.getParentId());
//        //先判断积分是否为空，防止空指针异常
//        if(chapter1.getScore()!=null && !StringUtils.isEmpty(chapter1.getScore())){
//            //如果章节的parentId不为0同时不为空
//            if (!StringUtils.isEmpty(chapter1.getParentId()) && !chapter1.getParentId().equals(0L)) {
//                //selectSumScoreByParentId(chapter1)根据章节的parentId获取总和再减去要删除章节的积分，设置给父章节
//                parent.setScore((chapterService.selectSumScoreByParentId(chapter1)) - chapter1.getScore());
//                //更新
//                chapterService.updateById(parent);
//            }
//        }
//        //删除，成功后完成事务，否则回滚
        Boolean f = chapterService.chapterDelete(chapter.getId());

        return f;
    }

    /**
     * 章节排序
     *
     * @param ids
     * @return
     */
    @PostMapping("/chapter/sort")
    public Boolean chapterSort(@RequestBody List<String> ids) {
        if (ids.size() != 2) {
            return Boolean.FALSE;
        } else {
            try {
                List<Chapter> list = chapterService.selectBatchIds(ids);
                Chapter chapter1 = list.get(0);
                Chapter chapter2 = list.get(1);
                Integer sort1 = chapter1.getSort();
                Integer sort2 = chapter2.getSort();
                String path1 = chapter1.getPath();
                String path2 = chapter2.getPath();

                chapter1.setSort(sort2);
                chapter1.setPath(path2);
                chapter2.setSort(sort1);
                chapter2.setPath(path1);
                chapterService.updateById(chapter1);
                chapterService.updateById(chapter2);
            } catch (Exception e) {
                e.getLocalizedMessage();
                return Boolean.FALSE;
            }
        }
        return Boolean.TRUE;
    }

    /**
     * 学员端 课程列表
     *
     * @param apiSeachCourseVo
     * @return
     */
    @GetMapping("/api/list")
    public Page<ApiListCourseVo> apiList(@RequestBody ApiSeachCourseVo apiSeachCourseVo) {
        LOGGER.info("微信端课程分类进入，查询课程列表参数={}", JSON.toJSONString(apiSeachCourseVo));
        Page<ApiListCourseVo> apiListCourseVos = courseService
                .apiList(apiSeachCourseVo.getName(), apiSeachCourseVo.getClassifyId(), apiSeachCourseVo.getType(), apiSeachCourseVo.getIds(),
                        apiSeachCourseVo.getCompanyId(), apiSeachCourseVo.getSiteId(), apiSeachCourseVo.getPageNo(), apiSeachCourseVo.getPageSize());
        return apiListCourseVos;
    }

    /**
     * 学员端 课程列表(包含子分类)
     *
     * @param apiSeachCourseVo
     * @return
     */
    @GetMapping("/api/classify/sub/list")
    public Page<ApiListCourseVo> searchPageSubClassify(@RequestBody ApiSeachCourseVo apiSeachCourseVo) {
        LOGGER.info("微信端课程分类进入，查询课程列表参数={}", JSON.toJSONString(apiSeachCourseVo));
        List<Long> classifyIds = null;
        Long classifyId = apiSeachCourseVo.getClassifyId();
        if (Objects.nonNull(classifyId) && classifyId.longValue() > 0) {
            classifyIds = new ArrayList<>();
            // 查询第一级的子分类
            Classify classify = classifyService.selectById(classifyId);
            Integer layer = classify.getLayer();
            classifyIds.add(classifyId);
            if (Objects.nonNull(layer) && layer.intValue() < 3) {
                Classify classify1 = new Classify();
                classify1.setParentId(classifyId);
                classify1.setStatus(1);
                List<Classify> firstIds = classifyService.selectList(QueryUtil.condition(classify1));
                Set<Long> lastIds = null;
                if (!CollectionUtils.isEmpty(firstIds)) {
                    lastIds = firstIds.stream().map(objfirst -> objfirst.getId()).collect(Collectors.toSet());
                    classifyIds.addAll(lastIds);
                }
                // 查询第二级的子分类
                if (Objects.nonNull(lastIds) && layer.intValue() < 2) {
                    EntityWrapper<Classify> ew = QueryUtil.condition(new Classify());
                    ew.in("parent_id", lastIds);
                    ew.eq("status", 1);
                    List<Classify> dataList = classifyService.selectList(ew);
                    if (!CollectionUtils.isEmpty(dataList)) {
                        classifyIds.addAll(dataList.stream().map(objLast -> objLast.getId()).collect(Collectors.toSet()));
                    }
                }
            }
        }
        Page<ApiListCourseVo> apiListCourseVos = courseService
                .searchPageSubClassify(apiSeachCourseVo.getName(), classifyIds
                        , apiSeachCourseVo.getType(), apiSeachCourseVo.getIds(),
                        apiSeachCourseVo.getCompanyId(), apiSeachCourseVo.getSiteId(), apiSeachCourseVo.getPageNo(), apiSeachCourseVo.getPageSize());
        return apiListCourseVos;
    }

    /**
     * 课程删除 支持批量
     *
     * @param courseDeleteVo
     * @return
     */
    @PostMapping("/delete")
    public boolean delete(@RequestBody CourseDeleteVo courseDeleteVo) {
        String[] courseIdArr = courseDeleteVo.getIds().split(",");
        if (courseIdArr.length == 1) {
            Course course = courseService.selectById(Long.valueOf(courseIdArr[0]));
            if (course.getShelves() == 1) {
                return Boolean.FALSE;
            }
        }
        Boolean flag = Boolean.FALSE;
        for (String courseId : courseIdArr) {
            Course course = courseService.selectById(Long.valueOf(courseId));
            if (course.getShelves() == 1) {
            } else {
                course.setDelFlg(1);
                courseService.updateById(course);
                flag = Boolean.TRUE;
                //删除课程，若有测验，则是需要修改测验状态的
                updateTestStatus(Long.valueOf(courseId));
            }
        }
        return flag;
    }

    /**
     * //删除课程，若有测验，则是需要修改测验状态的
     *
     * @param courseId
     */
    public void updateTestStatus(Long courseId) {
        Chapter chapter = new Chapter();
        chapter.setCourseId(courseId);
        List<Chapter> chapters = chapterService.courseChapterList(courseId);
        List<Long> chapterIds = new ArrayList<>();
        if (CollectionUtils.isEmpty(chapters)) {
            chapters.forEach(a -> {
                chapterIds.add(a.getId());
                //内容类型为测验
                if (a.getMaterialType().equals(7)) {
                    TestInfo testInfo = new TestInfo();
                    testInfo.setId(a.getMaterialId());
                    testInfo.setStatus(2);
                    testService.updateById(testInfo);
                }
            });
            chapterService.deleteBatchIds(chapterIds);
        }
    }


    /**
     * 课程指定范围保存
     *
     * @param courseAccounts
     * @return
     */
    @PostMapping("/courseAccount/save")
    public boolean saveCoursrAccount(@RequestBody List<CourseAccount> courseAccounts) {
        Boolean f = courseAccountService.saveCourseAccountList(courseAccounts);
        return f;
    }

    @PostMapping("/courseAccount/insert")
    public boolean insertCourseAccount(@RequestBody List<CourseAccount> courseAccounts) {
        Boolean f = courseAccountService.insertCourseAccountList(courseAccounts);
        return f;
    }


    /**
     * 已保存的指定范围集合
     *
     * @param courseId
     * @return
     */
    @GetMapping("/courseAccount/list")
    public List<CourseAccount> getCourseAccountList(@RequestParam(name = "courseId") Long courseId) {
        CourseAccount courseAccount = new CourseAccount();
        courseAccount.setCourseId(courseId);
        EntityWrapper<CourseAccount> wrapper = new EntityWrapper<CourseAccount>(courseAccount);
        List<CourseAccount> courseAccounts = courseAccountService.selectList(wrapper);
        return courseAccounts;
    }

    /**
     * 全文检索 课程搜索
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param orgId
     * @return
     */
    @GetMapping("/search/list")
    public List<Course> getCourseList(
            @RequestParam(name = "name") String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "orgId") Long orgId

    ) {
        List<Course> courses = courseService.selectCourseList(name, companyId, siteId, orgId);
        return courses;
    }

    /**
     * scorm 心跳更新 学时(心跳更新学时）
     *
     * @param scormStudyLog
     * @return
     */
    @PostMapping("/heart/scorm/save")
    public void saveScormHeartBeat(@RequestBody StudyLog scormStudyLog) {
        EntityWrapper<Recorde> ewR = QueryUtil.condition(new Recorde());
        ewR.eq("course_id", scormStudyLog.getCourseId());
        ewR.eq("account_id", scormStudyLog.getAccountId());
        //ewR.eq("site_id", scormStudyLog.getSiteId());
        //ewR.eq("data_type",0);
        ewR.orderBy("id", false);
        List<Recorde> oldRecord = recordeService.selectList(ewR);
        if (CollectionUtils.isEmpty(oldRecord)) {
            Date now = new Date();
            // 入库学习记录
            Recorde recordeNew = new Recorde();
            recordeNew.setId(idGenerator.generate());
            recordeNew.setAccountId(Long.valueOf(scormStudyLog.getAccountId()));
            recordeNew.setCourseId(scormStudyLog.getCourseId());
            recordeNew.setChapterId(scormStudyLog.getChapterId());
            //Calendar calendar= Calendar.getInstance();
            //calendar.add(Calendar.MINUTE, -2);
            //recordeNew.setStartTime(calendar.getTime());
            recordeNew.setStartTime(scormStudyLog.getCreateTime());
            recordeNew.setEndTime(scormStudyLog.getNginxTime());
            recordeNew.setDuration(Integer.valueOf(scormStudyLog.getTerminalDevice()));
            //recordeNew.setOvered(scormStatus);
            recordeNew.setExitstates(1);
            recordeNew.setValid(1);
            recordeNew.setCreateTime(now);
            //recordeNew.setOrgId(Long.valueOf(scormStudyLog.getOrgId()));
            //recordeNew.setCompanyId(scormStudyLog.getCompanyId());
            recordeNew.setSiteId(scormStudyLog.getSiteId());
            recordeNew.setTerminalType("新增");
            //recordeNew.setTerminalDevice("scorm学习时间:"+ DateUtil.toMinute(scormRecord.getCreateTime()));
            //recordeNew.setSessionId("scorm:"+context.getAccountId()+":"+scormRecord.getScormId());
            recordeNew.setDataType(0);
            recordeService.insert(recordeNew);
        } else {
            Recorde recordeOld = oldRecord.get(0);
            recordeService.updateByHeart(recordeOld.getId(), Integer.valueOf(scormStudyLog.getTerminalDevice()), scormStudyLog.getNginxTime());
        }
    }

    /**
     * scorm 进度 保存
     *
     * @param record
     * @return
     */
    @PostMapping("/scorm/save")
    public ScormRecord saveScorm(@RequestBody ScormRecord record) {
        String scormLockKey = SCORM_DISTRIBUTED_LOCK + ":" + record.getScormId() + "." + record.getAccountId();
        int maxRetry = 0;
        boolean courseExecLockStatus = false;

        try {
            do {
                courseExecLockStatus = distributedLock.lock(scormLockKey, 3000, 9, 300);
                LOGGER.info("{}锁的状态{}", JSON.toJSONString(record), courseExecLockStatus);
                maxRetry++;
            } while (!courseExecLockStatus && maxRetry < 2);

            return scormService.saveScormRecord(record);
        } finally {
            if (courseExecLockStatus) {
                distributedLock.releaseLock(scormLockKey);
            }
        }
    }

    /**
     * scorm 进度获取(scorm 学习记录
     *
     * @param scormId
     * @return
     */
    @GetMapping("/scorm/get")
    public ScormRecord getScorm(@RequestParam("scormId") Long scormId, @RequestParam("chapterId") Long chapterId) {
        Page<ScormRecord> data = null;
        try {
            ScormRecord record = new ScormRecord();
            //record.setScormId(scormId);
            record.setAccountId(ContextHolder.get().getAccountId());
            record.setChapterId(chapterId);
            EntityWrapper<ScormRecord> ew = new EntityWrapper<>(record);
            ew.orderBy("total_study_seconds", false);
            ew.orderBy("create_time", false);
            Page<ScormRecord> page = new Page<>(1, 1);
            data = scormRecordService.selectPage(page, ew);
        } catch (Exception e) {
            LOGGER.info("查询历史的scorm课程学习记录发生异常{}", e);
            return null;
        }
        List<ScormRecord> retList = data.getRecords();
        if (CollectionUtils.isEmpty(retList)) {
            return null;
        }
        ScormRecord record = retList.get(0);
        String jsonStr = record.getData();

        if (org.apache.commons.lang3.StringUtils.isNotBlank(jsonStr)) {
            JSONObject json = JSON.parseObject(jsonStr);
            json.replace("scormStartTime", null);
            if (!record.getScormId().equals(scormId)) {
                json.replace("cmi.core.exit", "suspend");
                json.replace("cmi.core.lesson_mode", "normal");
                json.replace("cmi.core.score._children", "raw");
                json.replace("cmi.core.lesson_status", "incomplete");
                json.replace("cmi.core.credit", "credit");
                json.replace("cmi.core.session_time", null);
            }
            record.setData(json.toJSONString());
        }

        return record;
    }

    /**
     * 离线课程下载列表
     *
     * @param ids
     * @param pageNo
     * @param pagesize
     * @return
     */
    @GetMapping("/offline/list")
    public List<ApiListCourseVo> getList(
            @RequestParam(name = "ids") String ids,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pagesize") Integer pagesize
    ) {
        if (StringUtils.isEmpty(ids)) {
            return new ArrayList<ApiListCourseVo>();
        }
        String[] courseIds = ids.split(",");
        List<Long> idList = new ArrayList<Long>();
        for (String id : courseIds) {
            try {
                idList.add(Long.valueOf(id));
            } catch (Exception e) {
                e.getLocalizedMessage();
            }
        }
        List<ApiListCourseVo> apiListCourseVos = courseService.selectOfflineCourseList(idList, pageNo, pagesize);
        return apiListCourseVos;
    }

    /**
     * 查询授权的课程列表
     *
     * @param name
     * @param classifyId
     * @param csiteId
     * @param pageNo
     * @param pageSize
     * @return
     */

    @GetMapping(value = "/get/list")
    public PageCourseAuthorizeVO queryeAuthorizeCoursesList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "companyId", required = false) Long companyId,
            @RequestParam(name = "csiteId", required = false) Long csiteId,
            @RequestParam(name = "classifyId", required = false) Long classifyId,
            @RequestParam(name = "pageNo", defaultValue = "1", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "20", required = false) Integer pageSize

    ) {
        // 站点id
        Long siteId = csiteId;

        PageCourseAuthorizeVO pageCourseAuthorizeVO = courseService.queryeAuthorizeCoursesList(name, siteId, classifyId, pageNo, pageSize);
        return pageCourseAuthorizeVO;
    }

    /**
     * 提交保存授权课程的接口
     *
     * @return
     */
    @PostMapping(value = "/insert/authorization")
    public CourseRespVO saveAuthorization(@RequestBody AuthorizeVO authorizeVO) {
        RequestContext context = ContextHolder.get();
//        String taskLogSerialNo = "CourseOpreate-" + DateUtil.getYMDHMSS();
//        Long taskId = idGenerator.generate();
//
//        TaskContext taskContext =
//                new TaskContext(taskId, taskLogSerialNo, "课程运营", context.getAccountId(), new Date(), context.getSiteId(), context.getCompanyId(), authorizeVO, context);
//        courseAuthorizeTask.bindContext(taskContext);
//        taskExecutor.asynExecute(courseAuthorizeTask);
//
//        // 发送消息
//        TaskContext taskContextTemp = new TaskContext(taskId, "课程运营", 888L, new Date(), 12L, 13L);
//        TaskLogEvent taskLogEvent = TaskLogEvent.working(taskContextTemp);
//        LogEvent<TaskLogEvent> event = new LogEvent<>(LogQueue.TASK_LOG_QUEUE, taskLogEvent);
//        publisher.publish(event);

        CourseRespVO courseRespVO = null;
        try {
            courseRespVO = saveCourseAuthorize(authorizeVO, context);
        } catch (Exception e) {
            log.info("课程授权失败：" + e);
        }

        return courseRespVO;

    }

    /**
     * 获取课程销售的列表
     *
     * @param courseKeys 关键字
     * @param startTime  开始时间
     * @param siteId     站点id
     * @param endTime    结束时间
     * @param pageNo     页码
     * @param pageSize   每页行数
     * @return
     */
    @GetMapping(value = "/get/authorized")
    public PageAuthorizedVO getAuthorizedList(
            @ApiParam(name = "courseKeys", value = "站点名称") @RequestParam(name = "courseKeys", required = false) String courseKeys,
            @ApiParam(name = "startTime", value = "开始时间") @RequestParam(name = "startTime", required = false) String startTime,
            @ApiParam(name = "endTime", value = "结束时间") @RequestParam(name = "endTime", required = false) String endTime,
            @ApiParam(name = "siteId", value = "站点id") @RequestParam(name = "siteId", required = true) Long siteId,
            @ApiParam(name = "activeType", value = "活跃状态") @RequestParam(name = "activeType") Integer activeType,
            @ApiParam(name = "type", value = "1 课程授权  2课程销售") @RequestParam(name = "type") Integer type,
            @ApiParam(name = "pageNo", value = "当前页") @RequestParam(name = "pageNo", defaultValue = "1", required = false) Integer pageNo,
            @ApiParam(name = "pageSize", value = "当前页条数") @RequestParam(name = "pageSize", defaultValue = "20", required = false) Integer pageSize) {
        log.info("activeType是多少：" + activeType);
        if (activeType != 0 && activeType != 1) {
            activeType = null;
        }
        PageAuthorizedVO pageAuthorizedVO = accreditService.getAuthorizedList(courseKeys, siteId, startTime, endTime, activeType, type, pageNo, pageSize);

        return pageAuthorizedVO;
    }


    /**
     * 查询课程 只包含课程信息无简介
     *
     * @param id 课程id
     * @return
     */
    @GetMapping("/view")
    public Course view(@RequestParam(value = "id") Long id) {
        Course course = courseService.selectById(id);
        return course;
    }


    /**
     * 查询指定范围可见id
     *
     * @param courseId
     * @return
     */
    @GetMapping("/getRange")
    public List<Long> getRange(@RequestParam("courseId") Long courseId) {
        return statisticsCourseMapper.getRangeByCourseId(courseId);
    }


    @ApiOperation(value = "可见范围导出", notes = "可见范围导出")
    @GetMapping("/export/visiblRange")
    public VisibleRangeExport vsibleRangeExport(@ApiParam(value = "课程的Id", name = "课程的Id", required = true) @RequestParam(name = "courseId", required = true) Long courseId) {
        return courseService.vsibleRangeExport(courseId);
    }


    /**
     * 根据课程id 获取课程评分和(课程学习人数)课程浏览人次
     *
     * @param courseId
     * @return
     */
    @GetMapping("/getManageCourseList")
    public HomePageCoursePageVo getManageCourseList(@RequestParam("courseId") Long courseId, @RequestParam("siteId") Long siteId, @RequestParam("companyId") Long companyId) {
        HomePageCoursePageVo homePageCoursePageVo = null;
        try {
            homePageCoursePageVo = courseService.selectMyManageCourseList(courseId);

        } catch (Exception e) {
            log.info("获取课程信息异常" + e);
        }

        if (homePageCoursePageVo != null) {
            return homePageCoursePageVo;
        } else {
            return null;
        }
    }


    /**
     * 授权到期的课程下架 失效
     *
     * @param id 授权id
     * @return
     */
    @PostMapping(value = "/invalid")
    public boolean invalidCourse(@RequestParam(value = "id") Long id) {
        // 授权到期的课程下架 失效
        return courseService.invalidCourse(id);
    }

    /**
     * 待授权课程列表
     *
     * @param map
     * @return
     */
    @GetMapping("/authorizelist")
    public Page<AuthCourseVO> getAuthorizeCourseList(@RequestBody Map<String, Object> map) {
        Integer pageNo = Integer.valueOf(map.get("pageNo").toString());
        Integer pageSize = Integer.valueOf(map.get("pageSize").toString());
        Long authorizeSiteId = Long.valueOf(map.get("authorizeSiteId").toString());

        Long companyId = null;
        Long siteId = null;
        List<Long> orgIds = null;
        if (map.containsKey("companyId")) {
            companyId = Long.valueOf(map.get("companyId").toString());
        }
        if (map.containsKey("siteId")) {
            siteId = Long.valueOf(map.get("siteId").toString());
        }
        if (map.containsKey("orgIds")) {
            orgIds = (List<Long>) map.get("orgIds");
        }
        Long classifyId = null;
        List<Long> classifyIds = null;
        if (map.get("classifyId") != null) {
            classifyId = Long.valueOf(map.get("classifyId").toString());
            //Long classifyId = apiSeachCourseVo.getClassifyId();
            if (Objects.nonNull(classifyId) && classifyId.longValue() > 0) {
                classifyIds = new ArrayList<>();
                // 查询第一级的子分类
                Classify classify = classifyService.selectById(classifyId);
                Integer layer = classify.getLayer();
                classifyIds.add(classifyId);
                if (Objects.nonNull(layer) && layer.intValue() < 3) {
                    Classify classify1 = new Classify();
                    classify1.setParentId(classifyId);
                    classify1.setStatus(1);
                    List<Classify> firstIds = classifyService.selectList(QueryUtil.condition(classify1));
                    log.info("返回的firstIds：" + JSON.toJSON(firstIds));
                    Set<Long> lastIds = null;
                    if (!org.springframework.util.CollectionUtils.isEmpty(firstIds)) {
                        lastIds = firstIds.stream().map(objfirst -> objfirst.getId()).collect(Collectors.toSet());
                        classifyIds.addAll(lastIds);
                        log.info("返回的classifyIds：" + JSON.toJSON(classifyIds));
                    }
                    log.info("返回的lastIds：" + JSON.toJSON(lastIds));

                    // 查询第二级的子分类
                    if (Objects.nonNull(lastIds) && layer.intValue() < 2) {
                        EntityWrapper<Classify> ew = QueryUtil.condition(new Classify());
                        ew.in("parent_id", lastIds);
                        ew.eq("status", 1);
                        List<Classify> dataList = classifyService.selectList(ew);
                        log.info("返回的dataList：" + JSON.toJSON(dataList));
                        log.info("返回的classifyIds：" + JSON.toJSON(classifyIds));
                        if (!org.springframework.util.CollectionUtils.isEmpty(dataList)) {
                            classifyIds.addAll(dataList.stream().map(objLast -> objLast.getId()).collect(Collectors.toSet()));
                            log.info("返回的classifyIds：" + JSON.toJSON(classifyIds));
                        }
                    }
                }
            }
        }
        Integer shelves = null;
        if (map.get("shelves") != null) {
            shelves = Integer.valueOf(map.get("shelves").toString());
        }
        String name = null;
        if (map.get("name") != null) {
            name = map.get("name").toString();
        }
        Page<AuthCourseVO> page = courseService.findCourseList(name, classifyIds, shelves, companyId, siteId, orgIds, authorizeSiteId, pageNo, pageSize);
        return page;
    }

    /**
     * 课程授权的方法
     *
     * @param authorizeVO
     * @param context
     * @return
     */
    public CourseRespVO saveCourseAuthorize(AuthorizeVO authorizeVO, RequestContext context) {

        CourseRespVO courseRespVO = new CourseRespVO();

        if (authorizeVO.getAccreditVO().getType() == 1) {
            List<Long> siteIds = new ArrayList<>();
            siteIds.add(context.getSiteId());
            List<SiteVO> siteVOS = siteClient.selectByIds(siteIds);
            log.info("返回的信息为：" + siteVOS);
            if (org.apache.commons.collections.CollectionUtils.isNotEmpty(siteVOS)) {

                if (authorizeVO.getAccreditVO().getCompanyId() == null &&
                        !siteVOS.get(0).getCompanyId().equals(context.getCompanyId())) {

                    courseRespVO.setCode(0);
                    courseRespVO.setMsg("课程授权仅支持在内公司内部操作！");

                    return courseRespVO;
                }
            }

        }
        log.info("before authorizeParam:" + authorizeVO);
        // 授权课程过去站点id
        Long siteId = authorizeVO.getAccreditVO().getSiteId();
        // 判断是否为主站点
        Boolean topSite = siteClient.isDefaultSite(siteId);
        Boolean currentSiteType = siteClient.isDefaultSite(context.getSiteId());

        log.info("after authorizeParam:" + authorizeVO);
        CourseVo courseVO = new CourseVo();

        try {
            for (AuthCourseVO authCourseVO : authorizeVO.getAuthorizationCourseVOs()) {

                // 课程id
                Long id = authCourseVO.getCourseId();

                if (accreditService.existAuthorize(id, siteId)) {
                    // 同一个站点一门有效课程成只能授权一次
                    log.info("同一个站点一门有效课程成只能授权一次");
                    continue;
                }
                if (!currentSiteType) {
                    if (topSite) {
                        log.info("授权站点为主站点，被授权站点为子站点");
                        //被授权站点为主站点
                        // 课程是否已收取到主站点
                        boolean f = accreditService.topSiteInAuthorized(siteId, id);
                        if (f) {
                            courseRespVO.setCode(0);
                            courseRespVO.setMsg("不能重复给此站点授权此课程！");
                            break;
                        } else {
                            courseRespVO.setCode(1);
                        }
                    }
                } else {
                    courseRespVO.setCode(1);
                }
                // 课程有效课程
                Date validDate = authCourseVO.getValidDate();
                log.info("课程id：" + id);
                Date sysDate = new Date();
                Course course = courseService.selectById(id);


                // 复制新生成一条记录
                Course newCourse = courseService.copyCourse(course, authorizeVO.getAccreditVO().getCompanyId(), siteId, authorizeVO.getAccreditVO().getType(), context);

                if (newCourse == null) {
                    log.info("there are no new course generated!");
                } else {
                    // 设置有效期时间
                    if (validDate != null && !"null".equals(validDate)) {
                        (authorizeVO.getAccreditVO()).setUseType(1);
                        (authorizeVO.getAccreditVO()).setUseDuration(validDate);
                    } else {
                        (authorizeVO.getAccreditVO()).setUseType(0);
                    }
                    accreditService.saveAuthorizeCourse(authorizeVO.getAccreditVO(), newCourse, id);
                }

            }
        } catch (Exception e) {
            log.info("授权的课程+" + e);
        }
        return courseRespVO;
    }

    /**
     * 异步任务被授权课章节的新增
     *
     * @param chapterVO
     * @return
     */
    @PostMapping("/authorize/chapter/save")
    public Boolean authorizeChapterSave(@RequestBody AuthorizeChapterVO chapterVO) {

        String taskLogSerialNo = "chapter-" + DateUtil.getYMDHMSS();
        Long taskId = idGenerator.generate();
        TaskContext taskContext =
                new TaskContext(taskId, taskLogSerialNo, "新增章节", taskId, new Date(), chapterVO, null);
        chapterAddTask.bindContext(taskContext);
        taskExecutor.asynExecute(chapterAddTask);

        return true;
    }

    /**
     * 异步任务删除被修改的章节
     *
     * @param chapterVO
     * @return
     */
    @PostMapping("/authorize/chapter/delete")
    public Boolean authorizeChapterDelete(@RequestBody AuthorizeChapterVO chapterVO) {

        String taskLogSerialNo = "chapter-" + DateUtil.getYMDHMSS();
        Long taskId = idGenerator.generate();
        TaskContext taskContext =
                new TaskContext(taskId, taskLogSerialNo, "删除章节", taskId, new Date(), chapterVO, null);
        chapterDeleteTask.bindContext(taskContext);
        taskExecutor.asynExecute(chapterDeleteTask);

        return true;
    }

    /**
     * 异步任务修改被授权课程的章节
     *
     * @param chapterVO 章节实体
     * @return
     */
    @PostMapping("/authorize/chapter/update")
    public Boolean authorizeChapterUpdate(@RequestBody AuthorizeChapterVO chapterVO) {
        String taskLogSerialNo = "chapter-" + DateUtil.getYMDHMSS();
        Long taskId = idGenerator.generate();
        TaskContext taskContext =
                new TaskContext(taskId, taskLogSerialNo, "修改章节", taskId, new Date(), chapterVO, null);
        chapterUpdateTask.bindContext(taskContext);
        taskExecutor.asynExecute(chapterUpdateTask);
        return true;
    }

    @PostMapping("/authorize/chapter/sort")
    public Boolean authorizeChapterSort(@RequestBody List<String> ids) {
        String taskLogSerialNo = "chapter-" + DateUtil.getYMDHMSS();
        Long taskId = idGenerator.generate();
        TaskContext taskContext =
                new TaskContext(taskId, taskLogSerialNo, "修改章节", taskId, new Date(), null, ids);
        chapterSortTask.bindContext(taskContext);
        taskExecutor.asynExecute(chapterSortTask);
        return true;
    }

    /**
     * 查询可见范围的课程list
     *
     * @return
     * @author lilingye
     */
    @GetMapping(value = "/home/courseList")
    public List<Long> findCourseList(@RequestParam(value = "pageNo", defaultValue = "0", required = false) Integer pageNo,
                                     @RequestParam(value = "pageSize", defaultValue = "20", required = false) Integer pageSize) {
        return courseService.findCourseIdList(pageNo, pageSize);
    }

    /**
     * 判断课程是否完成
     *
     * @param courseAccountIds
     * @param isTestSensitive  是否包括测验是否完成
     * @return
     */
    @PostMapping(value = "/record/isCourseFinished")
    public Map<String, Boolean> isCourseFinished(@RequestBody Set<String> courseAccountIds, @RequestParam(name = "isTestSensitive") Boolean isTestSensitive) {
        Map<String, Boolean> map = new HashMap<>();
        if (!CollectionUtils.isEmpty(courseAccountIds)) {
            Long accountId = null;
            Long courseId = null;
            for (String key : courseAccountIds) {
                courseId = Long.valueOf(key.split("_")[0]);
                accountId = Long.valueOf(key.split("_")[1]);
                map.put(key, courseService.isCourseFinished(courseId, accountId, isTestSensitive) == 1);
            }
        }
        return map;
    }


    @GetMapping(value = "/list/notIds")
    public Page<Course> listNotIds(@RequestParam(value = "siteId", required = false) Long siteId,
                                   @RequestParam(value = "name", required = false) String name,
                                   @RequestParam(value = "ids", required = false) List<Long> ids,
                                   @RequestParam(value = "pageNo", required = false) Integer pageNo,
                                   @RequestParam(value = "pageSize", required = false) Integer pageSize) {
        RequestContext context = ContextHolder.get();
        Page<Course> page = courseService.searchPage(name, null,
                1, context.getCompanyId(), siteId, context.getOrgIds(),
                pageNo, pageSize, null, null, ids);
//        return courseService.listNotIds(siteId,name,ids, pageNo, pageSize);
        return page;
    }


    @GetMapping(value = "/by/new/server")
    public List<Map<String, Object>> getServerByCompanyIdAndIds(@RequestParam("companyId") Long companyId, @RequestParam(value = "ids", required = false) List<Long> ids) {
        return courseService.getServerByCompanyIdAndIds(companyId, ids);
    }


    @GetMapping(value = "/get/course/time/message")
    public Map<String, Object> getCourseTimeMessage(@RequestParam("companyId") Long companyId, @RequestParam(value = "ids", required = false) List<Long> ids) {
        return courseService.getCourseTimeMessage(companyId, ids);
    }

    /**
     * 更新视频时间
     */
    @GetMapping(value = "/update/polyvideo")
    public void upateDuration() {
        HttpClientUtils httpClientUtils = new HttpClientUtils();
        Material material1 = new Material();
        Wrapper<Material> wrapper = QueryUtil.condition(material1);
        wrapper.where(" text_second = 1000 ").groupBy(" poly_vid ");

        List<Material> materialList = materialMapper.selectList(wrapper);
        LOGGER.info("结果：" + materialList);
        for (Material material : materialList) {
            String url = "http://v.polyv.net/uc/services/rest?method=editById&writetoken=lrKphGM4ZYuWtp2-K1SspyotFSciHBgL&vid=" + material.getPolyVid();
            String reStr = httpClientUtils.httpGet(url);
            JSONObject polyResult = JSON.parseObject(reStr);
            if (null != reStr && polyResult.containsKey("data")) {
                String data = polyResult.getString("data");
                JSONArray jsonArray = JSON.parseArray(data);
                JSONObject dat = JSON.parseObject(jsonArray.get(0).toString());
                String vid1 = dat.getString("vid");
                log.info("保利威视的vid：" + material.getPolyVid());
                String duration = dat.getString("duration");
                material.setPolyVid(material.getPolyVid());

                SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
                try {
                    Date date = sdf.parse(duration);
                    System.out.println(sdf.parse(duration));
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(date);
                    int hour = calendar.get(Calendar.HOUR_OF_DAY);
                    int minute = calendar.get(Calendar.MINUTE);
                    int second = calendar.get(Calendar.SECOND);
                    log.info("hour:" + hour + ",minute:" + minute + ",second:" + second);
                    material.setTextHour(hour);
                    material.setTextMinute(minute);
                    material.setTextSecond(second);
                } catch (ParseException e) {
                    log.info("异常{}", e);
                }

                //更新polyVid到素材中
                log.info("修改参数" + material);
                boolean ret = materialService.updateById(material);
                log.info("返回结果" + ret);
            }
        }
    }

    public void uploadPolyVideo() {
        HttpClientUtils httpClientUtils = new HttpClientUtils();
        Material material = new Material();
        Wrapper<Material> wrapper = QueryUtil.condition(material);
        wrapper.where(" poly_vid!=''");

        List<Material> materialList = materialMapper.selectList(wrapper);
        LOGGER.info("结果：" + materialList);
        for (Material m : materialList) {
        }
    }

    @ApiOperation(value = "站点-素材", notes = "站点-素材")
    @PostMapping(value = "/backupMaterial")
    public void updateMaterialData(@RequestBody List<String> codeList,
                                   @RequestParam(name = "afterSiteId", required = true) Long afterSiteId,
                                   @RequestParam(name = "beforeSiteId", required = true) Long beforeSiteId,
                                   @RequestParam(name = "companyId", required = true) Long companyId) {
        List<Material> materialList = materialService.queryMaterialList(codeList, beforeSiteId);

        Date date = new Date();
        for (Material m : materialList) {
            Long mId = m.getId();
            Long id = idGenerator.generate();
            m.setId(id);
            m.setSiteId(afterSiteId);

            m.setCompanyId((companyId != null ? companyId : null));
            m.setOrgId(null);
            m.setUpdateTime(date);
            m.setCreateTime(date);
            m.setUpdateByName("shyz_admin");
            boolean mIn = materialService.insert(m);

            Integer type = m.getType();
            switch (type) {
                case 1:
                    // 音频素材
                    AudioVideo audioVideo = new AudioVideo();
                    audioVideo.setMaterialId(mId);
                    audioVideo.setDeleted(0);
                    List<AudioVideo> audioVideoList = iAudioVideoService.selectList(QueryUtil.condition(audioVideo));
                    log.info("audioVideoList的长度：" + audioVideoList.size());
                    for (AudioVideo a : audioVideoList) {
                        a.setId(idGenerator.generate());
                        a.setMaterialId(id);
                        iAudioVideoService.insert(a);

                    }
                    break;
                case 2:
                    // 视频素材
                    AudioVideo video = new AudioVideo();
                    video.setMaterialId(mId);
                    video.setDeleted(0);
                    List<AudioVideo> videoList = iAudioVideoService.selectList(QueryUtil.condition(video));
                    log.info("audioVideoList的长度：" + videoList.size());
                    for (AudioVideo a : videoList) {
                        a.setId(idGenerator.generate());
                        a.setMaterialId(id);
                        iAudioVideoService.insert(a);

                    }
                    break;
                case 3:
                    //文档课程
                    DocPages docPages = new DocPages();
                    docPages.setMaterialId(mId);
                    docPages.setDeleted(0);
                    List<DocPages> docPagesList = docPagesService.selectList(QueryUtil.condition(docPages));
                    log.info("docPagesList：" + docPagesList.size());
                    for (DocPages d : docPagesList) {
                        d.setId(idGenerator.generate());
                        d.setMaterialId(id);
                        docPagesService.insert(d);

                    }
                    break;
                case 4:
                    // 富文本素材
                    RichMedia richMedia = new RichMedia();
                    richMedia.setMaterialId(mId);
                    richMedia.setDeleted(0);
                    List<RichMedia> richMediaList = iRichMediaService.selectList(QueryUtil.condition(richMedia));
                    log.info("richMediaList的长度：" + richMediaList.size());
                    for (RichMedia r : richMediaList) {
                        r.setId(idGenerator.generate());
                        r.setMaterialId(id);
                        iRichMediaService.insert(r);

                    }
                    break;
                case 5:
                    // scorm 素材
                    Scorm scorm = new Scorm();
                    scorm.setMaterialId(mId);
                    scorm.setDeleted(0);
                    List<Scorm> scormList = scormService.selectList(QueryUtil.condition(scorm));
                    log.info("scormList的长度：" + scormList.size());
                    for (Scorm s : scormList) {
                        s.setId(idGenerator.generate());
                        s.setMaterialId(id);
                        scormService.insert(s);

                    }
                    break;
            }
        }
    }

    /***
     *  课程列表
     * @param classifyId
     * @return
     */
    @GetMapping("/list/export")
    public Map<String, Object> listExport(@RequestParam(required = false, value = "type", defaultValue = "0") Integer type,
                                          @ApiParam(name = "name", value = "课程名称或作者", required = false) @RequestParam(name = "name", required = false) String name,
                                          @ApiParam(name = "classifyId", value = "分类id ", required = false) @RequestParam(name = "classifyId", required = false) Long classifyId,
                                          @ApiParam(name = "shelves", value = "0 未上架 1 已上架 2 草稿", required = false) @RequestParam(name = "shelves", required = false) Integer shelves,
                                          @ApiParam(name = "courseType", value = "课程类型", required = false) @RequestParam(required = false, value = "courseType") Integer courseType) {
        RequestContext requestContext = ContextHolder.get();
        Date submitTime = new Date();
        Long taskId = submitTime.getTime();
        String serialNo = "USER-EXPORT-" + taskId;
        String taskName = "导出课程清单-" + taskId;
        String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("accountId", requestContext.getAccountId());
        map.put("siteId", requestContext.getSiteId());
        map.put("companyId", requestContext.getCompanyId());
        map.put("serialNo", serialNo);
        map.put("taskId", taskId);
        map.put("submitTime", submitTime);
        map.put("taskName", taskName);
        map.put("type", type);
        map.put("name", name);
        map.put("classifyId", classifyId);
        map.put("shelves", shelves);
        map.put("courseType", courseType);
        map.put("result", result);
        courseListExportAsync.execute(map, true);
        return map;
    }

    /**
     * o
     * 自定义项目获取课程log
     *
     * @param courseIds
     * @return
     */
    @GetMapping("/getCourseLogos")
    Map<Long, String> getCourseLogos(@RequestParam("courseIds") List<Long> courseIds) {
        return courseService.getCourseLogos(courseIds);
    }

    @GetMapping("/getCourseCodeName")
    Map<Long, String> getCourseCodeName(@RequestParam(value = "courseIds", required = false) List<Long> courseIds) {
        return courseService.getCourseCodeName(courseIds);
    }

    @GetMapping("/VisibleRange")
    public List<CourseAccount> VisibleRange(@RequestParam("courseId") Long courseId) {
        CourseAccount courseAccount = new CourseAccount();
        courseAccount.setCourseId(courseId);
        courseAccount.setDeleted(0);
        EntityWrapper<CourseAccount> wrapper = new EntityWrapper<CourseAccount>(courseAccount);
        List<CourseAccount> list = courseAccountService.selectList(wrapper);
        return list;

    }

    @GetMapping("/getOne")
    public CourseVo getOne(@RequestParam("courseId") Long courseId) {
        return courseService.getCourseDetails(courseId);
    }

    @PostMapping("/getPageToCalendar")
    public Page<Course> getPageToCalendar(@ApiParam("paramVo") @RequestBody CalendarTaskParamVo paramVo) {
        Page<Course> page = new Page(paramVo.getPageNo(), paramVo.getPageSize());

        return courseService.getPageToCalendar(paramVo.getDate(), page);
    }

    /**
     * 嘉兴银行闯关排行
     *
     * @param courseId
     * @return
     */
    @GetMapping("/getRankings")
    Page<JaxinRankingVO> getRankings(@RequestParam("courseId") Long courseId,
                                     @RequestParam("pageNo") Integer pageNo,
                                     @RequestParam("pageSize") Integer pageSize) {
        return courseService.getRankings(courseId, pageNo, pageSize);
    }

    /**
     * 获取课程实体
     *
     * @return
     */
    @GetMapping(value = "/getCourseByCode")
    public Course getCourseByCode(@RequestParam(value = "courseNo") String courseNo,
                                  @RequestParam(value = "siteId") Long siteId,
                                  @RequestParam(value = "companyId") Long companyId
    ) {
        Course course = new Course();
        course.setCode(courseNo);
        course.setSiteId(siteId);
        course.setDelFlg(0);
        course.setShelves(1);
        EntityWrapper<Course> condition = QueryUtil.condition(course);
        return courseService.selectOne(condition);
    }

    @GetMapping("/getPageByDrools")
    Page<DroolsVo> getPageByDrools(@RequestParam("field") String field,
                                   @RequestParam(value = "value", required = false) String value,
                                   @RequestParam("pageNo") Integer pageNo,
                                   @RequestParam("pageSize") Integer pageSize) {
        Page<DroolsVo> page = new Page<>(pageNo, pageSize);
        return courseService.getPageByDrools(field, value, page);
    }

    @PostMapping("/ids/get")
    public List<Course> getCourseByIds(@RequestBody List<Long> ids) {
        return courseService.selectBatchIds(ids);
    }

    /**
     * 发送证书
     *
     * @param courseId      课程id
     * @param courseName    课程名
     * @param accountId     用户id
     * @param siteId        站点id
     * @param certificateId 证书id
     */
    private void sendCertificate(Long courseId, String courseName, Long accountId, Long siteId, String certificateId) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                try {
                    Map<String, Object> map = new HashMap<>();
                    map.put("siteId", siteId);
                    map.put("certificateId", certificateId);
                    map.put("accountId", accountId);
                    map.put("courseId", courseId);
                    map.put("courseName", courseName);
                    map.put("bizdType", 1);
                    String[] certificates = certificateId.split(",");
                    for (String certificateId : certificates) {
                        //sendCertificate(courseVo.getId(),courseVo.getName(),rc.getAccountId(),rc.getSiteId(),Long.valueOf(certificateId));
                        cloudEventPublisher.publish("myCertificate", new EventWrapper<Map>(Long.valueOf(certificateId), map));
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }
}
