package com.yizhi.course.application.controller;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.cache.CacheNamespace;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.course.application.domain.CourseStudyHour;
import com.yizhi.course.application.domain.CourseStudyHourDetailed;
import com.yizhi.course.application.service.ICourseStudyHourDetailedService;
import com.yizhi.course.application.service.ICourseStudyHourService;
import com.yizhi.course.application.vo.MyDurationVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 课程学时发放明细 前端控制器
 * </p>
 *
 * @author 谢海军123
 * @since 2019-05-14
 */
@RestController
@RequestMapping("/courseStudyHour")
public class CourseStudyHourController {

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private ICourseStudyHourDetailedService studyHourDetailedService;

    @Autowired
    private ICourseStudyHourService studyHourService;

    @Autowired
    private RedisCache redisCache;

    private Logger LOGGER = LoggerFactory.getLogger(CourseStudyHourController.class);
    /**
     * 学时统计
     * @param siteId
     * @param accountId
     * @param courseId
     * @return
     */
    @GetMapping("/hour/statistics")
    public Float hourStatistics(@RequestParam("siteId") Long siteId,
                               @RequestParam(value = "accountId",required = false) Long accountId,
                               @RequestParam(value = "courseId" ,required = false) Long courseId){
        LOGGER.info("学时统计 请求参数siteId={},accountId={},courseId={}",siteId,accountId,courseId);
        // 先查询 缓存
        if(Objects.nonNull(accountId) && Objects.nonNull(courseId)){
            String key = CacheNamespace.COURSE_STUDY_HOUR + siteId;
            String item = accountId.toString() + "-" + courseId.toString();
            Object object = redisCache.hget(key, item);
            if (object == null) {
                Float value = studyHourService.studyHourSum(siteId, accountId, courseId);
                redisCache.hset(key, item, String.valueOf(value), 600);
                return value;
            }
            return Float.valueOf(String.valueOf(object));
        }else {
            Float value = studyHourService.studyHourSum(siteId, accountId, courseId);return value;
        }
    }


    /**
     * 发放学时
     * @param entiy
     * @return
     */
    @PostMapping("/add")
    public Boolean add(@RequestBody CourseStudyHour entiy){
        Float totalDuration = entiy.getTotalDuration();

        if(Objects.nonNull(totalDuration)){
            Float hour = null;
            if(totalDuration>0f){
                hour = studyHourService.studyHourSum(entiy.getSiteId(),
                        entiy.getAccountId(),entiy.getCourseId());
            }
            if(hour == null ){
                entiy.setId(idGenerator.generate());
                entiy.setCreateTime(new Date());
                studyHourService.insert(entiy);
            }else if(totalDuration>hour){
                entiy.setId(idGenerator.generate());
                entiy.setTotalDuration(totalDuration-hour);
                entiy.setCreateTime(new Date());
                studyHourService.insert(entiy);
            }

            // 插入学时明细
            CourseStudyHourDetailed entiyDetailed = new CourseStudyHourDetailed();
            BeanUtils.copyProperties(entiy,entiyDetailed,"id","createTime");
            entiyDetailed.setId(idGenerator.generate());
            entiyDetailed.setCreateTime(new Date());
            studyHourDetailedService.insert(entiyDetailed);
        }
        // 删除 缓存
        try {
            String key = CacheNamespace.COURSE_STUDY_HOUR + entiy.getSiteId();
            String item = entiy.getAccountId().toString() + "-" + entiy.getCourseId().toString();
            String[] items = {item};
            redisCache.hdel(key, items);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    @GetMapping("/query/duration/ByAccountId")
    public Page<MyDurationVO> queryDurationByAccountId(@RequestParam("accountId")Long accountId,
                                                       @RequestParam("siteId")Long siteId,
                                                       @RequestParam(value = "pageNo")Integer pageNo,
                                                       @RequestParam("pageSize")Integer pageSize){

        return this.studyHourService.queryDurationByAccountId(accountId,siteId,pageNo,pageSize);
    }

}

