package com.yizhi.course.application.controller;

import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.course.application.task.CourseStudyHourRankingAsync;
import com.yizhi.course.application.util.JsonUtils;
import com.yizhi.course.application.vo.CourseStudyDrRankInfoVO;
import com.yizhi.course.application.vo.CourseStudyDurationRanking;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 课程学时排名
 */
@RestController
@RequestMapping("/studyHour/ranking")
public class CourseStudyHourRankingController {

    @Value("${ACTIVE}")
    public String active;

    @Autowired
    private CourseStudyHourRankingAsync courseStudyHourRankingAsync;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private AccountClient accountClient;

    private Logger LOGGER = LoggerFactory.getLogger(CourseStudyHourRankingController.class);

    /**
     * 查询当前人的排名和站点排名列表
     * @param siteId
     * @param accountId
     * @return
     */
    @GetMapping("/info")
    public CourseStudyDrRankInfoVO getListBySiteId(@RequestParam("siteId") Long siteId,
                                                   @RequestParam("accountId") Long accountId){

        LOGGER.info("排行榜数据获取开始，站点id={},账号id={}", siteId, accountId);
        Object obj = redisCache.hget(CourseStudyHourRankingAsync.REDIS_KEY, siteId.toString());
        AccountVO accountVO = accountClient.findCardById(accountId);
        CourseStudyDrRankInfoVO vo = new CourseStudyDrRankInfoVO();
        vo.setRankingDate(LocalDate.now().minusDays(1).format(DateTimeFormatter.ISO_LOCAL_DATE));
        vo.setProfilePicture(accountVO.getHeadPortrait());
        vo.setAccountId(accountVO.getId());
        vo.setAccountFullName(accountVO.getFullName());
        vo.setAccountName(accountVO.getName());
        vo.setOrgName(accountVO.getOrgName());
        vo.setRanking(0);
        vo.setDurationSum(0L);
        if(obj != null){

            List<CourseStudyDurationRanking>  listData = JsonUtils.json2JavaBeanList(obj.toString(),
                    CourseStudyDurationRanking.class);




            vo.setRankingList(listData);
            Long accountIdTmp = accountVO.getId();

                /*Optional<CourseStudyDurationRankingVO> rankings = listData.stream().filter(objItem->objItem.getAccountId()
                        .equals(accountId)).findFirst();*/
            CourseStudyDurationRanking rankings = null;
            int rankingIndex = 0;
            //int size = listData.size();
            for(CourseStudyDurationRanking rankingVO : listData){
                rankingIndex++;
                if(rankingVO.getAccountId().equals(accountIdTmp)){
                    rankings = rankingVO;
                    break;
                }

            }
            //if(rankings.isPresent()){
            if(rankings != null){
                //CourseStudyDurationRankingVO ranking = rankings.get();
                vo.setDurationSum(rankings.getDurationSum());
                vo.setRanking(rankingIndex);
            }else {
                vo.setRanking(-1);
                vo.setDurationSum(0L);
            }



        }

        LOGGER.info("排行榜数据获取redis不存在");
        return vo;
    }

    @GetMapping("/generated")
    public boolean rankingGenerated(@RequestParam("siteId") Long siteId){
        // 生成课程学习时长排名
        Map<String, Object> data = new HashMap<>();
        data.put("siteId",siteId);
        data.put("active",active);
        courseStudyHourRankingAsync.execute(data,true);
        return true;
    }

}
