package com.yizhi.course.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.course.application.domain.Favorite;
import com.yizhi.course.application.service.IFavoriteService;
import com.yizhi.course.application.vo.ApiListCourseVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.springframework.stereotype.Controller;

import java.util.List;

/**
 * <p>
 * 课程收藏 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-04-23
 */
@RestController
@RequestMapping("/favorite")
public class FavoriteController {

    @Autowired
    private IFavoriteService favoriteService;

    @Autowired
    IdGenerator idGenerator;

    /**
     * 课程收藏
     * @param favorite
     * @return
     */
    @PostMapping("/save")
    public Boolean saveFavorite(@RequestBody Favorite favorite){
        favorite.setId(idGenerator.generate());
        Boolean f = favoriteService.insert(favorite);
        return f;
    }

    /**
     * 取消课程收藏
     * @param favorite
     * @return
     */
    @PostMapping("/delete")
    public Boolean deleteFavorite(@RequestBody Favorite favorite){
        EntityWrapper<Favorite> entityWrapper = new EntityWrapper<Favorite>(favorite);
        Boolean f = favoriteService.delete(entityWrapper);
        return f;
    }

    /**
     * 课程收藏总数
     * @param accountId
     * @return
     */
    @GetMapping("/total")
    public Integer getMyFavoriteCount(
            @RequestParam(name = "accountId")Long accountId,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId

    ) {
        Integer count = favoriteService.selectMyCourseCount(accountId,companyId,siteId);
        return count;
    }


    /**
     * 我的课程收藏列表
     * @param accountId
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/list")
    public List<ApiListCourseVo> getMyFavoriteList(
            @RequestParam(name = "accountId")Long accountId,
            @RequestParam(name = "name",required = false)String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "pageNo")Integer pageNo,
            @RequestParam(name = "pageSize")Integer pageSize
    ) {
        List<ApiListCourseVo> apiListCourseVos = favoriteService.selectMyCourseList(name,accountId,companyId,siteId,pageNo,pageSize);
        return apiListCourseVos;
    }

}

