package com.yizhi.course.application.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.yizhi.course.application.service.ILessonReplyService;
import com.yizhi.course.application.service.IMaterialService;
import com.yizhi.course.application.vo.MaterialManageDetailsVo;
import com.yizhi.course.application.vo.MaterialManageVo;
import com.yizhi.course.application.vo.MaterialStateUpdateVo;
import com.yizhi.course.application.vo.NoPassReason;
import org.apache.commons.lang3.tuple.MutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.baomidou.mybatisplus.plugins.Page;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@RequestMapping("/manageLesson")
public class LessonManageController {

	    @Autowired
	    private IMaterialService materialService;
	    
	    @Autowired
	    private ILessonReplyService lessonReplyService;

		@ApiOperation(value = "后台做课列表", notes = "后台做课列表")
		@GetMapping("/lesson/list")
		public Page<MaterialManageVo> getLessonList(
				@ApiParam(name = "name", value = "模糊查询做课名称", required = false) @RequestParam(name = "name", required = false) String name,
				@ApiParam(name = "listCommintIdList", value = "提交人", required = false) @RequestParam(name = "listCommintIdList", required = false) List<Long> listCommintIdList,
				@ApiParam(name = "listAuditorIdList", value = "审批人", required = false) @RequestParam(name = "auditorName", required = false) List<Long> listAuditorIdList,
				@ApiParam(name = "state", value = "状态 1未提交 2审核中 3已通过 4未通过 5删除", required = false) @RequestParam(name = "state", required = false) Integer state,
				@ApiParam(name = "pageNo", value = "要跳转的页数", required = false) @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
				@ApiParam(name = "pageSize", value = "每页条数，默认：20", required = false) @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize,
				@ApiParam(name = "orgId", value = "组织id", required = true) @RequestParam(name = "orgId", required = true) Long orgId) {

				Page<MaterialManageVo> pageList = materialService.manageGetLessonList(name, listCommintIdList, listAuditorIdList, state,pageNo, pageSize,orgId);
			
				return pageList;
		}

		@ApiOperation(value = "根据素材id查看单个做课详情", notes = "后台单个做课详情")
		@GetMapping("/lesson/details")
		public MaterialManageDetailsVo getLesson(
				@ApiParam(name = "id", value = "id主键", required = true) @RequestParam(name = "id", required = true) Long id) {
			MaterialManageDetailsVo materialManageDetailsVo = materialService.getDetailsById(id);
			return materialManageDetailsVo;
		}

		@ApiOperation(value = "根据素材id审核/更改状态", notes = "审核/更改状态")
		@PostMapping("/update/state")
		public Boolean updateState(@RequestBody MaterialStateUpdateVo materialStateUpdateVo) {
			return materialService.updateStateById(materialStateUpdateVo);
		}
		
		
		 @ApiOperation(value="通过并查看下一个",notes="通过并查看下一个")
		 @PostMapping("/update/state/getNextId")
		 public Long updateStateGetNextId(@RequestBody MaterialStateUpdateVo materialStateUpdateVo){
		    return materialService.updateStateByIdAndNextId(materialStateUpdateVo);
	    }

		@ApiOperation(value = "通过做课id查看不通过的原因", notes = "查看不通过的原因")
		@GetMapping("/nopass/reason")
		public List<NoPassReason> selectNoPassReaon(
				@ApiParam(name = "lessonId", value = "做课id", required = true) @RequestParam(name = "lessonId", required = true) Long lessonId,
				@ApiParam(name = "pageNo", value = "要跳转的页数", required = false) @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
				@ApiParam(name = "pageSize", value = "每页条数，默认：20", required = false) @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize) {
			List<NoPassReason> list=new ArrayList<NoPassReason>();
			list=lessonReplyService.getNoPassReason(lessonId, pageNo, pageSize);
			return list;

		}
		
		
		@ApiOperation(value="统计做课记录",notes="统计做课记录")
		@GetMapping("/count/report")
		public  List<String> selectCountReport(
				@ApiParam(name = "startTime", value = "开始时间", required = false) @RequestParam(name = "startTime", required = false) Date startTime,
				@ApiParam(name = "endTime", value = "结束时间", required = false) @RequestParam(name = "endTime", required=false) Date endTime){
			
			return null;
		}
}
