package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.OfflineRecorde;
import com.yizhi.course.application.service.IOfflineRecordeService;
import com.yizhi.course.application.vo.ReportOfflineCourseDataStatisticsVo;
import com.yizhi.course.application.vo.ReportOfflineCourseStatisticsDetailVo;
import org.apache.commons.lang.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 线下课程报表
 *
 * @Author: shengchenglong
 * @Date: 2018/10/22 15:06
 */
@RestController
@RequestMapping("/manage/offlineCourseReport")
public class OfflineCourseReportController {

    private static final Logger LOGGER = LoggerFactory.getLogger(OfflineCourseReportController.class);

    @Autowired
    private IOfflineRecordeService offlineRecordeService;

    /**
     * 数据统计
     *
     * @param startTime
     * @param endTime
     * @return
     */
    @GetMapping("/data/statistics")
    public List<ReportOfflineCourseDataStatisticsVo> dataStatistics(@RequestParam("startTime") Long startTime,
                                                                    @RequestParam("endTime") Long endTime) {
        List<ReportOfflineCourseDataStatisticsVo> list = new ArrayList<>();
        RequestContext context = ContextHolder.get();

        String[] pattern = new String[]{"yyyy-MM-dd"};
        Date startTimeDate = null;
        Date endTimeDate = null;
        try {
            startTimeDate = new Date(startTime);
            endTimeDate = new Date(endTime);
        } catch (Exception e) {
            LOGGER.info("异常{}",e);
        }
        List<OfflineRecorde> records = offlineRecordeService.dataStatistics(startTimeDate, endTimeDate, context);
        if (!CollectionUtils.isEmpty(records)) {

            Map<Date, Integer> map = new TreeMap<>();
            while (DateUtils.truncatedCompareTo(startTimeDate, endTimeDate, Calendar.DATE) <= 0) {
                map.put(startTimeDate, 0);
                startTimeDate = DateUtils.addDays(startTimeDate, 1);
            }
            Set<Date> keys = map.keySet();

            for (OfflineRecorde record : records) {
                Date startTimeIn = record.getStartTime();
                Date endTimeIn = record.getEndTime();
                if(Objects.isNull(startTimeIn) || Objects.isNull(endTimeIn) ){
                    continue;
                }
                for (Date key : keys) {
                    if(Objects.isNull(key)){
                        continue;
                    }
                    if (DateUtils.truncatedCompareTo(startTimeIn, key, Calendar.DATE) <= 0
                            && DateUtils.truncatedCompareTo(endTimeIn, key, Calendar.DATE) >= 0) {
                        map.put(key, map.get(key) + 1);
                    }
                }
            }

            list = new ArrayList<>(keys.size());
            ReportOfflineCourseDataStatisticsVo vo = null;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (Date key : keys) {
                vo = new ReportOfflineCourseDataStatisticsVo(sdf.format(key), map.get(key));
                list.add(vo);
            }
        }
        return list;
    }


    /**
     * 数据统计
     *
     * @param name
     * @param startTime
     * @param endTime
     * @param pageSize
     * @param pageNo
     * @return
     */
    @GetMapping("/statistics/detail")
    public Page<ReportOfflineCourseStatisticsDetailVo> statisticsDetail(@RequestParam(name = "name") String name,
                                                                        @RequestParam(name = "startTime") Long startTime, @RequestParam(name = "endTime") Long endTime,
                                                                        @RequestParam(name = "pageSize") Integer pageSize, @RequestParam("pageNo") Integer pageNo
    ) {
        name = name.trim();
        String[] pattern = new String[]{"yyyy-MM-dd"};
        Date startTimeDate = null;
        Date endTimeDate = null;
        try {
            startTimeDate = new Date(startTime);
            endTimeDate = new Date(endTime);
        } catch (Exception e) {
            LOGGER.info("异常{}",e);
        }

        Page<ReportOfflineCourseStatisticsDetailVo> page = offlineRecordeService.statisticsDetail(startTimeDate, endTimeDate, name, name, pageSize, pageNo, ContextHolder.get());
        return page;
    }

    /**
     * 查询报表记录详情  根据课程id
     *
     * @param offlineCourseId
     * @return
     */
    @GetMapping("/queryDetail/list")
    public Page<OfflineRecorde> queryDetailList(@RequestParam(name = "offlineCourseId") Long offlineCourseId,
                                                @RequestParam(name = "pageNo") Integer pageNo,
                                                @RequestParam(name = "pageSize") Integer pageSize) {
        Page<OfflineRecorde> page = offlineRecordeService.queryDetailByOfflineCourseId(offlineCourseId, pageNo, pageSize);
        return page;
    }
}
