package com.yizhi.course.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.mapper.Wrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.cache.distributedlock.impl.RedisDistributedLock;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.vo.MyCourseRecordeVo;
import com.yizhi.course.application.vo.RecordeDurationVO;
import com.yizhi.course.application.vo.RecordeListVo;
import com.yizhi.course.application.vo.StudyDurationVO;
import com.yizhi.util.application.date.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**

 */
@RestController
@RequestMapping("/recorde")
public class RecordeController {

    private final static String COURSE_DISTRIBUTED_LOCK = "distributed.lock:course";

    private final static String COURSE_VIEW_DISTRIBUTED_LOCK = "distributed.lock:course.view";

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    IRecordeService recordeService;

    @Autowired
    private IMaterialService materialService;

    @Autowired
    private ICourseService courseService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    private RedisDistributedLock redisDistributedLock;

    @Autowired
    private CourseStudyController completionStatus;

    @Autowired
    private ILastViewChapterService lastViewChapterService;

    private Logger LOGGER = LoggerFactory.getLogger(RecordeController.class);


    /**
     * 学习记录保存
     *
     * @param recorde
     * @return
     */
    @PostMapping("/save")
    public Boolean save(@RequestBody Recorde recorde) {
        recorde.setId(idGenerator.generate());
        Boolean f = recordeService.insert(recorde);
        return f;
    }

    /**
     * 学习记录保存
     *
     * @param recorde
     * @return
     */
    @PostMapping("/completed/save")
    public Boolean completedSave(@RequestBody Recorde recorde) {

        String courseLockKey = COURSE_DISTRIBUTED_LOCK+":"+recorde.getCourseId()+"."+recorde.getAccountId();
        int maxRetry = 0;
        boolean courseExecLockStatus = false;

        Integer overed = recorde.getOvered();
        if(overed == null){
            overed = -1;
        }

        // 课程完成状态更新（发放积分等处理）
        try {
            do{
                courseExecLockStatus = redisDistributedLock.lock(courseLockKey,2500,4, 500);
                maxRetry++;
            }while (!courseExecLockStatus && maxRetry<2);

            try {
                completionStatus.calculateStudyStatusNew(recorde);
            }catch (Exception e){
                LOGGER.info("完成接口计算课程状态异常{}",e);
            }

            /*boolean execStatus = completionStatus.calculateStudyStatusNew(recorde);
            if(!execStatus){
                return Boolean.FALSE;
            }*/
            EntityWrapper<Recorde> ew = QueryUtil.condition(new Recorde());
            String sessionId = recorde.getSessionId();
            Wrapper<Recorde> recordeWrapper = ew.
                    eq("account_id", recorde.getAccountId());
                    //eq("chapter_id", recorde.getChapterId());
            if(!StringUtils.isEmpty(sessionId)){
                ew.eq("session_id",sessionId);
            }else{
                ew.isNull("terminal_type").
                eq("chapter_id", recorde.getChapterId());
            }
            ew.orderBy("id", false);
            List<Recorde> retData = recordeService.selectList(recordeWrapper);
            LOGGER.info("获取存在的学习记录信息={}", JSON.toJSONString(retData));
            Recorde old = null;
            if(!CollectionUtils.isEmpty(retData)){
                old = retData.get(0);
            }
            if(Objects.isNull(old)){
                recorde.setId(idGenerator.generate());
                String terminalDevice = recorde.getTerminalDevice();
                terminalDevice = "接口新增["+ DateUtil.toShortSeconds(new Date()) +recorde.getOvered()+",用户ID="+recorde.getAccountId()+",overed="+overed+"]";
                recorde.setTerminalDevice(terminalDevice);
                Chapter chapter = chapterService.selectById(recorde.getChapterId());
                try {
                    Integer mType = chapter.getMaterialType();
                    int materialType = mType==null?0:mType.intValue();
                    if(!recorde.getSessionId().isEmpty() && materialType>2){
                        Material material = materialService.selectById(chapter.getMaterialId());
                        if(material!=null){
                            Integer textSecond = material.getTextSecond();
                            if(textSecond != null && textSecond.intValue()>0){
                                recorde.setDuration(textSecond%5);
                            }
                        }
                    }
                }catch (Exception e){
                    LOGGER.info("处理偏移时间异常{}",e);
                }
                Boolean f = recordeService.insert(recorde);
            }else {
                Recorde up = new Recorde();
                up.setId(old.getId());
                // 更新组织和企业以及站点信息
                //up.setOrgId(recorde.getOrgId());
                //up.setCompanyId(recorde.getCompanyId());
                //up.setSiteId(recorde.getSiteId());
                Long siteId = recorde.getSiteId();
                String msg = "";
                if(siteId != null && !siteId.equals(old.getSiteId())){
                    msg += "站点不一致";
                }
                Long accountId = recorde.getAccountId();
                if(accountId != null && !accountId.equals(old.getAccountId())){
                    msg += "账号不一致";
                }
                Integer overdTmp = recorde.getOvered();
                if(Integer.valueOf(1).equals(overdTmp)){
                    up.setOvered(overdTmp);
                }
                String terminalDevice = old.getTerminalDevice();
                if(terminalDevice.indexOf("-->接口更新")<0){
                    terminalDevice+="-->接口更新["+ DateUtil.toShortSeconds(new Date()) +recorde.getOvered()+",用户ID="+recorde.getAccountId()+msg+",overed="+overed+"]";
                }else{
                    terminalDevice+="->"+recorde.getOvered()+"接口重复更新"+DateUtil.getSimpleDateFormat("dd日HH:mm:ss").format(new Date());
                }
                int lengthStr = terminalDevice.length();
                if(lengthStr>300){
                    terminalDevice = terminalDevice.substring(lengthStr-300);
                }

                up.setTerminalDevice(terminalDevice);
                recordeService.updateById(up);
            }
        }catch (Exception e){
            LOGGER.error("课程完成状态接口处理异常{}",e);
        } finally {
            if(courseExecLockStatus){
                redisDistributedLock.releaseLock(courseLockKey);
            }
        }
        return true;
    }

    @PostMapping("/last/study/save")
    public Boolean lastStudySave(@RequestBody Recorde recorde) {
        String courseViewLockKey = COURSE_VIEW_DISTRIBUTED_LOCK+":"+recorde.getCourseId()+"."+recorde.getAccountId();
        int maxRetry = 0;
        boolean courseExecLockStatus = false;
        do{
            courseExecLockStatus = redisDistributedLock.lock(courseViewLockKey,6500,2, 2000);
            maxRetry++;
        }while (!courseExecLockStatus && maxRetry<3);

        LastViewChapter ew = new LastViewChapter();
        ew.setAccountId(recorde.getAccountId());
        ew.setCourseId(recorde.getCourseId());
        ew = lastViewChapterService.selectOne(QueryUtil.condition(ew));
        try {
            if (ew == null) {
                LastViewChapter addObj = new LastViewChapter();
                addObj.setAccountId(recorde.getAccountId());
                addObj.setChapterId(recorde.getChapterId());
                addObj.setCompanyId(recorde.getCompanyId());
                addObj.setCourseId(recorde.getCourseId());
                addObj.setOrgId(recorde.getOrgId());
                addObj.setSiteId(recorde.getSiteId());
                addObj.setCreateTime(new Date());
                lastViewChapterService.insert(addObj);
            } else {
                LastViewChapter ewUp = new LastViewChapter();
                ewUp.setAccountId(recorde.getAccountId());
                ewUp.setCourseId(recorde.getCourseId());
                ew.setUpdateTime(new Date());
                ew.setAccountId(recorde.getAccountId());
                ew.setChapterId(recorde.getChapterId());
                ew.setCompanyId(recorde.getCompanyId());
                ew.setCourseId(recorde.getCourseId());
                ew.setOrgId(recorde.getOrgId());
                ew.setSiteId(recorde.getSiteId());
                lastViewChapterService.update(ew, QueryUtil.condition(ewUp));
            }
        }catch (Exception e){
            LOGGER.info("处理最后章节异常{}",e);
        }finally {
            if(courseExecLockStatus){
                redisDistributedLock.releaseLock(courseViewLockKey);
            }
        }

        return true;
    }

    /**
     * 学习记录修改
     *
     * @param recorde
     * @return
     */
    @PostMapping("/update")
    public Boolean update(@RequestBody Recorde recorde) {
        Boolean f = recordeService.insertOrUpdate(recorde);
        return f;
    }


    /**
     * 学习记录批量修改
     *
     * @param recordes
     * @return
     */
    @PostMapping("/update/list")
    public Boolean updateList(@RequestBody List<Recorde> recordes) {
        Boolean f = recordeService.updateBatchById(recordes, 500);
        return f;
    }


    /**
     * 学习记录详情
     *
     * @return
     */
    @GetMapping("/get")
    public Recorde get(@RequestParam("accountId") Long accountId,
                       @RequestParam("chapterId") Long chapterId,
                       @RequestParam("sessionId") String sessionId) {
        Recorde recorde = new Recorde();
        recorde.setAccountId(accountId);
        recorde.setChapterId(chapterId);
        recorde.setSessionId(sessionId);
        EntityWrapper<Recorde> entityWrapper = new EntityWrapper<Recorde>(recorde);
        Recorde recorde1 = recordeService.selectOne(entityWrapper);
        return recorde1;
    }

    /**
     * 学习记录批量保存
     *
     * @param recordes
     * @return
     */
    @PostMapping("/list/save")
    public List<Recorde> save(@RequestBody List<Recorde> recordes) {
        for (Recorde recorde : recordes) {
            recorde.setId(idGenerator.generate());
        }
        Boolean f = recordeService.insertBatch(recordes, 500);
        if (f) {
            return recordes;
        }
        return null;
    }

    /**
     * 学习记录批量更新
     *
     * @param recordes
     * @return boolean
     */
    @PostMapping("/list/update")
    public boolean batchUpdate(@RequestBody List<Recorde> recordes) {
        return recordeService.updateBatchById(recordes, 500);
    }

    /**
     * 学习记录批量删除
     *
     * @param recordes
     * @return
     */
    @PostMapping("/list/delete")
    public Boolean deleteList(@RequestBody List<Recorde> recordes) {
        recordeService.deleteBatchIds(recordes);
        return recordeService.deleteBatchIds(recordes);
    }


    /**
     * 我学习课程记录
     *
     * @param courseId
     * @param accountId
     * @return
     */
    @GetMapping("/list")
    public List<RecordeListVo> getMyRecorde(
            @RequestParam(name = "courseId") Long courseId,
            @RequestParam(name = "accountId") Long accountId
    ) {
        List<RecordeListVo> list = recordeService.selectRecordeList(accountId, courseId);
        return list;
    }

    /**
     * 学员端 我学习过的课程
     *
     * @param name
     * @param accountId
     * @param type
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/my/list")
    public Page<MyCourseRecordeVo> getMyCourseRecordeList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "type") Integer type,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize
    ) {
        Page<MyCourseRecordeVo> list = recordeService.selectMyCourseRecordeList(name, accountId, type, pageNo, pageSize);
        return list;
    }


    /**
     * 学员端 我学习过的课程数量
     *
     * @param accountId
     * @param type
     * @return
     */
    @GetMapping("/my/list/count")
    public Integer getMyCourseRecordeCount(
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "type", required = false) Integer type)
    {
        Integer count = recordeService.selectMyCourseRecordeCount(accountId, type);
        // 计算进行中的课程数
        RequestContext context = ContextHolder.get();
        Long siteId = context.getSiteId();
        // 获取学习记录信息


        return count;
    }


    /**
     * 查询所有课程
     *
     * @return
     */
    @GetMapping("/course/list")
    public List<Course> getCourseList() {
        List<Course> mapList = courseService.selectList(null);
        return mapList;
    }

    /**
     * 根据课程id查询存在的课程
     * @return
     */
    @PostMapping("/exist/course/list")
    public List<Course> selectExistCourse(@RequestBody Set<Long> courseIds) {
        EntityWrapper<Course> ew = QueryUtil.condition(new Course());
        ew.in("id",courseIds);
        List<Course> mapList = courseService.selectList(ew);
        return mapList;
    }

    /**
     * 查询所有章节
     *
     * @return
     */
    @GetMapping("/chapter/list")
    public List<Chapter> getChapterList() {
        List<Chapter> mapList = chapterService.selectList(null);
        return mapList;
    }

    @PostMapping("/exist/chapter/list")
    public List<Chapter> selectExistChapter(@RequestBody List<Long> chapterIds) {
        EntityWrapper<Chapter> ew = QueryUtil.condition(new Chapter());
        ew.in("id",chapterIds);
        List<Chapter> mapList = chapterService.selectList(ew);
        return mapList;
    }

    /**
     * 查询所有素材
     *
     * @return
     */
    @GetMapping("/material/list")
    public List<Material> getMaterialList() {
        List<Material> mapList = materialService.selectList(null);
        return mapList;
    }

    @PostMapping("/exist/material/list")
    public List<Material> selectExistMaterial(@RequestBody List<Long> materialIds) {
        EntityWrapper<Material> ew = QueryUtil.condition(new Material());
        ew.in("id",materialIds);
        List<Material> mapList = materialService.selectList(ew);
        return mapList;
    }

    /**
     * 查询所有学习记录
     *
     * @return
     */
    @GetMapping("/all/list")
    public List<Recorde> getAllRecordeList() {
        List<Recorde> mapList = recordeService.selectList(null);
        return mapList;
    }

    /**
     * 根据sessionIds查询存在的学习记录
     *
     * @return
     */
    @PostMapping("/exist/recorde/list")
    public List<Recorde> getExistRecorde(@RequestBody List<String> sessionIds) {
        EntityWrapper<Recorde> ew = QueryUtil.condition(new Recorde());
        //ew.in("session_id",sessionIds).gt("duration",0);
        ew.in("session_id",sessionIds);
        List<Recorde> mapList = recordeService.selectList(ew);
        if(!CollectionUtils.isEmpty(mapList)){
            List<Recorde> mapListLeft = mapList.stream().filter(obj->obj.getDuration()>0)
                    .collect(Collectors.toList());
            List<Recorde> mapListRht = mapList.stream().filter(obj->obj.getDuration()==0)
                    .collect(Collectors.toList());
            Map<String,Recorde> rhtmap = new HashMap<>();
            try {
                rhtmap = mapListRht.stream().collect(Collectors
                        .toMap(key->key.getSessionId(),val->val));
            }catch (Exception e){
                Map<String,List<Recorde>> rhtmapG = mapListRht.stream().collect(Collectors
                        .groupingBy(key->key.getSessionId()));
                for(Map.Entry<String,List<Recorde>> item : rhtmapG.entrySet()){
                    List<Recorde> dataTmp = item.getValue();
                    int size = dataTmp.size();
                    if(size==1){
                        rhtmap.put(item.getKey(), dataTmp.get(0));
                    }else {
                        dataTmp.sort(Comparator.comparing(Recorde::getOvered).reversed());
                        rhtmap.put(item.getKey(), dataTmp.get(0));
                    }
                }
            }
            List<Long> removeIds = new ArrayList<>();
            Map<String,Recorde> retDataMap = new HashMap<>();
            for(Recorde recorde : mapListLeft){
                String sesid = recorde.getSessionId();
                if(!CollectionUtils.isEmpty(rhtmap) && rhtmap.containsKey(sesid)){
                    Integer oIn = recorde.getOvered();
                    if(Objects.isNull(oIn) || oIn.intValue()==0){
                        Recorde rem = rhtmap.get(sesid);
                        recorde.setOvered(rem.getOvered());
                        removeIds.add(rem.getId());
                        rhtmap.remove(sesid);
                    }
                }
                retDataMap.put(sesid, recorde);
            }
            List<Recorde> mapListret=new ArrayList<>();
            if(!CollectionUtils.isEmpty(retDataMap)){
                for(Map.Entry<String,Recorde> ret : retDataMap.entrySet()){
                    mapListret.add(ret.getValue());
                }
            }
            if(!CollectionUtils.isEmpty(rhtmap)){
                for(Map.Entry<String,Recorde> ret1 : rhtmap.entrySet()){
                    mapListret.add(ret1.getValue());
                }
            }
            return mapListret;
        }
        return mapList;
    }

    /**
     * 查询所有的学员每个章节的总学习时长
     *
     * @return
     */
    @GetMapping("/duration/list")
    public List<RecordeDurationVO> getRecordeDuration() {
        List<RecordeDurationVO> mapList = recordeService.selectAllRecordeDuration();
        return mapList;
    }


    /**
     * 判断课程章节完成状态
     *
     * @param chapterId
     * @param accountId
     * @param duration
     * @return
     */
    @GetMapping("/get/completed")
    public Integer getCompleted(
            @RequestParam("chapterId") Long chapterId,
            @RequestParam("accountId") Long accountId,
            @RequestParam("duration") Integer duration
    ) {
        Chapter chapter = chapterService.selectById(chapterId);
        if (StringUtils.isEmpty(chapter)) {
            return 0;
        }

        Material material = materialService.selectById(chapter.getMaterialId());
        if (StringUtils.isEmpty(material)) {
            return 0;
        }
        Integer hour = material.getTextHour();
        Integer minute = material.getTextMinute();
        Integer second = material.getTextSecond();

        Integer total = 0;
        if (!StringUtils.isEmpty(hour)) {
            total = total + hour * 3600;
        }
        if (!StringUtils.isEmpty(minute)) {
            total = total + minute * 60;
        }
        if (!StringUtils.isEmpty(second)) {
            total = total + second;
        }

        //课程的完成规则
        //1点进去就算完成
        //2超过完成时间才算完成

        Integer completed = 0;
        Course course = courseService.selectById(chapter.getCourseId());
        if (null == course) {
            completed = 0;
        } else {
            switch (material.getType()) {
                case 1:
                    if (StringUtils.isEmpty(course.getAudioComplete())) {
                        break;
                    }
                    if (course.getAudioComplete() == 0) {
                        completed = 1;
                    } else {
                        completed = getCompletedStatus(chapterId, accountId, total, duration);
                    }
                    break;
                case 2:
                    if (StringUtils.isEmpty(course.getVideoComplete())) {
                        break;
                    }
                    if (course.getVideoComplete() == 0) {
                        completed = 1;
                    } else {
                        completed = getCompletedStatus(chapterId, accountId, total, duration);
                    }
                    break;
                case 3:
                    if (StringUtils.isEmpty(course.getDocComplete())) {
                        break;
                    }
                    if (course.getDocComplete() == 0) {
                        completed = 1;
                    } else {
                        completed = getCompletedStatus(chapterId, accountId, total, duration);
                    }
                    break;
                case 4:
                    if (StringUtils.isEmpty(course.getRichComplete())) {
                        break;
                    }
                    if (course.getRichComplete() == 0) {
                        completed = 1;
                    } else {
                        completed = getCompletedStatus(chapterId, accountId, total, duration);
                    }
                    break;
                case 5:
                    break;
                default:
                    break;
            }
        }
        return completed;
    }


    /**
     * 根据章节用户id 获得课程学习记录时间 用于判定是否课程完成
     *
     * @param chapterId 章节id
     * @param accountId 账号id
     * @param total     学习课程所需总数
     * @param duration  还未入库的学习时长
     * @return
     */
    private Integer getCompletedStatus(Long chapterId, Long accountId, Integer total, Integer duration) {
        Integer recordeDuration = courseService.getRecordeDuration(chapterId, accountId);
        if (!StringUtils.isEmpty(recordeDuration)) {
            if (recordeDuration + duration >= total) {
                return 1;
            } else {
                return 0;
            }
        }
        return 0;
    }

    /**
     * 查询课程是否完成
     *
     * @param chapterId
     * @return
     */
    @GetMapping("/course/isFinish")
    Long getCourseFinished(@RequestParam("accountId") Long accountId, @RequestParam("chapterId") Long chapterId) {
        return recordeService.getCourseFinished(accountId, chapterId);
    }

    /**
     * 查询课程的章节是否完成
     *
     * @param chapterId
     * @return
     */
    @GetMapping("/course/chapter/isFinish")
    public int getCourseChapterFinished(@RequestParam("accountId") Long accountId,
                                        @RequestParam("chapterId") Long chapterId) {
        Recorde recorde = new Recorde();
        recorde.setAccountId(accountId);
        recorde.setChapterId(chapterId);
        recorde.setOvered(1);
        int countNum = recordeService.selectCount(QueryUtil.condition(recorde));
        return countNum;
    }

    @GetMapping("course/finishAccountIds")
    List<Long> getFinishedAccountIds(@RequestParam("courseId") Long courseId) {
        return recordeService.getFinishedIds(courseId);
    }

    /**
     * 专为培训统计时长所用
     * @param courseId
     * @return
     */
    @GetMapping("/course/getStudyTime")
    Integer getStudyTime(@RequestParam(value = "courseId",required = false) List<Long> courseId) {
        return recordeService.getStudyTime(courseId);

    }

    /**
     * 查询用户学习章节的学习时长
     * @param accountId
     * @param SiteId
     * @return
     */
    @GetMapping("/count/study/duration")
    public Long countStudyDuration(@RequestParam("accountId") Long accountId,
                                   @RequestParam("SiteId") Long SiteId,
                                   @RequestParam(value = "chapterId" ,required = false) Long chapterId){
        Long sumDuration = recordeService.countStudyDuration(accountId,chapterId,SiteId);
        if(Objects.isNull(sumDuration)){
            return  0L;
        }
        return sumDuration;
    }

    @GetMapping("/count/study/duration/cids")
    public Long countStudyDurationByIds(@RequestParam("accountId") Long accountId,
                                        @RequestParam("SiteId") Long SiteId,
                                        @RequestParam(value = "courseIds") List<Long> courseIds){
        Long sumDuration = recordeService.countStudyDurationCourseIds(accountId, SiteId, courseIds);
        if(Objects.isNull(sumDuration)){
            return  0L;
        }
        return sumDuration;
    }

    @GetMapping("/studyDuration")
    public StudyDurationVO studyDuration(@RequestParam("accountId") Long accountId, @RequestParam("siteIds") List<Long> siteIds,
                                         @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate , @RequestParam("accounts") List<Long> accounts) {
    	RequestContext request = ContextHolder.get();
    	Long siteId = request.getSiteId();
    	StudyDurationVO vo = recordeService.studyDuration(accountId, siteIds, startDate, endDate, accounts);
    	return vo;
    }
}

