package com.yizhi.course.application.controller;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.sql.DataSource;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.domain.StatisticsCourse;
import com.yizhi.course.application.domain.StatisticsCourseLearn;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.service.IStatisticsCourseLearnService;
import com.yizhi.course.application.service.IStatisticsCourseService;
import com.yizhi.point.application.feign.PointUserFeignClients;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.OrgVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.util.application.domain.Response;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;
import com.yizhi.course.application.mapper.StatisticsCourseLearnMapper;
import com.yizhi.course.application.mapper.StatisticsCourseMapper;

@Component
public class StatisticsCourseRun extends AbstractDefaultTask<String, Map<String, Object>> {

	private Logger logger = LoggerFactory.getLogger(StatisticsCourseRun.class);
	
	// 分页获取学员学习信息
	private int pageNo = 1;
	private int pageSize = 30;
	
	//字段分割符
	private final static String FIELD_STERMINATED = "'\t'";
	
	//行分割符
	private final static String LINE_STERMINATED = "'\r\n'";
	
	private final static String FENGEFU = "\t";  //tab键
	private final static String HUANHANGFU = "\r\n";  //换行
	
	//复用对象
	private StringBuilder strBuilder = new StringBuilder();
	
	private SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	
	private final static String COLUMNS = "(id,course_id,course_code,course_name,course_create_time,course_state,course_duration,course_org_name,"
			+ "org_no,org_id,site_id,company_id,account_id,work_num,name,fullname,org_name,org_parent_names,account_state,"
			+ "account_org_id,account_site_id,account_company_id,learn_duration,start_time,end_time,finish_learn_time,learn_state,"
			+ "first_finish_time,can_state,join_state,learn_count,comment_account_count,comment_count,grade,finish_count,"
			+ "learn_point,learn_score)";
	
	private final static String EXECUTE_SQL = "load data local infile ''  into table statistics_course fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ COLUMNS;
	
	@Autowired
	private DataSource dataSource;	
	
	/**
	 * 缓存学员学习信息
	 */
	private final Map<String, Map<Long, StatisticsCourse>> accountMap = new HashMap<String, Map<Long, StatisticsCourse>>();
	
	/**
	 * 缓存部门信息
	 */
	private final Map<String, List<OrgVO>> orgMap = new HashMap<String, List<OrgVO>>();
		
	@Autowired
	private StatisticsCourseMapper statisticsCourseMapper;
	
	@Autowired
	private StatisticsCourseLearnMapper statisticsCourseLearnMapper;
	
	@Autowired
	private ReportClient reportClient;
	
	@Autowired
	private PointUserFeignClients pointUserFeignClients;
	
	@Autowired
	private IStatisticsCourseService statisticsCourseService;
	 
	@Autowired
	IdGenerator idGenerator;
	
	@Autowired
	IStatisticsCourseLearnService statisticsCourseLearnService;
	
	@Autowired
	private OrganizationClient organizationClient;
	
	@Autowired
	ICourseService courseService;
	/**
	 * map定义的键
	 * startDate 为空系统自动计算（从对中间表或主表中获取），不为空就用实际传参
	 * endDate 为空系统自动计算（当前日期-1），不为空就用实际传参
	 */
	@Override
	protected String execute(Map<String, Object> map) {  
		 
		Date startRunDate = new Date();
		
		SimpleDateFormat formatDate = new SimpleDateFormat("yyyy-MM-dd");
		
		Long taskId = idGenerator.generate();
		TaskContext taskContext = new TaskContext(taskId, formatDate.format(startRunDate) + " 在线课程报表分析", 999L, startRunDate);
    	working(taskContext);
    	
		try{
			String startDate = null;
			String endDate = null;
			Date dCurDate = null;
			
			if(map != null){
				startDate = (String) map.get("startDate");
				endDate = (String) map.get("endDate");
			}
			
			//判断是否首次初始化，不是的话删除之前的数据
			boolean isRunDel = true;
			
			if (startDate == null) {
				//从中间表取最大日期，如有有记录，那就是最大日期+1天
				Date middleMaxTime = statisticsCourseMapper.selectMaxDate();
				if (middleMaxTime != null) {
					//记录的最大一天加一天
					dCurDate = addDay(middleMaxTime, 1);
					startDate = formatDate.format(dCurDate);
					isRunDel = true;
				} else {
					/*//最小的课程创建时间开始
					Date recordMinTime = statisticsCourseMapper.selectRecordMinTime();
					startDate = formatDate.format(recordMinTime);*/
					isRunDel = false;
				}
			}
			//当前日期的前一天
			if (endDate == null) {
				dCurDate = addDay(new Date(), -1);
				endDate = formatDate.format(dCurDate);
			}
			Long day = 0L;
			Date beginDate = null;
			Date overDate = formatDate.parse(endDate);
			//不是首次初始化数据，删除 
	        if(isRunDel){
	        	try {
					beginDate = formatDate.parse(startDate);
					day = (overDate.getTime() - beginDate.getTime()) / (24 * 60 * 60 * 1000);    
				} catch (ParseException e1) {
					return null;
				}
	        	
	        	//先删除这天的记录再添加，防止一天添加多次记录
				for(int d = 0; d <= day; d++){
					String currentDate = startDate;
					if(d > 0){
						dCurDate = addDay(beginDate, d);
						currentDate = formatDate.format(dCurDate);
					}
					statisticsCourseMapper.deleteRecordeByDate(currentDate);
				}
	        }
	        
			// 获得所有的课程(条件：未上架/已上架，没有被删除的课程)
			List<Course> getAllCourse =null; /*statisticsCourseMapper.getAllCourseReport();*/
			//指定范围可见的课程人员的IdList
			List<Long> listAccountIdRange=null;
			

			List<StatisticsCourseLearn> accountLearns = null;
			List<StatisticsCourse> insertList =new ArrayList<StatisticsCourse>();
			
			String rangeKey = "";
			boolean isOk = true;
			String errMsg = "";
			String curCourseName = "";
			Long curCourseId = 0L;
			
			if(getAllCourse != null && getAllCourse.size() > 0){
				taskDetail(taskId, "***** 在线课程统计开始 ,课程数量:" + getAllCourse.size() +" *****");
				logger.info("***** 在线课程统计开始 ,课程数量:" + getAllCourse.size() +" *****");
			} else {
				taskDetail(taskId, "***** 在线课程统计开始 ,课程数量:0 *****");
				logger.info("***** 在线课程统计开始 ,课程数量:0 *****");
			}
			
			
			// 按课程遍历学员学习记录
			for (Course course : getAllCourse) {
				try {
					Long courseId = course.getId(); // 课程ID
					Long siteId = course.getSiteId(); // 课程的站点ID
					curCourseName = course.getName();
					curCourseId = courseId;
					/**
					 * 获取该门课程的课程时长（course_duration）（h:m:s）、
					 * 评分人数（comment_account_count）、评论数（comment_count）、课程评分（grade）、
					 * 得分（learn_score）
					 */
					// 根据课程ID获取课程时间长度(对章节的sum已经进行累加) //课程时间长度处理
					List<Long> courseIds=new ArrayList<Long>();
					courseIds.add(courseId);
					List<Material> list= statisticsCourseMapper.getCourseDurationById(courseIds);
					Material material =null;
					if(CollectionUtils.isNotEmpty(list)) {
						material = list.get(0);
					}
				
					String courseDuration = "0:0:0";
					//有些课程是没有时间长度的
					if (material != null) {
						Integer timeS = material.getTextHour() * 3600 + material.getTextMinute() * 60 + material.getTextSecond();
						Integer hour = timeS / 3600;
						Integer minute = (timeS - hour * 3600) / 60;
						Integer second = timeS - hour * 3600 - minute * 60;
						//课程时间长度
						courseDuration = hour + ":" + minute + ":" + second;
					}

					Integer scope = course.getScope(); // 可见范围 0 游客 1 平台用户 2 指定范围
					if (scope == 2 && course.getSiteId()!=null) { // 如果是指定范围 siteId=null 指定范围有值
						siteId = null;
						listAccountIdRange = statisticsCourseMapper.getRangeByCourseId(courseId);
						if (null == listAccountIdRange) {
							taskDetail(taskId,course.getName() + "(" + course.getId() + ")" + " 未设置指定范围，请确认是否为该课程设置的范围,无法统计数据");
							rangeKey = "";
						} else {
							rangeKey = course.getSiteId().toString() + listAccountIdRange.toString();
						}
					} else {
						listAccountIdRange = null;
						if (siteId != null) {
							rangeKey = siteId.toString();
						} else {
							rangeKey = "";
							taskDetail(taskId,course.getName() + "--课程异常。课程id(" + course.getId() + ")" + " 全平台可见的站点为空:(" + siteId + ") 请确认数据,无法统计数据");
						
						}
					}
					if("".equals(rangeKey)){
						continue;
					}
					
					// 从缓存中获取用户，没有找到再请求system查询
					Map<Long, StatisticsCourse> resultccountMap = accountMap.get(rangeKey);
					if(resultccountMap == null){  
						logger.info("***** 正在获取课程参加人员信息： " + course.getName() + "(" + course.getId() + ")");
						resultccountMap = new HashMap<Long, StatisticsCourse>();
						//查找人员信息
						getReportAccountRespVO(course.getSiteId(), listAccountIdRange, pageNo, pageSize, null, resultccountMap, rangeKey, taskId, course.getName() + "(" + course.getId() + ")");
						
						if(resultccountMap.size() == 0){
							if(siteId == null){
								taskDetail(taskId, course.getName() + "(" + course.getId() + ")" + " 没有查找到该站点(" + course.getSiteId() + ")下" +  "指定范围IDs:("+ listAccountIdRange.toString() + ") 的学员信息,无法统计数据");
							} else {
								taskDetail(taskId, course.getName() + "(" + course.getId() + ")" + " 没有查找到全平台:(" + siteId.toString() + ") 的学员信息,无法统计数据");
							}		
							
							logger.info("***** 获取课程参加人数： " + course.getName() + "(" + course.getId() + "), 0");
							continue;
						}
						logger.info("***** 获取课程参加人数： " + course.getName() + "(" + course.getId() + "), " + resultccountMap.size());
					}
					if(resultccountMap != null){
						//添加学员信息
						insertEmptyCourseLearn(resultccountMap, course, insertList, courseDuration, taskId);
					}
					
					// 循环统计每天的学习记录
					if(!isRunDel){
						//首次初始化数据时，开始时间就是课程创建时间
						beginDate = course.getCreateTime();
					}
					day = (overDate.getTime() - beginDate.getTime()) / (24 * 60 * 60 * 1000);    
					for (int d = 0; d <= day; d++) {
						String currentDate = null;
						dCurDate = addDay(beginDate, d);
						currentDate = formatDate.format(dCurDate);

						// 批量添加当前日期学习记录
						accountLearns = statisticsCourseService.insertAccountLearn(courseId, currentDate);
						// 没有学习记录，跳转
						if (accountLearns == null || accountLearns.size() == 0) {
							continue;
						}

						// 遍历有学习的学员信息统计他们的学习情况，并更新统计表中存在的记录：更新条件ID
						for (StatisticsCourseLearn statCourse : accountLearns) {
							//statCourse.setCourseId(courseId);
							Long accountId = statCourse.getAccountId();
							if(accountId == null){
								taskDetail(taskId, course.getName() + "(" + course.getId() + ") 学员ID为空，学习记录ID:(" + statCourse.getId() + ")" );
								continue;
							}
							
							// 实际参加学习状态
							statCourse.setJoinState(1);
							// 学习这门课的积分 
							try {
								statCourse.setLearnPoint(pointUserFeignClients.acquirePoint(accountId, courseId, currentDate));
							} catch (Exception e) {
								taskDetail(taskId,course.getName() + "(" + course.getId() + ") 学员ID:(" + accountId + ") 获取积分服务不可用: " + e.getMessage());
							}

							if(statCourse.getLearnPoint()==null) {
								statCourse.setLearnPoint(0);
							}
							statCourse.setLearnScore(0); // 暂时无法统计
							
							// 更新已学习的学员信息
							statisticsCourseLearnMapper.updateById(statCourse);
							//statisticsCourseLearnMapper.updateRecoredByCourseIdAndAccountId(statCourse);
							//statisticsCourseLearnService.updateRecoredByCourseIdAndAccountId(statCourse);	
						}
					}
					taskDetail(taskId, "成功统计课程 " + course.getName() + "(" + course.getId() + ")");
					logger.info("***** 成功统计课程 " + course.getName() + "(" + course.getId() + ")");
				}catch(Exception e){
					e.printStackTrace();
					errMsg += "   " + e.getMessage();
					isOk = false;
					taskDetail(taskId, "统计课程 " + curCourseName + "(" + curCourseId + ")异常:" + e.getMessage());
					logger.info("***** 统计课程 " + curCourseName + "(" + curCourseId + ")异常:" + e.getMessage());
				}
			}
			
			try{
				//添加课程，先删除后添加信息
				statisticsCourseMapper.deleteStatisticsCourseToGroupFind();
				statisticsCourseMapper.insertStatisticsCourseToGroupFind();
				
				//添加学员，先删除后添加信息
				statisticsCourseMapper.deleteStatisticsCourseToAccountGroupFind();
				statisticsCourseMapper.insertStatisticsCourseToAccountGroupFind();
				
				//添加学员应参加课程数量，先删除后添加信息
				statisticsCourseMapper.deleteStatisticsAccountJoinCount();
				statisticsCourseMapper.insertStatisticsAccountJoinCount();
				
				//添加部门学习数据，先删除后添加信息
				//statisticsCourseMapper.deleteStatisticsCourseToOrgMetadata();
				//statisticsCourseMapper.insertStatisticsCourseToOrgMetadata();
				 
				//更新部门编码
				/*List<Map<String, Long>> emptyOrg = statisticsCourseMapper.selectEmptyOrgCode();
				List<OrgVO> orgVos = null;
				for(Map<String, Long> mapItem : emptyOrg){
					Long companyId = mapItem.get("companyId");
					orgVos = organizationClient.fuzzySearchOrgByName("", companyId);
					if(orgVos != null){
						for(OrgVO item : orgVos){
							statisticsCourseMapper.updateEmptyOrgCode(item.getCode(), companyId, item.getId());
						}
					}
				}*/
			} catch(Exception e){
				logger.info("***** 添加数据到按部门统计表错误 *****");
				taskDetail(taskId, "添加数据到中间表异常:" + e.getMessage());
				errMsg += "   " + e.getMessage();
				isOk = false;
			}
			
			if(accountMap != null && accountMap.size() > 0){
				accountMap.clear();
			}
			
			String con = "在线课程报表分析完成耗时 :" + (System.currentTimeMillis() - startRunDate.getTime()) / 1000 + "秒,";
			if(isOk){
				con += "执行成功";
				taskDetail(taskId, con);
				success(taskContext, con);
			} else {
				con += "执行过程中有错误:" + errMsg;
				taskDetail(taskId, con);
				fail(taskContext, con);
			}
		} catch(Exception e){
			e.printStackTrace();
			String con = "在线课程报表分析异常结束：" + e.getMessage();
			logger.info("***** " + con);
			taskDetail(taskId, con);
			fail(taskContext, con);
		}
		

		return "123";
	}
 
	/**
	 * 创建和获取学员学习对象，缓存全平台可见的学员对象
	 * 
	 * @param siteId 站点ID，为空不缓存学员信息
	 * @param orgIdsAndAccountIds 指定范围可见ID集合 
	 * @param pageNo 页码
	 * @param pageSize 页数
	 * @param resultccountMap 保存当前查询的学员信息，每次会清空集合中的记录
	 * @return 当前查询的学员IDs
	 */
	private StatisticsCourse getReportAccountRespVO(Long siteId, List<Long> orgIdsAndAccountIds, Integer pageNo, Integer pageSize, 
			Long findAccountId, Map<Long, StatisticsCourse> resultccountMap, String rangeKey, Long taskId, String taskCon) {/*
		
		//查找指定的人
		StatisticsCourse learnStu = null;
		
		// 获取用户信息
		ReportRangeAccountReqVO rrarv = new ReportRangeAccountReqVO();
		rrarv.setSiteId(siteId);
		rrarv.setPageNo(pageNo);
		rrarv.setOrgIdsAndAccountIds(orgIdsAndAccountIds);
		rrarv.setPageSize(pageSize);
		PageInfo<ReportAccountRespVO> page = null;
		try{
			page = reportClient.getRangeAccounts(rrarv);
			if(page == null || page.getRecords() == null){
				return null;
			}			
		 
			learnStu = initStatisticsCourses(page.getRecords(), findAccountId, resultccountMap, rangeKey);
			
			//分页查询查找所有人
			int pageTotal = page.getPageTotal();
			//int pageCount = (int) Math.ceil(Double.valueOf(page.getPageRecords()) / pageSize);
			for (int i = 2; i <= pageTotal; i++) {
				rrarv.setSiteId(siteId);
				rrarv.setPageNo(i);
				rrarv.setOrgIdsAndAccountIds(orgIdsAndAccountIds);
				rrarv.setPageSize(pageSize);
				page = reportClient.getRangeAccounts(rrarv);
				if(page != null && page.getRecords().size() > 0){
					if(learnStu == null){
						learnStu = initStatisticsCourses(page.getRecords(), findAccountId, resultccountMap, rangeKey);
					} else {
						initStatisticsCourses(page.getRecords(), null, resultccountMap, rangeKey);
					}
				}
			}
		} catch(Exception e){
			e.printStackTrace();
			if(siteId == null){
				taskDetail(taskId, "获取" + taskCon + "的指定范围Ids:(" + orgIdsAndAccountIds + ") system服务不可用,:" + e.getMessage());
			} else {
				taskDetail(taskId, "获取" + taskCon + "的全平台:(" + siteId + ") system服务不可用,:" + e.getMessage());
			}
			
			return null;
		}
		
		return learnStu;
	*/
	return null;	
	}
				
	/**
	 * 创建和获取学员学习对象
	 * 
	 * @param listReportAccountRespVO 查询出的学员
	 * @param resultccountMap 保存当前查询的学员信息，每次会清空集合中的记录
	 * @return 当前查询的学员IDs
	 */
	private StatisticsCourse initStatisticsCourses(List<ReportAccountRespVO> listReportAccountRespVO, Long findAccountId,
												   Map<Long, StatisticsCourse> resultccountMap, String rangeKey) {
		
		//查找指定的人
		StatisticsCourse learnStu = null;
		
		//找到的人员信息
		// 初始化学员学习信息 
		for (ReportAccountRespVO item : listReportAccountRespVO) {
			Long accountId = item.getUserId();//账号
			StatisticsCourse stu = null;//学习记录对象
			if(accountMap.get(rangeKey) != null){
				stu = accountMap.get(rangeKey).get(accountId);
			} else {
				accountMap.put(rangeKey, resultccountMap);
			}
			
			if(stu == null){
				stu = new StatisticsCourse();
				stu.setId(null);
				stu.setAccountId(accountId);
				stu.setAccountState(item.getStatus());
				stu.setWorkNum(stu.getWorkNum());
				stu.setName(item.getUserName());
				stu.setFullname(item.getUserFullName());
				stu.setOrgName(item.getOrgName());
				stu.setAccountOrgId(item.getOrgId());
				String orgParentNames = null;
				if (item.getParentOrgNames() != null) {
					for (String orgName : item.getParentOrgNames()) {
						if (orgParentNames == null) {
							orgParentNames = orgName;
						} else {
							orgParentNames += "/" + orgName;
						}
					} 
					stu.setOrgParentNames(orgParentNames);
				}
				
				// 设置实际参加状态，用于实际人数的统计
				stu.setCanState(1);
				//设置已参加状态为0（未参加）
				stu.setJoinState(0);
				resultccountMap.put(accountId, stu);
			}
			if(findAccountId != null && findAccountId.compareTo(accountId) == 0){
				learnStu = stu;
			}
		}
		return learnStu;
	}
	
	/**
	 * 插入空的学习记录
	 * @param resultccountMap
	 * @param insertList
	 * @param course
	 * @param courseDuration
	 */
	private void insertEmptyCourseLearn(Map<Long, StatisticsCourse> resultccountMap, Course course, 
			List<StatisticsCourse> insertList, String courseDuration, Long taskId){
		if(insertList != null){
			insertList.clear();
		}
		
		//获取企业的所有部门信息
		String orgNo = "";
		String orgName = "";
		Long orgId = course.getOrgId(); // 部门ID
		if(orgId != null){
			List<OrgVO> orgs = orgMap.get(orgId.toString());
			if(orgs == null){
				logger.info("***** 获取" + orgId + " 部门信息");
				try {
					List<Long> ids = new ArrayList<Long>();
					ids.add(orgId);
					orgs = organizationClient.listByOrgIds(ids);
				} catch (Exception e) {
					e.printStackTrace();
					taskDetail(taskId, "获取" + orgId + "的部门信息错误system服务不可用:" + e.getMessage());
				}
				orgMap.put(orgId.toString(), orgs);
			}
			if(orgs != null){
				for(OrgVO item : orgs){
					if(orgId.compareTo(item.getId()) == 0){
						orgNo = item.getCode();
						orgName = item.getName();
						break;
					}
				}
			}
		}
		
		//获取该课程的学员信息
		Map<Long, Long> stuMap = statisticsCourseMapper.selectAccountLearnByCourseId(course.getId());
		
		Iterator<StatisticsCourse> statCourses = resultccountMap.values().iterator();
		StatisticsCourse statCourse;
        while (statCourses.hasNext()){
        	//更新课程信息
        	statCourse = statCourses.next();
        	statCourse.setCourseId(course.getId());
			statCourse.setCourseCode(course.getCode());
			statCourse.setCourseName(course.getName());
			statCourse.setCourseCreateTime(course.getCreateTime());
			statCourse.setCourseState(course.getShelves());
			statCourse.setCourseDuration(courseDuration);
			//课程的站点信息
			statCourse.setCompanyId(course.getCompanyId());
			statCourse.setSiteId(course.getSiteId());
			statCourse.setOrgId(course.getOrgId());
			statCourse.setOrgNo(orgNo);
			statCourse.setCourseOrgName(orgName);
			
			if(stuMap == null){
				insertList.add(statCourse);
			} else {
				if(stuMap.get(statCourse.getAccountId()) == null){
					insertList.add(statCourse);
				}
			}
        }
        //批量插入 
        if(!CollectionUtils.isEmpty(insertList)) {
        	try{
        		//批量插入太慢，因此采用导入插入的方式
        		//statisticsCourseService.insertBatch(insertList);
        		insertBatchStatisticsCourse(insertList);
        	} catch(Exception e){
        		logger.info("***insert data err *********" + course.getName() + "(" + course.getId() + ")"+ "数据错误:" + e.getMessage());
        		taskDetail(taskId, "批量添加" + course.getName() + "(" + course.getId() + ")"+ "数据错误:" + e.getMessage());
        		e.printStackTrace();
        	}
        }  
	}
	
	
	
	@SuppressWarnings("static-access")
	private Date addDay(Date date, int num) {
		Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		calendar.add(calendar.DATE, num);
		date = calendar.getTime();
		return date;
	}
		
	public Response<String> insertBatchStatisticsCourse(List<StatisticsCourse> listStatisticsCourse)
			throws SQLException {

		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try{
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(EXECUTE_SQL);
			mysqlPreStatement = preStatement
					.unwrap(com.mysql.jdbc.PreparedStatement.class);
			in = switchStream(listStatisticsCourse);
			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
		} catch(Exception e){
			e.printStackTrace();
			throw e;
		} finally {
			//大对象手动显示的设置为空保证GC时被释放
			if(in != null){
				in = null;
			}
			
			//关闭数据库连接
			if (mysqlPreStatement != null) {
				mysqlPreStatement.close();
			}
			if (preStatement != null) {
				preStatement.close();
			}
			if (conn != null) {
				DataSourceUtils.releaseConnection(conn, dataSource);
			}
		}
		 
		return Response.ok();
	}

	public InputStream switchStream(List<StatisticsCourse> list) {

		Long id = null;
		for (StatisticsCourse sc : list) {
			id = idGenerator.generate();
			strBuilder.append(id); // 主键
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCourseId());// 课程id
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCourseCode());// 课程编码
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCourseName());// 课程名字
			strBuilder.append(FENGEFU);
			if (sc.getCourseCreateTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getCourseCreateTime())); // 课程创建时间
			}
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCourseState());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCourseDuration());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCourseOrgName());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getOrgNo());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getOrgId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getSiteId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCompanyId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getAccountId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getWorkNum());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getName());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getFullname());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getOrgName());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getOrgParentNames());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getAccountState());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getAccountOrgId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getAccountSiteId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getAccountCompanyId());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getLearnDuration());
			strBuilder.append(FENGEFU);
			if (sc.getStartTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getStartTime()));
			}
			strBuilder.append(FENGEFU);
			if (sc.getEndTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getEndTime()));
			}
			strBuilder.append(FENGEFU);
			if (sc.getFinishLearnTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getFinishLearnTime()));
			}
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getLearnState());
			strBuilder.append(FENGEFU);
			if (sc.getFirstFinishTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getFirstFinishTime()));
			}
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCanState());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getJoinState());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getLearnCount());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCommentAccountCount());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getCommentCount());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getGrade());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getFinishCount());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getLearnPoint());
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getLearnScore());
			
			strBuilder.append(HUANHANGFU);

		}
		byte[] bytes = strBuilder.toString().getBytes();
		InputStream in = new ByteArrayInputStream(bytes);

		//大对象手动显示的设置为空保证GC时被释放
		bytes = null;
		
		//清空之前的内容
		strBuilder.delete(0, strBuilder.length());
		
		return in;
	}
}
