package com.yizhi.course.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.cache.distributedlock.impl.RedisDistributedLock;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.domain.TestCase;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.service.ITestCaseService;
import com.yizhi.course.application.vo.*;
import com.yizhi.util.application.date.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 测验案件 前端控制器
 * </p>
 *
 * @author 谢海军123
 * @since 2018-10-17
 */
@RestController
@RequestMapping("/testCase")
public class TestCaseController {

    private final static String COURSE_DISTRIBUTED_LOCK = "distributed.lock:course";

    @Autowired
    private ITestCaseService testCaseService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    private RedisDistributedLock redisDistributedLock;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private CourseStudyController completionStatus;

    @Autowired
    private IRecordeService recordeService;

    private Logger LOGGER = LoggerFactory.getLogger(TestCaseController.class);

    /**
     * 测验单次提交
     * @param vo
     */
    @PostMapping("/simple/submit")
    public TestCaseSimpleSubmitVO testCaseSimpleSubmit(@RequestBody TestCaseSimpleSubmitVO vo){
        return vo;
    }

    @GetMapping("/idGenerator/id")
    public Long idGeneratorId(){
        return idGenerator.generate();
    }

    /**
     * 总提交
     */
    @PostMapping("/submit")
    public TestCaseSubmitVO testSubmit(@RequestBody TestCaseVO vo){
        TestCaseSubmitVO caseSubmitVO = testCaseService.testSubmit(vo);

        RequestContext rt = ContextHolder.get();
        String courseLockKey = COURSE_DISTRIBUTED_LOCK+":"+caseSubmitVO.getCourseId()+"."+rt.getAccountId();
        int maxRetry = 0;
        boolean courseExecLockStatus = false;
        do{
            courseExecLockStatus = redisDistributedLock.lock(courseLockKey,5000,90,50);
            maxRetry++;
        }while (!courseExecLockStatus && maxRetry<3);
        try {
            // 获取以前测验的状态
            EntityWrapper<Recorde> testRecodeEw = QueryUtil.condition(new Recorde());
            testRecodeEw.eq("account_id", rt.getAccountId()).eq("chapter_id",caseSubmitVO.getChapterId());
            //String session = "TEST-"+caseSubmitVO.getTestId()+""+rt.getAccountId()+ DateUtil.getYMDHMSS();
            String session = "TEST-"+""+rt.getAccountId()+ DateUtil.getYMDHMSS();
            testRecodeEw.eq("session_id", session);
            Recorde exsits = recordeService.selectOne(testRecodeEw);
            // 测验完，新增一条学习记录
            boolean isPass = caseSubmitVO.getIsPass();

            Recorde recorde1 = new Recorde();
            recorde1.setOvered(isPass ? 1 : 0);
            recorde1.setSiteId(rt.getSiteId());
            recorde1.setChapterId(caseSubmitVO.getChapterId());
            recorde1.setDuration(0);
            Date date = new Date();
            recorde1.setEndTime(date);
            recorde1.setValid(1);
            recorde1.setStartTime(date);
            recorde1.setCreateTime(date);
            recorde1.setCourseId(caseSubmitVO.getCourseId());
            recorde1.setOrgId(rt.getOrgId());
            recorde1.setAccountId(rt.getAccountId());
            recorde1.setCompanyId(rt.getCompanyId());
            recorde1.setExitstates(1);
            recorde1.setTerminalDevice("测验记录");
            recorde1.setDataType(0);
            recorde1.setSessionId(session);
            completionStatus.calculateStudyStatusNew(recorde1);
            if(Objects.isNull(exsits)){
                recorde1.setId(idGenerator.generate());
                recorde1.setSessionId(session);
                recordeService.insert(recorde1);
            }else {
                Recorde recordeUp = new Recorde();
                int overed = exsits.getOvered();
                if(overed == 0 && isPass){
                    recordeUp.setOvered(1);
                }
                recordeUp.setId(exsits.getId());
                recordeUp.setEndTime(new Date());
                recordeService.updateById(recordeUp);
            }

            return caseSubmitVO;
        }finally {
            if(courseExecLockStatus){
                redisDistributedLock.releaseLock(courseLockKey);
            }
        }
    }

    /**
     * 根据测试答卷查询答卷信息
     * @param testCaseId
     * @return
     */
    @GetMapping("/record/get")
    public TestCaseRecordVO getRecordInfo(@RequestParam("testCaseId") Long testCaseId){
        TestCaseRecordVO vo = testCaseService.getRecordInfo(testCaseId);
        return vo;
    }

    /**
     * 判断该课程是否被该用户测验通过
     * @param courseId
     * @param accountId
     * @return
     */
    @GetMapping("/course/pass")
    public Boolean courseIsSucceed(@RequestParam("courseId") Long courseId, @RequestParam("accountId") Long accountId){
        Chapter chapter = new Chapter();
        chapter.setSource(9);
        chapter.setCourseId(courseId);
        List<Chapter> chapters = chapterService.selectList(QueryUtil.condition(chapter));
        // 如果没有关联测验的课程直接返回true
        if(CollectionUtils.isEmpty(chapters)){
            return true;
        }
        List<Long> chapterIds = new ArrayList<>();
        //List<Long> testIds = new ArrayList<>();
        for(Chapter obj : chapters){
            chapterIds.add(obj.getId());
            //testIds.add(obj.getMaterialId());
        }
        // 关联测验课程，查询各个章节的测验是否通过
        EntityWrapper<TestCase> ew = QueryUtil.condition(new TestCase());
        ew.eq("course_id",courseId).eq("status",1).eq("account_id",accountId)
                .in("chapter_id",chapterIds);
        List<TestCase> testCaseList = testCaseService.selectList(ew);
        if(CollectionUtils.isEmpty(testCaseList)){
            return false;
        }

        Set<Long> retDataIds = testCaseList.stream().map(item->item.getChapterId()).collect(Collectors.toSet());
        LOGGER.info("指定课程，查询的测验章节ids={},通过测验的章节ids={}",
                JSON.toJSONString(chapterIds),JSON.toJSONString(retDataIds));
        chapterIds.removeAll(retDataIds);
        return CollectionUtils.isEmpty(chapterIds)?true:false;
    }

    @PostMapping("/chapters/pass")
    public Boolean chaptersIsSucceed(@RequestBody ChaptersPassReqVO vo){
        List<Long> chapterIds = vo.getChapterIds();
        // 关联测验课程，查询各个章节的测验是否通过
        EntityWrapper<TestCase> ew = QueryUtil.condition(new TestCase());
        ew.eq("status",1).eq("account_id",vo.getAccountId())
                .in("chapter_id",chapterIds);
        List<TestCase> testCaseList = testCaseService.selectList(ew);
        if(CollectionUtils.isEmpty(testCaseList)){
            return false;
        }
        Set<Long> retDataIds = testCaseList.stream().map(item->item.getChapterId()).collect(Collectors.toSet());
        LOGGER.info("指定章节，查询的测验章节ids={},通过测验的章节ids={}",
                JSON.toJSONString(chapterIds),JSON.toJSONString(retDataIds));
        chapterIds.removeAll(retDataIds);
        return CollectionUtils.isEmpty(chapterIds)?true:false;
    }
}

