package com.yizhi.course.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.course.application.domain.TestInfo;
import com.yizhi.course.application.domain.TestSubject;
import com.yizhi.course.application.domain.TestSubjectOption;
import com.yizhi.course.application.service.ITestService;
import com.yizhi.course.application.service.ITestSubjectOptionService;
import com.yizhi.course.application.service.ITestSubjectService;
import com.yizhi.course.application.vo.*;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 测验表 前端控制器
 * </p>
 *
 * @author 谢海军123
 * @since 2018-10-11
 */
@RestController
@RequestMapping("/test")
public class TestController {

    private Logger logger = LoggerFactory.getLogger(TestController.class);

    @Autowired
    private ITestService testService;

    @Autowired
    private ITestSubjectService testSubjectService;

    @Autowired
    private ITestSubjectOptionService testSubjectOptionService;

    /**
     * 管理端测验信息列表查询
     * @return
     */
    @PostMapping("/list")
    public Page<TestInfoVO> list(@RequestBody TestInfoListReqVO reqVO){
        Page<TestInfoVO> data = testService.listPage(reqVO);
        return data;
    }

    /**
     * 查询测验的试题列表
     * @param id
     * @return
     */
    @GetMapping("/subject/list")
    public List<TestSubjectVO> listSubject(@RequestParam("id") Long id){
        TestSubject ts = new TestSubject();
        ts.setTestId(id);
        EntityWrapper<TestSubject> ew = QueryUtil.condition(ts);
        ew.orderBy("sort");
        List<TestSubject> data = testSubjectService.selectList(ew);
        List<TestSubjectVO> ret = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(data)){
            data.forEach(obj->{
                TestSubjectVO vo = new TestSubjectVO();
                BeanUtils.copyProperties(obj,vo);
                ret.add(vo);
            });
        }

        return ret;
    }

    /**
     * 删除测试
     * @param deleteTestIds
     * @return
     */
    @PostMapping("/ids/delete")
    public Boolean delete(@RequestBody List<Long> deleteTestIds){
        return testService.deleteIds(deleteTestIds);
    }


    /**
     * 保存数据
     * @param vo
     * @return
     */
    @PostMapping("/save")
    public TestInfoVO save(@RequestBody TestInfoVO vo){
        TestInfoVO voData = null;
        try {
            voData = testService.saveTest(vo);
        }catch (BizException e){
            if(voData == null ){
                voData = new TestInfoVO();
            }
            voData.setMsgCode(e.getCode());
            voData.setMsgContext(e.getMsg());
        }
        return voData;
    }



    /**
     * 根据id查询（查看）
     * @param id
     * @return
     */
    @GetMapping("/view")
    public TestInfoViewRespVO view(@RequestParam(value = "id") Long id ){
        TestInfoViewRespVO ret = null;
        TestInfo testInfo = testService.selectById(id);
        if(testInfo != null){
            ret = new TestInfoViewRespVO();
            BeanUtils.copyProperties(testInfo,ret);
            EntityWrapper<TestSubject> ew = QueryUtil.condition(new TestSubject());
            ew.eq("test_id",id);
            ew.orderBy("sort",true);
            List<TestSubject> dataList = testSubjectService.selectList(ew);
            if(CollectionUtils.isNotEmpty(dataList)){
                if("02".equals(testInfo.getType())){
                    ret.setSumScore(dataList.stream().mapToInt(TestSubject::getScore).sum());
                }
                List<TestSubjectVO> voData = new ArrayList<>();
                dataList.forEach(obj->{
                    TestSubjectVO vo = new TestSubjectVO();
                    BeanUtils.copyProperties(obj,vo);
                    voData.add(vo);
                });
                ret.setTestSubjectVOList(voData);
            }
        }
        return ret;
    }

    /**
     * 查询测验全部信息（包含测验试题以及试题的答案选项）
     * @param id
     * @return
     */
    @GetMapping("/all/view")
    public TestInfoViewRespVO viewAll(@RequestParam(value = "id") Long id ){
        // 查询测验信息
        TestInfoViewRespVO ret = null;
        TestInfo testInfo = testService.selectById(id);
        if(testInfo == null){
            return null;
        }
        ret = new TestInfoViewRespVO();
        BeanUtils.copyProperties(testInfo,ret);

        // 查询测验试题信息
        EntityWrapper<TestSubject> ew = QueryUtil.condition(new TestSubject());
        ew.eq("test_id",id);
        ew.orderBy("sort",true);
        List<TestSubject> dataList = testSubjectService.selectList(ew);
        List<Long> subjectIds = dataList.stream().map(obj -> obj.getId()).collect(Collectors.toList());

        // 查询测验试题的答案选项
        EntityWrapper<TestSubjectOption> ew1 = QueryUtil.condition(new TestSubjectOption());
        ew1.in("test_subject_id",subjectIds);
        ew1.orderAsc(Arrays.asList("test_subject_id","sort"));
        List<TestSubjectOption> optionData = testSubjectOptionService.selectList(ew1);
        Map<Long,List<TestSubjectOption>> optionMap = optionData.stream()
                .collect(Collectors.groupingBy(obj->obj.getTestSubjectId()));

        // 随机题目
        if(ret.getSubjectRandomStrategy()){
            Collections.shuffle(dataList);
        }
        List<TestSubjectVO> retSubjectList = new ArrayList<>();
        for(TestSubject subject : dataList){
            TestSubjectVO subjectVo = new TestSubjectVO();
            BeanUtils.copyProperties(subject,subjectVo);
            List<TestSubjectOptionVO> optionVOList = new ArrayList<>();
            Long testSubjectId = subject.getId();
            List<TestSubjectOption> options = optionMap.get(testSubjectId);
            if(options == null){
                logger.warn("请注意 testSubjectId={}测验试题没有设答案",testSubjectId);
            }
            options.forEach(obj-> {
                TestSubjectOptionVO vo = new TestSubjectOptionVO();
                //BeanUtils.copyProperties(obj,vo,"answerResult");
                BeanUtils.copyProperties(obj,vo);
                optionVOList.add(vo);
            });
            // 添加试题选项(随机答案选项)
            if(ret.getOptionRandomStrategy()){
                Collections.shuffle(optionVOList);
            }else{
                optionVOList.stream().sorted(Comparator.comparing(obj->obj.getSort()));
            }
            subjectVo.setOptionVOS(optionVOList);
            retSubjectList.add(subjectVo);
        }

        ret.setTestSubjectVOList(retSubjectList);
        return ret;
    }

    /**
     * 更新测验是否是使用的状态
     * @param testId
     * @return
     */
    @GetMapping("/status/update")
    public Boolean updateStatus(@RequestParam(value = "testId") Long testId,
                                @RequestParam(required = false, value = "chapterId") Long chapterId,
                                @RequestParam(value = "type") Integer type){
        testService.updateStatus(testId,chapterId,type);
        return true;
    }

}

