package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.vo.ApiClassifyVo;
import com.yizhi.course.application.vo.ClassifyVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.session.RowBounds;

import java.util.List;


/**
 * <p>
 * 课程分类 Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface ClassifyMapper extends BaseMapper<Classify> {
    /**
     * 查找这个父级分类下的最大排序数
     * @param parentId
     * @return
     */
//    @Select("select ifnull(max(sort),0) " +
//            "from cloud_course.classify " +
//            "where parent_id=#{ParentId} " +
//            "limit 1")
    int selectSort(@Param("ParentId") Long parentId);

    /**
     *查找这个分类下的层数
     * @param parentId
     * @return
     */
    @Select("select ifnull(layer,0)\n" +
            "from cloud_course.classify\n" +
            "where parent_id=#{ParentId}\n" +
            "limit 1")
    int selectLayer(@Param("ParentId") Long parentId);

    /**
     * 根据id查找层级
     * @param Id
     * @return
     */
//    @Select("select layer\n" +
//            "from cloud_course.classify\n" +
//            "where id=#{Id}\n" +
//            "limit 1")
    int selectParentLayer(@Param("Id") Long Id);

    /**
     * 根据id查找root_id
     * @param Id
     * @return
     */
//    @Select("select root_id\n" +
//            "from cloud_course.classify\n" +
//            "where id=#{Id}\n" +
//            "limit 1")
    Long selectRootid(@Param("Id") Long Id);

    /**
     * 根据id查找其有没有子级分类
     * @param Id
     * @return
     */
//    @Select("select ifnull(id,0)\n" +
//            "from cloud_course.classify\n" +
//            "where parent_id=#{Id}\n"+
//            "and status = 1 " +
//            "limit 1")
    Long selectChild(@Param("Id") Long Id);

    /**
     * 根据分类id查找下面所挂的课程
     * @param Id
     * @return
     */
//    @Select("select ifnull(id,0)\n" +
//            "from cloud_course.course\n" +
//            " where classify_id=#{Id}\n"+
//            " and del_flg = 0 "+
//            "limit 1")
    Long selectCourse(@Param("Id") Long Id, @Param("siteId") Long siteId);

    List<ApiClassifyVo> selectListByCompanyId(@Param("companyId") Long companyId, @Param("name") String name,
                                              @Param("orgIds") List<Long> orgIds, @Param("siteId") Long siteId,
                                              RowBounds rowBounds);

    Integer selectCountByCompanyId(@Param("companyId") Long companyId,@Param("name") String name,@Param("orgIds") List<Long> orgIds,@Param("siteId") Long siteId);

    List<Classify> selectClassifyByRootId(@Param("ids") List<Long> ids);


    List<Classify> selectClassifyByLayer(@Param("companyId") Long companyId,@Param("siteId") Long siteId,@Param("layer") Integer layer);


    @Select("<script>" +
            "SELECT parent_id AS id ,COUNT(0) AS total FROM classify " +
            "WHERE parent_id in " +
            "<foreach collection=\"classifyIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">"+
            "#{cid}"+
            "</foreach>"+
            " GROUP BY parent_id" +
            "</script>")
    List<ClassifyVo> findExistChildNodes(@Param("classifyIds") List<Long> classifyIds);

}
