package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.course.application.domain.CourseShare;
import com.yizhi.course.application.vo.CourseForwardUserVo;
import com.yizhi.course.application.vo.CourseShareVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * <p>
 *  Mapper 接口
 * </p>
 *
 * @author 谢海军123
 * @since 2018-04-09
 */
public interface CourseMapperShareMapper extends BaseMapper<CourseShare> {

    @Select("<script>" +
            "SELECT " +
            " count(DISTINCT course_id) FROM course_share " +
            " WHERE site_id = #{siteId} AND account_id = #{accountId} AND share_url IS NOT NULL " +
            "</script>")
    Integer countMyShareCourseNum(@Param("siteId") Long siteId ,@Param("accountId") Long accountId);

    @Select("<script>" +
            "SELECT " +
            " cs.id AS id,cs.course_id AS courseId,cs.account_id AS accountId,cs.share_url AS shareUrl, " +
            " cs.share_time AS shareTime,SUM(cs.read_count) AS readCount,c.name AS courseName,c.image AS courseImage " +
            " FROM course_share cs LEFT JOIN course c " +
            " ON cs.course_id = c.id " +
            " WHERE cs.share_url IS NOT NULL AND cs.account_id = #{accountId} " +
            "<if test='searchFiled != null'>" +
            " AND c.name LIKE #{searchFiled} " +
            "</if>" +
            "<if test='siteId != null'>" +
            " AND cs.site_id = #{siteId} " +
            "</if>" +
            " GROUP BY cs.course_id" +
            "</script>")
    List<CourseShareVo> searchMyShareInfo(@Param("accountId") Long accountId, @Param("searchFiled") String searchFiled, @Param("siteId") Long siteId);

    @Select("<script>" +
            "SELECT " +
            " t1.course_id courseId, count(DISTINCT br.account_id) readUserCount " +
            " FROM ( " +
            " SELECT id, course_id FROM course_share cs " +
            " WHERE account_id = #{accountId} " +
            "<if test='courseIds != null'>" +
            " AND course_id IN " +
            "<foreach collection=\"courseIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">" +
            " #{cid} " +
            "</foreach>" +
            "</if>" +
            " ) t1 LEFT JOIN browse_records br ON t1.id = br.share_id WHERE br.avatar IS NOT NULL GROUP BY t1.course_id " +
            "</script>")
    List<CourseShareVo> searchMyUserInfo(@Param("accountId") Long accountId,
                                         @Param("courseIds") List<Long> courseIds);

    @Select("<script>" +
            "SELECT " +
            "name AS loginName, full_name AS userName,tel, " +
            "account_id AS accountId,count(0) AS forwardCount " +
            "FROM course_share " +
            "WHERE course_id=#{conrseId} " +
            "<if test='username != null'>" +
            "AND name LIKE #{username} " +
            "</if>"+
            "GROUP BY account_id "+
            "</script>")
    List<CourseForwardUserVo> courseShareCount(@Param("conrseId") Long conrseId, @Param("username") String username);

    @Select("<script>" +
            "SELECT " +
            " cs.id AS id,cs.course_id AS courseId,cs.account_id AS accountId, cs.first_share_id AS firstShareId,  " +
            " cs.second_share_id AS secondShareId,SUM(cs.read_count) AS readCount " +
            " FROM course_share cs " +
            " WHERE cs.account_id = #{accountId} AND cs.course_id = #{courseId} " +
            " <choose>"+
            "    <when test=\"firstShareId != null and firstShareId != ''\">"+
            "       AND cs.first_share_id = #{firstShareId}  AND cs.second_share_id IS NOT NULL " +
            "    </when>"+
            "    <otherwise>"+
            "       AND cs.first_share_id IS NOT NULL " +
            "    </otherwise>"+
            " </choose>" +
            "<if test='siteId != null'>" +
            " AND cs.site_id = #{siteId} " +
            "</if>" +
            " GROUP BY cs.course_id, cs.first_share_id" +
            "<if test=\"firstShareId != null and firstShareId != '' \">" +
            " , cs.second_share_id" +
            "</if>" +
            " ORDER BY cs.id" +
            "</script>")
    List<CourseShareVo> findShareSpreadList(@Param("accountId") Long accountId,
                                            @Param("courseId") Long courseId,
                                            @Param("siteId") Long siteId,
                                            @Param("firstShareId") String firstShareId);

    @Select("<script>" +
            " SELECT first_share_id AS firstShareId,MAX(second_share_id) AS secondShareId,COUNT(first_share_id) AS shareUserCount " +
            " FROM course_share WHERE account_id = #{accountId} AND course_id = #{courseId} AND site_id = #{siteId} " +
            "<if test='firstShareIds != null'>" +
            " AND first_share_id IN " +
            "<foreach collection=\"firstShareIds\" open=\"(\" close=\")\" separator=\",\" item=\"fid\" index=\"index\">" +
            " #{fid} " +
            "</foreach>" +
            "</if>" +
            " GROUP BY first_share_id" +
            "</script>")
    List<CourseShareVo> findShareSpreadNextList(@Param("accountId") Long accountId,
                                                @Param("courseId") Long courseId,
                                                @Param("siteId") Long siteId,
                                                @Param("firstShareIds") List<String> firstShareIds);
}
