package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.domain.dto.MaterialExportDto;
import com.yizhi.course.application.vo.MaterialManageVo;
import com.yizhi.course.application.vo.MaterialVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.session.RowBounds;

import java.util.List;
import java.util.Set;

/**
 * <p>
 * 素材库 Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface MaterialMapper extends BaseMapper<Material> {

    List<Material> searchPage(@Param("name") String name, @Param("type") Integer type, @Param("source") Integer source,
                              @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") List<Long> orgId, RowBounds rowBounds);

    Integer searchPageCount(@Param("name") String name, @Param("type") Integer type, @Param("source") Integer source,
                            @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") List<Long> orgId);

    /**
     * 管理员得到做客列表
     *
     * @param name
     * @param listCommintIdList
     * @param listAuditorIdList
     * @param state
     * @param page
     * @param orgId
     * @return
     */
    List<MaterialManageVo> manageGetLessonList(@Param("name") String name, @Param("listCommintIdList") List<Long> listCommintIdList, @Param("listAuditorIdList") List<Long> listAuditorIdList, @Param("state") Integer state, Page<MaterialManageVo> page, @Param("orgId") Long orgId);

    /**
     * 得到下一个id
     *
     * @param id
     * @return
     */
    Long getNextId(@Param("id") Long id);


    List<Material> studentMyLessonList(@Param("name") String name, @Param("accountId") Long accountId, @Param("page") Page<Material> page);


    /**
     * 已通过的做课列表
     *
     * @param name
     * @param accountId
     * @param page
     * @return
     */
    List<Material> getPassMaterialList(@Param("name") String name, @Param("accountId") Long accountId, @Param("page") Page<Material> page);


    /**
     * 已通过的做课数量
     *
     * @param name
     * @param accountId
     * @return
     */
    Integer getPassMaterialListCount(@Param("name") String name, @Param("accountId") Long accountId);


    Integer getListNumByAccountIdAndName(@Param("name") String name, @Param("accountId") Long accountId);


    /**
     * 查询素材列表  ( Scorm课程 )
     *
     * @return
     */
    List<Material> queryScormCourseList(@Param("name") String name, @Param("type") Integer type, @Param("source") Integer source,
                                        @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") List<Long> orgId, RowBounds rowBounds);

    Integer queryScormCourseListCount(@Param("name") String name, @Param("type") Integer type, @Param("source") Integer source,
                                      @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") List<Long> orgId);

    /**
     * 查询素材列表  ( 一般课程  )
     *
     * @param name
     * @param type
     * @param source
     * @param companyId
     * @param siteId
     * @param orgId
     * @param rowBounds
     * @return
     */
    List<Material> queryCommonCourseList(@Param("name") String name, @Param("type") Integer type, @Param("source") Integer source,
                                         @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") List<Long> orgId,
                                         @Param("excludType") Integer excludType, RowBounds rowBounds);

    Integer queryCommonCourseListCount(@Param("name") String name, @Param("type") Integer type, @Param("source") Integer source,
                                       @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") List<Long> orgId);


    List<Material> getCourseTimeMessage(@Param("companyId") Long companyId, @Param("ids") List<Long> ids);

    /**
     * 修改一下sql需注意
     * 当 audio_video没有数据时，material表的数据要正常获取
     * @param materialIds
     * @return
     */
    @Select("<script>" +
            "SELECT " +
            "m.id," +
            "m.name," +
            "m.type," +
            "m.code," +
            "m.original_file AS originalFile," +
            "m.original_file_url AS originalFileUrl,  " +
            "m.source," +
            "m.tags," +
            "m.type," +
            "m.text_hour AS textHour," +
            "m.text_minute AS textMinute," +
            "m.text_second AS textSecond, " +
            "m.producer," +
            "m.exam_id AS examId," +
            "m.exam_name AS examName," +
            "m.poly_vid AS polyVid," +
            "m.size," +
            "m.display_mark as displayMark," +
            "m.word_type as wordType," +
            "m.word as word," +
            " a.image," +
            "a.ld_url AS ldUrl," +
            "a.sd_url AS sdUrl," +
            "a.fd_url AS fdUrl," +
            "a.explanation " +
            " FROM material m LEFT JOIN audio_video a " +
            " ON m.id = a.material_id and a.deleted = 0 " +
            " WHERE m.id IN " +
            "<foreach collection=\"materialIds\" open=\"(\" close=\")\" separator=\",\" item=\"mid\" index=\"index\">" +
            "#{mid}" +
            "</foreach>" +
            "and m.deleted=0" +
            "group by m.id" +
            "</script>")
    List<MaterialVo> getMaterialDetails(@Param("materialIds") Set<Long> materialIds);


    List<Material> queryMaterialList(@Param("codeList") List<String> codeList, @Param("beforeSiteId") Long beforeSiteId);

    List<MaterialExportDto> getMaterialExportDtoList(@Param("courseIds")List<Long> courseIds);

    /**
     * 根据素材id查询素材
     * @param materialIds 素材id
     */
    List<Material> selectMaterialsByIds(@Param("materialIds") List<Long> materialIds,@Param("companyId") Long companyId,@Param("siteId") Long siteId);
}
