package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.domain.ScormRecord;
import com.yizhi.course.application.vo.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.session.RowBounds;

import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 学习记录 Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface RecordeMapper extends BaseMapper<Recorde> {

    @Update("update recorde set duration = duration + #{progress}, end_time=#{endDate} where id = #{dataId} ")
    Integer updateByHeart(@Param("dataId") Long dataId, @Param("progress") Integer progress, @Param("endDate") Date endDate);


    @Select("<script>" +
            " SELECT site_id AS siteId, account_id AS accountId, org_id AS orgId, SUM(duration) AS durationSum FROM recorde  " +
            " WHERE site_id = #{siteId} AND org_id > 0 AND account_id > 0 AND duration>0" +
            "<choose>" +
            "<when test=\"active != '' and active=='prod'\">" +
            " AND <![CDATA[ end_time < curdate() ]]> " +
            "</when>" +
            "<otherwise>" +
            " AND <![CDATA[ end_time < date_sub(curdate(),interval -1 day) ]]> " +
            "</otherwise>" +
            "</choose>" +
            " GROUP BY site_id , account_id " +
            "</script>")
    List<CourseStudyDurationRanking> studyHourRanking(@Param("siteId") Long siteId,
                                                      @Param("active") String active);

    	@Select("<script>" +
			" SELECT COUNT(0) AS studyCount,SUM( duration ) AS studyDuration " +
			" FROM recorde " +
			" WHERE course_id = #{courseId} AND account_id = #{accountId} AND site_id = #{siteId} AND duration>0" +
			"</script>")
    CourseStudyStatisticVO getCourseStudyStatistic(@Param("courseId") Long courseId,
                                                   @Param("accountId") Long accountId,
                                                   @Param("siteId") Long siteId);
    @Select("<script> " +
            "SELECT " +
            "duration, " +
            "start_time AS startTime," +
            "end_time AS endTime " +
            "FROM " +
            "recorde " +
            "WHERE " +
            "course_id = #{courseId} AND account_id = #{accountId} AND site_id = #{siteId} " +
            "</script>")
    List<CourseStudyRecordeVo> getCourseStudyRecorde(@Param("courseId") Long courseId,
                                               @Param("accountId") Long accountId,
                                               @Param("siteId") Long siteId);
    // 记录列表
    List<RecordeListVo> selectRecordeList(@Param("accountId") Long accountId, @Param("courseId") Long courseId,
                                          @Param("siteId") Long siteId);

    // 章节学习记录完成状态列表
    List<RecordeListVo> selectRecordeOveredList(@Param("accountId") Long accountId, @Param("courseId") Long courseId,
                                                @Param("siteId") Long siteId);

    // 我的课程记录列表
    List<MyCourseRecordeVo> selectMyCourseRecordeList(@Param("name") String name, @Param("accountId") Long accountId,
                                                      @Param("type") Integer type, RowBounds rowBounds, @Param("siteId") Long siteId);

    Integer selectDurationByCourseId(@Param("courseId") Long courseId);

    /**
     * 统计我的课程记录数量
     *
     * @param accountId
     * @param type
     * @param siteId
     * @return
     */
    Integer selectMyCourseRecordeCount(@Param("accountId") Long accountId, @Param("type") Integer type,
                                       @Param("siteId") Long siteId);

    Integer selectMyCourseRecordeNumber(@Param("name") String name, @Param("accountId") Long accountId,
                                        @Param("type") Integer type, @Param("siteId") Long siteId);

    List<RecordeDurationVO> selectAllRecordeDuration();

    /**
     * 根据用户 id 和 章节 id 获取 完成条数
     *
     * @param accountId
     * @param chapterIds
     * @return
     */
    Integer getCountByAccountIdAndChapterIds(@Param("accountId") Long accountId,
                                             @Param("chapterIds") Set<Long> chapterIds);

    @Select("SELECT chapter_id FROM recorde WHERE course_id=#{courseId} and account_id=#{accountId} and data_type=1 ORDER BY start_time DESC LIMIT 0,1")
    Long selectLastStudy(@Param("courseId") Long courseId, @Param("accountId") Long accountId);

    @Select("SELECT SUM(duration) FROM recorde WHERE site_id = #{siteId} and account_id = #{accountId} and duration > 0")
    Long countStudyDurationByUser(@Param("siteId") Long siteId, @Param("accountId") Long accountId);

    //@Select("SELECT COUNT(DISTINCT(account_id)) FROM recorde WHERE course_id=#{courseId}")
    @Select("SELECT COUNT(DISTINCT(account_id)) AS study_num FROM course_view_record WHERE course_id = #{courseId}")
    Integer getCourseNumByCourseId(@Param("courseId") Long courseId);

    List<Long> getFinishedIds(@Param("courseId") Long courseId);

    Integer getStudyTime(@Param("courseIds") List<Long> courseIds, @Param("siteId") Long siteId,
                         @Param("accountId") Long accountId);

    @Select("<script>"
            + "SELECT SUM(duration) AS durationSum FROM recorde WHERE account_id=#{accountId} AND site_id=#{siteId} AND chapter_id=#{chapterId} AND <![CDATA[ duration > 0 ]]>"
            + "</script>")
    Long countStudyDuration(@Param("accountId") Long accountId, @Param("chapterId") Long chapterId,
                            @Param("siteId") Long siteId);

    @Select("<script>"
            + "SELECT SUM(duration) AS durationSum FROM recorde " +
            "WHERE account_id=#{accountId} AND site_id=#{siteId} AND course_id IN " +
            "<foreach collection=\"courseIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">" +
            "#{cid} " +
            "</foreach>" +
            "</script>")
    Long countStudyDurationCourseIds(@Param("accountId") Long accountId, @Param("siteId") Long siteId,
                                     @Param("courseIds") List<Long> courseIds);

    List<MyCourseRecordeVo> findCourseBaseInfoByIds(@Param("courseIds") List<Long> courseIds,
                                                    @Param("siteId") Long siteId, @Param("accountId") Long accountId);

    @Select("<script>   SELECT course_id AS courseId,account_id AS accountId,site_id AS siteId,COUNT(id) AS learnCount,MIN(start_time) AS startTime,MAX(end_time) AS endTime,SUM(duration) AS learnTime,DATE_FORMAT(MIN(start_time),'%Y-%m-%d') AS recordeTime FROM recorde WHERE \r\n"
            + "			course_id=#{cId}  and duration>0 and"
            + "			<![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')<=DATE_FORMAT(#{endDate,jdbcType=VARCHAR},'%Y-%m-%d')]]>   and "
            + "			<![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')>=DATE_FORMAT(#{startDate,jdbcType=VARCHAR},'%Y-%m-%d')]]>  "
            + "			GROUP BY account_id,<![CDATA[ DATE_FORMAT(start_time,'%Y-%m-%d')]]>,<![CDATA[ DATE_FORMAT(end_time,'%Y-%m-%d')]]>" + "</script>")
    List<StatisticsCourseRecorde> getCourseRecordeByCourseId(@Param("cId") Long cId,
                                                             @Param("startDate") String startDate, @Param("endDate") String endDate);

    @Select("<script>"
            + "SELECT siteId,course_id AS courseId,account_id AS accountId,LEVEL,COMMENT,finish_time AS finishTime,TYPE,record_time AS recordeTime,collect_num AS collectNum  FROM ("

            + "SELECT #{siteId} as siteId,course_id,account_id,NULL AS LEVEL,1 AS COMMENT,NULL AS finish_time ,2 AS TYPE,create_time AS record_time,0 AS collect_num FROM `comment`  WHERE course_id=#{cId} "
            + " and <![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')>=DATE_FORMAT(#{startDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "
            + " and <![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')<=DATE_FORMAT(#{endDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "

            + "			UNION ALL   "

            + "	SELECT #{siteId} as siteId,course_id,account_id,LEVEL,NULL AS COMMENT,NULL AS finish_time ,1 AS TYPE,create_time AS record_time,0 AS collect_num FROM course_grade WHERE course_id=#{cId} "
            + " and <![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')>=DATE_FORMAT(#{startDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "
            + " and <![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')<=DATE_FORMAT(#{endDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "

            + "			UNION ALL  "

            + "	SELECT #{siteId} as siteId,course_id,account_id,NULL AS LEVEL,NULL AS COMMENT,first_finish_time AS finish_time,3 AS TYPE,last_student_time AS record_time,0 AS collect_num  FROM course_student_status WHERE course_id=#{cId} AND student_status=1	"
            + " and <![CDATA[ DATE_FORMAT(last_student_time,'%Y-%m-%d')>=DATE_FORMAT(#{startDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "
            + " and <![CDATA[ DATE_FORMAT(last_student_time,'%Y-%m-%d')<=DATE_FORMAT(#{endDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "

            + "			UNION ALL  "

            + "SELECT #{siteId} as siteId,course_id,account_id,NULL AS LEVEL,NULL AS COMMENT,NULL AS finish_time ,4 AS TYPE,create_time AS record_time,1 AS collect_num FROM `favorite`  WHERE course_id=#{cId} "
            + " and <![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')>=DATE_FORMAT(#{startDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "
            + " and <![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')<=DATE_FORMAT(#{endDate,jdbcType=VARCHAR},'%Y-%m-%d')]]> "

            + ") AS tem" + "  </script>")
    List<StatisticsCourseMessage> getCourseMessageByCourseId(@Param("cId") Long cId,
                                                             @Param("startDate") String startDate, @Param("endDate") String endDate, @Param("siteId") Long siteId);

    @Select("<script>" + "SELECT MIN(create_time) FROM (" +

            " SELECT create_time FROM `course` WHERE id=#{cId} " +

            " UNION ALL " +

            " SELECT MIN(start_time) FROM `recorde` WHERE course_id=#{cId} " +

            " UNION ALL " +

            " SELECT MIN(end_time) FROM `recorde` WHERE course_id=#{cId} " +

            " UNION ALL " +

            " SELECT MIN(create_time) FROM `comment` WHERE course_id=#{cId} " +

            " UNION ALL " +

            " SELECT MIN(create_time) FROM course_grade WHERE course_id=#{cId} " + ") AS t</script>")
    Date getCourseMinTimeByCourseId(@Param("cId") Long cId);


    @Select("SELECT courseId as scormId,accountId as accountId,testScore as score from (SELECT b.course_id courseId,a.account_id accountId,MAX(a.score) testScore from scorm_record a LEFT JOIN chapter b on a.chapter_id=b.id where a.chapter_id is not null and a.score is not null and b.id is not null GROUP BY\n" +
            "						b.course_id,a.account_id\n" +
            "			union  all \n" +
            "			SELECT course_id as courseId,account_id as accountId,MAX(test_score) as testScore FROM `test_case` WHERE test_score IS NOT NULL  \n" +
            "			GROUP BY course_id,account_id ORDER BY null) a")
    List<ScormRecord> getCourseGrade();

    @Select("<script>" + " SELECT course_id AS courseId,account_id AS accountId,site_id siteId FROM recorde "
            + " WHERE account_id IS NOT NULL and account_id > 0 " + "<if test='siteId != null'>"
            + " AND site_id = #{siteId} " + "</if>" + " GROUP BY account_id,course_id,site_id" + "</script>")
    List<RecordeListVo> findStuByAccountG(@Param("siteId") Long siteId);

    @Select("<script>"
            + " SELECT c.id,r.account_id accountId,r.company_id companyId,r.site_id siteId,"
            + " r.org_id orgId,r.course_id courseId,r.chapter_id chapterId,r.overed,r.end_time endTime FROM "
            + " (SELECT id FROM chapter WHERE material_id IS NOT NULL and course_id=#{courseId}) c LEFT JOIN"
            + " (SELECT account_id,site_id,company_id,org_id,course_id,chapter_id,MAX(overed) overed,MAX(end_time) end_time FROM recorde"
            + " WHERE account_id IS NOT NULL and account_id > 0 and site_id=#{siteId} and course_id=#{courseId} and account_id=#{accountId} and valid = 1"
            + " GROUP BY chapter_id ) r" + " on c.id = r.chapter_id" + " ORDER BY r.end_time DESC"
            + "</script>")
    List<RecordeVO> findStuChapterByAccountG(@Param("siteId") Long siteId, @Param("courseId") Long courseId,
                                             @Param("accountId") Long accountId);


    Integer findRankingNumById(@Param("siteId") Long siteId, @Param("accountId") Long accountId);

    List<OrganizationDurationVO> getOrgDuration(@Param("siteId") Long siteId, @Param("top") Integer top,
                                                @Param("accountIds") List<Long> accountIds);


    @Select("<script>SELECT course_id AS courseId,account_id AS accountId,site_id AS siteId,0 AS learnCount,MIN(start_time) AS startTime,MAX(end_time) AS endTime,SUM(duration) AS learnTime,DATE_FORMAT(MIN(start_time),'%Y-%m-%d') AS recordeTime,site_id as siteId FROM recorde WHERE " +
            "		course_id=#{cId}   and  duration>0 and " +
            "		<![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')<=DATE_FORMAT(#{endDate,jdbcType=VARCHAR},'%Y-%m-%d')]]>   and " +
            "		<![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')>=DATE_FORMAT(#{startDate,jdbcType=VARCHAR},'%Y-%m-%d')]]>  " +
            "		GROUP BY account_id,<![CDATA[ DATE_FORMAT(create_time,'%Y-%m-%d')]]></script>")
    List<StatisticsCourseRecorde> getCourseRecordeByCourseIdStep1(@Param("cId") Long cId, @Param("startDate") String startDate, @Param("endDate") String endDate);

    @Select("<script>SELECT COUNT(1) FROM `recorde` WHERE DATE_FORMAT(create_time,'%Y-%m-%d')=DATE_FORMAT(#{selectDateString,jdbcType=VARCHAR},'%Y-%m-%d') AND course_id=#{cId} AND duration>0 and account_id=#{accountId}</script>")
    Long learnCountByCourseIdAndAccountIdAndSelectDate(@Param("cId") Long cId, @Param("accountId") Long accountId, @Param("selectDateString") String selectDateString);

    @Select("<script>" +
            "SELECT IFNULL(SUM( duration ), 0) " +
            "FROM recorde " +
            "WHERE site_id = #{siteId} " +
            "AND <![CDATA[ DATE_FORMAT(create_time, '%Y-%m-%d') >= DATE_FORMAT(#{startDate}, '%Y-%m-%d') ]]> " +
            "AND <![CDATA[ DATE_FORMAT(create_time, '%Y-%m-%d') <= DATE_FORMAT(#{endDate}, '%Y-%m-%d') ]]> " +
            "</script>")
    Long courseTimeInWeek(@Param("siteId") long siteId, @Param("startDate") Date startDate,
                          @Param("endDate") Date endDate);

    Integer countFinished(@Param("accountId") Long accountId, @Param("siteIds") List<Long> siteIds,
                          @Param("startDate") String startDate, @Param("endDate") String endDate);

    Integer proportion(@Param("accountId") Long accountId, @Param("siteIds") List<Long> siteIds,
                       @Param("startDate") String startDate, @Param("endDate") String endDate);

    Long totalDuration(@Param("accountId") Long accountId, @Param("siteIds") List<Long> siteIds,
                       @Param("startDate") String startDate, @Param("endDate") String endDate);

    StudyDurationVO studyDuration(@Param("accountId") Long accountId, @Param("siteIds") List<Long> siteIds,
                                  @Param("startDate") String startDate, @Param("endDate") String endDate);

    Date lateDate(@Param("accountId") Long accountId, @Param("siteIds") List<Long> siteIds,
                  @Param("startDate") String startDate, @Param("endDate") String endDate);
}
