package com.yizhi.course.application.mapper;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.vo.*;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.session.RowBounds;
import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author wangfeida123
 * @since 2018-07-26
 */
public interface StatisticsCourseMapper extends BaseMapper<StatisticsCourse> {

	/**
	 * 按课程统计查询条件
	 * @param paramMap
	 * @return
	 */
	List<Long> selectLearnOnCourses(@Param("paramMap") Map<String, Object> paramMap, @Param("rowBounds") RowBounds rowBounds);
	
	/**
	 * 按课程统计总记录数
	 * @param paramMap
	 * @return
	 */
	Integer selectLearnOnCoursesTotal(@Param("paramMap") Map<String, Object> paramMap);
	
	/**
	 * 按用户统计查询条件
	 * @param paramMap
	 * @return
	 */
	List<Long> selectLearnOnAccounts(@Param("paramMap") Map<String, Object> paramMap, @Param("rowBounds") RowBounds rowBounds);

	/**
	 * 统计用户总数 
	 * @param paramMap
	 * @return
	 */
	Integer selectLearnOnAccountsTotal(@Param("paramMap") Map<String, Object> paramMap);
	
	/**
	 * 查询统计课程信息，分页查询 
	 * @param paramMap
	 * @return
	 */
	@Deprecated
	List<Long> selectAccountLearnList(@Param("paramMap") Map<String, Object> paramMap, @Param("page") Page<Long> page);
	
	
	/**
	 * 报表得到所有的课程
	 * 
	 * @return
	 */
	List<Course> getAllCourseReport(@Param("companyIdParam")Long companyIdParam, @Param("siteIdParam")Long siteIdParam);
	
	
	
	/**
	 * 根据课程Id得到课程时长
	 * @param cId
	 * @return
	 */
	List<Material> getCourseDurationById(@Param("ids")List<Long> ids);
	
	
	
	
	
	/**
	 * 得到当天这个人对于这门课程的评论状态
	 * @param cId
	 * @return
	 */
	Integer getCommentAccountCount(@Param("cId")Long cId,@Param("accountId")Long accountId,@Param("startDate")String startDate);
	
	/**
	 * 得到当天评论数目
	 * @param cId
	 * @param startDate
	 * @return
	 */
	Integer getCommentCount(@Param("cId")Long courseId,@Param("startDate")String startDate,@Param("accountId")Long accountId);
	
	/**
	 * 根据账号和课程  得到学员对这门课程的评分
	 * @param courseId
	 * @param accountId
	 * @return
	 */
	CourseGrade getCourseGradeByAccountIdAndCourseId(@Param("cId")Long courseId, @Param("accountId")Long accountId, @Param("startDate")String startDate);
	
	/**
	 * 指定人
	 * @param courseId
	 * @return
	 */
	List<Long> getRangeByCourseId(@Param("courseId")Long courseId);

	/**
	 * 获取学习记录
	 * 
	 * @param startDate
	 * @return
	 */
	List<Recorde> getAllRecordeReport(@Param("courseId") Long courseId, @Param("accountIds") List<Long> accountIds,
									  @Param("startDate") String startDate);
	
	/**
	 * 查询当天的完成的记录
	 * @param accountId
	 * @return
	 */
	Date getRecordsByTimeAndAccountId(@Param("accountId")Long accountId,@Param("courseId")Long courseId);
	
	
	
	
	/**
	 * 课程----图形报表
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param siteId
	 * @return
	 */
	List<ReportStudyCourseChartsVo> chartGroup(@Param("startDate") String startDate, @Param("endDate") String endDate, @Param("companyId")Long companyId, @Param("orgIds")List<Long> orgIds, @Param("siteId")Long siteId);

	/**
	 * 课程---按课程统计
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	//List<ReportStudyCourseVo> courseGroup(@Param("kwd")String kwd,@Param("startDate") String startDate, @Param("endDate") String endDate,@Param("companyId")Long companyId,@Param("orgId")Long orgId,@Param("siteId")Long siteId,Page<ReportStudyCourseVo> page);
	List<ReportStudyCourseVo> courseGroup(@Param("courseIds")List<Long> courseIds, @Param("startDate") String startDate, @Param("endDate") String endDate);
	
	
	
	
	
	/**
	 * 课程---按课程统计---查看详情
	 * @param orgKwd
	 * @param accountKwd
	 * @param courseId
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	List<ReportStudyCourseViewVo> CourseGroupView(@Param("orgKwd")String orgKwd, @Param("accountKwd")String accountKwd, @Param("courseId")Long courseId, @Param("startDate") String startDate, @Param("endDate") String endDate, Page<ReportStudyCourseViewVo> page);
	
	
	
	
	/**
	 * 课程---按用户统计
	 * @param
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	//List<ReportStudyCourseAccountVo> courseAccountGroup(@Param("accountKwd")String accountKwd,@Param("orgKwd")String orgKwd,@Param("startDate") String startDate, @Param("endDate") String endDate,@Param("companyId")Long companyId,@Param("orgId")Long orgId,@Param("siteId")Long siteId,Page<ReportStudyCourseAccountVo> page);
	List<ReportStudyCourseAccountVo> courseAccountGroup(@Param("accountIds")List<Long> accountIds, @Param("startDate") String startDate,
														@Param("endDate") String endDate);
	 
	List<ReportStudyCourseAccountVo> courseAccountGroupExport(@Param("paramMap") Map<String, Object> paramMap,@Param("rowBounds") RowBounds rowBounds);
	
	/**
	 * 课程---按用户统计---下载学习明细
	 */
	List<ReportStudyCourseAccountDetailsVo> courseAccountDetails(@Param("paramMap") Map<String, Object> paramMap);
	
	
	/**
	 * 课程---按用户统计---下载学习明细---总数据量
	 */
	Integer courseAccountDetailsDataTotal(@Param("paramMap") Map<String, Object> paramMap);
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * 课程---按用户统计---详情
	 * @param accountId
	 * @param startDate
	 * @param endDate
	 * @param rowBounds
	 * @return
	 */
	List<ReportStudyCourseAccountViewVo> courseAccountGroupView(@Param("accountId")Long accountId,@Param("kwd")String kwd,@Param("startDate") String startDate, @Param("endDate") String endDate,Page<ReportStudyCourseAccountViewVo> page);
	
	
	
	String selectAccountNameByaccountId(@Param("accountId")Long accountId);
	
	
	
	/**
	 * 课程----按照部门统计
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @return
	 */
	List<ReportStudyCourseOrgVo> courseOrgGroup(@Param("companyId")Long companyId, @Param("siteIds")List<Long> siteIds, @Param("orgIds")List<Long> orgIds,
			@Param("startDate") String startDate, @Param("endDate") String endDate);

	
	
	Date selectMaxDate();
	
	Date selectRecordMinTime();
	
	/**
	 * 
	 * @param currentDate
	 * @return
	 */
	Integer deleteRecordeByDate(@Param("currentDate")String currentDate);
	
	/**
	 * 批量插入学习记录
	 * @param courseId 课程ID
	 * @param curDate 学习日期
	 * @return 
	 */
	int insertAccountLearn(@Param("courseId")Long courseId, @Param("curDate")String curDate);
	/**
	 * 评分
	 * @param courseId
	 * @param curDate
	 * @return
	 */
	int insertAccountLearnGrade(@Param("courseId")Long courseId, @Param("curDate")String curDate);
	/**
	 * 评论
	 * @param courseId
	 * @param curDate
	 * @return
	 */
	int insertAccountLearnCommen(@Param("courseId")Long courseId, @Param("curDate")String curDate);
	
	/**
	 * 获取学习记录
	 * @param courseId 课程ID
	 * @param curDate 学习日期
	 * @return 返回学习的学生ID集合
	 */
	List<StatisticsCourseLearn> selectAccountLearn(@Param("courseId")Long courseId, @Param("curDate")String curDate);

	/**
	 * 删除没有学习记录的课程信息
	 */
	void deleteEmptyCourseLearn();

	/**
	 * 删除没有学习记录的学员信息
	 */
	void deleteEmptyAccountLearn();
	
	/**
	 * 查找该课程有没有人学习了
	 * @param courseId
	 * @return
	 */
	Integer selectEmptyCourseLearnCount(@Param("courseId")Long courseId);

	/**
	 * 查找学员有没有学了该课程
	 * @param courseId
	 * @param accountId
	 * @return
	 */
	Integer selectEmptyCourseAccountLearnCount(@Param("courseId") Long courseId, @Param("accountId") Long accountId);
	
	/**
	 * 查找学员有没有学了该课程
	 * @param courseId
	 * @return
	 */
	@MapKey("accountId")
	Map<Long, Long> selectAccountLearnByCourseId(@Param("courseId") Long courseId);
	
	/**
	 * 按部门纬度统计数据量会很大,影响效率.对其进行拆分，按部门统计从此表中查询
	 */
	void insertStatisticsCourseToOrgMetadata();
	
	void deleteStatisticsCourseToOrgMetadata();
	
	/**
	 * 课程信息直接从statistics_course表中查询效率很慢，对其进行拆分.按课程和用户统计的查询条件从此获取
	 */
	void insertStatisticsCourseToGroupFind();
	
	void deleteStatisticsCourseToGroupFind();
	
	/**
	 * 获取部门编码为空记录
	 * @return
	 */
	List<Map<String, Long>> selectEmptyOrgCode();
	
	/**
	 * 更新部门编码
	 * @param orgCode
	 * @param companyId
	 * @param orgId
	 * @return
	 */
	int updateEmptyOrgCode(@Param("orgCode") String orgCode, @Param("companyId") Long companyId, @Param("orgId") Long orgId);
	
	/**
	 * 添加学员
	 */
	void insertStatisticsCourseToAccountGroupFind();
	
	/**
	 * 删除学员
	 */
	void deleteStatisticsCourseToAccountGroupFind();
	
	/**
	 * 删除应参加人数
	 */
	void deleteStatisticsAccountJoinCount();
	
	/**
	 * 添加应参加人数
	 */
	void insertStatisticsAccountJoinCount();
}
