package com.yizhi.course.application.service;


import java.util.List;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.core.application.vo.AccreditVO;
import com.yizhi.course.application.domain.Accredit;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.vo.AuthorizeCourseVO;
import com.yizhi.course.application.vo.PageAuthorizeVO;
import com.yizhi.course.application.vo.PageAuthorizedVO;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author fulan123
 * @since 2018-04-25
 */
public interface IAccreditService extends IService<Accredit> {


    /**
     * 根据课程关键字（课程名称 name或者标签tags）授权站点id、 站点id 查询已选择课程（分页并显示总条数)
     *
     * @param name
     * @param siteId
     *            站点名称
     * @return List<CourseListVO>
     */

    Page<Course> courseList(String name, Long companyId, Long siteId, Integer pageNo, Integer pageSize);

    /**
     * 根据课程关键字（课程名称 name或者标签tags）、授权站点id,站点id 查询未选择课程（分页并显示总条数)
     *
     */

    Page<Course> noCourseList(String name, Long companyId, Long siteId, Integer pageNo, Integer pageSize);


    /**
     * 根据课程关键字（课程名称 name或者标签tags）授权站点id、 站点id 查询已选择课程（不用分页）
     * 作用：供 保存、 授权接口调用  与传进来的已选课程对象集合相比较
     *
     */
    List<Course> YXcourseList(String name, Long companyId, Long siteId);

    /**
     * 授权
     * @param accreditVO
     * @param course
     * @param id
     * @return
     */
    Boolean saveAuthorizeCourse(AccreditVO accreditVO, Course course, Long id);

    /**
     * 根据站点名模糊查询
     * @param siteName
     * @return
     */
    PageAuthorizeVO queryAuthorizeCourse(String type, String siteName, String companyName, Integer pageNo, Integer pageSize);

    Page<AuthorizeCourseVO> queryAuthorizedCourseList(String type, String siteName, String companyName, Integer pageNo, Integer pageSize);

    /**
     * 获取已经授权的列表
     * @param courseKeys
     * @param startTime
     * @param endTime
     * @param pageNo
     * @param pageSize
     * @return
     */
    PageAuthorizedVO getAuthorizedList(String courseKeys, Long siteId, String startTime, String endTime, Integer activeType, Integer type, Integer pageNo, Integer pageSize);

    /**
     * 授权编辑
     * @param accreditVO
     * @return
     */
    Boolean updateAuthorize(AccreditVO accreditVO);

    /**
     * 指定时间失效课程
     * @param accredit
     * @return
     */
    boolean invalidCourse( Accredit accredit);

    /**
     * 通过课程id获取授权实体
     * @param id
     * @return
     */
    Integer getAccreditByCourseId(Long id);

    /**
     *  递归查询授权课程授权后所有的课程id
     * @param accredits
     * @param beforeCourseId
     * @return
     */
    List<Accredit> getAccreditList(List<Accredit> accredits,Long beforeCourseId);

    /**
     *
     * @param siteId 被授权的siteId
     * @param courseId 课程id
     * @return
     */
    Boolean topSiteInAuthorized(Long siteId, Long courseId);

    /**
     * 查询课程是否被授权
     * @param courseId
     * @param siteId
     * @return
     */
    Boolean existAuthorize(Long courseId,Long siteId);
}