package com.yizhi.course.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.vo.ApiClassifyVo;
import com.yizhi.course.application.vo.ClassifyVo;

import java.util.List;

/**
 * <p>
 * 课程分类 服务类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface IClassifyService extends IService<Classify> {

    /**
     * 检查是否存在分类名称
     * @param name 分类名称
     * @param parentId 分类的上级id（一级分类为0）
     * @param siteId 站点id
     * @param type 检查类型（1：新增，2：修改(需要传classifyId)）
     * @param classifyId 分类id，修改检查时用到
     * @return 不存在返回true，存放相同名返回false
     */
    boolean checkExtisClassifyName(String name, Long parentId,Long siteId,
                                   int type, Long classifyId);
    /**
     * 添加一个新的课程分类
     * @param classify
     * @return
     */
    int saveClassify(Classify classify);

    /**
     * 根据父级分类查询是否存在子分类
     * @param classifyIds
     * @return
     */
    List<ClassifyVo> findExistChildNodes(List<Long> classifyIds);


    /**
     * 根据id,删除分类
     * @param id
     * @return
     */
    int deleteClassify(Long id);

    /**
     * 根据分类对象修改分类
     * @param classify
     * @return
     */
    int updateClassify(Classify classify);


    /**
     * 根据名称查询分类
     * @param name
     * @return
     */
    Page<Classify> listClassify(String name, Integer pageNo, Integer pageSize,Long companyId,List<Long> orgIds,Long siteId,Boolean isStartHQ );


    /**
     * 学员端查询课程分类
     * @return
     */
    List<Classify> listStuClassify(Long companyId,Long siteId);

    /**
     * 更换排序
     * @param list
     * @return int
     */
    int updateClassiyfsort(List<Classify> list);

    /**
     *  管理端课程分类分页查询
     * @param companyId 企业id
     * @param name      分类名
     * @param orgIds    管辖区 部门id list
     * @param pageNo    分页 第几页
     * @param pageSize  分页数
     * @return Page<ApiClassifyVo>
     */
    Page<ApiClassifyVo> selectListByCompanyId(Long companyId, String name, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize);


    List<Classify> selectClassifyByPath(List<Long> ids);


    /**
     *  根据分类层级查询分类
     * @param companyId  企业id
     * @param siteId   站点id
     * @param layer    分类层级
     * @return
     */
    List<Classify> selectClassifyByLayer(Long companyId, Long siteId,Integer layer);


    Page getClassifyNameByDrools(String field,String value, Page page);




}
