package com.yizhi.course.application.service;

import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.course.application.domain.StatisticsCourse;
import com.yizhi.course.application.domain.StatisticsCourseLearn;
import com.yizhi.course.application.vo.*;

/**
 * <p>
 *  服务类
 * </p>
 *
 * @author wangfeida123
 * @since 2018-07-26
 */
public interface IStatisticsCourseService extends IService<StatisticsCourse> {
	
	/**
	 * 查询统计课程信息，分页查询 
	 * 查询效率慢，对齐进行了拆分成selectLearnOnCourses和selectLearnOnAccounts
	 * @param paramMap
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@Deprecated
	Page<Long> selectAccountLearnList(Map<String, Object> paramMap, int pageNo, int pageSize);
	
	/**
	 * 按课程统计查询条件
	 * @param paramMap
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	Page<Long> selectLearnOnCourses(Map<String, Object> paramMap, int pageNo, int pageSize);
	
	/**
	 * 按用户统计查询条件
	 * @param paramMap
	 * @return
	 */
	Page<Long> selectLearnOnAccounts(Map<String, Object> paramMap, int pageNo, int pageSize);
	
	/**
	 * 课程----图形报表
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param orgId
	 * @param siteId
	 * @return
	 */
	List<ReportStudyCourseChartsVo> chartGroup(String startDate, String endDate, Long companyId, List<Long> orgIds, Long siteId);
	
	
	
	/**
	 * 课程----按照课程统计
	 * 对统计进行了优化：
	 * 1、根据查询条件和分页，统计出要查询的课程IDs
	 * 2、然后拿着这些课程IDs去进行统计，提高查询效率
	 * @param kwd
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param orgId
	 * @param siteId
	 * @return
	 */
	Page<ReportStudyCourseVo> courseGroup(String kwd, String startDate, String endDate, Long companyId, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize);
	
	
	
	/**
	 * 课程----按照课程统计----详情
	 * @param orgKwd
	 * @param accountKwd
	 * @param startDate
	 * @param endDate
	 * @param courseId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	Map<String,Object> CourseGroupView(String orgKwd,String accountKwd,String startDate, String endDate, Long courseId,Integer pageNo,Integer pageSize);
	
	
	
	/**
	 * 课程----按照用户统计
	 * @param orgKwd
	 * @param accountKwd
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param orgId
	 * @param siteId
	 * @return
	 */
	Page<ReportStudyCourseAccountVo> courseAccountGroup(String orgKwd, String accountKwd, String startDate, String endDate, Long companyId, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize);
	
	
	
	
	/**
	 * 课程----按照用户统计----下载   因为通过上边的那个进行下载速度会特别慢，大概15分钟，这个是进行优化过的
	 * @param orgKwd
	 * @param accountKwd
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param orgId
	 * @param siteId
	 * @return
	 */
	Page<ReportStudyCourseAccountVo> courseAccountGroupExport(String orgKwd,String accountKwd,String startDate, String endDate, 
			Long companyId, List<Long> orgIds,Long siteId,Integer pageNo,Integer pageSize);
	
	
	
	
	/**
	 * 课程----按照用户统计---下载学习明细
	 */
	List<ReportStudyCourseAccountDetailsVo> courseAccountDetails(String orgKwd, String accountKwd, String startDate, String endDate,
																 Long companyId, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize);
	
	/**
	 * 课程----按照用户统计---下载学习明细---得到下载总数量
	 */
	Integer courseAccountDetailsDataTotal(String orgKwd,String accountKwd,String startDate, String endDate, 
			Long companyId, List<Long> orgIds,Long siteId);
	
	
	
	/**
	 * 课程----按照用户统计----详情
	 * @param accountId
	 * @param startDate
	 * @param endDate
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	Map<String,Object> courseAccountGroupView(Long accountId,String kwd,String startDate, String endDate, Integer pageNo,Integer pageSize);

	
	
	
	/**
	 * 课程----按照部门统计
	 * @param kwd
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param orgId
	 * @param siteId
	 * @return
	 */
	Page<ReportStudyCourseOrgVo> courseOrgGroup(String kwd, String startDate, String endDate, Long companyId, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize);
	
	
	/**
	 * 批量插入学习记录
	 * @param courseId 课程ID
	 * @param curDate 学习日期
	 * @return 返回学习的学生ID集合
	 */
	List<StatisticsCourseLearn> insertAccountLearn(Long courseId, String curDate);
}
