package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
//
//import com.yizhi.application.job.remote.vo.VmyJobVO;
import com.yizhi.core.application.vo.AccreditVO;
//import com.yizhi.application.job.remote.JobClient;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.course.application.domain.Accredit;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.mapper.AccreditMapper;
import com.yizhi.course.application.service.IAccreditService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.vo.*;
import com.yizhi.system.application.system.remote.CompanyClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.SiteVO;
import com.yizhi.util.application.date.DateUtil;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-04-25
 */
@Service
@Log4j2
public class AccreditServiceImpl extends ServiceImpl<AccreditMapper, Accredit> implements IAccreditService {

    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(AccreditServiceImpl.class);
    private static final String jobGroup="authorization";
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    private AccreditMapper accreditMapper;
    @Autowired
    private CompanyClient companyClient;

    @Autowired
    ICourseService courseService;

//    @Autowired
//    JobClient jobClient;

    @Autowired
    SiteClient siteClient;

    /**
     * 根据课程关键字（课程名称 name或者标签tags）、企业id,站点id 查询已选择课程（分页并显示总条数)
     *
     */
    @Override
    public Page<Course> courseList(String name, Long companyId, Long siteId, Integer pageNo,
                                   Integer pageSize) {
        Page<Course> page = new Page<Course>(pageNo, pageSize);

        List<Course> CourseList = this.baseMapper.courseList(name, companyId, siteId,
                new RowBounds(page.getOffset(), page.getLimit()));
        page.setRecords(CourseList);
        Integer count = this.baseMapper.getCourseListCount(name, companyId, siteId);
        page.setTotal(count);
        return page;
    }


    /**
     * 根据课程关键字（课程名称 name或者标签tags）、企业id,站点id 查询未选择课程（分页并显示总条数)
     *
     */
    @Override
    public Page<Course> noCourseList(String name, Long companyId, Long siteId, Integer pageNo,
                                     Integer pageSize) {
        Page<Course> page = new Page<Course>(pageNo, pageSize);

        List<Course> CourseList = this.baseMapper.noCourseList(name, companyId, siteId,
                new RowBounds(page.getOffset(), page.getLimit()));
        page.setRecords(CourseList);
        Integer count = this.baseMapper.getNoCourseListCount(name, companyId, siteId);
        page.setTotal(count);
        return page;
    }


    /**
     * 根据课程关键字（课程名称 name或者标签tags）授权站点id、 站点id 查询已选择课程（不用分页）
     * 作用：供 保存、 授权接口调用  与传进来的已选课程对象集合相比较
     *
     */
    @Override
    public List<Course> YXcourseList(String name, Long companyId, Long siteId) {
        List<Course> CourseList=this.baseMapper.YXcourseList(name, companyId, siteId);
        return CourseList;
    }

    @Override
    public boolean invalidCourse(Accredit accredit) {

        Long id = accredit.getId();

        // 批处理时间要在当前时间前十分钟哦
        Calendar nowTime = Calendar.getInstance();
        nowTime.add(Calendar.MINUTE, -10);


//        try {
//            VmyJobVO job=new VmyJobVO();
//            job.setGroup(jobGroup);
//            job.setJobName(id.toString());
//            job.setJobClassName("com.yizhi.course.application.handle.quartz.job.CourseInvalidJob");
//            String cron= DateUtil.getCron(accredit.getUseDuration());
//            String[] cronArr=cron.split(" ");
//            job.setSecond(cronArr[0]);
//            job.setMinute(cronArr[1]);
//            job.setHour(cronArr[2]);
//            job.setDay(cronArr[3]);
//            job.setMonth(cronArr[4]);
//            job.setWeek(cronArr[5]);
//            job.setYear(cronArr[6]);
//            jobClient.insert(job);
//
//        } catch (Exception e) {
//            LOGGER.error("创建任务失败,id="+id+"错误消息={}",e);
//        }
        return true;
    }

    /**
     *
     * @param accreditVO 授权VO
     * @param course 新的课程实体
     * @param id  老的课程id
     * @return
     */
    @Override
    public Boolean saveAuthorizeCourse(AccreditVO accreditVO, Course course, Long id) {
        Accredit accreditParam = new Accredit();
        accreditParam.setCourseIdAfter(id);
        Accredit ac = this.selectOne(QueryUtil.condition(accreditParam));
        Date sysDate = new Date();
        Accredit accredit = new Accredit();

        BeanUtils.copyProperties(accreditVO,accredit);
        Long aId = idGenerator.generate();
        accredit.setId(aId);
        accredit.setCourseIdBefore(id);
        accredit.setCourseIdAfter(course.getId());
        if (accreditVO.getUseDuration()!=null&&!"null".equals(accreditVO.getUseDuration())){
            Calendar c = getNextDayZero(accreditVO.getUseDuration());
            accredit.setUseDuration(c.getTime());
        } else {
            accredit.setUseDuration(null);
        }




        if (accreditVO.getType()==2){
            if (accreditVO.getSiteId()!=null&&!"".equals(accreditVO.getSiteId())){
                accredit.setSiteId(accreditVO.getSiteId());
                List<Long> siteIds = new ArrayList<>(1);
                siteIds.add(accreditVO.getSiteId());
                SiteVO site = (siteClient.selectByIds(siteIds)).get(0);
                LOGGER.info("siteVO "+site);
                if (site!=null) {
                    accredit.setSiteName(site.getName());
                    accredit.setSiteCode(site.getCode());
                }
            }

            accredit.setCompanyName(accreditVO.getCompanyName());
            accredit.setUseDuration(accreditVO.getUseDuration());
            accredit.setSaleAuthorizeStatus(accreditVO.getSaleAuthorizeStatus());


        }else {
            accredit.setSiteId(accreditVO.getSiteId());
        }

        accredit.setSaleAuthorizeStatus(1);
        accredit.setAccreditTime(sysDate);
        accredit.setSaveTime(sysDate);
        // 授权来源的站站点id
        accredit.setAccreditSiteId(ContextHolder.get().getSiteId());
        // 授权来源的公司id
        accredit.setSourceCompanyId(ContextHolder.get().getCompanyId());


        try {
            this.insert(accredit);

            if (accreditVO.getUseType()==1){
                invalidCourse(accredit);
            }
        } catch (Exception e) {
            LOGGER.info("授权失败"+e);
        }
        return true;
    }

    @Override
    public PageAuthorizeVO queryAuthorizeCourse(String type, String siteName, String companyName, Integer pageNo, Integer pageSize) {

        Integer start = pageSize*(pageNo -1);

        Integer end = pageSize;
        RowBounds rowBounds = new RowBounds(start,end);
        List<AuthorizeCourseVO> authorizeVOList =  accreditMapper.queryAuthorizeCourse(type,siteName,companyName,ContextHolder.get().getSiteId(),rowBounds);

        Integer totalCount = accreditMapper.queryAuthorizeCount(type,siteName,companyName,ContextHolder.get().getSiteId()).size();
        PageAuthorizeVO pageAuthorizeVO = new PageAuthorizeVO();
        pageAuthorizeVO.setPageNo(pageNo);
        pageAuthorizeVO.setPageSize(pageSize);
        pageAuthorizeVO.setPageTotal(totalCount%pageSize==0?totalCount/pageSize:totalCount/pageSize+1);
        pageAuthorizeVO.setRecords(authorizeVOList);
        pageAuthorizeVO.setPageRecords(totalCount);


        return pageAuthorizeVO;
    }

    @Override
    public Page<AuthorizeCourseVO> queryAuthorizedCourseList(String type, String siteName, String companyName, Integer pageNo, Integer pageSize) {
        Page<AuthorizeCourseVO> page = new Page<>();
        page.setCurrent(pageNo);
        page.setSize(pageSize);

        List<AuthorizeCourseVO> authorizeVOList =  accreditMapper.queryAuthorizeCourse(page,type,siteName,companyName,ContextHolder.get().getSiteId());
        page.setRecords(authorizeVOList);
        return page;
    }

    @Override
    public PageAuthorizedVO getAuthorizedList(String courseKeys, Long siteId, String startTime, String endTime, Integer activeType, Integer type, Integer pageNo, Integer pageSize) {

        // 课程授权站点
        Long sourceSiteId = ContextHolder.get().getSiteId();
        Integer start = pageSize*(pageNo -1);

        Integer end = pageSize;
        RowBounds rowBounds = new RowBounds(start,end);
        List<AuthorizedCourseVO> authorizedVOList
                = accreditMapper.getAuthorizedList(courseKeys,startTime,endTime,siteId,activeType, type, sourceSiteId,rowBounds);

        LOGGER.info("传的参数是："+activeType);

        Integer totalCount = accreditMapper.getAuthorizedCount(courseKeys, startTime, endTime,siteId,  activeType, type,sourceSiteId);
        PageAuthorizedVO pageAuthorizedVO = new PageAuthorizedVO();

        pageAuthorizedVO.setPageNo(pageNo);
        pageAuthorizedVO.setPageSize(pageSize);
        pageAuthorizedVO.setPageTotal(totalCount%pageSize==0?totalCount/pageSize:totalCount/pageSize+1);
        pageAuthorizedVO.setRecords(authorizedVOList);
        pageAuthorizedVO.setPageRecords(totalCount);

        return pageAuthorizedVO;
    }

    /**
     * 授权编辑
     * @param accreditVO
     * @return
     */

    @Override
    public Boolean updateAuthorize(AccreditVO accreditVO) {

        // 当前时间
        Calendar currentDate = Calendar.getInstance();
        currentDate.setTime(new Date());
        Date useDuration = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar validDate;
        if (accreditVO.getUseDuration()!=null) {
            validDate = getNextDayZero(accreditVO.getUseDuration());
            useDuration = validDate.getTime();
        } else {
            validDate = null;
            useDuration = null;
        }

        Accredit a = this.selectById(accreditVO.getId());
        Long acId = a.getId();
        List<Accredit> accreditList = new ArrayList<>();
        accreditList.add(a);

        // 获取课程授权的列表
        List<Accredit> accredits = getAccreditList(accreditList,a.getCourseIdAfter());
        // 初始化次级时间
        for (Accredit accredit:accredits) {
            Calendar curUseDuration = Calendar.getInstance();
            if (accreditVO.getUseDuration()!=null) {
                try {
                    Date durEndTime = sdf.parse(sdf.format(useDuration));
                    curUseDuration.setTime(durEndTime);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }

            Long courseIdAfter = accredit.getCourseIdAfter();
            accreditVO.setUseType(accredit.getUseType());

            Course course = courseService.selectById(courseIdAfter);

            if (accreditVO.getUseDuration()!=null&&!"null".equals(accreditVO.getUseDuration())){
                // 授权站点指定授权截止时间
                if (validDate.after(currentDate)){
                    if (validDate.after(curUseDuration) || validDate.equals(curUseDuration)){
                        // 指定有效期大于当前当授权日期 仅最大父级做相应的修改 所有被授权的站点保持授权状况不变



                        if (acId.equals(accredit.getId())){
                            accredit.setUseType(1);
                            accredit.setValid(1);
                            accredit.setUseDuration(useDuration);
                            course.setValid(1);
                        } else {
                            // 父级的授权是指定日期的
                            accredit.setUseType(1);
                            accredit.setValid(1);
                            accredit.setUseDuration(useDuration);
                            course.setValid(1);
                        }

                    } else if (validDate.before(accredit.getUseDuration())) {
                        // 指定有效期小于当前授权时间
                        accredit.setValid(0);
                        course.setValid(0);
                        accredit.setUseType(1);

                    }
                } else if (validDate.before(currentDate)){
                    // 授权站点指定被授权的课程失效
                    accredit.setUseType(1);
                    accredit.setValid(0);
                    accredit.setUseDuration(useDuration);
                    course.setValid(0);
                }
            } else {
                // 授权站点指定被授权的课程长期使用
                accredit.setUseType(0);
                accredit.setValid(1);
                accredit.setUseDuration(null);
                course.setValid(1);
            }
            // 定时任务的修改
            try {
                // 授权课程的操作
                int updateResult = accreditMapper.updateAllColumnById(accredit);
                // 课程失效/生效操作
                boolean updateCourseResult= courseService.updateById(course);

                if (updateResult>0&&updateCourseResult){
                    try {
                        // 删除定时任务
                        delAuthorizeCourse(accredit.getId());
                        // 新增定时任务
                        invalidCourse(accredit);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            } catch (Exception e) {
                LOGGER.info("授权课程失败！"+e.getLocalizedMessage());
            }
        }
        return true;
    }

    @Override
    public Integer getAccreditByCourseId(Long id) {

        Accredit accredit = new Accredit();
        accredit.setCourseIdBefore(id);
        accredit.setValid(1);

        return accreditMapper.selectCount(QueryUtil.condition(accredit));
    }

    /**
     * 递归查询授权课程授权后所有的课程id
     * @param accredits
     * @param beforeCourseId
     * @return
     */
    @Override
    public List<Accredit> getAccreditList(List<Accredit> accredits,Long beforeCourseId){

        log.info("beforeCourseId："+beforeCourseId);
        Accredit accredit = new Accredit();
        accredit.setCourseIdBefore(beforeCourseId);
        List<Accredit> accreditList = this.selectList(QueryUtil.condition(accredit));
        // 判断集合非空
        if (CollectionUtils.isNotEmpty(accreditList)) {
            Long courseIdBefore = accreditList.get(0).getCourseIdAfter();
            accredits.addAll(accreditList);
            accreditList.clear();
            getAccreditList(accredits,courseIdBefore);
        }
        return accredits;

    }

    @Override
    public Boolean topSiteInAuthorized(Long siteId, Long courseId) {
        Accredit accredit = new Accredit();
        accredit.setCourseIdAfter(courseId);
        accredit.setValid(1);
        Accredit a = this.selectOne(QueryUtil.condition(accredit));
        Long beforeCourseId;
        if (a!=null) {
            beforeCourseId = this.selectOne(QueryUtil.condition(accredit)).getCourseIdBefore();
            accredit.setCourseIdAfter(beforeCourseId);
            accredit.setSiteId(siteId);
            accredit.setValid(1);
            List<Accredit> accreditList = this.selectList(QueryUtil.condition(accredit));

            if (CollectionUtils.isNotEmpty(accreditList)){
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    @Override
    public Boolean existAuthorize(Long courseId,Long siteId) {
        Accredit accredit = new Accredit();
        accredit.setSiteId(siteId);
        accredit.setCourseIdBefore(courseId);
        accredit.setValid(1);
        List<Accredit> accreditList = this.selectList(QueryUtil.condition(accredit));
        if (CollectionUtils.isNotEmpty(accreditList)){
            return true;
        } else {
            return false;
        }
    }


    public DurationVO getDurationVO(Date useDuration){

        DurationVO durationVO = new DurationVO();
        Calendar currentDate = Calendar.getInstance();
        currentDate.setTime(new Date());
        Calendar validCalendar = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            if (useDuration!=null){
                Date endTime = sdf.parse(sdf.format(useDuration));
                // 指定时间 加一天(统计)
                validCalendar.setTime(endTime);
                validCalendar.add(Calendar.DAY_OF_MONTH, 1);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        durationVO.setCurrentCalender(currentDate);
        durationVO.setValidCalendar(validCalendar);

        return durationVO;
    }

    /**
     * 获取下一天的零点
     * @param durationDate
     * @return
     */
    public Calendar getNextDayZero(Date durationDate){

        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(durationDate);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);

        Date zero = calendar.getTime();
        calendar.setTime(zero);
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        log.info("增加一天后日期:"+sf.format(calendar.getTime()));
        return  calendar;
    }

    /**
     * 删除授权的任务
     * @param id 任务id
     */
    private void delAuthorizeCourse(Long id){
//        try {
//            // 删除定时任务
//            jobClient.delete(jobGroup+"."+id);
//        } catch (Exception e) {
//            log.info("删除报错："+e.getLocalizedMessage());
//        }
    }

}
