package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.course.application.domain.AudioVideo;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.mapper.AudioVideoMapper;
import com.yizhi.course.application.mapper.MaterialMapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;

import com.yizhi.course.application.service.IAudioVideoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;

/**
 * <p>
 * 音视频 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class AudioVideoServiceImpl extends ServiceImpl<AudioVideoMapper, AudioVideo> implements IAudioVideoService {

	@Autowired
	MaterialMapper materialMapper;

	@Override
	public AudioVideo getAudioVideoByMaterialId(Long materialId) {
		AudioVideo audioVideo=new AudioVideo();
		audioVideo.setMaterialId(materialId);
		audioVideo.setDeleted(0);
		EntityWrapper<AudioVideo> wrapper=new EntityWrapper<AudioVideo>(audioVideo);
		return this.selectOne(wrapper);
	}

	@Override
	public Boolean insertAudioVideo(AudioVideo audioVideo) {
		return this.insert(audioVideo);
	}

	@Override
	public Boolean deleteAudioVideo(Long materialId) {
		AudioVideo audioVideo=new AudioVideo();
		audioVideo.setMaterialId(materialId);
		audioVideo.setDeleted(1);
		EntityWrapper<AudioVideo> wrapper=new EntityWrapper<AudioVideo>(audioVideo);
		return this.update(audioVideo,wrapper);
	}

	@Override
	public Boolean updateAudioVideo(Map<String, Object> map) {
		try {
			AudioVideo video = JSON.parseObject(JSON.toJSONString(map.get("video")), new TypeReference<AudioVideo>() {});
			this.baseMapper.updateById(video);

			Material material =JSON.parseObject(JSON.toJSONString(map.get("material")),new TypeReference<Material>() {});
			materialMapper.updateById(material);
		} catch (Exception e) {
			return false;
		}
		return true;
	}

}
