package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.mapper.ChapterMapper;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.service.ITestService;
import com.yizhi.course.application.vo.*;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程章节 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ChapterServiceImpl extends ServiceImpl<ChapterMapper, Chapter> implements IChapterService {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ITestService testService;

    @Autowired
    private IRecordeService recordeService;

    private static final Logger LOG = LoggerFactory.getLogger(ChapterServiceImpl.class);


    /**
     * 获取章节信息
     *
     * @param chapterId
     * @return
     */
    @Override
    public ChapterDetailedInfoVO getChapterDetailedInfo(Long chapterId) {
        return this.baseMapper.getChapterDetailedInfo(chapterId);
    }

    /**
     * 数据缺失
     *
     * @param ids
     * @return
     */
    @Override
    public List<Long> dataLack(List<Long> ids) {
        return this.baseMapper.dataLack(ids);
    }

    @Override
    public List<ChapterVo> getCourseScoreByIds(List<Long> ids){
        if(org.apache.commons.collections4.CollectionUtils.isEmpty(ids)){
            return null;
        }
        List<ChapterVo> data = this.baseMapper.getCourseScoreByIds(ids);
        return data;
    }

    @Override
    public Chapter getById(Long chapterId) {
        return this.baseMapper.lockById(chapterId);
    }

    @Override
    public Integer getSort(Long parentId) {
        return this.baseMapper.getSort(parentId);
    }

    @Override
    public List<ChapterJoinTestVo> selectChapterTest(List<Long> materialIds) {
        return this.baseMapper.selectChapterTest(materialIds);
    }

    @Override
    public List<Chapter> listByCourseId(Long parentId) {
        return this.baseMapper.listByCourseId(parentId);
    }

    @Override
    public boolean saveList(List<ChapterVo> chapterVos, Long courceId) {

        List<Chapter> addObj = new ArrayList<>();
        for (ChapterVo item : chapterVos) {
            Chapter chapter = new Chapter();
            BeanUtils.copyProperties(item, chapter);
            chapter.setId(idGenerator.generate());
            chapter.setCourseId(courceId);
            addObj.add(chapter);
        }
        return this.insertBatch(addObj);
    }

    @Override
    public List<ChapterDetailsVo> selectChapterList(Long courseId, Long accountId) {
        return this.baseMapper.selectChapterList(courseId, accountId);
    }

    @Override
    public List<ChapterDetailsVo> selectChapterListUp(Long courseId, Long accountId) {
        return this.baseMapper.selectChapterListUp(courseId, accountId);
    }

    @Override
    public List<ChapterDetailsVo> selectChapterListMarketing(Long courseId) {
        return this.baseMapper.selectChapterListMarketing(courseId);
    }

    @Override
    public List<ChapterDetailsVo> selectChapterListUpMg(Long courseId, Long accountId,Long siteId) {
        List<ChapterDetailsVo> data = this.baseMapper.selectChapterListUpOptimize(courseId, accountId);
        if(CollectionUtils.isNotEmpty(data)){
            ////发送时间正序排序
            //msgList.sort((Message m1, Message m2) -> m1.getSendDate().compareTo(m2.getSendDate()));
            //消息ID倒序排序（可能是正序）
            //msgList.sort((Message m1, Message m2) -> m1.getId().compareTo(m2.getId()));
            //消息内容字典倒序排序
            //msgList.sort((Message m1, Message m2) -> m2.getMsg().compareTo(m1.getMsg()));
            try {
                //data = data.stream().sorted(Comparator.comparing(ChapterDetailsVo::getPaths)).collect(Collectors.toList());
                data.sort((ChapterDetailsVo m1, ChapterDetailsVo m2) -> m1.getPaths().compareTo(m2.getPaths()));
            } catch (Exception e) {
                LOG.info("排序异常{}",e);
            }
            // 查询课程章节学习记录
            List<RecordeListVo> recodeData = recordeService.selectRecordeOveredList(accountId,courseId,siteId);
            Map<Long,RecordeListVo> dataMap = new HashMap<>();
            if(CollectionUtils.isNotEmpty(recodeData)){
                dataMap = recodeData.stream().collect(Collectors.toMap(key->key.getChapterId(),val->val));
            }
            for(ChapterDetailsVo item : data){
                Long chapterId = item.getId();
                RecordeListVo objTem = dataMap.get(chapterId);
                if(dataMap != null && objTem != null){
                    Integer overd = objTem.getOvered();
                    item.setOvered(overd==null?0:overd);
                }
            }
        }
        //return this.baseMapper.selectChapterListUp(courseId, accountId);
        return data;
    }

    @Override
    public Integer selectSumScoreByParentId(Chapter param) {
        return this.baseMapper.selectSumScoreByParentId(param.getParentId());
    }

    @Override
    public List<Chapter> courseChapterList(Long courseId) {
        Chapter chapter = new Chapter();
        chapter.setCourseId(courseId);
        return this.selectList(QueryUtil.condition(chapter));
    }

    /**
     * 涉及到多个事务要写在service
     *
     * @param chapterId
     * @return
     */
    @Override
    public Boolean chapterDelete(Long chapterId) {
        // 查出将要删除的章节
        Chapter chapter = new Chapter();
        chapter.setId(chapterId);
        //获取章节
        chapter = chapter.selectById();
        // 如果删除的章节是测验的，
        Integer source = chapter.getSource();
        LOG.info("根据章节id={}，查询章节信息={}", chapterId, JSON.toJSONString(chapter));
        if (source != null && source.intValue() == 9) {
            // 获取测验的id
            testService.updateStatus(chapter.getMaterialId(), chapterId, 3);
        }
        boolean result = false;
        //判断非空防止空指针异常
        if (null != chapter) {
            // 删除当前
            result = chapter.deleteById();
            // 跟新父节点积分总和
            if (null != chapter.getScore() && !chapter.getScore().equals(0)) {
                //判断父节点的ParentId
                if (null != chapter.getParentId() && !chapter.getParentId().equals(0L)) {
                    //new父章节,设值
                    Chapter parent = new Chapter();
                    parent.setId(chapter.getParentId());
                    //获取父章节
                    parent = parent.selectById();
                    if (null != parent) {
                        if (null != parent.getScore()) {
                            //重新new的目的是保证只更新积分的字段
                            Chapter parentUpdate = new Chapter();
                            parentUpdate.setId(parent.getId());
                            //设值，更新
                            parentUpdate.setScore(parent.getScore() - chapter.getScore());
                            parentUpdate.updateById();
                        }
                    }
                }
            }
        }
        return result;
    }

    @Override
    public List<Chapter> getChapterList(List<Chapter> chapters, Long chapterBeforeId) {

        Chapter chapter = new Chapter();
        chapter.setChapterBeforeId(chapterBeforeId);
        List<Chapter> chapterList = this.selectList(QueryUtil.condition(chapter));

        if (CollectionUtils.isNotEmpty(chapterList)) {
            Long tempChapterBeforeId = chapterList.get(0).getId();
            chapters.addAll(chapterList);

            getChapterList(chapters, tempChapterBeforeId);
        }
        return chapters;
    }

    @Override
    public Integer getMaxIndex(Long courseId) {
        return this.baseMapper.getMaxIndex(courseId);
    }

    @Override
    public List<Long> getChapterIds(List<Long> chapterIds, Long chapterBeforeId) {
        Chapter chapter = new Chapter();
        chapter.setChapterBeforeId(chapterBeforeId);

        List<Chapter> chapterList = this.selectList(QueryUtil.condition(chapter));
        List<Long> ids = new ArrayList<>();
        for (Chapter c : chapterList) {
            ids.add(c.getId());
        }

        if (CollectionUtils.isNotEmpty(chapterList)) {

            Long tempChapterBeforeId = ids.get(0);
            chapterIds.addAll(ids);
            ids.clear();

            getChapterIds(chapterIds, tempChapterBeforeId);
        }
        return chapterIds;
    }

    @Override
    public List<Long> getChapters(Long courseId) {
        Chapter chapter = new Chapter();
        chapter.setCourseId(courseId);
        EntityWrapper<Chapter> entityWrapper = new EntityWrapper<>(chapter);
        List<Chapter> list = this.selectList(entityWrapper);
        List<Long> chapterIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)) {
            for (Chapter chapter1 : list) {
                if (null != chapter1.getMaterialId()) {
                    chapterIds.add(chapter1.getId());
                }
            }
        }
        return chapterIds;
    }

    /**
     * 查询课程完成的章节
     *
     * @param courseId
     * @param accountId
     * @param siteId
     * @return
     */
    @Override
    public List<Chapter> selectCourseFinishChapter(Long courseId, Long accountId, Long siteId) {
        return this.baseMapper.selectCourseFinishChapter(courseId,accountId,siteId);
    }


    /**
     * 查询已经完成的课程章节id集合
     *
     * @param accountId
     * @param chapterIds
     * @return
     */
    @Override
    public List<Long> getFinishChapterIds(Long accountId, List<Long> chapterIds) {
        return this.baseMapper.getFinishChapterIds(accountId,chapterIds);
    }
}
