package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.mapper.ClassifyMapper;
import com.yizhi.course.application.service.IClassifyService;
import com.yizhi.course.application.vo.ApiClassifyVo;
import com.yizhi.course.application.vo.ClassifyVo;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 课程分类 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Service
@Transactional
public class ClassifyServiceImpl extends ServiceImpl<ClassifyMapper, Classify> implements IClassifyService {

    private Logger logger = LoggerFactory.getLogger(ClassifyServiceImpl.class);

    @Autowired
    private ClassifyMapper classifyMapper;

    @Autowired
    private IdGenerator idGenerator;

    /**
     * 检查是否存在分类名称
     *
     * @param name       分类名称
     * @param parentId   分类的上级id（一级分类为0）
     * @param siteId     站点id
     * @param type       检查类型（1：新增，2：修改(需要传classifyId)）
     * @param classifyId 分类id，修改检查时用到
     * @return 不存在返回true，存放相同名返回false
     */
    @Override
    public boolean checkExtisClassifyName(String name, Long parentId, Long siteId, int type, Long classifyId) {
        Classify classify = new Classify();
        classify.setName(name);
        classify.setStatus(1);
        classify.setParentId(parentId == null ? 0 : parentId.longValue());
        classify.setSiteId(siteId);
        EntityWrapper wrapper = new EntityWrapper(classify);
        if (type == 2) {
            wrapper.ne("id", classifyId);
        }
        return classifyMapper.selectCount(wrapper) == 0;
    }

    /**
     * 添加课程分类
     *
     * @param classify
     * @return
     */
    @Override
    public int saveClassify(Classify classify) {
        Classify exitsNameClassify = new Classify();
        exitsNameClassify.setName(classify.getName());
        exitsNameClassify.setParentId(classify.getParentId());
        exitsNameClassify.setStatus(1);
        exitsNameClassify.setSiteId(classify.getSiteId());
        int count = exitsNameClassify.selectCount(new EntityWrapper(exitsNameClassify));
        // 如果同级下存在相同名称的分类，返回标识符3
        if (count > 0) {
            return 3;
        }
        Long id = idGenerator.generate();
        classify.setId(id);
        Integer sort = classifyMapper.selectSort(classify.getParentId());
        classify.setSort(++sort);
        if (classify.getParentId() == 0L) {
            classify.setLayer(1);
            classify.setRootId(id);
            classify.setPath("" + id);
        } else {
            int layer = classifyMapper.selectParentLayer(classify.getParentId());
            if (layer > 2) {
                logger.error("添加超过三级");
                return 0;
            }
            if (layer == 1) {
                classify.setPath(classify.getParentId() + "," + id);
                classify.setLayer(++layer);
                classify.setRootId(classify.getParentId());
            } else {
                classify.setLayer(++layer);
                Long rootid = classifyMapper.selectRootid(classify.getParentId());
                classify.setRootId(rootid);
                classify.setPath(rootid + "," + classify.getParentId() + "," + id);
            }
        }
        return classifyMapper.insert(classify);
    }

    @Override
    public List<ClassifyVo> findExistChildNodes(List<Long> classifyIds){
        List<ClassifyVo> data = null;
        data = this.baseMapper.findExistChildNodes(classifyIds);
        return data;
    }

    /**
     * 删除分类
     *
     * @param id
     * @return
     */
    @Override
    public int deleteClassify(Long id) {
        Classify classify = new Classify();
        classify.setId(id);
        //查找有没有子级分类
        Long childid = classifyMapper.selectChild(id);
        int i = 0;
        if (childid == null) {
            //查找有没有挂有课程
            Long i1 = classifyMapper.selectCourse(id, ContextHolder.get().getSiteId());
            if (i1 == null) {
                try {
                    classify.setStatus(0);
                    i = classifyMapper.updateById(classify);
                } catch (Exception e) {
                    logger.error("删除失败", e);
                    return 0;
                }
            }
            else{ return -1;} //分类下挂有课程需要在web进行提示
        }
        return i;
    }

    /**
     * 修改分类名称
     *
     * @param classify
     * @return
     */
    @Override
    public int updateClassify(Classify classify) {
        try {
            Classify oldClassify = classify.selectById();
            if (oldClassify == null) {
                return 0;
            }
           /* Classify existUpdateClassify = new Classify();
            existUpdateClassify.setParentId(oldClassify.getParentId());
            existUpdateClassify.setName(classify.getName());
            existUpdateClassify.setSiteId(classify.getSiteId());
            EntityWrapper<Classify> ew = new EntityWrapper(existUpdateClassify);
            ew.notIn("id", oldClassify.getId());
            int count = existUpdateClassify.selectCount(ew);
            // 如果同级下存在相同名称的分类，返回标识符3
            if (count > 0) {
                return 3;
            }*/
            return classifyMapper.updateById(classify);
        } catch (Exception e) {
            logger.error("修改分类失败");
            return 0;
        }
    }

    /**
     * 根据名称模糊查询分类
     *
     * @param name
     * @return
     */
    @Override
    public Page<Classify> listClassify(String name, Integer pageNo, Integer pageSize, Long copanyId, List<Long> orgIds,Long siteId,Boolean isStartHQ) {
        List<String> list = new ArrayList<String>();
        list.add("path");
        Page<Classify> page = new Page<Classify>(pageNo, pageSize);
        Classify classify = new Classify();
        classify.setStatus(1);
        classify.setCompanyId(copanyId);
        classify.setSiteId(siteId);
        EntityWrapper<Classify> entityWrapper = new EntityWrapper<Classify>(classify);
        entityWrapper.orderAsc(list);
        if (StringUtils.isEmpty(name)) {
            entityWrapper.like("name", name);
        }
        /*if (null != orgIds && orgIds.size() > 0) {
            entityWrapper.in("org_id", orgIds);
        }*/
        // 分级授权修改
        if(isStartHQ) HQueryUtil.startHQ(Classify.class);
        page = this.selectPage(page, entityWrapper);
        return page;
    }

    /**
     * 获取学员端分类列表
     *
     * @return
     */
    @Override
    public List<Classify> listStuClassify(Long companyId,Long siteId) {
        try {

            List<String> list = new ArrayList<String>();
            list.add("path");
            Classify classify = new Classify();
            classify.setStatus(1);
            classify.setCompanyId(companyId);
            classify.setSiteId(siteId);
            EntityWrapper<Classify> entityWrapper = new EntityWrapper<Classify>(classify);
            entityWrapper.orderAsc(list);
            List<Classify> classifies1 = classifyMapper.selectList(entityWrapper);

//            classify.setLayer(2);
//            List<Classify> classifies2 = classifyMapper.selectList(entityWrapper);
//
//            classify.setLayer(3);
//            List<Classify> classifies3 = classifyMapper.selectList(entityWrapper);
//
//            Map<Long, List<ApiClassifyVo>> map3 = new HashMap<Long, List<ApiClassifyVo>>();
//            for (Classify c3:classifies3) {
//                if (map3.containsKey(c3.getParentId())) {
//                    List<ApiClassifyVo> apiClassifyVos = map3.get(c3.getParentId());
//                    ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
//                    apiClassifyVo.setClassify(c3);
//                    apiClassifyVos.add(apiClassifyVo);
//                } else {
//                    List<ApiClassifyVo> apiClassifyVos = new ArrayList<ApiClassifyVo>();
//                    ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
//                    apiClassifyVo.setClassify(c3);
//                    apiClassifyVos.add(apiClassifyVo);
//                    map3.put(c3.getParentId(), apiClassifyVos);
//                }
//            }
//
//            Map<Long, List<ApiClassifyVo>> map2 = new HashMap<Long, List<ApiClassifyVo>>();
//            for (Classify c2:classifies2) {
//                if (map2.containsKey(c2.getParentId())) {
//                    List<ApiClassifyVo> apiClassifyVos = map2.get(c2.getParentId());
//                    ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
//                    apiClassifyVo.setClassify(c2);
//                    apiClassifyVo.setApiClassifyVos(map3.get(c2.getId()));
//                    apiClassifyVos.add(apiClassifyVo);
//                } else {
//                    List<ApiClassifyVo> apiClassifyVos = new ArrayList<ApiClassifyVo>();
//                    ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
//                    apiClassifyVo.setClassify(c2);
//                    apiClassifyVo.setApiClassifyVos(map3.get(c2.getId()));
//                    apiClassifyVos.add(apiClassifyVo);
//                    map2.put(c2.getParentId(), apiClassifyVos);
//                }
//            }
//
//            List<ApiClassifyVo> classifies = new ArrayList<ApiClassifyVo>();
//            for (Classify c1 : classifies1) {
//                ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
//                apiClassifyVo.setClassify(c1);
//                apiClassifyVo.setApiClassifyVos(map2.get(c1.getId()));
//                classifies.add(apiClassifyVo);
//            }

            return classifies1;
        } catch (Exception e) {
            logger.error("", e);
            return null;
        }
    }

    /**
     * 课程分类排序
     *
     * @param list
     * @return
     */
    @Override
    public int updateClassiyfsort(List<Classify> list) {

        if (list.size() == 2) {
            Classify classifyFirst = list.get(0);
            Classify classifyLast = list.get(1);
            if(classifyFirst.getSort().equals(classifyLast.getSort())){
                classifyLast.setSort(classifyLast.getSort().intValue()-1);
            }
            classifyMapper.updateById(classifyFirst);
            classifyMapper.updateById(classifyLast);
        }
        return 1;
    }

    @Override
    public Page<ApiClassifyVo> selectListByCompanyId(Long companyId, String name, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize) {
        Page<ApiClassifyVo> page = new Page<ApiClassifyVo>(pageNo, pageSize);
//        // 分级授权修改   bug12380要求修改为站点共享
//        HQueryUtil.startHQ(Classify.class);
        List<ApiClassifyVo> apiClassifyVos = this.baseMapper.selectListByCompanyId(companyId, name, orgIds,siteId, new RowBounds(page.getOffset(), page.getLimit()));
        Integer count = this.baseMapper.selectCountByCompanyId(companyId, name, orgIds,siteId);
        page.setTotal(count);
        page.setRecords(apiClassifyVos);
        return page;
    }

    @Override
    public List<Classify> selectClassifyByPath(List<Long> ids) {
        return this.baseMapper.selectClassifyByRootId(ids);
    }

    @Override
    public List<Classify> selectClassifyByLayer(Long companyId, Long siteId, Integer layer) {
//        // 分级授权修改
//        HQueryUtil.startHQ(Classify.class);
        return this.baseMapper.selectClassifyByLayer(companyId,siteId,layer);
    }


    @Override
    public Page getClassifyNameByDrools(String field, String value, Page page) {
        if (StringUtils.isEmpty(field)) {
            logger.info("列名不能为空！");
            return page;
        }
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();
        List<DroolsVo> voList = null;

        Classify classify = new Classify();
        classify.setSiteId(siteId);
        classify.setCompanyId(companyId);
        classify.setStatus(1);
        EntityWrapper wrapper = new EntityWrapper(classify);
        wrapper.setSqlSelect("distinct(" + field + "),"+"id ")
                .isNotNull(field)
                .like(field, value)
                .addFilter(field + "!=''")
                .orderBy("create_time", false);

        List<Classify> list = this.classifyMapper.selectPage(page, wrapper);
        if (!CollectionUtils.isEmpty(list)){
            voList = new ArrayList<>(list.size());
            for (Classify a:list){
                DroolsVo vo = new DroolsVo();
                vo.setTaskId(a.getId());
                vo.setTaskFieldValue(a.getName());
                vo.setTaskParamsType(field);
                voList.add(vo);
            }
        }
        page.setRecords(voList);
        return page;
    }
}
