package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.plugins.pagination.PageHelper;
import com.baomidou.mybatisplus.plugins.pagination.Pagination;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.core.application.enums.TaskParamsEnums;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.feign.CoursePcClient;
import com.yizhi.course.application.mapper.*;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.util.MaterialDateUtil;
import com.yizhi.course.application.vo.*;
import com.yizhi.site.application.enums.FunctionTypeCode;
import com.yizhi.site.application.feign.api.FunctionDisplayConfigApiClients;
import com.yizhi.site.application.vo.domain.FunctionDisplayConfigVo;
import com.yizhi.util.application.clazz.ClassUtil;
import com.yizhi.util.application.domain.BizResponse;
import com.yizhi.util.application.domain.Response;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Service
@Transactional
@Log4j2
public class CourseServiceImpl extends ServiceImpl<CourseMapper, Course> implements ICourseService {

    private static final Logger LOGGER = LoggerFactory.getLogger(CourseServiceImpl.class);

    private static final List<Long> scopes = Arrays.asList(0L, 1L);

    @Autowired
    private CourseDescriptionMapper courseDescriptionMapper;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    FunctionDisplayConfigApiClients configApiClients;

    @Autowired
    private CourseAccountMapper courseAccountMapper;

    @Autowired
    private AccreditMapper accreditMapper;

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    private IMaterialService materialService;

    @Autowired
    private ICourseDescriptionService courseDescriptionService;

    @Autowired
    private ICourseAccountService courseAccountService;

    @Autowired
    private ChapterMapper chapterMapper;

    @Autowired
    private TestCaseMapper testCaseMapper;

    @Autowired
    private RecordeMapper recordeMapper;

    @Autowired
    private ITestCaseService testCaseService;

    @Autowired
    private MaterialMapper materialMapper;

    @Autowired
    private ICourseStudentStatusService studentStatusService;

    @Autowired
    private ClassifyMapper classifyMapper;

    @Autowired
    private IClassifyService classifyService;

    @Autowired
    private CourseStudentStatusMapper courseStudentStatusMapper;


    /**
     * 获取课程的总学时
     *
     * @param courseId
     * @return
     */
    @Override
    public Long getTotalDuration(Long courseId) {
        return this.baseMapper.getTotalDuration(courseId);
    }

    @Override
    public CourseVo getCourseDetails(Long id) {
        CourseVo cv = this.baseMapper.getCourseDetails(id);
        return cv;
    }

    @Override
    public CourseVo save(CourseVo courseVo) {

        if (! Integer.valueOf(4).equals(courseVo.getSource()) && !this.vailCode(null, courseVo.getCode(), courseVo.getSiteId())) {
            return null;
        }

        try {
            Course course = new Course();
            course.setCode(courseVo.getCode());
            course.setTotalDurationGrant(courseVo.getTotalDurationGrant());

            Integer playModel = courseVo.getVideoPlayModel();
            course.setVideoPlayModel(Objects.isNull(playModel) ? 0 : playModel);

            Integer publishingTerminal = courseVo.getPublishingTerminal();
            if (Objects.nonNull(publishingTerminal)) {
                course.setPublishingTerminal(publishingTerminal);
            }

            Float totalDuration = courseVo.getTotalDuration();
            if (Objects.nonNull(totalDuration)) {
                course.setTotalDuration(totalDuration);
            }

            course.setId(idGenerator.generate());
            course.setName(courseVo.getName());
            course.setTags(courseVo.getTags());
            course.setImage(courseVo.getImage());
            course.setClassifyId(courseVo.getClassifyId());
            course.setShelves(courseVo.getShelves());
            course.setCourseType(courseVo.getCourseType());
            course.setSource(1);
            // 营销课程名片和分享标题/描述修改
            course.setCardShow(courseVo.getCardShow());
            course.setShareTitle(courseVo.getShareTitle());
            course.setShareDesc(courseVo.getShareDesc());

            Integer source = courseVo.getSource();
            if (courseVo.getSource() != null) {
                course.setSource(courseVo.getSource());
            }

            if (courseVo.getShelves() == null) {
                course.setShelves(2);
            }
            course.setDelFlg(0);
            course.setScope(courseVo.getScope());
            course.setAuthorUnit(courseVo.getAuthorUnit());
            course.setCompanyId(courseVo.getCompanyId());
            course.setSiteId(courseVo.getSiteId());
            course.setOrgId(courseVo.getOrgId());
            course.setCreateById(courseVo.getCreateById());
            course.setCreateByName(courseVo.getCreateByName());
            course.setCreateTime(new Date());

            // 营销课程除外
            if (Objects.nonNull(source) && source.intValue() == 4) {
                course.setPortRule(2);
                course.setDownEnable(0);
            } else {
                //为规则加入默认值
                if (courseVo.getCourseType().equals(1)) {
                    course.setPortRule(1);
                    course.setDownEnable(1);
                } else {
                    course.setPortRule(2);
                    course.setDownEnable(0);
                }
            }
            course.setJumpEnable(0);
            course.setGradeEnable(0);
            course.setFastForwardEnable(0);
            course.setAudioComplete(0);
            course.setVideoComplete(1);
            course.setDocComplete(1);
            course.setRichComplete(1);
            course.setAutoEnable(0);
            course.setCommentEnable(0);
            this.baseMapper.insert(course);

            // 如果存在课程简介信息，执行简介大字段表保存
            if (StringUtils.isNotEmpty(courseVo.getDescription())) {
                CourseDescription courseDescription = new CourseDescription();
                courseDescription.setId(idGenerator.generate());
                courseDescription.setCourseId(course.getId());
                courseDescription.setDescription(courseVo.getDescription());
                courseDescriptionMapper.insert(courseDescription);
            }
            courseVo.setId(course.getId());
            return courseVo;
        } catch (Exception e) {
            LOGGER.info("新增课程异常" + JSON.toJSONString(e));
            System.out.println(e.getLocalizedMessage());
            return null;
        }


    }

    @Override
    public BizResponse<CourseVo> saveV2(CourseVo courseVo) {

        if (!this.vailCodeExclMarketing(null, courseVo.getCode(), courseVo.getSiteId())) {
            log.error("保存课程失败;课程编码重复;入参: courseVo = {}", JSONObject.toJSONString(courseVo));
            return BizResponse.fail(InternationalEnums.COURSECONTROLLER1.getCode(),InternationalEnums.COURSECONTROLLER1.getName(),courseVo);
        }

        try {
            Course course = new Course();
            course.setCode(courseVo.getCode());
            course.setTotalDurationGrant(courseVo.getTotalDurationGrant());

            Integer playModel = courseVo.getVideoPlayModel();
            course.setVideoPlayModel(Objects.isNull(playModel) ? 0 : playModel);

            Integer publishingTerminal = courseVo.getPublishingTerminal();
            if (Objects.nonNull(publishingTerminal)) {
                course.setPublishingTerminal(publishingTerminal);
            }

            Float totalDuration = courseVo.getTotalDuration();
            if (Objects.nonNull(totalDuration)) {
                course.setTotalDuration(totalDuration);
            }

            course.setId(idGenerator.generate());
            course.setName(courseVo.getName());
            course.setTags(courseVo.getTags());
            course.setImage(courseVo.getImage());
            course.setClassifyId(courseVo.getClassifyId());
            course.setShelves(courseVo.getShelves());
            course.setCourseType(courseVo.getCourseType());
            course.setSource(1);
            // 营销课程名片和分享标题/描述修改
            course.setCardShow(courseVo.getCardShow());
            course.setShareTitle(courseVo.getShareTitle());
            course.setShareDesc(courseVo.getShareDesc());

            Integer source = courseVo.getSource();
            if (courseVo.getSource() != null) {
                course.setSource(courseVo.getSource());
            }

            if (courseVo.getShelves() == null) {
                course.setShelves(2);
            }
            course.setDelFlg(0);
            course.setScope(courseVo.getScope());
            course.setAuthorUnit(courseVo.getAuthorUnit());
            course.setCompanyId(courseVo.getCompanyId());
            course.setSiteId(courseVo.getSiteId());
            course.setOrgId(courseVo.getOrgId());
            course.setCreateById(courseVo.getCreateById());
            course.setCreateByName(courseVo.getCreateByName());
            course.setCreateTime(new Date());

            // 营销课程除外
            if (Objects.nonNull(source) && source.intValue() == 4) {
                course.setPortRule(2);
                course.setDownEnable(0);
            } else {
                //为规则加入默认值
                if (courseVo.getCourseType().equals(1)) {
                    course.setPortRule(1);
                    course.setDownEnable(1);
                } else {
                    course.setPortRule(2);
                    course.setDownEnable(0);
                }
            }
            course.setJumpEnable(0);
            course.setGradeEnable(0);
            course.setFastForwardEnable(0);
            course.setAudioComplete(0);
            course.setVideoComplete(1);
            course.setDocComplete(1);
            course.setRichComplete(1);
            course.setAutoEnable(0);
            course.setCommentEnable(0);
            this.baseMapper.insert(course);

            // 如果存在课程简介信息，执行简介大字段表保存
            if (StringUtils.isNotEmpty(courseVo.getDescription())) {
                CourseDescription courseDescription = new CourseDescription();
                courseDescription.setId(idGenerator.generate());
                courseDescription.setCourseId(course.getId());
                courseDescription.setDescription(courseVo.getDescription());
                courseDescriptionMapper.insert(courseDescription);
            }
            courseVo.setId(course.getId());
            return BizResponse.ok(courseVo);
        } catch (Exception e) {
            LOGGER.error("新增课程异常;入参: {}",JSONObject.toJSONString(courseVo),e);
            return BizResponse.fail(InternationalEnums.TPPLANCONTROLLER6.getCode(),InternationalEnums.TPPLANCONTROLLER6.getName(),courseVo);
        }


    }

    @Override
    public Boolean update(CourseVo courseVo) {
        if (!this.vailCodeExclMarketing(courseVo.getId(), courseVo.getCode(), courseVo.getSiteId())) {
            log.error("更新课程信息失败;课程编码重复;入参: = {}",JSONObject.toJSONString(courseVo));
            return Boolean.FALSE;
        }

        try {
            Course course = this.baseMapper.selectById(courseVo.getId());
            Integer publishingTerminal = courseVo.getPublishingTerminal();

            Integer playModel = courseVo.getVideoPlayModel();
            if (Objects.nonNull(playModel)) {
                course.setVideoPlayModel(playModel);
            }

            if (Objects.nonNull(publishingTerminal)) {
                course.setPublishingTerminal(publishingTerminal);
            }
            Float totalDuration = courseVo.getTotalDuration();
            if (Objects.nonNull(totalDuration)) {
                course.setTotalDuration(totalDuration);
            }
            Boolean totalDurationGrant = courseVo.getTotalDurationGrant();
            if (Objects.nonNull(totalDurationGrant)) {
                course.setTotalDurationGrant(totalDurationGrant);
            }

            if (StringUtils.isEmpty(courseVo.getCode())) {
                course.setAutoEnable(courseVo.getAutoEnable());
                course.setCommentEnable(courseVo.getCommentEnable());
                course.setScope(courseVo.getScope());
                course.setJumpEnable(courseVo.getJumpEnable());
                course.setDownEnable(courseVo.getDownEnable());
                course.setGradeEnable(courseVo.getGradeEnable());
                course.setFastForwardEnable(courseVo.getFastForwardEnable());
                course.setAudioComplete(courseVo.getAudioComplete());
                course.setVideoComplete(courseVo.getVideoComplete());
                course.setDocComplete(courseVo.getDocComplete());
                course.setRichComplete(courseVo.getRichComplete());
                course.setPortRule(courseVo.getPortRule());
                course.setUpdateById(courseVo.getUpdateById());
                course.setUpdateByName(courseVo.getUpdateByName());
                course.setUpdateTime(courseVo.getUpdateTime());
            } else {
                course.setName(courseVo.getName());
                course.setCode(courseVo.getCode());
                course.setTags(courseVo.getTags());
                course.setImage(courseVo.getImage());
                course.setClassifyId(courseVo.getClassifyId());
                course.setAuthorUnit(courseVo.getAuthorUnit());

                if (courseVo.getCourseType() != null) {
                    course.setCourseType(courseVo.getCourseType());
                }
                CourseDescription courseDescription = courseDescriptionMapper.getByCourseId(course.getId());
                if (courseDescription != null) {
                    courseDescription.setDescription(courseVo.getDescription());
                    courseDescriptionMapper.updateById(courseDescription);
                } else {
                    CourseDescription newCourseDescription = new CourseDescription();
                    newCourseDescription.setId(idGenerator.generate());
                    newCourseDescription.setCourseId(course.getId());
                    newCourseDescription.setDescription(courseVo.getDescription());
                    courseDescriptionMapper.insert(newCourseDescription);
                }

            }
            course.setEnableTask(courseVo.getEnableTask());
            if (courseVo.getEnableTask() != null && courseVo.getEnableTask() == 1) {
                course.setTaskStartDate(courseVo.getTaskStartDate());
                course.setTaskEndDate(courseVo.getTaskEndDate());
            }

            // 如果关联证书
            Boolean grantCertificate = courseVo.getGrantCertificate();
            if(grantCertificate != null){
                course.setGrantCertificate(grantCertificate);
                if(grantCertificate.booleanValue()){
                    List<CourseCertificateVo> CourseCertificateVos = courseVo.getCertificates();
                    // 设置关联的证书id
                    course.setCertificates(CourseCertificateVos.stream().
                            map(itemObj -> itemObj.getId().toString()).collect(Collectors.joining(",")));
                }else {
                    course.setCertificates("");
                }
            }
            RequestContext context = ContextHolder.get();
            if(context != null && context.getAccountId() != null){
                course.setUpdateTime( new Date());
                course.setUpdateByName(context.getAccountName());
                course.setUpdateById(context.getAccountId());
            }
            this.baseMapper.updateById(course);
            return Boolean.TRUE;
        } catch (Exception e) {
            LOGGER.error("更新课程异常;入参= {} ;错误信息如下：",JSONObject.toJSONString(courseVo), e);
            return Boolean.FALSE;
        }

    }

    @Override
    public BizResponse<Boolean> updateV2(CourseVo courseVo) {
        if (!this.vailCodeExclMarketing(courseVo.getId(), courseVo.getCode(), courseVo.getSiteId())) {
            log.error("更新课程信息失败;课程编码重复;入参: = {}",JSONObject.toJSONString(courseVo));
            return BizResponse.fail(InternationalEnums.COURSECONTROLLER1.getCode(),InternationalEnums.COURSECONTROLLER1.getName(), Boolean.FALSE);
        }

        try {
            Course course = this.baseMapper.selectById(courseVo.getId());
            Integer publishingTerminal = courseVo.getPublishingTerminal();

            Integer playModel = courseVo.getVideoPlayModel();
            if (Objects.nonNull(playModel)) {
                course.setVideoPlayModel(playModel);
            }

            if (Objects.nonNull(publishingTerminal)) {
                course.setPublishingTerminal(publishingTerminal);
            }
            Float totalDuration = courseVo.getTotalDuration();
            if (Objects.nonNull(totalDuration)) {
                course.setTotalDuration(totalDuration);
            }
            Boolean totalDurationGrant = courseVo.getTotalDurationGrant();
            if (Objects.nonNull(totalDurationGrant)) {
                course.setTotalDurationGrant(totalDurationGrant);
            }

            if (StringUtils.isEmpty(courseVo.getCode())) {
                course.setAutoEnable(courseVo.getAutoEnable());
                course.setCommentEnable(courseVo.getCommentEnable());
                course.setScope(courseVo.getScope());
                course.setJumpEnable(courseVo.getJumpEnable());
                course.setDownEnable(courseVo.getDownEnable());
                course.setGradeEnable(courseVo.getGradeEnable());
                course.setFastForwardEnable(courseVo.getFastForwardEnable());
                course.setAudioComplete(courseVo.getAudioComplete());
                course.setVideoComplete(courseVo.getVideoComplete());
                course.setDocComplete(courseVo.getDocComplete());
                course.setRichComplete(courseVo.getRichComplete());
                course.setPortRule(courseVo.getPortRule());
                course.setUpdateById(courseVo.getUpdateById());
                course.setUpdateByName(courseVo.getUpdateByName());
                course.setUpdateTime(courseVo.getUpdateTime());
            } else {
                course.setName(courseVo.getName());
                course.setCode(courseVo.getCode());
                course.setTags(courseVo.getTags());
                course.setImage(courseVo.getImage());
                course.setClassifyId(courseVo.getClassifyId());
                course.setAuthorUnit(courseVo.getAuthorUnit());

                if (courseVo.getCourseType() != null) {
                    course.setCourseType(courseVo.getCourseType());
                }
                CourseDescription courseDescription = courseDescriptionMapper.getByCourseId(course.getId());
                if (courseDescription != null) {
                    courseDescription.setDescription(courseVo.getDescription());
                    courseDescriptionMapper.updateById(courseDescription);
                } else {
                    CourseDescription newCourseDescription = new CourseDescription();
                    newCourseDescription.setId(idGenerator.generate());
                    newCourseDescription.setCourseId(course.getId());
                    newCourseDescription.setDescription(courseVo.getDescription());
                    courseDescriptionMapper.insert(newCourseDescription);
                }

            }
            course.setEnableTask(courseVo.getEnableTask());
            if (courseVo.getEnableTask() != null && courseVo.getEnableTask() == 1) {
                course.setTaskStartDate(courseVo.getTaskStartDate());
                course.setTaskEndDate(courseVo.getTaskEndDate());
            }

            // 如果关联证书
            Boolean grantCertificate = courseVo.getGrantCertificate();
            if(grantCertificate != null){
                course.setGrantCertificate(grantCertificate);
                if(grantCertificate.booleanValue()){
                    List<CourseCertificateVo> CourseCertificateVos = courseVo.getCertificates();
                    // 设置关联的证书id
                    course.setCertificates(CourseCertificateVos.stream().
                            map(itemObj -> itemObj.getId().toString()).collect(Collectors.joining(",")));
                }else {
                    course.setCertificates("");
                }
            }
            RequestContext context = ContextHolder.get();
            if(context != null && context.getAccountId() != null){
                course.setUpdateTime( new Date());
                course.setUpdateByName(context.getAccountName());
                course.setUpdateById(context.getAccountId());
            }
            this.baseMapper.updateById(course);
            return BizResponse.ok(Boolean.TRUE);
        } catch (Exception e) {
            LOGGER.error("更新课程异常;入参= {} ;错误信息如下：",JSONObject.toJSONString(courseVo), e);
            return BizResponse.fail(InternationalEnums.TPPLANCONTROLLER6.getCode(),InternationalEnums.TPPLANCONTROLLER6.getName(), Boolean.FALSE);
        }

    }

    @Override
    public Page<Course> searchPage(String name, Long classifyId, Integer shelves, Long companyId, Long siteId,
                                   List<Long> orgIds, int pageNo, int pageSize, Integer type, Integer courseType,
                                   List<Long> ids) {
        Page<Course> page = new Page<Course>(pageNo, pageSize);
        // 分级授权修改
        HQueryUtil.startHQ(Course.class);
        List<Course> data = this.baseMapper.searchPage(name, classifyId, shelves, companyId, siteId, orgIds,
                ids, type, page, courseType);
        if (CollectionUtils.isNotEmpty(data)) {
            List<Long> courseIds = data.stream().map(Course::getId).collect(Collectors.toList());
            List<ApiListCourseVo> vos = this.baseMapper.getCourseDurationByIds(courseIds);
            if (CollectionUtils.isNotEmpty(vos)) {
                Map<Long, Integer> dataMap = vos.stream().collect(Collectors.
                        toMap(ApiListCourseVo::getId, ApiListCourseVo::getDuration));
                for (Course itemCourse : data) {
                    itemCourse.setDuration(dataMap.get(itemCourse.getId()));
                }
            }
        }
        page.setRecords(data);
        return page;
    }

    @Override
    public Page<Course> searchPageByClassifyAllMg(String name, Long classifyId, Integer shelves,
                                           Long companyId, Long siteId, List<Long> orgIds, int pageNo, int pageSize,
                                           Integer type, Integer courseType, List<Long> courseIds) {

        List<Long> classifyIds = null;
        //Long classifyId = apiSeachCourseVo.getClassifyId();
        if (Objects.nonNull(classifyId) && classifyId.longValue() > 0) {
            classifyIds = new ArrayList<>();
            // 查询第一级的子分类
            Classify classify = classifyService.selectById(classifyId);
            Integer layer = classify.getLayer();
            classifyIds.add(classifyId);
            if (Objects.nonNull(layer) && layer.intValue() < 3) {
                Classify classify1 = new Classify();
                classify1.setParentId(classifyId);
                classify1.setStatus(1);
                List<Classify> firstIds = classifyService.selectList(QueryUtil.condition(classify1));
                Set<Long> lastIds = null;
                if (!org.springframework.util.CollectionUtils.isEmpty(firstIds)) {
                    lastIds = firstIds.stream().map(objfirst -> objfirst.getId()).collect(Collectors.toSet());
                    classifyIds.addAll(lastIds);
                }
                // 查询第二级的子分类
                if (Objects.nonNull(lastIds) && layer.intValue() < 2) {
                    EntityWrapper<Classify> ew = QueryUtil.condition(new Classify());
                    ew.in("parent_id", lastIds);
                    ew.eq("status", 1);
                    List<Classify> dataList = classifyService.selectList(ew);
                    if (!org.springframework.util.CollectionUtils.isEmpty(dataList)) {
                        classifyIds.addAll(dataList.stream().map(objLast -> objLast.getId()).collect(Collectors.toSet()));
                    }
                }
            }
        }
        Page<Course> page = new Page<Course>(pageNo, pageSize);

        // 分级授权修改
        HQueryUtil.startHQ(Course.class);

        List<Course> data = this.baseMapper.searchPageByClassifyAllMg(name, classifyIds, shelves, companyId, siteId, orgIds,
                courseIds, type, page, courseType);
        if (CollectionUtils.isNotEmpty(data)) {
            List<Long> courseIdsAll = data.stream().map(Course::getId).collect(Collectors.toList());
            List<ApiListCourseVo> vos = this.baseMapper.getCourseDurationByIds(courseIdsAll);
            if (CollectionUtils.isNotEmpty(vos)) {
                Map<Long, Integer> dataMap = vos.stream().collect(Collectors.
                        toMap(ApiListCourseVo::getId, ApiListCourseVo::getDuration));
                for (Course itemCourse : data) {
                    itemCourse.setDuration(dataMap.get(itemCourse.getId()));
                }
            }
        }
        page.setRecords(data);
        return page;
    }

    @Override
    public Boolean vailCode(Long id, String code, Long siteId) {
        Integer count = this.baseMapper.vailCode(id, code, siteId);
        if (count > 0) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }


    @Override
    public Boolean vailCodeExclMarketing(Long id, String code, Long siteId) {
        Integer count = courseMapper.vailCodeExclMarketing(id, code, siteId);
        if (count > 0) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public Page<ApiListCourseVo> apiList(String name, Long classifyId, Integer type, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (relationIds != null && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<ApiListCourseVo> page = new Page<ApiListCourseVo>(pageNo, pageSize);
        List<ApiListCourseVo> data = this.baseMapper.apiList(name, classifyId, type, ids,
                companyId, siteId, new RowBounds(page.getOffset(), page.getLimit()));
        if (CollectionUtils.isNotEmpty(data)) {
            List<Long> courseIds = data.stream().map(ApiListCourseVo::getId).collect(Collectors.toList());
            List<ApiListCourseVo> vos = this.baseMapper.getCourseDurationByIds(courseIds);
            LOGGER.info("课程时长={}", JSON.toJSONString(vos));
            if (CollectionUtils.isNotEmpty(vos)) {
                Map<Long, Integer> dataMap = vos.stream().collect(Collectors.
                        toMap(key -> key.getId(), val -> val.getDuration()));
                for (ApiListCourseVo itemCourse : data) {
                    itemCourse.setDuration(dataMap.get(itemCourse.getId()));
                }
            }
        }
        page.setRecords(data);
        Integer count = this.baseMapper.apiListCount(name, classifyId, type, ids, companyId, siteId);
        page.setTotal(count);
        return page;
    }

    @Override
    public Page<ApiListCourseVo> searchPageSubClassify(String name, List<Long> classifyIds
            , Integer type, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (relationIds != null && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<ApiListCourseVo> page = new Page<ApiListCourseVo>(pageNo, pageSize);
        List<ApiListCourseVo> data = this.baseMapper.searchPageSubClassify(name, classifyIds, type, ids,
                companyId, siteId, page);
        if (CollectionUtils.isNotEmpty(data)) {
            List<Long> courseIds = data.stream().map(ApiListCourseVo::getId).collect(Collectors.toList());
            List<ApiListCourseVo> vos = this.baseMapper.getCourseDurationByIds(courseIds);
            LOGGER.info("课程时长={}", JSON.toJSONString(vos));
            if (CollectionUtils.isNotEmpty(vos)) {
                Map<Long, Integer> dataMap = vos.stream().collect(Collectors.
                        toMap(key -> key.getId(), val -> val.getDuration()));
                for (ApiListCourseVo itemCourse : data) {
                    itemCourse.setDuration(dataMap.get(itemCourse.getId()));
                }
            }
        }
        page.setRecords(data);
        return page;
    }

    @Override
    public Double getAvgLevel(Long courseId) {
        return this.baseMapper.getAvgLevel(courseId);
    }

    @Override
    public Page<ProjectCourseVo> searchProjectCoursePage(Long id, String name, Long classifyId, Long companyId, Long siteId, String[] orgId, int pageNo, int pageSize) {
        Page<ProjectCourseVo> page = new Page<ProjectCourseVo>(pageNo, pageSize);
        page.setRecords(this.baseMapper.searchProjectCoursePage(id, name, classifyId, companyId, siteId, orgId, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.searchProjectCoursePageCount(id, name, classifyId, companyId, siteId, orgId);
        page.setTotal(count);
        return page;
    }

    /**
     * 分页查询
     * .groupBy("x1").groupBy("x2,x3");.having("x1=11").having("x3=433"), .orderBy("dd").orderBy("d1,d2");
     *
     * @param courseVo
     * @param page
     */
    @Override
    public Page<Course> list(CourseVo courseVo, Page page) {
        Course course = new Course();
        if (courseVo.getClassifyId() != null) {
            course.setClassifyId(courseVo.getClassifyId());
        }
        course.setDelFlg(0);
        course.setSource(4);

        // 数据过滤
        RequestContext requestContext = ContextHolder.get();
        course.setCompanyId(requestContext.getCompanyId());
        course.setSiteId(requestContext.getSiteId());

        EntityWrapper ew = QueryUtil.condition(course);

        // 数据过滤
        if (CollectionUtils.isNotEmpty(requestContext.getRelationIds())) {
            ew.and().eq("shelves", 1);
            // 学员端待过滤
            List<Long> ids = courseAccountMapper.selectCourseIdByRelationId(requestContext.getRelationIds());
            if (CollectionUtils.isNotEmpty(ids)) {
                ew.andNew().in("scope", scopes).or().in("id", ids);
            } else {
                ew.and().in("scope", scopes);
            }
        } else if (!requestContext.isAdmin()) {
            HQueryUtil.startHQ(Course.class);
        }
        String name = courseVo.getName();
        if (StringUtils.isNotEmpty(name)) {
            ew.andNew()
                    .like("name", courseVo.getName())
                    .or().like("code", courseVo.getCode())
                    .or().like("tags", courseVo.getName());
        }

        List<String> list = new ArrayList<>();
        list.add("update_time");
        list.add("create_time");
        page.setDescs(list);
        return selectPage(page, ew);
    }

    @Override
    public List<HomePageCoursePageVo> homePageCoursePageList(String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (null != relationIds && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<HomePageCoursePageVo> page = new Page<HomePageCoursePageVo>(pageNo, pageSize);
        List<HomePageCoursePageVo> homePageCoursePageVos = this.baseMapper.homePageCoursePageList(name, companyId, siteId, ids, new RowBounds(page.getOffset(), page.getLimit()));
        return homePageCoursePageVos;
    }

    @Override
    public List<HomePageCoursePageVo> homePageCoursePageListNew(String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (null != relationIds && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<HomePageCoursePageVo> page = new Page<HomePageCoursePageVo>(pageNo, pageSize);
        List<HomePageCoursePageVo> homePageCoursePageVos = this.baseMapper.homePageCoursePageListNew(name, companyId, siteId, ids, new RowBounds(page.getOffset(), page.getLimit()));
        return homePageCoursePageVos;
    }

    @Override
    public List<HomePageCourseVo> homePageClassifyList(String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (null != relationIds && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<HomePageCourseVo> page = new Page<HomePageCourseVo>(pageNo, pageSize);
        List<HomePageCourseVo> homePageCoursePageVos = this.baseMapper.homePageClassifyList(name, companyId, siteId, ids, new RowBounds(page.getOffset(), page.getLimit()));
        return homePageCoursePageVos;
    }

    @Override
    public List<ApiListCourseVo> classifyTopCourseList(Long classifyId, String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (null != relationIds && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<ApiListCourseVo> page = new Page<ApiListCourseVo>(pageNo, pageSize);
        List<ApiListCourseVo> homePageCoursePageVos = this.baseMapper.classifyTopCourseList(classifyId, name, companyId, siteId, ids, new RowBounds(page.getOffset(), page.getLimit()));
        return homePageCoursePageVos;
    }

    @Override
    public Integer getRecordeDuration(Long chapterId, Long accountId) {
        return this.baseMapper.getRecordeDuration(chapterId, accountId);
    }

    @Override
    public List<Course> selectCourseList(String name, Long companyId, Long siteId, Long orgId) {
        orgId = 0L;
        return this.baseMapper.selectCourseList(name, companyId, siteId, orgId);
    }


    @Override
    public List<ApiListCourseVo> selectOfflineCourseList(List<Long> ids, int pageNo, int pageSize) {
        Page<ApiListCourseVo> page = new Page<ApiListCourseVo>(pageNo, pageSize);
        return this.baseMapper.selectOfflineCourseList(ids, new RowBounds(page.getOffset(), page.getLimit()));
    }

    /**
     * 查询站点的
     *
     * @param name
     * @param siteId
     * @param classifyId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public PageCourseAuthorizeVO queryeAuthorizeCoursesList(String name, Long siteId, Long classifyId, Integer pageNo, Integer pageSize) {

        Integer start = pageSize * (pageNo - 1);

        Integer end = pageSize;

        Long sourceSiteId = ContextHolder.get().getSiteId();

        RowBounds rowBounds = new RowBounds(start, end);
        List<AuthorizedCourseVO> courseList = accreditMapper.getAuthorizeList(classifyId, siteId, name, sourceSiteId, rowBounds);
        Integer totalCount = accreditMapper.getAuthorizeListCount(classifyId, siteId, name, sourceSiteId);

        PageCourseAuthorizeVO pageCourseAuthorizeVO = new PageCourseAuthorizeVO();
        pageCourseAuthorizeVO.setPageNo(pageNo);
        pageCourseAuthorizeVO.setPageSize(pageSize);
        pageCourseAuthorizeVO.setPageTotal(totalCount % pageSize == 0 ? totalCount / pageSize : totalCount / pageSize + 1);
        pageCourseAuthorizeVO.setRecords(courseList);

        pageCourseAuthorizeVO.setPageRecords(totalCount);

        return pageCourseAuthorizeVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Course copyCourse(Course course, Long companyId, Long siteId, Integer type, RequestContext context) {

        Long oldCourseId = course.getId();
        Date curDate = new Date();
        // 生成
        Long newCourseId = idGenerator.generate();
        course.setId(newCourseId);
        course.setCreateTime(curDate);
        course.setUpdateTime(curDate);
        course.setCreateByName(context.getAccountName());

        course.setSiteId(siteId);
        course.setOrgId(null);
        // 可见范围默认平台
        course.setScope(1);
        //草稿状态
        course.setShelves(2);
        course.setClassifyId(null);
        course.setClassifyName(null);

        if (type == 1) {
            // 1 课程授权
            course.setSource(6);
            course.setCompanyId(context.getCompanyId());
        } else if (type == 2) {
            // 2课程销售',
            course.setSource(5);
            course.setCompanyId(companyId);
        }
        // 新增课程
        try {
            // 新增课程完成
            this.insert(course);
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.info(e + "");
        }
        // 课程描述
        CourseDescription courseDescription = new CourseDescription();
        courseDescription.setCourseId(oldCourseId);

        courseDescription = courseDescriptionService.selectOne(QueryUtil.condition(courseDescription));
        // 新增课程描述
        if (courseDescription != null) {
            courseDescription.setId(idGenerator.generate());
            courseDescription.setCourseId(newCourseId);
            try {
                courseDescriptionService.insert(courseDescription);
            } catch (Exception e) {
                log.info("" + e);
            }
        }


        // 获取课程章节
        List<Chapter> chapterList = chapterService.courseChapterList(oldCourseId);
        //初始化章节list供插入用
        List<Chapter> newChapterList = new ArrayList<Chapter>();

        // 建中间list 复制章节的 父id使用
        Map<Long, Long> midMap = new HashMap<Long, Long>();
        ;
        // 初始化素材类

        Material material = null;


        // 章节的循环
        for (Chapter chapter : chapterList) {
            // 元章节id
            Long oldChapterId = chapter.getId();
            // 生成章节的id
            Long newChapterId = idGenerator.generate();

            chapter.setCourseId(newCourseId);
            chapter.setId(newChapterId);
            chapter.setChapterBeforeId(oldChapterId);
            if (chapter.getParentId() == 0) {
                chapter.setPath(newChapterId.toString());
                // 保存中间表id的关系
                midMap.put(oldChapterId, newChapterId);
            } else {
                for (Long key : midMap.keySet()) {
                    if (chapter.getParentId().equals(key)) {
                        Long newParentId = midMap.get(key);
                        log.info("新的父级章节idnewParentId：" + chapter.getParentId());
                        chapter.setPath(newParentId + "," + newChapterId);
                        chapter.setParentId(newParentId);
                        break;
                    }
                }

                material = materialService.selectById(chapter.getMaterialId());
                if (material != null) {
                    chapter.setMaterialId(material.getId());
                }

            }

            newChapterList.add(chapter);
        }
        log.info("保存新的章节的关系：" + midMap);


        // 新增章节
        if (CollectionUtils.isNotEmpty(newChapterList)) {
            chapterService.insertBatch(newChapterList);
        }
        return course;
    }

    //获取课程浏览人次
    @Override
    public Integer courseViewCount(Long id, Long accountId) {

        return courseMapper.courseViewCountNum(id);
        //List<ViewCourseCount> viewCourseCounts = courseMapper.courseViewCountNum(id);
        /*if (CollectionUtils.isNotEmpty(viewCourseCounts)) {
            log.info("数量 count:" + viewCourseCounts.size());
            return viewCourseCounts.size();
        } else {
            return 0;
        }*/
    }


    @Override
    public VisibleRangeExport vsibleRangeExport(Long courseId) {
        // TODO Auto-generated method stub
        VisibleRangeExport visibleRangeExport = new VisibleRangeExport();
        List<Long> accountIds = new ArrayList<Long>();
        List<Long> orgIds = new ArrayList<Long>();


        Course course = courseMapper.selectById(courseId);
        if (course != null) {
            visibleRangeExport.setBizId(course.getId());
            visibleRangeExport.setBizName(course.getName());
        }

        List<CourseAccount> listStudent = courseAccountService.listCourseAccountByCourseId(courseId);
        if (listStudent != null && listStudent.size() > 0) {
            CourseAccount courseAccount = null;
            for (int i = 0; i < listStudent.size(); i++) {
                courseAccount = listStudent.get(i);
                if (courseAccount != null && courseAccount.getType() != null) {
                    if (courseAccount.getType() == 2) {
                        accountIds.add(courseAccount.getRelationId());
                    }
                    if (courseAccount.getType() == 1) {
                        orgIds.add(courseAccount.getRelationId());
                    }
                }
                visibleRangeExport.setAccountIds(accountIds);
                visibleRangeExport.setOrgIds(orgIds);
            }
        }
        return visibleRangeExport;
    }

    //获取评分和浏览人次
    @Override
    public HomePageCoursePageVo selectMyManageCourseList(Long courseId) {

        return courseMapper.findCourseById(courseId);
    }


    @Override
    public boolean invalidCourse(Long id) {
        try {
            Accredit accredit = accreditMapper.selectById(id);

            Long courseId = accredit.getCourseIdAfter();

            accredit.setValid(0);
            //授权课程失效
            int r = accreditMapper.updateById(accredit);
            log.info("到期的课程授权设置无效：" + r);

            Course course = this.selectById(courseId);
            course.setValid(0);
            // 课程失效
            boolean f = this.updateById(course);
            log.info("到期的课程设置无效课程：" + f);
        } catch (Exception e) {
            log.info("到期失效课程失败：" + e);
        }

        return true;
    }

    /**
     * 火热课程为了优化sql新写的
     *
     * @param relationIds
     * @param companyId
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public List<HomePageCoursePageVo> homePageCourseNewPageList(List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize) {

        List<Long> ids = null;
        if (null != relationIds && relationIds.size() > 0) {
            ids = courseAccountMapper.selectCourseIdByRelationId(relationIds);
        }
        Page<HomePageCoursePageVo> page = new Page<HomePageCoursePageVo>(pageNo, pageSize);
        List<HomePageCoursePageVo> homePageCoursePageVos = this.baseMapper.homePageCourseNewPageList(companyId, siteId, ids, new RowBounds(page.getOffset(), page.getLimit()));
        return homePageCoursePageVos;
    }

    @Override
    public Page<AuthCourseVO> findCourseList(String name, List<Long> classifyIds, Integer shelves, Long companyId, Long siteId, List<Long> orgIds, Long authorizeSiteId, int pageNo, int pageSize) {
        Page<AuthCourseVO> page = new Page<AuthCourseVO>(pageNo, pageSize);
        List<Long> courseIds = new ArrayList<>();
        Accredit accredit = new Accredit();
        accredit.setSiteId(authorizeSiteId);
        List<Accredit> accreditList = accreditMapper.selectList(QueryUtil.condition(accredit));
        for (Accredit a : accreditList) {
            if (!courseIds.contains(a.getCourseIdBefore())) {
                courseIds.add(a.getCourseIdBefore());
            }
        }
        // 分级授权修改
        HQueryUtil.startHQ(Course.class);
        log.info("参数classifyIds："+JSON.toJSON(classifyIds));
        page.setRecords(this.baseMapper.searchAuthorizePage(name, classifyIds, companyId, siteId, orgIds, courseIds, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.searchAuthorizePageCount(name, classifyIds, companyId, siteId, orgIds, courseIds);
        page.setTotal(count);
        return page;
    }

    @Override
    public List<Long> findCourseIdList(Integer pageNo, Integer pageSize) {

        RequestContext context = ContextHolder.get();
        RowBounds rowBounds = new RowBounds(pageNo, pageSize);
        List<Long> courseIds = null;
        if (CollectionUtils.isNotEmpty(context.getRelationIds())) {
            courseIds = courseAccountMapper.selectCourseIdByRelationId(context.getRelationIds());
        }


        List<Long> courseIdList = this.baseMapper.getCourseIdList(context.getCompanyId(), context.getSiteId(), courseIds, rowBounds);
        log.info("返回的courseIdList长度：" + courseIdList.size());
        return courseIdList;
    }

    @Override
    public List<HomePageCoursePageVo> findHomeCourseList(List<Long> courseIds) {
        log.info("courseids 参数:" + courseIds);
        List<HomePageCoursePageVo> homePageCoursePageVos = this.baseMapper.findHomeCourseList(courseIds);
        log.info("courseids 参数:" + homePageCoursePageVos.size());
        log.info("homePageCoursePageVos:" + homePageCoursePageVos);
        return homePageCoursePageVos;
    }

    @Override
    public Page<HomePageCoursePageVo> findCourseList(List<Long> courseIds, Integer pageNo, Integer pageSize) {
        Page<HomePageCoursePageVo> retPage = new Page<>(pageNo, pageSize);
        log.info("courseids:" + courseIds);
        //排序
        PageHelper.startPage(pageNo, pageSize);
        List<HomePageCoursePageVo> homePageCoursePageVos = this.baseMapper.findPcCourseList(courseIds);
        Pagination pagination = PageHelper.getPagination();
        PageHelper.remove();
        retPage.setTotal(pagination.getTotal());
        List<Long> findCourses = homePageCoursePageVos.stream().map(HomePageCoursePageVo::getCourseId)
                .collect(Collectors.toList());

        if (CollectionUtils.isEmpty(findCourses)) {
            return retPage;
        }
        boolean levelFlg = false;
        Map<Long, HomePageCoursePageVo> courseLevelMap = null;
        List<HomePageCoursePageVo> courseLevelList = this.baseMapper.findCourseLevelList(findCourses);
        if (CollectionUtils.isNotEmpty(courseLevelList)) {
            levelFlg = true;
            courseLevelMap = courseLevelList.stream()
                    .collect(Collectors.toMap(key -> key.getCourseId(), val -> val));
        }

        boolean studyNumFlg = false;
        Map<Long, HomePageCoursePageVo> courseStudyNumMap = null;
        List<HomePageCoursePageVo> courseStudyNumList = this.baseMapper.findCourseStudyNumList(findCourses);
        if (CollectionUtils.isNotEmpty(courseStudyNumList)) {
            studyNumFlg = true;
            courseStudyNumMap = courseStudyNumList.stream()
                    .collect(Collectors.toMap(key -> key.getCourseId(), val -> val));
        }

        for (HomePageCoursePageVo item : homePageCoursePageVos) {
            //item.setStudyNum(0);
            item.setLevel(5.0);
            Long courseId = item.getCourseId();
            if (levelFlg && courseLevelMap.containsKey(courseId)) {
                item.setLevel(courseLevelMap.get(courseId).getLevel());
            }
            if (studyNumFlg && courseStudyNumMap.containsKey(courseId)) {
                item.setStudyNum(courseStudyNumMap.get(courseId).getStudyNum());
            }
        }
        retPage.setRecords(homePageCoursePageVos);
        //log.info("course：——+++" + homeCourseList);
        return retPage;
    }

    @Override
    public Page<HomePageCoursePageVo> findCourseListV2(List<Long> courseIds, Integer pageNo, Integer pageSize, Boolean hotDesc, Boolean gradeDesc) {
        Page<HomePageCoursePageVo> retPage = new Page<>(pageNo, pageSize);
        log.info("courseids:" + courseIds);
        //排序
        List<HomePageCoursePageVo> homePageCoursePageVos = new ArrayList<>();
        if (null != hotDesc && hotDesc){
            //人气最高倒叙;点击量倒叙
            PageHelper.startPage(pageNo, pageSize);
            homePageCoursePageVos = this.baseMapper.findPcCourseListByHotDesc(courseIds,hotDesc);
        }else if (null != gradeDesc && gradeDesc){
            //好评最高倒叙
            PageHelper.startPage(pageNo, pageSize);
            homePageCoursePageVos = this.baseMapper.findPcCourseListByGradeDesc(courseIds,gradeDesc);
        }else {
            PageHelper.startPage(pageNo, pageSize);
            homePageCoursePageVos = this.baseMapper.findPcCourseList(courseIds);
        }
        Pagination pagination = PageHelper.getPagination();
        PageHelper.remove();
        retPage.setTotal(pagination.getTotal());
        List<Long> findCourses = homePageCoursePageVos.stream().map(HomePageCoursePageVo::getCourseId)
                .collect(Collectors.toList());

        if (CollectionUtils.isEmpty(findCourses)) {
            return retPage;
        }
        boolean levelFlg = false;
        Map<Long, HomePageCoursePageVo> courseLevelMap = null;
        List<HomePageCoursePageVo> courseLevelList = this.baseMapper.findCourseLevelList(findCourses);
        if (CollectionUtils.isNotEmpty(courseLevelList)) {
            levelFlg = true;
            courseLevelMap = courseLevelList.stream()
                    .collect(Collectors.toMap(key -> key.getCourseId(), val -> val));
        }

        boolean studyNumFlg = false;
        Map<Long, HomePageCoursePageVo> courseStudyNumMap = null;
        List<HomePageCoursePageVo> courseStudyNumList = this.baseMapper.findCourseStudyNumList(findCourses);
        if (CollectionUtils.isNotEmpty(courseStudyNumList)) {
            studyNumFlg = true;
            courseStudyNumMap = courseStudyNumList.stream()
                    .collect(Collectors.toMap(key -> key.getCourseId(), val -> val));
        }

        for (HomePageCoursePageVo item : homePageCoursePageVos) {
            //item.setStudyNum(0);
            item.setLevel(5.0);
            Long courseId = item.getCourseId();
            if (levelFlg && courseLevelMap.containsKey(courseId)) {
                item.setLevel(courseLevelMap.get(courseId).getLevel());
            }
            if (studyNumFlg && courseStudyNumMap.containsKey(courseId)) {
                item.setStudyNum(courseStudyNumMap.get(courseId).getStudyNum());
            }
        }
        retPage.setRecords(homePageCoursePageVos);
        //log.info("course：——+++" + homeCourseList);
        return retPage;
    }

    /**
     * 查询推介课程
     *
     * @param courseIds
     * @param companyId
     * @param siteId
     * @param accountId
     * @return
     */
    @Override
    public List<HomePageCoursePageVo> findRecommendCourseList(List<Long> courseIds, Long companyId, Long siteId, Long accountId) {
        return this.baseMapper.findRecommendCourseList(courseIds, companyId, siteId, accountId);
    }

    /**
     * 查询微信的课程列表数据
     *
     * @param courseIds
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<HomePageCourseVo> findWechatCourseList(List<Long> courseIds, Integer pageNo,
                                                       Integer pageSize, Long siteId) {
        LOGGER.info(" 请求的分页信息：pageNo={},pageSize={}", pageNo, pageSize);
        Page<HomePageCourseVo> retPage = new Page<>(pageNo, pageSize);
        //log.info("courseids:" + courseIds);
        //排序
        PageHelper.startPage(pageNo, pageSize);
        List<HomePageCoursePageVo> homePageCoursePageVos = this.baseMapper.findWechatCourseList(courseIds);
        Pagination pagination = PageHelper.getPagination();
        PageHelper.remove();
        retPage.setTotal(pagination.getTotal());
        List<HomePageCourseVo> data = null;
        if (CollectionUtils.isNotEmpty(homePageCoursePageVos)) {
            Set<Long> cids = new HashSet<>();
            data = new ArrayList<>();
            for (HomePageCoursePageVo item : homePageCoursePageVos) {
                HomePageCourseVo vo = new HomePageCourseVo();
                Long cid = item.getClassifyId();
                vo.setClassifyId(cid);
                vo.setClassifyName(item.getClassifyName());
                data.add(vo);
                cids.add(cid);
            }
            //LOGGER.info("分类列表={}",JSON.toJSONString(data));
            List<ApiListCourseVo> courseData = selectCourseData(courseIds, new ArrayList<>(cids), siteId);

            //List<ApiListCourseVo> courseData = this.baseMapper.findWechatCourseInfoList(courseIds,new ArrayList<>(cids));
            //List<Long> courseIdList = courseData.stream().map(course->course.getId()).collect(Collectors.toList());
            Map<Long, List<ApiListCourseVo>> courseDataMap = courseData.stream().collect(Collectors.groupingBy(key -> key.getClassifyId()));
            //LOGGER.info("最后分组的数据={}",JSON.toJSONString(courseDataMap));
            //List<ApiListCourseVo> listLevel = this.baseMapper.findWechatCourseLevelList(courseIdList);
            //List<ApiListCourseVo> listLevel = selectListLevel(courseIdList);
            //LOGGER.info("查询的结果={}",JSON.toJSONString(listLevel));
            //Map<Long,Double> listLevelMap = listLevel.stream().collect(Collectors.toMap(key->key.getId(),val->val.getLevel()));

            for (HomePageCourseVo voData : data) {
                Long classifyIdG = voData.getClassifyId();
                List<ApiListCourseVo> subData = courseDataMap.get(classifyIdG);
                voData.setApiListCourseVos(subData);
            }
            retPage.setRecords(data);
        }

        //log.info("course：——+++" + homeCourseList);
        return retPage;
    }

    private List<ApiListCourseVo> selectListLevel(List<Long> courseIds) {
        List<ApiListCourseVo> data = new ArrayList<>();
        StringBuilder sql = new StringBuilder("SELECT course_id AS id, ROUND(AVG(`level`), 1) AS levelVal")
                .append(" FROM course_grade")
                .append(" WHERE ");
        try (Connection conn = this.sqlSessionBatch().getConnection();
             Statement pst = conn.createStatement();
        ) {
            String courseIdsStr = courseIds.stream().map(courseid -> courseid.toString())
                    .collect(Collectors.joining(","));
            sql.append(" course_id IN ( ").append(courseIdsStr).append(")");

            sql.append(" GROUP BY course_id")
                    .append("");
            //PreparedStatement pst = conn.prepareStatement(sql.toString());
            LOGGER.info("执行的sql={}", sql.toString());
            ResultSet rs = pst.executeQuery(sql.toString());
            //5.处理ResultSet
            data = new ArrayList<>();
            while (rs.next()) {
                ApiListCourseVo vo = new ApiListCourseVo();
                //rs.get+数据库中对应的类型+(数据库中对应的列别名)
                vo.setId(rs.getLong("id"));
                vo.setLevel(rs.getDouble("levelVal"));
                data.add(vo);
            }
            rs.close();
        } catch (SQLException e) {
            LOGGER.info("执行sql异常{}", e);
        }
        return data;
    }

    private List<ApiListCourseVo> selectCourseData(List<Long> courseIds, List<Long> classifyIds, Long siteId) {
        List<ApiListCourseVo> data = null;
        /*StringBuilder sql = new StringBuilder("SELECT id,classify_id,`name`,`code`,total_duration,`image`,author_unit,valid ");
        sql.append(" FROM ( ")
                .append(" SELECT id,classify_id,`name`,`code`,total_duration,`image`,author_unit,valid, ")
                //需要分组的字段和下面变量字段进行比较
                .append(" @num :=IF( @classifyId = classify_id, @num + 1, 1 ) AS row_number, ")
                //分组赋值给变量
                .append(" @classifyId := classify_id AS dummy ")
                .append(" FROM course WHERE `shelves` = 1 AND valid = 1 AND del_flg = 0 AND classify_id IS NOT NULL ORDER BY classify_id,id desc ")
                .append(" ) AS x ")
                .append(" WHERE x.row_number <= 2");*/


        Connection conn = this.sqlSessionBatch().getConnection();
        try (
                Statement pst = conn.createStatement();
        ) {
            StringBuilder sql = new StringBuilder("SELECT id,classify_id,`name`,`code`,total_duration,`image`,author_unit,valid FROM ( ");
            sql.append(" select id,classify_id,`name`,`code`,total_duration,`image`,author_unit,valid,")
                    .append(" @a:= case when @cid=classify_id then @a+1 else 1 end as num,ifnull(@a,@a:=0),@cid:=classify_id")
                    .append(" from course WHERE `shelves` = 1 AND valid = 1 AND del_flg = 0");
            // 关键过滤字段必须放在分组序列生成之后查询
            if (CollectionUtils.isNotEmpty(courseIds)) {
                String courseIdsStr = courseIds.stream().map(courseid -> courseid.toString())
                        .collect(Collectors.joining(","));
                //sql.append(" AND x.id IN ( ").append(courseIdsStr).append(")");
                sql.append(" AND id IN ( ").append(courseIdsStr).append(")");
            }
            if (CollectionUtils.isNotEmpty(classifyIds)) {
                String classifyIdsStr = classifyIds.stream().map(courseid -> courseid.toString())
                        .collect(Collectors.joining(","));
                //sql.append(" AND x.classify_id IN(  ").append(classifyIdsStr).append(")");
                sql.append(" AND classify_id IN(  ").append(classifyIdsStr).append(")");
            }

            sql.append(" order by classify_id asc, id asc")
                    // 最后筛选
                    .append(") AS x WHERE x.num <= 2");
            //PreparedStatement pst = conn.prepareStatement(sql.toString());
            LOGGER.info("执行的sql={}", sql.toString());
            ResultSet rs = pst.executeQuery(sql.toString());
            //5.处理ResultSet
            data = new ArrayList<>();
            // 获取课时显示配置
            boolean courseDurationShow = false;
            List<FunctionDisplayConfigVo> dataFunction = configApiClients.getConfigBySiteId(siteId);
            LOGGER.info("查询的配置信息={}", JSON.toJSONString(dataFunction));
            if (CollectionUtils.isNotEmpty(dataFunction)) {
                for (FunctionDisplayConfigVo item : dataFunction) {

                    if (FunctionTypeCode.COURSE.getCode()
                            .equals(item.getFunctionType())) {
                        courseDurationShow = item.getShowDisplay();
                    }
                }
 /*               FunctionDisplayConfig config = dataFunction.stream().filter(item -> FunctionTypeCode.COURSE.getCode()
                        .equals(item.getFunctionType())).findAny().get();
                courseDurationShow = config.getShowDisplay();*/
            }

            while (rs.next()) {
                ApiListCourseVo vo = new ApiListCourseVo();
                //rs.get+数据库中对应的类型+(数据库中对应的列别名)
                vo.setId(rs.getLong("id"));
                vo.setName(rs.getString("name"));
                vo.setClassifyId(rs.getLong("classify_id"));
                vo.setImage(rs.getString("image"));
                vo.setAuthorUnit(rs.getString("author_unit"));
                vo.setValid(rs.getInt("valid"));
                vo.setLevel(5.0);
                vo.setTotalDuration(rs.getFloat("total_duration"));
                vo.setDuration(0);
                vo.setCourseDurationShow(courseDurationShow);
                data.add(vo);
            }
            //LOGGER.info("查询课程信息={}",JSON.toJSONString(data));
            // 查询课程星级
            List<Long> courseIdList = data.stream().map(course -> course.getId()).collect(Collectors.toList());
            courseIdList = courseIdList.stream().filter(itemObj -> Objects.nonNull(itemObj)).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(courseIdList)) {
                sql = new StringBuilder("SELECT course_id AS id, ROUND(AVG(`level`), 1) AS levelVal")
                        .append(" FROM course_grade")
                        .append(" WHERE ");
                String courseIdsStr = courseIdList.stream().map(courseid -> courseid.toString())
                        .collect(Collectors.joining(","));
                sql.append(" course_id IN ( ").append(courseIdsStr).append(")");

                sql.append(" GROUP BY course_id")
                        .append("");
                LOGGER.info("执行的sql={}", sql.toString());
                rs = pst.executeQuery(sql.toString());
                while (rs.next()) {
                    long id = rs.getLong("id");
                    Double level = rs.getDouble("levelVal");
                    for (ApiListCourseVo retItem : data) {
                        if (retItem.getId().longValue() == id) {
                            retItem.setLevel(level);
                            break;
                        }
                    }
                }

                // 获取时长
                sql = new StringBuilder("SELECT c.course_id,SUM(m.text_hour*3600+m.text_minute*60+m.text_second) AS duration from(").
                        append(" select material_id,course_id FROM chapter where course_id IN ( ")
                        .append(courseIdsStr).append(")");

                sql.append(" ) c left join material m on c.material_id = m.id GROUP BY c.course_id")
                        .append("");
                LOGGER.info("执行的sql={}", sql.toString());
                rs = pst.executeQuery(sql.toString());
                while (rs.next()) {
                    long id = rs.getLong("course_id");
                    Integer duration = rs.getInt("duration");
                    for (ApiListCourseVo retItem : data) {
                        if (retItem.getId().longValue() == id) {
                            retItem.setDuration(duration);
                            break;
                        }
                    }
                }
            }
            rs.close();
        } catch (SQLException e) {
            LOGGER.info("执行sql异常{}", e);
        }
        return data;
    }

    @Override
    public Integer isCourseFinished(Long courseId, Long accountId, Boolean isTestSensitive) {
        Integer result = -1;
        Set<Long> chapterIds = chapterMapper.getIdsByCourseId(courseId);
        if (CollectionUtils.isNotEmpty(chapterIds)) {
            int finishedChapterCount = recordeMapper.getCountByAccountIdAndChapterIds(accountId, chapterIds);
            // 如果课程通过了，再判断测验是否通过
            if (finishedChapterCount >= chapterIds.size()) {
                if (isTestSensitive) {
                    Boolean testPassed = testCaseService.courseIsSucceed(courseId, accountId);
                    if (testPassed) {
                        result = 1;
                    } else {
                        result = 0;
                    }
                } else {
                    result = 1;
                }
            } else {
                result = 0;
            }
        }
        return result;
    }

    @Override
    public Page<Map<String, Object>> experiencePoolList(String name, List<Long> ids, Integer pageNo, Integer pageSize) {
        RequestContext res = ContextHolder.get();
        Long companyId = res.getCompanyId();
        Long siteId = res.getSiteId();
        List<Long> orgIds = null;
        Page<Map<String, Object>> page = new Page<>(pageNo, pageSize);
        List<Map<String, Object>> list = new ArrayList<>();
//        if (!res.isAdmin() && CollectionUtils.isNotEmpty(res.getOrgIds())) {
//            orgIds = res.getOrgIds();
//        }
        String name1 = null;
        if (StringUtils.isNotBlank(name)) {
            name1 = name.trim();
        }
        HQueryUtil.startHQ(Course.class);
        list = courseMapper.experiencePoolList(name1, ids, companyId, siteId, orgIds, page);
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public List<ModuleAllVoCourse> listCoursePCgrade(List<Long> ids) {
        // TODO Auto-generated method stub
        List<ModuleAllVoCourse> list = courseMapper.listCoursePCgrade(ids);
        if (CollectionUtils.isNotEmpty(list)) {
            for (ModuleAllVoCourse moduleAllVoCourse : list) {
                Long cId = moduleAllVoCourse.getId();
                Integer courseNum = recordeMapper.getCourseNumByCourseId(cId);
                moduleAllVoCourse.setCourseNum(courseNum == null ? 0 : courseNum);
            }
        }
        return list;
    }

    @Override
    public Page<Course> listNotIds(Long siteId, String name, List<Long> ids, Integer pageNo, Integer pageSize) {
        // TODO Auto-generated method stub
        Page<Course> page = new Page<Course>(pageNo, pageSize);
        Course course = new Course();
        course.setSiteId(siteId);
        course.setShelves(1);
        EntityWrapper<Course> wrapper = new EntityWrapper<Course>(course);
//        wrapper.and()
        if (CollectionUtils.isNotEmpty(ids)) {
            wrapper.notIn("id", ids);
        }
        if (StringUtils.isNotEmpty(name)) {
            wrapper.like("name", name);
        }
        wrapper.orderBy("create_time", false);
        return this.selectPage(page, wrapper);
    }

    @Override
    public List<ModuleAllVoCourse> listCourseRelationId(List<Long> relationIds, Integer num, List<Long> listIds) {
        Long siteId = ContextHolder.get().getSiteId();
        List<Long> listCourseIds = courseMapper.queryCourseByRelationId(relationIds, siteId, num, listIds);
        List<ModuleAllVoCourse> listModuleAllVoCourse = new ArrayList<ModuleAllVoCourse>();
        if (CollectionUtils.isNotEmpty(listCourseIds)) {
            List<ModuleAllVoCourse> list = courseMapper.listCoursePCgrade(listCourseIds);
            if (CollectionUtils.isNotEmpty(list)) {
                for (ModuleAllVoCourse moduleAllVoCourse : list) {
                    Long cId = moduleAllVoCourse.getId();
                    Integer courseNum = recordeMapper.getCourseNumByCourseId(cId);
                    moduleAllVoCourse.setCourseNum(courseNum == null ? 0 : courseNum);
                    listModuleAllVoCourse.add(moduleAllVoCourse);
                }
            }
        }
        return listModuleAllVoCourse;
    }

    @Override
    public List<Map<String, Object>> getServerByCompanyIdAndIds(Long companyId, List<Long> ids) {
        // TODO Auto-generated method stub
        List<Map<String, Object>> listMap = null;
        //查询
        Course course = new Course();
        course.setCompanyId(companyId);
        EntityWrapper<Course> wrapper = new EntityWrapper<Course>(course);
        if (CollectionUtils.isNotEmpty(ids)) {
            wrapper.in("id", ids);
        }
        List<Course> listCourse = this.selectList(wrapper);
        //循环组装到输出对象
        Map<String, Object> map = null;
        if (CollectionUtils.isNotEmpty(listCourse)) {
            listMap = new ArrayList<Map<String, Object>>();
            for (Course c : listCourse) {
                map = new HashMap<String, Object>();
                map.put("catalog", 0);
                map.put("id", c.getId());
                map.put("name", c.getName());
                map.put("logo_url", c.getImage());
                listMap.add(map);
            }
        }
        return listMap;
    }

    @Override
    public Map<String, Object> getCourseTimeMessage(Long companyId, List<Long> ids) {
        // TODO Auto-generated method stub
        List<Material> listMaterial = materialMapper.getCourseTimeMessage(companyId, ids);
        Map<String, Object> map = null;
        if (CollectionUtils.isNotEmpty(listMaterial)) {
            map = new HashMap<String, Object>(listMaterial.size());//map大小为数组的大小
            for (Material material : listMaterial) {
                Integer sumSecond = material.getTextHour() * 3600 + material.getTextMinute() * 60 + material.getTextSecond();
                double sumMinuteDouble = sumSecond / 60.0;
                String sumMinute = String.format("%.1f", sumMinuteDouble);//   %.1f表示保留后1位，能四舍五入。
                map.put(material.getId().toString(), sumMinute);
            }
        }
        return map;
    }

    /**
     * 查询我的课程（进行中和已完成的课程Id） 进行中和完成的章节列表（按课程id分类）
     *
     * @param siteId
     * @param accountId
     * @return
     */
    @Override
    public Map<String, Map<Long, List<Chapter>>> getMyStudyCourseIds(Long siteId, Long accountId) {
        Map<String, Map<Long, List<Chapter>>> data = new HashMap<>();

        // 查询学习的课程id列表
        List<Long> studyCourseId = this.baseMapper.getStudyCourseId(siteId, accountId);
        if (CollectionUtils.isEmpty(studyCourseId)) {
            return null;
        }
        Collections.sort(studyCourseId);
        List<Chapter> stduyChapterList = chapterMapper.selectStudyCourseChapter(studyCourseId, accountId, siteId);
        // 学习中的课程
        Map<Long, List<Chapter>> studyCourseGroup = stduyChapterList.stream().collect(Collectors.groupingBy(key -> key.getCourseId()));
        Map<Long, List<Chapter>> studying = new HashMap<>();
        // 完成的课程
        Map<Long, List<Chapter>> complete = new HashMap<>();

        for (Map.Entry<Long, List<Chapter>> item : studyCourseGroup.entrySet()) {
            Long studyCourseIdTem = item.getKey();
            List<Chapter> studyItem = item.getValue();
            long studyingChapter = studyItem.stream().filter(studyChapterList -> Objects.isNull(studyChapterList.getOvered())).count();
            if (studyingChapter > 0) {
                studying.put(studyCourseIdTem, studyItem);
            } else {
                complete.put(studyCourseIdTem, studyItem);
            }
        }
        data.put(CoursePcClient.COURSE_STATUS_STUDYING, studying);
        data.put(CoursePcClient.COURSE_STATUS_COMPLETE, complete);
        return data;
    }

    @Override
    public Map<String, List<CourseStudentStatus>> getMyStudyCourseStatus(Long siteId, Long accountId, Date startDate, Date endDate) {
        Map<String, List<CourseStudentStatus>> data = new HashMap<>();

        //CourseStudentStatus studentStatusEw = new CourseStudentStatus();
        //studentStatusEw.setSiteId(siteId);
        //studentStatusEw.setAccountId(accountId);
        //studentStatusEw.setStudentStatus(0);
        // 查询学习中的课程
        //List<CourseStudentStatus> courseStatus = studentStatusService.selectList(QueryUtil.condition(studentStatusEw));
        List<CourseStudentStatus> courseStatus = studentStatusService.findValidStudyCourse(siteId, accountId, startDate, endDate);
        if (CollectionUtils.isNotEmpty(courseStatus)) {
            Map<Integer, List<CourseStudentStatus>> gData = courseStatus.stream().parallel().collect(Collectors.groupingBy(key -> key.getStudentStatus()));
            data.put(CoursePcClient.COURSE_STATUS_STUDYING, gData.get(Integer.valueOf(0)));
            data.put(CoursePcClient.COURSE_STATUS_COMPLETE, gData.get(Integer.valueOf(1)));
        }

        return data;
    }

    /**
     * 根据课程id查询课程基本信息
     *
     * @param courseIds
     * @return
     */
    @Override
    public Page<MyCourseRecordeVo> findCourseBaseInfoByIds(Set<Long> courseIds, Long siteId,
                                                           Long accountId,
                                                           Integer pageNo, Integer pageSize) {
        Page<MyCourseRecordeVo> pageData = new Page<>();
        pageData.setCurrent(pageNo);
        pageData.setSize(pageSize);

        // 排序
        List<Long> courseIdList = new ArrayList<>(courseIds);
        Collections.sort(courseIdList);

        PageHelper.startPage(pageNo, pageSize);
        List<MyCourseRecordeVo> data = recordeMapper.findCourseBaseInfoByIds(courseIdList, siteId, accountId);
        Pagination pagination = PageHelper.getPagination();
        PageHelper.remove();
        pageData.setTotal(pagination.getTotal());
        pageData.setRecords(data);
        return pageData;
    }

    /**
     * 查询课程评级
     *
     * @param courseIds
     * @return
     */
    @Override
    public List<ApiListCourseVo> findWechatCourseLevelList(List<Long> courseIds) {
        return this.baseMapper.findWechatCourseLevelList(courseIds);
    }

    @Override
    public List<HomePageCoursePageVo> findCourseIdList(List<Long> courseIds) {
        RequestContext context = ContextHolder.get();
        List<Long> ids = null;
        if (CollectionUtils.isNotEmpty(context.getRelationIds())) {
            ids = courseAccountMapper.selectCourseIdByRelationId(context.getRelationIds());
        }
        courseIds.retainAll(ids);

        List<HomePageCoursePageVo> homePageCoursePageVos = new ArrayList<>();
        if (courseIds.size() > 7) {
            homePageCoursePageVos = this.baseMapper.findHomeCourseList(courseIds.subList(0, 8));
        } else {
            homePageCoursePageVos = this.baseMapper.findHomeCourseList(courseIds);
        }
        if (CollectionUtils.isNotEmpty(homePageCoursePageVos)) {
            homePageCoursePageVos.parallelStream().map(hc -> {
                Double level = hc.getLevel();
                if (level == null) {
                    hc.setLevel(0D);
                }
                return hc;
            }).collect(Collectors.toList());
        }


        return homePageCoursePageVos;
    }


    @Override
    public List<CourseListExportVO> listExport(String name, Long classifyId, Integer shelves, Long companyId,
                                               Long siteId, List<Long> orgIds, Integer type, Integer courseType) {

        List<CourseListExportVO> list = courseMapper.listExport(name, classifyId, shelves, companyId, siteId, orgIds, null, type, courseType);
        if (CollectionUtils.isNotEmpty(list)) {
            for (CourseListExportVO course : list) {
                String firstName = "";
                String secondName = "";
                String thirdName = "";
                String courseName = "";
                if (course.getClassifyId() != null && course.getClassifyId() != 0) {
                    Classify classify1 = classifyMapper.selectById(course.getClassifyId());
                    if (null != classify1) {
                        thirdName = classify1.getName();
                        if (classify1.getParentId() != null && classify1.getParentId() != 0) {
                            Classify classify2 = classifyMapper.selectById(classify1.getParentId());
                            if (null != classify2) {
                                secondName = classify2.getName();
                                if (classify2.getParentId() != null && classify2.getParentId() != 0) {
                                    Classify classify3 = classifyMapper.selectById(classify2.getParentId());
                                    if (null != classify3) {
                                        firstName = classify3.getName();
                                        courseName = firstName + "/" + secondName + "/" + thirdName;
                                    } else {
                                        courseName = secondName + "/" + thirdName;
                                    }
                                } else {
                                    courseName = secondName + "/" + thirdName;
                                }
                            } else {
                                courseName = thirdName;
                            }
                        } else {
                            courseName = thirdName;
                        }
                    } else {
                        courseName = firstName;
                    }
                } else {
                    courseName = firstName;
                }
                Integer score = chapterMapper.getScoreByCourseId(course.getId());
                course.setClassify(courseName);
                course.setScore(score);
                // 获取课程总学习时长 =课程下所有素材时长的和  单位是秒
                Integer totalSeconds = getCourseMaterialDurationsSeconds(course.getId(),companyId,siteId);
                String totalStudyHourStr = MaterialDateUtil.transformSecondsToHms(totalSeconds);
                course.setTotalHourString(totalStudyHourStr);
            }
        }
        return list;
    }

    /**
     * 获取课程的总学习时长(=课程下素材总时长)
     * @param courseId 课程id
     * @return 课程总时长；单位是秒
     */
    @Override
    public Integer getCourseMaterialDurationsSeconds(Long courseId,Long companyId,Long siteId) {

        if (null == courseId || courseId.compareTo(0L) < 0){
            return 0;
        }
        Course course = courseMapper.selectById(courseId);
        if (null == course){
            log.error("获取课程总学习时长;根据课程id未查询到课程信息;courseId = {}",courseId);
            return 0;
        }
        // 获取课程下所有素材
        List<Long> materialIds = chapterMapper.selectMaterialIdsByCourse(courseId);
        if (org.springframework.util.CollectionUtils.isEmpty(materialIds)){
            return 0;
        }
        List<Material> materialList = materialMapper.selectMaterialsByIds(materialIds,companyId,siteId);
        // 遍历素材；获取素材时长 时分秒；并转换为秒 多个时长相加总和
        if (org.springframework.util.CollectionUtils.isEmpty(materialList)){
            return 0;
        }
        AtomicInteger totalSecond = new AtomicInteger(0);
        materialList.parallelStream().forEach(material -> {
            Integer textHour = material.getTextHour();
            if (null != textHour && textHour.compareTo(0) > 0){
                totalSecond.addAndGet(textHour *60*60);
            }
            Integer textMinute = material.getTextMinute();
            if (null != textMinute && textMinute.compareTo(0) > 0){
                totalSecond.addAndGet(textMinute*60);
            }
            Integer textSecond = material.getTextSecond();
            if (null != textSecond && textSecond.compareTo(0) > 0){
                totalSecond.addAndGet(textSecond);
            }
        });
        return totalSecond.intValue();
    }

    @Override
    public Map<Long, String> getCourseLogos(List<Long> courseIds) {

        Map<Long, String> map = new HashMap<>();
        List<CourseLogoVO> list = courseMapper.getCourseLogos(courseIds);
        if (CollectionUtils.isNotEmpty(list)) {
            for (CourseLogoVO vo : list) {
                map.put(vo.getCourseId(), vo.getCourseLogoUrl());
            }
        }
        return map;
    }

    @Override
    public Map<Long, String> getCourseCodeName(List<Long> courseIds) {
        Map<Long, String> codeMap = null;
        if (CollectionUtils.isNotEmpty(courseIds)) {
            codeMap = new HashMap<>(courseIds.size());

            EntityWrapper<Course> wrapper = new EntityWrapper();
            wrapper.in("id", courseIds);
            wrapper.eq("del_flg", 0);
            wrapper.setSqlSelect("id", "code");
            List<Course> list = courseMapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(list)) {
                for (Course c : list) {
                    if (!codeMap.containsKey(c.getId())) {
                        codeMap.put(c.getId(), c.getCode());
                    }
                }
            }
        }
        return codeMap;
    }

    @Override
    public Page<Course> getPageToCalendar(Date date, Page page) {
        RequestContext context = ContextHolder.get();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();
        //获取该时间参数的课程ids
        List<Long> ids = courseMapper.getIdsByDate(date, siteId);
        if (CollectionUtils.isEmpty(ids)) {
            return page;
        }

        //根据可见范围获取课程ids
        List<Long> courseIdsFromVisibRange = courseAccountMapper.getUsefulIds(ids, context.getRelationIds(), siteId);
        //获取已完成的课程IDS

        List<Long> finishCourseIds = courseStudentStatusMapper.getCourseIds(siteId, accountId, ids);

        page.setRecords(courseMapper.getPageToCalendar(finishCourseIds, courseIdsFromVisibRange, date, siteId, page));
        page.setTotal(courseMapper.getPageToCalendarNum(finishCourseIds, courseIdsFromVisibRange, date, siteId));
        return page;

    }

    @Override
    public Page<JaxinRankingVO> getRankings(Long courseId, Integer pageNo, Integer pageSize) {

        Page<JaxinRankingVO> page = new Page<>(pageNo, pageSize);
        List<JaxinRankingVO> list = courseMapper.getRankings(courseId, page);
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page) {

        if (StringUtils.isBlank(field)) {
            LOGGER.info("列名不能为空！");
            return page;
        }
        if (field.equalsIgnoreCase(TaskParamsEnums.CLASSIFY.getCode())) {
            field = "name";
            page = classifyService.getClassifyNameByDrools(field, value, page);
        } else {
            if (field.equalsIgnoreCase(TaskParamsEnums.NAME.getCode()) || field.equalsIgnoreCase(TaskParamsEnums.CODE.getCode())) {
                return getPage(field, value, page);
            } else if (field.equalsIgnoreCase(TaskParamsEnums.KEYWORD.getCode())) {
                field = "tags";
                return getPage(field, value, page);
            }
        }
        return page;
    }

    public Page getPage(String field, String value, Page<DroolsVo> page) {
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();
        String upperField = ClassUtil.getFieldName(field);

        Course course = new Course();
        course.setSiteId(siteId);
        course.setCompanyId(companyId);
        course.setDelFlg(0);
        course.setShelves(1);
        EntityWrapper wrapper = new EntityWrapper(course);
        wrapper.setSqlSelect("distinct(" + field + "),"+"id ")
                .isNotNull(field)
                .like(field, value)
                .addFilter(field+"!=''")
                .orderBy("create_time", false);

        List<DroolsVo> voList = null;
        List<Course> list = this.courseMapper.selectPage(page, wrapper);
        if (!CollectionUtils.isEmpty(list)) {
            voList = new ArrayList<>(list.size());

            for (Course a : list) {
                DroolsVo vo = new DroolsVo();
                vo.setTaskId(a.getId());
                vo.setTaskFieldValue(ClassUtil.invokeMethod(a, upperField));
                vo.setTaskParamsType(field);
                voList.add(vo);
            }
            page.setRecords(voList);
        }
        return page;
    }
}
