package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.plugins.pagination.PageHelper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.course.application.domain.CourseShare;
import com.yizhi.course.application.domain.WechatInfo;
import com.yizhi.course.application.mapper.CourseMapperShareMapper;
import com.yizhi.course.application.service.ICourseShareService;
import com.yizhi.course.application.service.IWechatInfoService;
import com.yizhi.course.application.vo.CourseForwardUserVo;
import com.yizhi.course.application.vo.CourseShareVo;
import com.yizhi.util.application.page.PageInfo;
import com.yizhi.util.application.page.PageParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2018-04-09
 */
@Service
public class CourseShareServiceImpl extends ServiceImpl<CourseMapperShareMapper, CourseShare> implements ICourseShareService {

    private Logger LOGGER= LoggerFactory.getLogger(CourseShareServiceImpl.class);

    @Autowired
    private CourseMapperShareMapper courseShareMapper;

    @Autowired
    private IWechatInfoService wechatInfoService;

    @Override
    public PageInfo<CourseShareVo> searchMyShareInfo(Long accountId, PageParam<CourseShareVo> pageParam, String searchFiled) {
        int currentPage = 1;
        int pageSize = 10;
        if(pageParam != null && pageParam.getPageNo() != null && pageParam.getPageNo()>0){
            currentPage = pageParam.getPageNo();
        }
        if(pageParam != null && pageParam.getPageSize() != null && pageParam.getPageSize()>0){
            pageSize = pageParam.getPageSize();
        }

        if(StringUtils.trimToEmpty(searchFiled).equals(StringUtils.EMPTY)){
            searchFiled = null;
        }else {
            searchFiled = "%"+searchFiled+"%";
        }
        Long siteId = ContextHolder.get().getSiteId();
        if(siteId == null || siteId==0L){
            siteId = -1L;
        }
        PageInfo<CourseShareVo> pageInfo = new PageInfo<>();

        PageHelper.startPage(currentPage, pageSize);
        List<CourseShareVo> cvList = courseShareMapper.searchMyShareInfo(accountId,searchFiled,siteId );
        if(!CollectionUtils.isEmpty(cvList)){
            pageInfo.setPageNo(PageHelper.getPagination().getCurrent());//当前页
            pageInfo.setPageSize(PageHelper.getPagination().getSize());//每页条数
            pageInfo.setPageTotal(PageHelper.getPagination().getPages());//总页数
            pageInfo.setPageRecords(PageHelper.getTotal());//总条数
        }
        PageHelper.remove();


        if(!CollectionUtils.isEmpty(cvList)){
            LOGGER.info("cvList查询课程分享信息 = "+JSON.toJSONString(cvList));
            List<Long> courseIds = cvList.stream().map(CourseShareVo::getCourseId).collect(Collectors.toList());
            List<CourseShareVo> data = courseShareMapper.searchMyUserInfo(accountId, courseIds);
            if(!CollectionUtils.isEmpty(data)){
                LOGGER.info("查询课程分享信息，分组可从分享查看人数统计 = "+JSON.toJSONString(data));
                Map<Long , Integer> mapData = data.stream().collect(Collectors.
                        toMap(key -> key.getCourseId(), mapKey->mapKey.getReadUserCount()));
                for(CourseShareVo itemCourseVo : cvList){
                    if(mapData.containsKey(itemCourseVo.getCourseId())){
                        itemCourseVo.setReadUserCount(mapData.get(itemCourseVo.getCourseId()));
                    }
                }
            }
        }
        pageInfo.setRecords(cvList);
        return pageInfo;
    }

    @Override
    public List<CourseForwardUserVo> courseShareCount(Long conrseId, String username) {
        if(StringUtils.trimToEmpty(username).equals(StringUtils.EMPTY)){
            username = null;
        }
        return courseShareMapper.courseShareCount(conrseId, username);
    }

    /**
     * 统计我的分享课程数
     *
     * @param siteId
     * @param accountId
     * @return
     */
    @Override
    public Integer countMyShareCourseNum(Long siteId, Long accountId) {
        return this.baseMapper.countMyShareCourseNum(siteId,accountId);
    }


    /**
     * 查询我分享的推广信息
     *
     * @param accountId
     * @param courseId
     * @param pageNo
     * @param pageSize
     * @param firstShareId
     * @return
     */
    @Override
    public PageInfo<CourseShareVo> findShareSpreadList(Long accountId, Long courseId, Long siteId,
                                                       int pageNo,
                                                       Integer pageSize,
                                                       String firstShareId) {
        LOGGER.info("findShareSpreadList 处理凯撒 accountId={}， courseId={}, siteId={}, firstShareId={}",accountId,courseId,siteId,firstShareId);
        int currentPage = 1;

        if(pageNo>0){
            currentPage = pageNo;
        }
        if(pageSize == null || pageSize < 10){
            pageSize = 10;
        }
        if(StringUtils.isBlank(firstShareId) || firstShareId.trim().equals(StringUtils.EMPTY)){
            firstShareId = null;
        }
        PageInfo<CourseShareVo> pageInfo = new PageInfo<>();
        PageHelper.startPage(currentPage, pageSize);
        List<CourseShareVo> cvList = courseShareMapper.findShareSpreadList(accountId,courseId,siteId,firstShareId);
        LOGGER.info("查询");
        if(!CollectionUtils.isEmpty(cvList)){
            pageInfo.setPageNo(PageHelper.getPagination().getCurrent());//当前页
            pageInfo.setPageSize(PageHelper.getPagination().getSize());//每页条数
            pageInfo.setPageTotal(PageHelper.getPagination().getPages());//总页数
            pageInfo.setPageRecords(PageHelper.getTotal());//总条数
        }
        PageHelper.remove();


        if(!CollectionUtils.isEmpty(cvList)){
            List<WechatInfo> wechatList = null;
            Set<String> openIds = null;
            if(firstShareId == null){
                // 获取第一级的分享用户信息
                openIds = cvList.stream().map(CourseShareVo::getFirstShareId).collect(Collectors.toSet());
                LOGGER.info("accountId={},courseId={},siteId={},openIds={}",accountId,courseId,siteId,JSON.toJSONString(openIds));
                List<CourseShareVo> nextList = this.baseMapper.findShareSpreadNextList(
                        accountId,courseId,siteId, new ArrayList<>(openIds));
                if(Objects.nonNull(nextList)){
                    Map<String, CourseShareVo> dataNextMap = nextList.stream().collect(Collectors.
                            toMap(CourseShareVo::getFirstShareId,val->val));
                    for(CourseShareVo itemFirst : cvList){
                        String firstShareIdTmp = itemFirst.getFirstShareId();
                        if(dataNextMap.containsKey(firstShareIdTmp)){
                            CourseShareVo courseShareVo = dataNextMap.get(firstShareIdTmp);

                            String hasNext = courseShareVo.getSecondShareId();
                            if(Objects.nonNull(hasNext) && hasNext.length()>0){
                                itemFirst.setShareUserCount(courseShareVo.getShareUserCount()-1);
                                itemFirst.setHasNext(true);
                            }
                        }
                    }
                }

            }else {
                openIds = cvList.stream().map(CourseShareVo::getSecondShareId).collect(Collectors.toSet());
            }
            wechatList = wechatInfoService.selectBatchIds(openIds);
            if(!CollectionUtils.isEmpty(wechatList)){
                Map<String, WechatInfo> firstMap = wechatList.parallelStream().collect(Collectors.toMap(firstKey -> firstKey.getOpenId(), firstVal -> firstVal ));
                for(CourseShareVo itemFirst : cvList){
                    if(firstShareId == null && firstMap.containsKey(itemFirst.getFirstShareId())){
                        WechatInfo firstInfo = firstMap.get(itemFirst.getFirstShareId());
                        itemFirst.setName(firstInfo.getNikeName());
                        itemFirst.setAvatar(firstInfo.getAvatar());
                        itemFirst.setTel(firstInfo.getTel());
                    }else if (firstShareId != null && firstMap.containsKey(itemFirst.getSecondShareId())){
                        WechatInfo firstInfo = firstMap.get(itemFirst.getSecondShareId());
                        itemFirst.setName(firstInfo.getNikeName());
                        itemFirst.setTel(firstInfo.getTel());
                        itemFirst.setAvatar(firstInfo.getAvatar());
                    }
                }
            }
            pageInfo.setRecords(cvList);
        }
        return pageInfo;
    }
}
