package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.course.application.domain.CourseStudyHour;
import com.yizhi.course.application.mapper.CourseStudyHourMapper;
import com.yizhi.course.application.service.ICourseStudyHourService;
import com.yizhi.course.application.vo.MyDurationVO;
import com.yizhi.course.application.vo.StudyHourReportVO;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.HQAccountInManageParam;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 课程学时发放明细 服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2019-05-14
 */
@Service
public class CourseStudyHourServiceImpl extends ServiceImpl<CourseStudyHourMapper, CourseStudyHour> implements ICourseStudyHourService {


    @Autowired
    private ReportClient reportClient;

    @Autowired
    private CourseStudyHourMapper courseStudyHourMapper;

    @Autowired
    private OrganizationClient organizationClient;


    /**
     * 根据站点id或者（用户id，课程id）统计学时
     *
     * @param siteId
     * @param accountId
     * @param courseId
     * @return
     */
    @Override
    public Float studyHourSum(Long siteId, Long accountId, Long courseId) {
        return this.baseMapper.studyHourSum(siteId, accountId, courseId);
    }

    /**
     * 学时统计获取列表(加入分级授权)
     *
     * @param companyId
     * @param siteId
     * @param isAdmin
     * @param accountId
     * @param userName
     * @param orgNameLike
     * @param pageNo
     * @param pageSize
     * @return
     * @author ding
     */
    @Override
    public Page<StudyHourReportVO> getStudyHourList(Long companyId, Long siteId, boolean isAdmin, Long accountId,
                                                    String userName, String orgNameLike, Integer pageNo, Integer pageSize) {
        Page<StudyHourReportVO> page = new Page<>(pageNo, pageSize);
        List<Long> accountIds = null;
        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>();
        if (isAdmin) {
            ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
            reqVO.setPage(page);
            if (StringUtils.isNotBlank(userName)) {
                reqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reqVO.setOrgName(orgNameLike.trim());
            }
            reqVO.setSiteId(siteId);

            Page<ReportAccountRespVO> accountsPage = reportClient.getRangeAccountsPage(reqVO);
            if (null != accountsPage && CollectionUtils.isNotEmpty(accountsPage.getRecords())) {
                page.setTotal(accountsPage.getTotal());
                accountIds = new ArrayList<>();
                for (ReportAccountRespVO respVO : accountsPage.getRecords()) {
                    accountIds.add(respVO.getUserId());
                    accountMap.put(respVO.getUserId(), respVO);
                }
            }
        } else {
            HQAccountInManageParam reqVO = new HQAccountInManageParam();
            if (StringUtils.isNotBlank(userName)) {
                reqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reqVO.setOrgName(orgNameLike.trim());
            }
            reqVO.setPage(page);
            reqVO.setAccountId(accountId);
            reqVO.setSiteId(siteId);
            reqVO.setModuleType(AuthzConstant.moduleType.statistics_course);
            Page<ReportAccountRespVO> accountsPage = reportClient.getRangeAccountsInManagePage(reqVO);
            if (null != accountsPage && CollectionUtils.isNotEmpty(accountsPage.getRecords())) {
                page.setTotal(accountsPage.getTotal());
                accountIds = new ArrayList<>();
                for (ReportAccountRespVO respVO : accountsPage.getRecords()) {
                    accountIds.add(respVO.getUserId());
                    accountMap.put(respVO.getUserId(), respVO);
                }
            }
        }
        if (CollectionUtils.isEmpty(accountIds)) {
            return page;
        } else {
            List<StudyHourReportVO> list = courseStudyHourMapper.getStudyHourList(companyId, siteId, accountIds);
            List<StudyHourReportVO> resultList = new ArrayList<>();
            Integer index = null;
            if (pageSize != Integer.MAX_VALUE) {
                index = (pageNo - 1) * pageSize;
            }
            for (Long id : accountIds) {
                StudyHourReportVO studyHourReportVO = new StudyHourReportVO();
                Float studyHour = null;
                ReportAccountRespVO account = accountMap.get(id);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (StudyHourReportVO studyHourReportVO1 : list) {
                        if (studyHourReportVO1.getAccountId().equals(id)) {
                            studyHour = studyHourReportVO1.getStudyHour();
                        }
                    }
                }
                if (null == studyHour) {
                    studyHour = Float.valueOf(0);
                }
                if (null == account) {
                    return page;
                }
                if (null != index) {
                    studyHourReportVO.setIndex(accountIds.indexOf(id) + 1 + index);
                }
                studyHourReportVO.setUserName(account.getUserName());
                studyHourReportVO.setFullName(account.getUserFullName());
                studyHourReportVO.setOrgId(account.getOrgId());
                studyHourReportVO.setOrgName(account.getOrgName());
                studyHourReportVO.setAccountState(account.getStatus() == 1 ? "启用" : "停用");
                studyHourReportVO.setStudyHour(studyHour);
                resultList.add(studyHourReportVO);
            }
            if (CollectionUtils.isNotEmpty(resultList)) {
                page.setRecords(resultList);
            }

            return page;
        }
    }

    @Override
    public List<StudyHourReportVO> studyHourDetailsDownload(Long companyId, Long siteId, boolean isAdmin, Long accountId, String userName, String orgNameLike) {
        List<Long> accountIds = null;
        Page<StudyHourReportVO> page = new Page<>(1, Integer.MAX_VALUE);
        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>();
        List<StudyHourReportVO> resultList = new ArrayList<>();
        if (isAdmin) {
            ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
            reqVO.setPage(page);
            if (StringUtils.isNotBlank(userName)) {
                reqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reqVO.setOrgName(orgNameLike.trim());
            }
            reqVO.setSiteId(siteId);

            Page<ReportAccountRespVO> accountsPage = reportClient.getRangeAccountsPage(reqVO);
            if (null != accountsPage && CollectionUtils.isNotEmpty(accountsPage.getRecords())) {
                accountIds = new ArrayList<>();
                for (ReportAccountRespVO respVO : accountsPage.getRecords()) {
                    accountIds.add(respVO.getUserId());
                    accountMap.put(respVO.getUserId(), respVO);
                }
            }
        } else {
            HQAccountInManageParam reqVO = new HQAccountInManageParam();
            if (StringUtils.isNotBlank(userName)) {
                reqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reqVO.setOrgName(orgNameLike.trim());
            }
            reqVO.setPage(page);
            reqVO.setAccountId(accountId);
            reqVO.setSiteId(siteId);
            reqVO.setModuleType(AuthzConstant.moduleType.statistics_course);
            Page<ReportAccountRespVO> accountsPage = reportClient.getRangeAccountsInManagePage(reqVO);
            if (null != accountsPage && CollectionUtils.isNotEmpty(accountsPage.getRecords())) {
                accountIds = new ArrayList<>();
                for (ReportAccountRespVO respVO : accountsPage.getRecords()) {
                    accountIds.add(respVO.getUserId());
                    accountMap.put(respVO.getUserId(), respVO);
                }
            }
        }
        List<StudyHourReportVO> list = null;
        if (CollectionUtils.isNotEmpty(accountIds)) {
            list = courseStudyHourMapper.getStudyHourDetails(companyId, siteId, accountIds);
        } else {
            return null;
        }
        Map<Long, String> orgNamesMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list)) {
            for (Long id : accountIds) {
                ReportAccountRespVO account = accountMap.get(id);
                for (StudyHourReportVO studyHourReportVO : list) {
                    if (studyHourReportVO.getAccountId().equals(id)) {
                        studyHourReportVO.setUserName(account.getUserName());
                        studyHourReportVO.setFullName(account.getUserFullName());
                        studyHourReportVO.setOrgName(account.getOrgName());
                        if (orgNamesMap.get(account.getOrgId()) != null) {
                            studyHourReportVO.setOrgNames(orgNamesMap.get(account.getOrgId()));
                        } else {
                            String orgNames = "";
                            List<String> orgNameList = organizationClient.getChiefOrgNames(account.getOrgId(), companyId);
                            if (CollectionUtils.isNotEmpty(orgNameList)) {
                                for (String s : orgNameList) {
                                    if (StringUtils.isBlank(orgNames)) {
                                        orgNames = orgNames + s;
                                    } else {
                                        orgNames = orgNames + "/" + s;
                                    }
                                }
                                studyHourReportVO.setOrgNames(orgNames);
                                orgNamesMap.put(account.getOrgId(), orgNames);
                            } else {
                                studyHourReportVO.setOrgNames("--");
                                orgNamesMap.put(account.getOrgId(), "--");
                            }
                        }

                        if (account.getStatus() == 1) {
                            studyHourReportVO.setAccountState("启用");
                        } else {
                            studyHourReportVO.setAccountState("停用");
                        }

                        resultList.add(studyHourReportVO);
                    }
                }
            }
            return resultList;
        }

        return null;
    }

    @Override
    public Page<MyDurationVO> queryDurationByAccountId(Long accountId, Long siteId, Integer pageNo, Integer pageSize) {
        Page<MyDurationVO> page = new Page(pageNo, pageSize);
        SimpleDateFormat monthFormat = new SimpleDateFormat("MM");
        SimpleDateFormat yearFormat = new SimpleDateFormat("yyyy");
        List<MyDurationVO> list = this.courseStudyHourMapper.queryDurationByAccountId(accountId, siteId, page);
        if (CollectionUtils.isNotEmpty(list)) {

            for (int i = 0; i < list.size(); i++) {

                int index = list.size() - i - 1;

                Float currentDuration = 0.0f;
                currentDuration = list.get(index).getCurrentDuration();
                currentDuration = (currentDuration == null ? 0.0f : currentDuration);
                list.get(index).setCurrentDuration(currentDuration);
                String m = monthFormat.format(list.get(index).getCreateTime());
                Integer month = Integer.valueOf(m);
                if (month > 0 && month < 10) {
                    m = m.substring(1);
                }
                list.get(index).setMonth(m);

                String year = yearFormat.format(list.get(index).getCreateTime());
                list.get(index).setYear(year);
            }
        }

        page.setTotal(this.courseStudyHourMapper.queryDurationByAccountIdNum(accountId, siteId));
        page.setRecords(list);
        return page;
    }

    /**
     * 保留一位小数
     *
     * @param f
     * @return
     */
    public static String strOnePointOfFloat(Float f) {
        f = (f == null ? 0.0f : f);
        return String.format("%.1f", Double.valueOf(String.valueOf(f)));
    }
}
