package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.domain.dto.MaterialExportDto;
import com.yizhi.course.application.mapper.*;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.vo.*;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 素材库 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Log4j2
@Service
@Transactional(rollbackFor = Exception.class)
public class MaterialServiceImpl extends ServiceImpl<MaterialMapper, Material> implements IMaterialService {

    private static final Logger LOG = LoggerFactory.getLogger(MaterialServiceImpl.class);

    @Autowired
    private IDocPagesService docPagesService;

    @Autowired
    private DocPagesMapper docPagesMapper;

    @Autowired
    private AudioVideoMapper audioVideoMapper;

    @Autowired
    private ScormMapper scormMapper;

    @Autowired
    private IAudioVideoService audioVideoService;

    @Autowired
    private IRichMediaService richMediaService;

    @Autowired
    private ILessonReplyService lessonReplyService;

    @Autowired
    private MaterialMapper materialMapper;

    @Autowired
    private RichMediaMapper richMediaMapper;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ChapterMapper chapterMapper;

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private PolyUploadRemoteVideoTask polyUploadRemoteVideoTask;
    @Autowired
    TaskExecutor taskExecutor;

    @Autowired
    private IMaterialService materialService;

    @Value("${polyv.remote.url}")
    private String polyvUrl;

    @Value("${polyv.writetoken}")
    private String writeToken;

    @Value("${polyv.cataid}")
    private String cataId;

    @Override
    public Material saveMaterial(Material material, Map<String, Object> map) {

        try {

            Long id = material.getId();
            // 增加支持外部的新增id方式
            if(id == null){
                id = idGenerator.generate();
                material.setId(id);
            }

            this.baseMapper.insert(material);
            switch (material.getType()) {
                case 1:
                    AudioVideo audio = JSON.parseObject(JSON.toJSONString(map.get("audio")), new TypeReference<AudioVideo>() {
                    });
                    audio.setMaterialId(material.getId());
                    audioVideoMapper.insert(audio);
                    break;
                case 2:
                    AudioVideo video = JSON.parseObject(JSON.toJSONString(map.get("video")), new TypeReference<AudioVideo>() {
                    });
                    video.setMaterialId(material.getId());
                    audioVideoMapper.insert(video);
                    break;
                case 3:
                    List<DocPages> docPages = JSON.parseObject(JSON.toJSONString(map.get("docPages")), new TypeReference<List<DocPages>>() {
                    });
                    for (DocPages s : docPages) {
                        s.setMaterialId(material.getId());
                    }
                    docPagesService.insertBatch(docPages, docPages.size());
                    break;
                case 4:
                    RichMedia richMedia = JSON.parseObject(JSON.toJSONString(map.get("richMedia")), new TypeReference<RichMedia>() {
                    });
                    richMedia.setMaterialId(material.getId());
                    richMediaMapper.insert(richMedia);
                    break;
                case 5:
                    Scorm scorm = JSON.parseObject(JSON.toJSONString(map.get("scorm")), new TypeReference<Scorm>() {
                    });
                    scorm.setMaterialId(material.getId());
                    scormMapper.insert(scorm);
                    break;
                default:
                    break;
            }
            return material;
        } catch (Exception e) {
            System.out.println(e.getLocalizedMessage());
            return null;
        }

    }


    @Override
    public Boolean deleteMaterial(List<Long> ids) {
        try {


            List<Long> notDeleteIds = new ArrayList<Long>();
            for (Long id : ids) {
                //先验证素材是否有跟已有课程关联
                List<Course> courses = courseMapper.selectCourseByMaterialId(id);

                if (courses.size() > 0) {
                    notDeleteIds.add(id);
                } else {
                    docPagesMapper.deleteByMaterialId(id);
                    richMediaMapper.deleteByMaterialId(id);
                    audioVideoMapper.deleteByMaterialId(id);
                    scormMapper.deleteByMaterialId(id);
                }
            }
            Iterator<Long> iter = ids.iterator();
            while (iter.hasNext()) {
                Long id= iter.next();
                if (notDeleteIds.contains(id)) {
                    iter.remove();
                }
            }
            List<Material> materials=this.baseMapper.selectBatchIds(ids);
            for (Material m:materials
                 ) {
                m.setDeleted(1);
                m.updateById();
            }
            return Boolean.TRUE;
        } catch (Exception e) {
            return Boolean.FALSE;
        }
    }

    @Override
    public MaterialVo getById(Long id) {
        MaterialVo materialVo = new MaterialVo();
        Material material = this.baseMapper.selectById(id);
        if (StringUtils.isEmpty(material)) {
            return null;
        }
        materialVo.setName(material.getName());
        materialVo.setCode(material.getCode());
        materialVo.setId(material.getId().toString());
        materialVo.setOriginalFile(material.getOriginalFile());
        materialVo.setOriginalFileUrl(material.getOriginalFileUrl());
        materialVo.setSource(material.getSource());
        materialVo.setTags(material.getTags());
        materialVo.setType(material.getType());
        materialVo.setTextHour(material.getTextHour());
        materialVo.setTextMinute(material.getTextMinute());
        materialVo.setTextMinute(material.getTextMinute());
        materialVo.setTextSecond(material.getTextSecond());
        materialVo.setProducer(material.getProducer());
        materialVo.setExamId(material.getExamId());
        materialVo.setExamName(material.getExamName());
        materialVo.setPolyVid(material.getPolyVid());
        materialVo.setDisplayMark(material.getDisplayMark());
        materialVo.setWord(material.getWord());
        materialVo.setWordType(material.getWordType());
        String fileSize = "0";
        try {
            // 转化文件大小
            fileSize = getPrintSize(Long.valueOf(material.getSize()));
        } catch (Exception e) {
            e.getLocalizedMessage();
        }
        materialVo.setSize(fileSize);

        switch (material.getType()) {
            case 1:
                AudioVideo audio = audioVideoMapper.getByMaterialId(id);
                if (StringUtils.isEmpty(audio)) {
                    break;
                }
                materialVo.setImage(audio.getImage());
                materialVo.setLdUrl(audio.getLdUrl());
                materialVo.setSdUrl(audio.getSdUrl());
                materialVo.setFdUrl(audio.getFdUrl());
                materialVo.setExplanation(audio.getExplanation());
                break;
            case 2:
                AudioVideo video = audioVideoMapper.getByMaterialId(id);
                if (StringUtils.isEmpty(video)) {
                    break;
                }
                materialVo.setImage(video.getImage());
                materialVo.setLdUrl(video.getLdUrl());
                materialVo.setSdUrl(video.getSdUrl());
                materialVo.setFdUrl(video.getFdUrl());
                break;
            case 3:
                List<DocPagesVO> docPages = docPagesMapper.getByMaterialId(id);
                if (StringUtils.isEmpty(docPages)) {
                    break;
                }
                materialVo.setDocPages(docPages);
                materialVo.setPages(docPages.size());
                break;
            case 4:
                List<RichMedia> richMedia = richMediaMapper.getByMaterialId(id);
                if (CollectionUtils.isEmpty(richMedia)) {
                    break;
                }
                RichMedia oneRichMedia = richMedia.get(0);
                materialVo.setContent(oneRichMedia.getContent());
                materialVo.setAuthor(oneRichMedia.getAuthor());
                materialVo.setTitle(oneRichMedia.getTitle());
                break;
            case 5:
                ScormVO scorm = scormMapper.getByMaterialId(id);
                materialVo.setScorm(scorm);
                break;
            default:
                break;
        }
        return materialVo;
    }

    /**
     * 查询章节列表的素材详情
     *
     * @param materialIds
     * @return
     */
    @Override
    public List<MaterialVo> getMaterialDetails(Set<Long> materialIds) {
        if(CollectionUtils.isEmpty(materialIds)){
            return null;
        }
        LOG.info("查询章节列表的素材id = {}",JSON.toJSONString(materialIds));
        List<MaterialVo> data = this.baseMapper.getMaterialDetails(materialIds);
        if(CollectionUtils.isNotEmpty(data)){
            List<Long> docPagesMids = new ArrayList<>();
            List<Long> richMediaMids = new ArrayList<>();
            List<Long> scormMids = new ArrayList<>();
            for(MaterialVo item : data){
                int type = item.getType().intValue();
                Long mid = Long.valueOf(item.getId());
                switch (type){
                    case 3:
                        docPagesMids.add(mid);
                        break;
                    case 4:
                        richMediaMids.add(mid);
                        break;
                    case 5:
                        scormMids.add(mid);
                        break;
                    default:
                        break;
                }
            }
            // 查询文档详细数据
            boolean docPagesFlg = false;
            Map<Long,List<DocPagesVO>> docPagesG = null;
            if(CollectionUtils.isNotEmpty(docPagesMids)){
                List<DocPagesVO> docPages = docPagesMapper.getByMaterialIds(docPagesMids);
                if(CollectionUtils.isNotEmpty(docPages)){
                    docPagesFlg = true;
                    docPagesG = docPages.stream().collect(Collectors.groupingBy(key->key.getMaterialId()));
                }
            }

            // 查询多媒体
            boolean richMediaFlg = false;
            Map<Long,List<RichMedia>> richMediaG = null;
            if(CollectionUtils.isNotEmpty(richMediaMids)){
                List<RichMedia> richMedias = richMediaMapper.getByMaterialIds(richMediaMids);
                if(CollectionUtils.isNotEmpty(richMedias)){
                    richMediaFlg = true;
                    richMediaG = richMedias.stream().collect(Collectors.groupingBy(key->key.getMaterialId()));
                }
            }
            // scorm课程
            boolean scormFlg = false;
            Map<Long, List<ScormVO>> scormG = null;
            if(CollectionUtils.isNotEmpty(scormMids)){
                List<ScormVO> scormList = scormMapper.getByMaterialIds(scormMids);
                if(CollectionUtils.isNotEmpty(scormList)){
                    scormFlg = true;
                    scormG = scormList.stream().collect(Collectors.groupingBy(key->key.getMaterialId()));
                }
            }

            for(MaterialVo item : data){
                Long mid = Long.valueOf(item.getId());
                // 设置文档素材信息
                if(docPagesFlg && docPagesG.containsKey(mid)){
                    List<DocPagesVO> docPages = docPagesG.get(mid);
                    int size = docPages.size();
                    if(size>1){
                        docPages.sort(new Comparator<DocPagesVO>() {
                            @Override
                            public int compare(DocPagesVO o1, DocPagesVO o2) { return o1.getDocPage().compareTo(o2.getDocPage()); }});
                    }
                    item.setDocPages(docPages);
                    item.setPages(size);
                }
                // 多媒体
                if(richMediaFlg && richMediaG.containsKey(mid)){
                    RichMedia oneRichMedia = richMediaG.get(mid).get(0);
                    item.setContent(oneRichMedia.getContent());
                    item.setAuthor(oneRichMedia.getAuthor());
                    item.setTitle(oneRichMedia.getTitle());
                }
                // scorm 课程
                if(scormFlg && scormG.containsKey(mid)){
                    item.setScorm(scormG.get(mid).get(0));
                }
            }

        }
        return data;
    }


    public String getPrintSize(long size) {
        //如果字节数少于1024，则直接以B为单位，否则先除于1024，后3位因太少无意义
        if (size < 1024) {
            return String.valueOf(size) + "B";
        } else {
            size = size / 1024;
        }
        //如果原字节数除于1024之后，少于1024，则可以直接以KB作为单位
        //因为还没有到达要使用另一个单位的时候
        //接下去以此类推
        if (size < 1024) {
            return String.valueOf(size) + "KB";
        } else {
            size = size / 1024;
        }
        if (size < 1024) {
            //因为如果以MB为单位的话，要保留最后1位小数，
            //因此，把此数乘以100之后再取余
            size = size * 100;
            return String.valueOf((size / 100)) + "."
                    + String.valueOf((size % 100)) + "MB";
        } else {
            //否则如果要以GB为单位的，先除于1024再作同样的处理
            size = size * 100 / 1024;
            return String.valueOf((size / 100)) + "."
                    + String.valueOf((size % 100)) + "GB";
        }
    }


    @Override
    @Transactional
    public Material updateMaterial(Material material, Map<String, Object> map) {
        try {


            switch (material.getType()) {
                case 1:

                    AudioVideo audio = JSON.parseObject(JSON.toJSONString(map.get("audio")), new TypeReference<AudioVideo>() {
                    });
                    if(Objects.nonNull(audio)){
                        audioVideoMapper.deleteByMaterialId(material.getId());
                        audio.setMaterialId(material.getId());
                        audioVideoMapper.insert(audio);
                    }
                    break;
                case 2:
                    AudioVideo video = JSON.parseObject(JSON.toJSONString(map.get("video")), new TypeReference<AudioVideo>() {
                    });
                    if(Objects.nonNull(video)){
                        audioVideoMapper.deleteByMaterialId(material.getId());
                        video.setMaterialId(material.getId());
                        audioVideoMapper.insert(video);
                        LOG.info("视频素材更新时，material={},map={}",JSON.toJSONString(material),JSON.toJSONString(map));
                    }

                    break;
                case 3:
                    List<DocPages> docPages = JSON.parseObject(JSON.toJSONString(map.get("docPages")), new TypeReference<List<DocPages>>() {
                    });
                    if(CollectionUtils.isNotEmpty(docPages)){
                        docPagesMapper.deleteByMaterialId(material.getId());
                        for (DocPages s : docPages) {
                            s.setMaterialId(material.getId());
                        }
                        docPagesService.insertBatch(docPages, docPages.size());
                    }

                    break;
                case 4:

                    RichMedia richMedia = JSON.parseObject(JSON.toJSONString(map.get("richMedia")), new TypeReference<RichMedia>() {
                    });
                    if(Objects.nonNull(richMedia)){
                        richMediaMapper.deleteByMaterialId(material.getId());
                        richMedia.setMaterialId(material.getId());
                        richMediaMapper.insert(richMedia);
                    }

                    break;
                case 5:
                    Scorm scorm = JSON.parseObject(JSON.toJSONString(map.get("scorm")), new TypeReference<Scorm>() {
                    });
                    if(Objects.nonNull(scorm)){
                        scormMapper.deleteByMaterialId(material.getId());
                        scorm.setMaterialId(material.getId());
                        scormMapper.insert(scorm);
                    }
                    break;
                default:
                    break;
            }
            this.baseMapper.updateById(material);
            LOG.info("更新素材信息={}",JSON.toJSONString(material));
            return material;
        } catch (Exception e) {
            LOG.error("素材更新异常----------------->：");
            e.printStackTrace();
            return null;
        }


    }

    @Override
    public Page<Material> searchPage(String name, Integer type, Integer source, Long companyId, Long siteId, List<Long> orgId,
                                     int pageNo, int pageSize) {
        Page<Material> page = new Page<Material>(pageNo, pageSize);
        // 分级授权修改
        HQueryUtil.startHQ(Material.class);
        page.setRecords(baseMapper.searchPage(name, type, source, companyId, siteId, orgId, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.searchPageCount(name, type, source, companyId, siteId, orgId);
        page.setTotal(count);
        return page;
    }


    /**
     * 得到我的做课列表
     */
    @Override
    public Page<CourseStudentListVo> listMyLesson(String name, Integer pageNo, Integer pageSize, Long accountId) {
        // TODO Auto-generated method stub
        //条件
        Page<Material> page = new Page<Material>(pageNo, pageSize, "create_time", false);
        //数据
        List<Material> pageList = materialMapper.studentMyLessonList(name, accountId, page);
        Integer pageListNum = materialMapper.getListNumByAccountIdAndName(name, accountId);
        //将数据全部给条件的page  这样page就完整了
        if (pageList != null) {
            page.setRecords(pageList);
            page.setTotal(pageListNum);
        }
        List<CourseStudentListVo> listCourseStudentListVo = new ArrayList<CourseStudentListVo>();   //vo类
        if (page != null) {
            List<Material> list = page.getRecords();
            for (int i = 0; i < list.size(); i++) {
                CourseStudentListVo cslv = new CourseStudentListVo();
                if (list.get(i).getType() == 1 || list.get(i).getType() == 2) {//1音频 2 视频
                    AudioVideo audioVideo = audioVideoService.getAudioVideoByMaterialId(list.get(i).getId());
                    cslv.setId(list.get(i).getId());                //主素材库id
                    cslv.setTitle(list.get(i).getName());           //标题
                    cslv.setContent(list.get(i).getContent());      //内容
                    cslv.setImg(audioVideo.getImage());             //封面图片
                    cslv.setCommitTime(list.get(i).getCreateTime());
                    if(list.get(i).getState()==0) {   //管理端有已撤回0  学员端已撤回按照未提交的状态
                         cslv.setState(1);
                    }
                    else {
                         cslv.setState(list.get(i).getState());
                    }
                    cslv.setType(list.get(i).getType());
                }

                if (list.get(i).getType() == 3) {//3 文件
                    DocPages docPages = docPagesService.getDocPagesByMaterialId(list.get(i).getId());
                    cslv.setId(list.get(i).getId());
                    cslv.setTitle(list.get(i).getName());           //标题
                    cslv.setContent(list.get(i).getContent());      //内容
                    cslv.setImg(docPages.getLogo());                //logo
                    cslv.setCommitTime(list.get(i).getCreateTime());
                    if(list.get(i).getState()==0) {   //管理端有已撤回0  学员端已撤回按照未提交的状态
                        cslv.setState(1);
                    }
                    else {
                        cslv.setState(list.get(i).getState());
                    }
                    cslv.setType(list.get(i).getType());
                }
                ;

                if (list.get(i).getType() == 4) {// 4 富媒体
                    RichMedia richMedia = richMediaService.getRichMediaByMaterialId(list.get(i).getId());
                    cslv.setId(list.get(i).getId());
                    cslv.setTitle(list.get(i).getName());           //标题
                    cslv.setContent(list.get(i).getContent());      //内容
                    cslv.setImg(richMedia.getLogo());               //logo
                    cslv.setCommitTime(list.get(i).getCreateTime());
                    if(list.get(i).getState()==0) {   //管理端有已撤回0  学员端已撤回按照未提交的状态
                        cslv.setState(1);
                    }
                    else {
                        cslv.setState(list.get(i).getState());
                    }
                    cslv.setType(list.get(i).getType());
                }
                listCourseStudentListVo.add(cslv);
            }
        }
        Page<CourseStudentListVo> page2 = new Page<CourseStudentListVo>(pageNo, pageSize);
        page2.setRecords(listCourseStudentListVo);
        page2.setTotal(page.getTotal());
        return page2;
    }


    @Override
    public CourseStudentListVo getCourseStudentListVo(Long id) {
        // TODO Auto-generated method stub
        Material material = new Material();
        material = this.selectById(id);
        CourseStudentListVo cslv = new CourseStudentListVo();
        cslv.setId(id);
        cslv.setContent(material.getContent());
        if (material.getType() == 1 || material.getType() == 2) {
            AudioVideo audioVideo = audioVideoService.getAudioVideoByMaterialId(id);
            cslv.setTitle(material.getName());           //标题
            cslv.setImg(audioVideo.getImage());             //封面图片  logo
            cslv.setUrl(audioVideo.getFdUrl());             //视频地址
            if(material.getState()==0) {
                 cslv.setState(1);   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
            }
            else {
            	 cslv.setState(material.getState());   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
            }
            cslv.setType(material.getType());
            LessonReply lastReply = lessonReplyService.getLastNoPassReasonByLessonId(id);
            if (lastReply != null) {
                cslv.setLastNoPassReason(lastReply.getReason());
                cslv.setCommitTime(lastReply.getCreateByTime());
            }
        }
        if (material.getType() == 3) {//3 文件
            DocPages docPages = docPagesService.getDocPagesByMaterialId(id);
            cslv.setTitle(material.getName());           //标题
            cslv.setImg(docPages.getLogo());                //logo
            cslv.setUrl(docPages.getImageUrl());
            cslv.setState(material.getState());   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
            cslv.setType(material.getType());
            LessonReply lastReply = lessonReplyService.getLastNoPassReasonByLessonId(id);
            if (lastReply != null) {
                cslv.setLastNoPassReason(lastReply.getReason());
                cslv.setCommitTime(lastReply.getCreateByTime());
            }
        }

        if (material.getType() == 4) {// 4 富媒体
            RichMedia richMedia = richMediaService.getRichMediaByMaterialId(id);
            cslv.setTitle(material.getName());           //标题
            cslv.setImg(richMedia.getLogo());
            cslv.setUrl(richMedia.getContent());
            if(material.getState()==0) {
                cslv.setState(1);   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
           }
           else {
           	    cslv.setState(material.getState());   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
           }
            cslv.setType(material.getType());
            LessonReply lastReply = lessonReplyService.getLastNoPassReasonByLessonId(id);
            if (lastReply != null) {
                cslv.setLastNoPassReason(lastReply.getReason());
                cslv.setCommitTime(lastReply.getCreateByTime());
            }
        }
        return cslv;
    }


    @Override
    public Long saveMaterialByType(MaterialSaveVO materialSaveVO) {
        // TODO Auto-generated method stub
        Long mid = null;
        if (1 == materialSaveVO.getStyle()) {   //1新建  2更改
            mid = idGenerator.generate();
            Material material = new Material();
            material.setId(mid);
            material.setName(materialSaveVO.getName());
            material.setContent(materialSaveVO.getContent());
            material.setType(materialSaveVO.getType());
            material.setSource(2);           //素材来源 2 做课
            material.setAccountId(materialSaveVO.getContext().getAccountId());
            material.setState(1);           //1未提交 2审核中 3已通过 4未通过 5删除
            material.setCreateById(materialSaveVO.getContext().getAccountId());
            material.setCreateByName(materialSaveVO.getContext().getAccountName());
            material.setCreateTime(new Date());
            material.setOrgId(materialSaveVO.getContext().getOrgId());
            material.setCompanyId(materialSaveVO.getContext().getCompanyId());
            material.setSiteId(materialSaveVO.getContext().getSiteId());
            this.insert(material);  //主类插入完成
            /**
             *   开始插入子类
             */
            switch (materialSaveVO.getType()) {
                case 1:                                                      //音频 有简介
                    AudioVideo audioVideo = new AudioVideo();
                    audioVideo.setId(idGenerator.generate());
                    audioVideo.setImage(materialSaveVO.getLogo());           //logo
                    audioVideo.setMaterialId(mid);                           //所属素材id
                    audioVideo.setFdUrl(materialSaveVO.getFdUrl());          //音频地址
                    audioVideoService.insert(audioVideo);
                    break;
                case 2:
                    AudioVideo audioVideo2 = new AudioVideo();
                    audioVideo2.setId(idGenerator.generate());
                    audioVideo2.setImage(materialSaveVO.getLogo());    //logo
                    audioVideo2.setMaterialId(mid);                    //所属素材id
                    audioVideo2.setFdUrl(materialSaveVO.getFdUrl());   //流畅视频地址
                    audioVideo2.setLdUrl(materialSaveVO.getLdUrl());   //标清视频地址
                    audioVideo2.setSdUrl(materialSaveVO.getSdUrl());   //高清视频地址
                    audioVideoService.insert(audioVideo2);
                    break;
                case 3:
                    DocPages docPages = new DocPages();                 //文档
                    docPages.setId(idGenerator.generate());
                    docPages.setImageUrl(materialSaveVO.getFdUrl());  //文档地址
                    docPages.setMaterialId(mid);
                    docPages.setLogo(materialSaveVO.getLogo());
                    docPagesService.insert(docPages);
                    break;
                case 4:                                                //富文本
                    RichMedia richMedia = new RichMedia();
                    richMedia.setId(idGenerator.generate());
                    richMedia.setMaterialId(mid);
                    richMedia.setContent(materialSaveVO.getFdUrl());               //富文本内容
                    richMedia.setLogo(materialSaveVO.getLogo());   //logo图标
                    richMediaService.insert(richMedia);
                    break;
                default:
                    break;
            }
            return mid;
        }

        if (2 == materialSaveVO.getStyle()) {   //1新建  2更改
            mid = materialSaveVO.getId();
            Material material = new Material();
            material.setId(materialSaveVO.getId());
            material.setName(materialSaveVO.getName());
            material.setContent(materialSaveVO.getContent());
            material.setType(materialSaveVO.getType());
            material.setSource(2);
            material.setAccountId(materialSaveVO.getContext().getAccountId());
            material.setState(1);           //1未提交 2审核中 3已通过 4未通过 5删除
            material.setUpdateById(materialSaveVO.getContext().getAccountId());
            material.setUpdateByName(materialSaveVO.getContext().getAccountName());
            material.setUpdateTime(new Date());
            material.setOrgId(materialSaveVO.getContext().getOrgId());
            material.setCompanyId(materialSaveVO.getContext().getCompanyId());
            material.setSiteId(materialSaveVO.getContext().getSiteId());
            this.updateById(material);  //主类更新完成
            /**
             *   开始更新子类
             */
            switch (materialSaveVO.getType()) {
                case 1:                                                      //音频 有简介
                    //先删除
                    audioVideoService.deleteAudioVideo(materialSaveVO.getId());
                    //添加
                    AudioVideo audioVideo = new AudioVideo();
                    audioVideo.setId(idGenerator.generate());
                    audioVideo.setImage(materialSaveVO.getLogo());           //logo
                    audioVideo.setMaterialId(materialSaveVO.getId());                           //所属素材id
                    audioVideo.setFdUrl(materialSaveVO.getFdUrl());          //音频地址
                    audioVideoService.insert(audioVideo);
                    break;
                case 2:
                	//先删除
                    audioVideoService.deleteAudioVideo(materialSaveVO.getId());
                    //再添加
                    AudioVideo audioVideo2 = new AudioVideo();
                    audioVideo2.setId(idGenerator.generate());
                    audioVideo2.setImage(materialSaveVO.getLogo());    //logo
                    audioVideo2.setMaterialId(materialSaveVO.getId()); //所属素材id
                    audioVideo2.setFdUrl(materialSaveVO.getFdUrl());   //流畅视频地址
                    audioVideo2.setLdUrl(materialSaveVO.getLdUrl());   //标清视频地址
                    audioVideo2.setSdUrl(materialSaveVO.getSdUrl());   //高清视频地址
                    audioVideoService.insert(audioVideo2);
                    break;
                case 3:
                    docPagesService.deleteDocPagesByMaterialId(materialSaveVO.getId());
                    DocPages docPages = new DocPages();                 //文档
                    docPages.setId(idGenerator.generate());
                    docPages.setImageUrl(materialSaveVO.getFdUrl());  //文档地址
                    docPages.setMaterialId(materialSaveVO.getId());
                    docPages.setLogo(materialSaveVO.getLogo());
                    docPagesService.insert(docPages);
                    break;
                case 4:                                                //富文本
                    richMediaService.deleteByMaterialId(materialSaveVO.getId());
                    RichMedia richMedia = new RichMedia();
                    richMedia.setId(idGenerator.generate());
                    richMedia.setMaterialId(materialSaveVO.getId());
                    richMedia.setContent(materialSaveVO.getFdUrl());               //富文本地址
                    richMedia.setLogo(materialSaveVO.getLogo());   //logo图标
                    richMediaService.insert(richMedia);
                    break;
                default:
                    break;
            }
            return mid;
        }
        return mid;
    }


    @Override
    public Boolean updateStateById(MaterialStateUpdateVo materialStateUpdateVo) {
        // TODO Auto-generated method stub
        Material material = new Material();
        material.setId(materialStateUpdateVo.getId());
        material.setState(materialStateUpdateVo.getState());
        material.setUpdateById(materialStateUpdateVo.getContext().getAccountId());
        material.setUpdateByName(materialStateUpdateVo.getContext().getAccountName());
        material.setUpdateTime(new Date());
        // 4代表不通过，填写不通过原因
        if (materialStateUpdateVo.getState() == 4) {
            LessonReply lessonReply = new LessonReply();
            lessonReply.setId(idGenerator.generate());
            lessonReply.setLessonId(materialStateUpdateVo.getId());
            lessonReply.setAuditorId(materialStateUpdateVo.getContext().getAccountId());
            lessonReply.setReason(materialStateUpdateVo.getReason());
            lessonReplyService.insert(lessonReply);
        }
        Boolean x = this.updateById(material);
        return x;
    }

    @Override
    public Long updateStateByIdAndNextId(MaterialStateUpdateVo materialStateUpdateVo) {
        // TODO Auto-generated method stub
        Material material = new Material();
        material.setId(materialStateUpdateVo.getId());
        // 通过
        material.setState(3);
        Long id = materialStateUpdateVo.getId();
        //先得到下一个id，再执行更新
        Long nextId = materialMapper.getNextId(id);
        this.updateById(material);
        return nextId;
    }


    @Override
    public Page<MaterialManageVo> manageGetLessonList(String name, List<Long> listCommintIdList, List<Long> listAuditorIdList, Integer state,
                                                      Integer pageNo, Integer pageSize, Long orgId) {
        // TODO Auto-generated method stub
        Page<MaterialManageVo> page = new Page<MaterialManageVo>(pageNo, pageSize);
        List<MaterialManageVo> pageList = materialMapper.manageGetLessonList(name, listCommintIdList, listAuditorIdList, state, page,orgId);
        page.setRecords(pageList);
        System.out.println("----");
        return page;
    }


    /**
     * 查看单个详情
     */
    @Override
    public MaterialManageDetailsVo getDetailsById(Long id) {
        // TODO Auto-generated method stu
        MaterialManageDetailsVo materialManageDetailsVo = new MaterialManageDetailsVo();
        Material material = this.selectById(id);
        materialManageDetailsVo.setId(id);
        materialManageDetailsVo.setName(material.getName());
        materialManageDetailsVo.setContent(material.getContent());
        materialManageDetailsVo.setType(material.getType());
        materialManageDetailsVo.setResultTime(material.getUpdateTime());
        materialManageDetailsVo.setState(material.getState());
        if (material.getType() == 1 || material.getType() == 2) {
            AudioVideo audioVideo = new AudioVideo();
            audioVideo.setMaterialId(id);
            EntityWrapper<AudioVideo> wrapper = new EntityWrapper<AudioVideo>(audioVideo);
            AudioVideo audioVideo2 = audioVideoService.selectOne(wrapper);
            materialManageDetailsVo.setLogo(audioVideo2.getImage());
            materialManageDetailsVo.setUrl(audioVideo2.getFdUrl());//中等
            LessonReply lastNoPass = lessonReplyService.getLastNoPassReasonByLessonId(id);
            if (lastNoPass != null) {
                materialManageDetailsVo.setLastNoPassReason(lastNoPass.getReason());
            } else {
                materialManageDetailsVo.setLastNoPassReason(null);
            }
            return materialManageDetailsVo;
        }
        if (material.getType() == 3) {
            DocPages docPages = new DocPages();
            docPages.setMaterialId(id);
            EntityWrapper<DocPages> wrapper = new EntityWrapper<DocPages>(docPages);
            DocPages docPages2 = new DocPages();
            docPages2 = docPagesService.selectOne(wrapper);
            materialManageDetailsVo.setLogo(docPages2.getLogo());
            materialManageDetailsVo.setUrl(docPages2.getImageUrl());
            LessonReply lastNoPass = lessonReplyService.getLastNoPassReasonByLessonId(id);
            if (lastNoPass != null) {
                materialManageDetailsVo.setLastNoPassReason(lastNoPass.getReason());
            } else {
                materialManageDetailsVo.setLastNoPassReason(null);
            }
            return materialManageDetailsVo;
        }
        if (material.getType() == 4) {
            RichMedia richMedia = new RichMedia();
            richMedia.setMaterialId(id);
            EntityWrapper<RichMedia> wrapper = new EntityWrapper<RichMedia>(richMedia);
            RichMedia richMedia2 = new RichMedia();
            richMedia2 = richMediaService.selectOne(wrapper);
            materialManageDetailsVo.setLogo(richMedia2.getLogo());
            materialManageDetailsVo.setUrl(richMedia2.getContent());
            LessonReply lastNoPass = lessonReplyService.getLastNoPassReasonByLessonId(id);
            if (lastNoPass != null) {
                materialManageDetailsVo.setLastNoPassReason(lastNoPass.getReason());
            } else {
                materialManageDetailsVo.setLastNoPassReason(null);
            }
            return materialManageDetailsVo;
        }
        return materialManageDetailsVo;
    }


    @Override
    public Page<CourseStudentListVo> getPassMaterialList(String name, Long accountId, Integer pageNo, Integer pageSize) {
        // TODO Auto-generated method stub
    	//要求的page
    	Page<CourseStudentListVo> pageCourseStudentListVo = new Page<CourseStudentListVo>();
    	//装数据
        List<CourseStudentListVo> listCourseStudentListVo = new ArrayList<CourseStudentListVo>();
        
        Page<Material> page = new Page<Material>(pageNo, pageSize);
        List<Material> materials = materialMapper.getPassMaterialList(name, accountId, page);
        Integer totalMaterials=materialMapper.getPassMaterialListCount(name, accountId);
        if (CollectionUtils.isNotEmpty(materials)) {
            for (int i = 0; i < materials.size(); i++) {
                CourseStudentListVo cslv = new CourseStudentListVo();
                cslv.setId(materials.get(i).getId());
                Material material = materials.get(i);
                if (material.getType() == 1 || material.getType() == 2) {
                    AudioVideo audioVideo = audioVideoService.getAudioVideoByMaterialId(material.getId());
                    cslv.setTitle(material.getName());           //标题
                    cslv.setContent(material.getContent());   //内容
                    cslv.setImg(audioVideo.getImage());             //封面图片  logo
                    cslv.setUrl(audioVideo.getFdUrl());             //视频地址
                    cslv.setState(material.getState());   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
                }
                if (material.getType() == 3) {//3 文件
                    DocPages docPages = docPagesService.getDocPagesByMaterialId(material.getId());
                    cslv.setTitle(material.getName());           //标题
                    cslv.setContent(material.getContent());
                    cslv.setImg(docPages.getLogo());                //logo
                    cslv.setUrl(docPages.getImageUrl());
                    cslv.setState(material.getState());   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
                };
                if (material.getType() == 4) {// 4 富媒体
                    RichMedia richMedia = richMediaService.getRichMediaByMaterialId(material.getId());
                    cslv.setTitle(material.getName());
                    cslv.setContent(material.getContent());//标题
                    cslv.setImg(richMedia.getLogo());
                    cslv.setUrl(richMedia.getContent());   //富媒体内容
                    cslv.setState(material.getState());   //状态  0已撤回 1未提交 2审核中 3已通过 4未通过 5删除
                };
                listCourseStudentListVo.add(cslv);
                pageCourseStudentListVo.setRecords(listCourseStudentListVo);
                pageCourseStudentListVo.setTotal(totalMaterials);
            }
            return pageCourseStudentListVo;
        } else {
            return pageCourseStudentListVo;
        }
    }


    @Override
    public Integer getMyLessonAllNum(Long accountId) {
        // TODO Auto-generated method stub
        Material material = new Material();
        material.setAccountId(accountId);
        material.setSource(2);//做课
        EntityWrapper<Material> wrapper = new EntityWrapper<Material>(material);
        Integer allNum = this.selectCount(wrapper);

        Material material2 = new Material();
        material2.setAccountId(accountId);
        material2.setSource(2);//做课
        material2.setState(5);  //已删除的
        EntityWrapper<Material> wrapper2 = new EntityWrapper<Material>(material2);
        Integer deleteNum = this.selectCount(wrapper2);
        //做课的全部减去做课的已经删除的
        return allNum - deleteNum;
    }

    /**
     * 查询素材列表  ( Scorm课程 )
     *
     * @param name
     * @param type
     * @param source
     * @param companyId
     * @param siteId
     * @param orgId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<Material> queryScormCourseList(String name, Integer type, Integer source, Long companyId, Long siteId, List<Long> orgId, Integer pageNo, Integer pageSize) {

        Page<Material> page = new Page<Material>(pageNo, pageSize);
        // 分级授权修改
        HQueryUtil.startHQ(Material.class);

        page.setRecords(baseMapper.queryScormCourseList(name, type, source, companyId, siteId, orgId, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.queryScormCourseListCount(name, type, source, companyId, siteId, orgId);
        page.setTotal(count);
        return page;
    }

    /**
     * 查询素材列表 ( 一般课程  )
     *
     * @param name
     * @param type
     * @param source
     * @param companyId
     * @param siteId
     * @param orgId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<Material> queryCommonCourseList(String name, Integer type, Integer source,
                                                Long companyId, Long siteId, List<Long> orgId,
                                                Integer pageNo, Integer pageSize,Integer excludType) {

        Page<Material> page = new Page<Material>(pageNo, pageSize);
        // 分级授权修改
        HQueryUtil.startHQ(Material.class);
        page.setRecords(baseMapper.queryCommonCourseList(name, type, source, companyId, siteId, orgId, excludType, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.queryCommonCourseListCount(name, type, source, companyId, siteId, orgId);
        page.setTotal(count);
        return page;
    }

    @Override
    public void uploadPolyVideoRemote(Material material) {

//        Long taskId = material.getId();
//        TaskContext taskContext =
//                new TaskContext(taskId,null,null,new Date());
//        polyUploadRemoteVideoTask.bindContext(taskContext);
//        taskExecutor.asynExecute(polyUploadRemoteVideoTask);
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                upLoadPolyVideoAndAudio(material);
            }
        });
    }

    private boolean upLoadPolyVideoAndAudio(Material material){
        if (material.getType() != 2 && material.getType() != 1) {
            return false;
        }
        log.info("当前的类型：" + material.getType());
        HttpClientUtils httpClientUtils = new HttpClientUtils();
        // 远程上传保利威视 视频
        Map<String, String> polyvMap = new HashMap<String, String>(16);
        polyvMap.put("fileUrl", material.getOriginalFileUrl());
        polyvMap.put("writetoken", writeToken);
        polyvMap.put("title", material.getName());
        polyvMap.put("cataid", cataId);
        log.info("参数的polyvMap：" + polyvMap);
        String reStr = httpClientUtils.httpPost(polyvUrl, polyvMap);
        log.info("上传保利威视的远程视的结果：" + reStr);

        JSONObject polyResult = JSON.parseObject(reStr);
        log.info("保利威视返回的结果：" + reStr);
        if (null != reStr && polyResult.containsKey("data")) {
            String data = polyResult.getString("data");
            JSONArray jsonArray = JSON.parseArray(data);
            JSONObject dat = JSON.parseObject(jsonArray.get(0).toString());
            String vid = dat.getString("vid");
            log.info("保利威视的vid：" + vid);
            String duration = dat.getString("duration");


            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");

            Material newMaterial = new Material();
            newMaterial.setId(material.getId());
            newMaterial.setPolyVid(vid);
            try {
                Date date = sdf.parse(duration);
                System.out.println(sdf.parse(duration));
                java.util.Calendar calendar = java.util.Calendar.getInstance();
                calendar.setTime(date);
                int hour = calendar.get(java.util.Calendar.HOUR_OF_DAY);
                int minute = calendar.get(java.util.Calendar.MINUTE);
                int second = calendar.get(java.util.Calendar.SECOND);
                log.info("hour:" + hour + ",minute:" + minute + ",second:" + second);
                newMaterial.setTextHour(hour);
                newMaterial.setTextMinute(minute);
                newMaterial.setTextSecond(second);
            } catch (ParseException e) {
                log.info("异常{}", e);
            }
            //更新polyVid到素材中
            log.info("修改参数" + material);
            boolean ret = materialService.updateById(newMaterial);

            log.info("material表:" + (ret ? "跟新成功" : "更新失败"));
        }
        return true;
    }

    @Override
    public String getPolyVId(Material material) {
        HttpClientUtils httpClientUtils = new HttpClientUtils();

        // 远程上传保利威视 视频
        Map<String,String> polyvMap = new HashMap<String,String>(16);
        polyvMap.put("fileUrl",material.getOriginalFileUrl());
        polyvMap.put("writetoken",writeToken);
        polyvMap.put("title",material.getName());
        log.info("参数的polyvMap："+polyvMap);
        String reStr = httpClientUtils.httpPost(polyvUrl,polyvMap);
        log.info("上传保利威视的远程视的结果："+reStr);

        JSONObject polyResult = JSON.parseObject(reStr);
        log.info("保利威视返回的结果："+reStr);
        String vid = "";
        if (null!=reStr&&polyResult.containsKey("data")){
            String data = polyResult.getString("data");
            JSONArray jsonArray = JSON.parseArray(data);
            JSONObject dat = JSON.parseObject(jsonArray.get(0).toString());
             vid = dat.getString("vid");
            log.info("保利威视的vid："+vid);
            material.setPolyVid(vid);
            //更新polyVid到素材中
            log.info("修改参数"+material);
            boolean ret = this.updateById(material);
            log.info("返回结果"+ret);
        }

        return vid;
    }

    @Override
    public List<Material> queryMaterialList(List<String> codeList,Long beforeSiteId) {

        return materialMapper.queryMaterialList(codeList,beforeSiteId);
    }

    /**
     * 获取导出素材明细vo
     * @param courseIds
     * @return
     */
    @Override
    public List<MaterialExportVO> getMaterialExportVOList(List<Long> courseIds) {
        List<MaterialExportVO> voList = null;
        List<MaterialExportDto> dtoList = baseMapper.getMaterialExportDtoList(courseIds);
        if (CollectionUtils.isNotEmpty(dtoList)) {
            voList = new ArrayList<>(dtoList.size());
            for (MaterialExportDto dto : dtoList) {
                MaterialExportVO vo  = new MaterialExportVO();
                BeanUtils.copyProperties(dto, vo);
                if (dto.getTextHour() != null || dto.getTextMinute() != null || dto.getTextHour() != null) {
                    String hour = dto.getTextHour() == null? "0" : String.valueOf(dto.getTextHour());
                    String minute = dto.getTextMinute() == null? "0" : String.valueOf(dto.getTextMinute());
                    String second = dto.getTextSecond() == null? "0" : String.valueOf(dto.getTextSecond());
                    vo.setMaterialDuration(hour + ":" + minute + ":" + second);
                } else {
                    vo.setMaterialDuration("--");
                }
                vo.setMaterialName(null == dto.getTestName()? dto.getMaterialName() : dto.getTestName());
                vo.setMaterialCode(null == dto.getTestCode()? dto.getMaterialCode() : dto.getTestCode());
                voList.add(vo);
            }
        }
        return voList;
    }
}
