package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.OfflineClassify;
import com.yizhi.course.application.domain.OfflineCourse;
import com.yizhi.course.application.eum.DeletedEnum;
import com.yizhi.course.application.mapper.OfflineClassifyMapper;
import com.yizhi.course.application.service.IOfflineClassifyService;
import com.yizhi.course.application.vo.OfflineClassifyVo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 线下课程分类 服务实现类
 * </p>
 *
 * @author huangliang123
 * @since 2018-10-16
 */
@Service
@Transactional
public class OfflineClassifyServiceImpl extends ServiceImpl<OfflineClassifyMapper, OfflineClassify> implements IOfflineClassifyService {

    private Logger logger = LoggerFactory.getLogger(OfflineClassifyServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private OfflineClassifyMapper offlineClassifyMapper;

    /**
     * 根据id删除线下课程分类
     *
     * @param id
     * @return
     */
    @Override
    public Integer deleteById(Long id) {
        // 1. 先确认是否有子分类
        OfflineClassify child = new OfflineClassify();
        child.setParentId(id);
        child.setDeleted(DeletedEnum.NO.getValue());
        int num = child.selectCount(new EntityWrapper(child));
        if (num > 0) {
            return 0;
        }
        // 2.确认是否有有关联课程
        OfflineCourse offlineCourse = new OfflineCourse();
        offlineCourse.setOfflineClassifyId(id);
        offlineCourse.setDeleted(DeletedEnum.NO.getValue());
        num = offlineCourse.selectCount(new EntityWrapper(offlineCourse));
        if (num > 0) {
            return 0;
        }
        OfflineClassify currentClassify = new OfflineClassify();
        currentClassify.setId(id);
        currentClassify.setDeleted(DeletedEnum.YES.getValue());
        RequestContext context = ContextHolder.get();
        currentClassify.setUpdateById(context.getAccountId());
        currentClassify.setUpdateByName(context.getAccountName());
        currentClassify.setUpdateTime(new Date());
        return currentClassify.updateById() ? 1 : 0;


    }

    /**
     * 添加线下课程分类
     *
     * @param offlineClassify
     * @return
     */
    @Override
    public Integer saveOfflineClassify(OfflineClassify offlineClassify) {
        OfflineClassify exsitOfflineClassify = new OfflineClassify();
        exsitOfflineClassify.setParentId(offlineClassify.getParentId());
        exsitOfflineClassify.setName(offlineClassify.getName());
        exsitOfflineClassify.setDeleted(0);
        exsitOfflineClassify.setSiteId(ContextHolder.get().getSiteId());
        int count = exsitOfflineClassify.selectCount(new EntityWrapper(exsitOfflineClassify));
        if (count > 0) {
            return -1;
        }

        Long id = idGenerator.generate();
        offlineClassify.setId(id);
        offlineClassify.setDeleted(DeletedEnum.NO.getValue());

        if (null != offlineClassify.getParentId() && !offlineClassify.getParentId().equals(0L)) {
            OfflineClassify parent = offlineClassifyMapper.selectById(offlineClassify.getParentId());
            if (null != parent) {
                offlineClassify.setLayer(parent.getLayer() + 1);
                offlineClassify.setRootId(parent.getRootId());
                offlineClassify.setPath(parent.getPath() + "," + id);
            }
        } else {
            offlineClassify.setLayer(1);
            offlineClassify.setRootId(id);
            offlineClassify.setParentId(0L);
            offlineClassify.setPath(String.valueOf(id));
        }

        OfflineClassify offlineClassifyEx = new OfflineClassify();
        offlineClassifyEx.setDeleted(DeletedEnum.NO.getValue());
        offlineClassifyEx.setCompanyId(offlineClassify.getCompanyId());
        offlineClassifyEx.setSiteId(offlineClassify.getSiteId());

        EntityWrapper<OfflineClassify> ew = new EntityWrapper<>(offlineClassifyEx);
        ew.orderBy("sort", false);
        OfflineClassify lastBrother = offlineClassifyEx.selectOne(ew);
        if (null != lastBrother) {
            offlineClassify.setSort(lastBrother.getSort() + 1);
        } else {
            offlineClassify.setSort(1);
        }

        return offlineClassifyMapper.insert(offlineClassify);
    }

    /**
     * 修改线下课程分类
     *
     * @param offlineClassify
     * @return
     */
    @Override
    public Integer updateOfflineClassify(OfflineClassify offlineClassify) {
        try {
            OfflineClassify oldOfflineClassify = offlineClassify.selectById();
            if (oldOfflineClassify == null) {
                // 如果根据传过来的ID在数据库里面未找到数据，返回标识符-1
                return -1;
            }
            OfflineClassify exsitOfflineClassify = new OfflineClassify();
            exsitOfflineClassify.setName(offlineClassify.getName());
            exsitOfflineClassify.setParentId(offlineClassify.getParentId());
            exsitOfflineClassify.setDeleted(0);
            exsitOfflineClassify.setSiteId(ContextHolder.get().getSiteId());
            EntityWrapper<OfflineClassify> a = new EntityWrapper<>(exsitOfflineClassify);
            a.notIn("id", oldOfflineClassify.getId());
            int count = exsitOfflineClassify.selectCount(a);
            if (count > 0) {
                // 如果同级下存在相同名称的分类，返回标识符-2
                return -2;
            }


            return offlineClassifyMapper.updateById(offlineClassify);
        } catch (Exception e) {
            logger.error("修改分类失败");
            return 0;
        }
    }

    /**
     * 线下课程分类排序
     *
     * @param list
     * @return
     */
    @Override
    public Integer offlineClassifySort(List<OfflineClassify> list) {

        if (list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                try {
                    offlineClassifyMapper.updateById(list.get(i));
                } catch (Exception e) {
                    logger.error("排序失败", e);
                    return 0;
                }
            }
        }
        return 1;
    }

    /**
     * 线下课程分类分页查询
     *
     * @param companyId
     * @param name
     * @param orgIds
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<OfflineClassifyVo> selectListByCompanyId(Long companyId, String name, List<Long> orgIds, Long siteId, Integer pageNo, Integer pageSize) {
        HQueryUtil.startHQ(OfflineClassify.class);
        Page<OfflineClassifyVo> page = new Page<OfflineClassifyVo>(pageNo, pageSize);
        List<OfflineClassify> offlineClassifies = this.baseMapper.selectListByCompanyId(companyId, name, orgIds, siteId, new RowBounds(page.getOffset(), page.getLimit()));
        List<OfflineClassifyVo> vos = null;
        if (CollectionUtils.isNotEmpty(offlineClassifies)) {
            vos = new ArrayList<>(offlineClassifies.size());
            OfflineClassifyVo vo = null;
            for (OfflineClassify classify : offlineClassifies) {
                vo = new OfflineClassifyVo();
                vo.setCompanyId(classify.getCompanyId());
                vo.setName(classify.getName());
                vo.setId(classify.getId());
                vo.setDescription(classify.getDescription());
                vo.setLayer(classify.getLayer());
                vo.setOrgId(classify.getOrgId());
                vo.setRootId(classify.getRootId());
                vo.setSiteId(classify.getSiteId());
                vo.setSort(classify.getSort());
                vo.setSource(classify.getSource());
                vo.setCreateTime(classify.getCreateTime());
                vos.add(vo);
            }
        }
        HQueryUtil.startHQ(OfflineClassify.class);
        Integer count = this.baseMapper.selectCountByCompanyId(companyId, name, orgIds, siteId);
        page.setTotal(count);
        page.setRecords(vos);
        return page;
    }

    /**
     * 分类层级查询线下课程分类
     *
     * @param companyId
     * @param siteId
     * @param layer
     * @return
     */
    @Override
    public List<OfflineClassify> selectOfflineClassifyByLayer(Long companyId, Long siteId, Integer layer) {
        HQueryUtil.startHQ(OfflineClassify.class);
        return this.baseMapper.selectOfflineClassifyByLayer(companyId, siteId, layer);
    }

    /**
     * 根据名称模糊查询分类
     *
     * @return
     */
    @Override
    public List<OfflineClassify> listOfflineClassify() {
        RequestContext context = ContextHolder.get();

        OfflineClassify classify = new OfflineClassify();
        classify.setCompanyId(context.getCompanyId());
        classify.setSiteId(context.getSiteId());
        classify.setDeleted(DeletedEnum.NO.getValue());
        EntityWrapper<OfflineClassify> entityWrapper = new EntityWrapper<>(classify);
        entityWrapper.orderBy("path");
        HQueryUtil.startHQ(OfflineClassify.class);
        List<OfflineClassify> list = classify.selectList(entityWrapper);
        return list;
    }

}
