package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.course.application.domain.OfflineCourse;
import com.yizhi.course.application.eum.DeletedEnum;
import com.yizhi.course.application.mapper.OfflineCourseMapper;
import com.yizhi.course.application.service.IOfflineCourseService;
import com.yizhi.course.application.vo.OfflineCourseVo;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 线下课程表 服务实现类
 * </p>
 *
 * @author huangliang123
 * @since 2018-10-16
 */
@Service
@Transactional
public class OfflineCourseServiceImpl extends ServiceImpl<OfflineCourseMapper, OfflineCourse> implements IOfflineCourseService {

    private Logger logger = LoggerFactory.getLogger(OfflineCourseServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private OfflineCourseMapper offlineCourseMapper;

    /**
     * 添加线下课程
     *
     * @param offlineCourseVo
     * @return
     */
    @Override
    public OfflineCourseVo save(OfflineCourseVo offlineCourseVo) {
        if (!this.verifyCourseCode(null, offlineCourseVo.getCode(), offlineCourseVo.getSiteId())) {
            return null;
        }
        try {
            OfflineCourse offlineCourse = new OfflineCourse();
            offlineCourse.setId(idGenerator.generate());
            offlineCourse.setName(offlineCourseVo.getName());
            offlineCourse.setCode(offlineCourseVo.getCode());
            offlineCourse.setImage(offlineCourseVo.getImage());
            offlineCourse.setOfflineClassifyId(offlineCourseVo.getOfflineClassifyId());
            offlineCourse.setStatus(offlineCourseVo.getStatus());
            if (offlineCourseVo.getStatus() != null) {
                offlineCourse.setStatus(2);
            }
            offlineCourse.setDeleted(DeletedEnum.NO.getValue());
            offlineCourse.setAuthorUnit(offlineCourseVo.getAuthorUnit());
            offlineCourse.setCompanyId(offlineCourseVo.getCompanyId());
            offlineCourse.setSiteId(offlineCourseVo.getSiteId());
            offlineCourse.setOrgId(offlineCourseVo.getOrgId());
            offlineCourse.setDescription(offlineCourseVo.getDescription());
            offlineCourse.setTags(offlineCourseVo.getTags());
            offlineCourse.setOssUrl(offlineCourseVo.getOssUrl());

            offlineCourse.setCreateTime(new Date());
            offlineCourse.setCreateByName(ContextHolder.get().getAccountName());
            offlineCourse.setCreateById(ContextHolder.get().getAccountId());

            this.baseMapper.insert(offlineCourse);

            return offlineCourseVo;
        } catch (Exception e) {
            System.out.println(e.getLocalizedMessage());
            return null;
        }

    }

    /**
     * 修改线下课程
     *
     * @param offlineCourseVo
     * @return
     */
    @Override
    public Boolean update(OfflineCourseVo offlineCourseVo) {
        if (!this.verifyCourseCode(offlineCourseVo.getId(), offlineCourseVo.getCode(), offlineCourseVo.getSiteId())) {
            return Boolean.FALSE;
        }
        try {
            OfflineCourse offlineCourse = this.baseMapper.selectById(offlineCourseVo.getId());
            offlineCourse.setName(offlineCourseVo.getName());
            offlineCourse.setCode(offlineCourseVo.getCode());
            offlineCourse.setImage(offlineCourseVo.getImage());
            offlineCourse.setTags(offlineCourseVo.getTags());
            offlineCourse.setOfflineClassifyId(offlineCourseVo.getOfflineClassifyId());
            offlineCourse.setAuthorUnit(offlineCourseVo.getAuthorUnit());
            offlineCourse.setDescription(offlineCourseVo.getDescription());
            offlineCourse.setOssUrl(offlineCourseVo.getOssUrl());
            offlineCourse.setDeleted(DeletedEnum.NO.getValue());
            offlineCourse.setUpdateById(offlineCourseVo.getUpdateById());
            offlineCourse.setUpdateByName(offlineCourseVo.getUpdateByName());
            offlineCourse.setUpdateTime(offlineCourseVo.getUpdateTime());
            this.baseMapper.updateById(offlineCourse);
            return Boolean.TRUE;

        } catch (Exception e) {
            System.out.println(e.getLocalizedMessage());
            return Boolean.FALSE;
        }

    }

    /**
     * 验证线下课程code唯一性
     *
     * @param id
     * @param code
     * @param siteId
     * @return
     */
    @Override
    public Boolean verifyCourseCode(Long id, String code, Long siteId) {
        Integer count = this.baseMapper.verifyCourseCode(id, code, siteId);
        if (count > 0) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 线下课程分页查询
     *
     * @param name
     * @param offlineClassifyId
     * @param status
     * @param companyId
     * @param siteId
     * @param orgIds
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<OfflineCourse> searchPage(String name, Long offlineClassifyId, Integer status, Long companyId, Long siteId, List<Long> orgIds, int pageNo, int pageSize) {
        HQueryUtil.startHQ(OfflineCourse.class);
        Page<OfflineCourse> page = new Page<OfflineCourse>(pageNo, pageSize);
        page.setRecords(this.baseMapper.searchPage(name, offlineClassifyId, status, companyId, siteId, orgIds, null, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.searchPageCount(name, offlineClassifyId, status, companyId, siteId, orgIds, null);
        if (!StringUtils.isEmpty(count)) {
            page.setTotal(count);
        }

        return page;
    }

    /**
     * 全文检索
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param orgId
     * @return
     */
    @Override
    public List<OfflineCourse> selectCourseList(String name, Long companyId, Long siteId, Long orgId) {
        return this.baseMapper.selectCourseList(name, companyId, siteId, orgId);
    }

    /**
     * 上传文件
     *
     * @param offlineCourseVo
     * @return
     */
    @Override
    public Integer upload(OfflineCourseVo offlineCourseVo) {

        OfflineCourse offlineCourse = new OfflineCourse();
        Integer up = offlineCourseVo.getUp();
        if (null != up && up.equals(1)) {
            offlineCourse.setStatus(0);
        }
        offlineCourse.setId(offlineCourseVo.getId());
        offlineCourse.setOssUrl(offlineCourseVo.getOssUrl());
        offlineCourse.setFileName(offlineCourseVo.getFileName());
        offlineCourse.setUpdateById(offlineCourseVo.getUpdateById());
        offlineCourse.setUpdateByName(offlineCourseVo.getUpdateByName());
        offlineCourse.setUpdateTime(offlineCourseVo.getUpdateTime());
        return this.baseMapper.updateById(offlineCourse);
    }
}
