package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.course.application.controller.CourseStudyController;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.mapper.ScormMapper;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.vo.ProjectsDurationParamVO;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.date.DateUtil;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * scorm 课程包解析保存 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ScormServiceImpl extends ServiceImpl<ScormMapper, Scorm> implements IScormService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ScormServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private CloudEventPublisher cloudEventPublisher;
    @Autowired
    private PointRedisFeignClients pointRedisFeignClients;
    /*@Autowired
    private AmqpTemplate amqpTemplate;*/
    @Autowired
    private IRecordeService recordeService;
    @Autowired
    private IChapterService chapterService;
    @Autowired
    private ICourseStudentStatusService studentStatusService;
    @Autowired
    private IScormRecordService scormRecordService;

    @Autowired
    private ICourseStudyHourDetailedService studyHourDetailedService;

    @Autowired
    private ICourseStudyHourService studyHourService;

    @Autowired
    private ICourseService courseService;

    @Autowired
    private ICourseMqRecordService courseMqRecordService;

    @Resource
    private ScormMapper scormMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private TaskExecutor taskExecutor;

    private static final String COMPLETED_STR = "completed,passed";

    // 课程积分发放MQ队列名称
    private static final String COURSE_MQ_SEND_SCORE = "course";

    private static final String EXPERIENCE_COURSE = "experienceCourse";

    @Override
    public ScormRecord saveScormRecord(ScormRecord scormRecord) {

        //return getScormRecord(scormRecord);

        // 上下文信息
        RequestContext context = ContextHolder.get();
        if(scormRecord.getAccountId() == null){
            scormRecord.setAccountId(context.getAccountId());
        }
        scormRecord.setSiteId(context.getSiteId());
        scormRecord.setCompanyId(context.getCompanyId());
        // 获取章节信息
        Chapter chapter = chapterService.selectById(scormRecord.getChapterId());
        // 获取scorm信息
        //Scorm scorm = this.baseMapper.selectById(scormRecord.getScormId());

        // 解析JsonData数据
        String data = scormRecord.getData();
        JSONObject json = JSON.parseObject(data);
        String status = (String) json.get("cmi.core.lesson_status");
        String score = String.valueOf(json.get("cmi.core.score.raw"));
        String totalTime = String.valueOf(json.get("cmi.core.total_time"));
        long totalTimeSeconds = countSeconds(totalTime);
        String scormStartTimeT = json.get("scormStartTime").toString();
        String terminalType = (String) json.get("terminal_type");
        String sessionTime = (String) json.get("cmi.core.session_time");
        long sessionTimeSeconds = countSeconds(sessionTime);
        // 把时间精度从毫秒到秒转换
        Date scormStartTimeDate;
        try{
            SimpleDateFormat timeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            LOGGER.info("scormStartTimeT={}",scormStartTimeT);
            scormStartTimeDate = timeFormat.parse(timeFormat.format( new Date(Long.valueOf(scormStartTimeT).longValue()) ));
            LOGGER.info("scormStartTimeDate={}", DateUtil.format(scormStartTimeDate,"yyyy-MM-dd-HH:mm:ss.SSS"));
        }catch (Exception e){
            LOGGER.info("转换日期参赛异常{}",e);
            scormStartTimeDate = new Date();
        }
        // 是否以前也有记录
        EntityWrapper<ScormRecord> ew = QueryUtil.condition(new ScormRecord());
        ew.eq("scorm_id",scormRecord.getScormId());
        ew.eq("account_id",context.getAccountId() );
        ew.eq("start_study_time", scormStartTimeDate);
        ScormRecord scormRecordDB = scormRecordService.selectOne(ew);
        //LOGGER.info("scormRecordDB={}",JSON.toJSONString(scormRecordDB));

        // scorm 完成状态
        int scormStatus = 0;
        // 判断状态是否是完成
        if (org.apache.commons.lang3.StringUtils.indexOf(COMPLETED_STR, status.trim()) >= 0) {
            scormStatus = 1;
        }
        boolean recodeGroup = false; // 是否是
        // 当前时间
        Date now = new Date();
        if(scormRecordDB == null){ //数据库不存在就新增
            scormRecord.setId(idGenerator.generate());
            scormRecord.setScormId(scormRecord.getScormId());
            scormRecord.setAccountId(context.getAccountId());
            scormRecord.setData(data);
            scormRecord.setFinished(scormStatus);
            if(!StringUtils.isEmpty(score)){
                scormRecord.setScore(Float.valueOf(score));
            }
            scormRecord.setCreateTime(now);
            scormRecord.setUpdateTime(now);
            scormRecord.setSiteId(context.getSiteId());
            scormRecord.setCompanyId(context.getCompanyId());
            scormRecord.setDeleted(0);

            scormRecord.setTotalStudySeconds(totalTimeSeconds);
            scormRecord.setChapterId(scormRecord.getChapterId());
            scormRecord.setCurrentStudySeconds(sessionTimeSeconds);
            scormRecord.setStartStudyTime(scormStartTimeDate);
            scormRecord.setTerminalType(terminalType);
            scormRecordService.insert(scormRecord);
        }else {
            recodeGroup = true;
            // 存在则更新数据
            if(scormRecordDB.getFinished().intValue()==1 && scormStatus==0){
                ScormRecord scormRecordUp = new ScormRecord();
                scormRecordUp.setId(scormRecordDB.getId());
                //scormRecordUp.setUpdateTime(now);
                scormRecordUp.setTerminalType("标记完成状态从1到0更新");
                scormRecordService.updateById(scormRecordUp);
            }else{
                ScormRecord scormRecordUp = new ScormRecord();
                scormRecordUp.setFinished(scormStatus);
                Float oldScoreObj =  scormRecordDB.getScore();
                float oldScore = oldScoreObj==null?0:oldScoreObj.floatValue();
                if(!StringUtils.isEmpty(score) && Float.valueOf(score) > oldScore){
                    scormRecordUp.setScore(Float.valueOf(score));
                }

                scormRecordUp.setUpdateTime(now);
                scormRecordUp.setData(data);

                Long totalStudySecondsObj =  scormRecordDB.getTotalStudySeconds();
                long totalStudySeconds = totalStudySecondsObj==null?0:totalStudySecondsObj.longValue();
                if(totalTimeSeconds>totalStudySeconds){
                    scormRecordUp.setTotalStudySeconds(totalTimeSeconds);
                }

                Long currentStudySecondsObj = scormRecordDB.getCurrentStudySeconds();
                long currentStudySeconds = currentStudySecondsObj == null?0:currentStudySecondsObj.longValue();
                if(sessionTimeSeconds>currentStudySeconds){
                    scormRecordUp.setCurrentStudySeconds(sessionTimeSeconds);
                }

                scormRecordUp.setId(scormRecordDB.getId());
                scormRecordService.updateById(scormRecordUp);
            }
        }
        // 查询学习记录表
        EntityWrapper<Recorde> ewR = QueryUtil.condition(new Recorde());
        ewR.eq("course_id",chapter.getCourseId());
        ewR.eq("account_id",context.getAccountId());
        //ewR.eq("site_id", context.getSiteId());
        //ewR.eq("data_type",0);
        ewR.orderBy("id",false);
        List<Recorde> oldRecord = recordeService.selectList(ewR);

        if (CollectionUtils.isEmpty(oldRecord)) {
            // 入库学习记录
            Recorde recordeNew = new Recorde();
            recordeNew.setId(idGenerator.generate());
            recordeNew.setAccountId(context.getAccountId());
            recordeNew.setCourseId(chapter.getCourseId());
            recordeNew.setChapterId(chapter.getId());
            recordeNew.setStartTime(scormStartTimeDate);
            recordeNew.setEndTime(scormRecord.getCreateTime());
            recordeNew.setDuration(Integer.valueOf(totalTimeSeconds+""));
            recordeNew.setOvered(scormStatus);
            recordeNew.setExitstates(1);
            recordeNew.setValid(1);
            recordeNew.setCreateTime(now);
            recordeNew.setOrgId(context.getOrgId());
            recordeNew.setCompanyId(context.getCompanyId());
            recordeNew.setSiteId(context.getSiteId());
            recordeNew.setTerminalType("新增");
            recordeNew.setTerminalDevice("scorm学习时间:"+ DateUtil.toMinute(scormRecord.getCreateTime()));
            recordeNew.setSessionId("scorm:"+context.getAccountId()+":"+scormRecord.getScormId());
            recordeNew.setDataType(0);
            recordeService.insert(recordeNew);
        }else{
            Recorde recordeOld = oldRecord.get(0);
            Recorde recordeUp = new Recorde();
            /*if(Objects.isNull(recordeOld.getCompanyId()) || recordeOld.getCompanyId().longValue()==0L){
                recordeUp.setCompanyId(context.getCompanyId());
            }
            if(Objects.isNull(recordeOld.getSiteId()) || recordeOld.getSiteId().longValue()==0L){
                recordeUp.setSiteId(context.getSiteId());
            }
            if(Objects.isNull(recordeOld.getOrgId())|| recordeOld.getSiteId().longValue()==0L){
                recordeUp.setOrgId(context.getOrgId());

            }*/
            recordeUp.setCompanyId(context.getCompanyId());
            recordeUp.setSiteId(context.getSiteId());
            recordeUp.setOrgId(context.getOrgId());
            if(recordeOld.getOvered().intValue()==1 && scormStatus==0){
                recordeUp.setId(recordeOld.getId());
                recordeUp.setTerminalType("数据状态从1到0的更新。");
                recordeService.updateById(recordeUp);
            }else{
                if(Objects.isNull(recordeOld.getStartTime())){
                    recordeUp.setStartTime(scormStartTimeDate);
                }
                recordeUp.setId(recordeOld.getId());
                recordeUp.setEndTime(scormRecord.getCreateTime());
                Integer oldDurationObj = recordeOld.getDuration();
                int oldDuration = oldDurationObj== null?0:oldDurationObj.intValue();
                if(totalTimeSeconds > oldDuration){
                    recordeUp.setDuration(Integer.valueOf(totalTimeSeconds+""));
                }
                recordeUp.setOvered(scormStatus);
                recordeService.updateById(recordeUp);
            }

        }

        // 插入学习状态表，并标识课程是否已经学习完成
        // 查询课程学习状态
        EntityWrapper<CourseStudentStatus> studentStatusEw = QueryUtil.condition(new CourseStudentStatus());
        studentStatusEw.eq("course_id",chapter.getCourseId()).eq("account_id",context.getAccountId());
        studentStatusEw.orderBy("id",false);
        CourseStudentStatus studentStatus = studentStatusService.selectOne(studentStatusEw);
        if (studentStatus == null) {
            studentStatus = new CourseStudentStatus();
            studentStatus.setId(idGenerator.generate());
            studentStatus.setCompanyId(context.getCompanyId());
            studentStatus.setSiteId(context.getSiteId());
            studentStatus.setOrgId(context.getOrgId());
            studentStatus.setAccountId(context.getAccountId());
            studentStatus.setCourseId(chapter.getCourseId());
        }
        boolean retStatus = false;
        studentStatus.setLastStudentTime(now); // 最后学习时间
        if(scormStatus==1){ // 如果scorm学习状态=1（完成）
            retStatus = true;
            if(studentStatus.getFirstFinishTime() == null){ // 首次完成时间为空，则本次完成时间就三完成时间
                studentStatus.setFirstFinishTime(now);
            }
            studentStatus.setStudentStatus(1);
            studentStatus.setPercent(100);

            // 发送课程学时
            Course course = courseService.selectById(chapter.getCourseId());
            scormRecord.setSiteId(course.getSiteId());
            Float totalDuration = course.getTotalDuration();
            if (Objects.nonNull(totalDuration) && Boolean.TRUE.equals(course.getTotalDurationGrant())) {
                // 判断是否是已经已经发放学时的
                String keyWord = ""+scormStartTimeDate.getTime();
                EntityWrapper<CourseStudyHourDetailed> ewDetai = QueryUtil.condition(new CourseStudyHourDetailed());
                ewDetai.eq("account_id",context.getAccountId()).eq("course_id",course.getId())
                        .eq("key_word",keyWord);
                int countNum = studyHourDetailedService.selectCount(ewDetai);
                if(countNum == 0){
                    // 获取课程的证书
                    if(course.getGrantCertificate()){ // 如果需要颁发证书
                        // 发放课程证书
                        String[] certificates = course.getCertificates().split(",");
                        Map<String, Object> map = new HashMap<>();
                        map.put("siteId", context.getSiteId());
                        map.put("accountId", context.getAccountId());
                        map.put("courseId", course.getId());
                        map.put("courseName", course.getName());
                        map.put("bizdType", 1);
                        for(String certificateId : certificates){
                            map.put("certificateId", certificateId);
                            cloudEventPublisher.publish("myCertificate", new EventWrapper<Map>(Long.valueOf(certificateId), map));
                        }
                        saveSendRQRecode("scorm课程发放证书", chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
                    }

                    CourseStudyHour hourEntiy = new CourseStudyHour();
                    hourEntiy.setTotalDuration(course.getTotalDuration());
                    hourEntiy.setAccountId(context.getAccountId());
                    hourEntiy.setCompanyId(context.getCompanyId());
                    hourEntiy.setCourseId(chapter.getCourseId());
                    hourEntiy.setSiteId(context.getSiteId());
                    hourEntiy.setOrgId(context.getOrgId());
                    this.addCourseStudyHour(hourEntiy,keyWord);
                }
            }
            // 发送经验
            String mqSendStatu =  toPoint(chapter, scormRecord, EXPERIENCE_COURSE,context.getOrgId());
            //LOGGER.info("插入的学习记录对象Record={}",JSON.toJSONString(recorde));
            saveSendRQRecode("scorm课程按节完成发放经验"+"-"+mqSendStatu, chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
            // 发送积分
            Integer scoreTmp = chapter.getScore();
            if(scoreTmp != null && scoreTmp.intValue()>0){
                mqSendStatu =  toPoint(chapter, scormRecord, COURSE_MQ_SEND_SCORE,context.getOrgId());
                saveSendRQRecode("scorm课程按节完成发放积分"+"-"+mqSendStatu, chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
            }

        }else{
            if(studentStatus.getStudentStatus() == null || studentStatus.getStudentStatus() == 0){
                studentStatus.setStudentStatus(0);
                studentStatus.setPercent(0);
            }
        }

        // 保存课程学习状态
        studentStatusService.insertOrUpdate(studentStatus);

        // 发送培训消息
        try {
            /*cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE,
                    new EventWrapper<>(chapter.getCourseId(),
                            TrainingProjectEvent.getInstanceCourse(chapter.getCourseId(), context.getAccountId(),
                                    now, totalTimeSeconds, retStatus, context.getSiteId())));*/
            sendFinishStatus(chapter.getCourseId(), context.getAccountId(), totalTimeSeconds, retStatus, context.getSiteId(), now);
            saveSendRQRecode("scorm课程按节发送课程完成状态成功"+"-"+retStatus, chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
        } catch (Exception e) {
            LOGGER.info("往培训项目发送消息发送异常");
            saveSendRQRecode("scorm课程按节发送课程完成状态异常，完成状态"+e.getMessage(), chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
        }

        return scormRecord;
    }


    /**
     * 给项目发送课程完成消息
     */
    private void sendFinishStatus(Long courseId, Long accountId, Long duration, boolean finishStatus, Long siteId, Date sendDate) {
        //Date sendDate = new Date();
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                try {
                    cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, new EventWrapper<>(
                            courseId, // 课程id
                            TrainingProjectEvent.getInstanceCourse(
                                    courseId, // 课程id
                                    accountId, // 账号id
                                    sendDate,
                                    duration,//学习时长
                                    finishStatus,//完成状态
                                    siteId//站点id
                            )
                    ));
                    cloudEventPublisher.publish(QueueConstant.ALBUM_EVENT_QUEUE, new EventWrapper<>(
                            courseId, // 课程id
                            TrainingProjectEvent.getInstanceCourse(
                                    courseId, // 课程id
                                    accountId, // 账号id
                                    sendDate,
                                    duration,//学习时长
                                    finishStatus,//完成状态
                                    siteId//站点id
                            )
                    ));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    private ScormRecord getScormRecord(ScormRecord scormRecord) {
        LOGGER.info("scorm课程保存处理请求进入时间={}，请求参数={}", System.currentTimeMillis(), JSON.toJSONString(scormRecord));
        RequestContext context = ContextHolder.get();
        // 获取章节信息，替换
        Chapter chapter = chapterService.selectById(scormRecord.getChapterId());
        // 获取scorm信息
        Scorm scorm = this.baseMapper.selectById(scormRecord.getScormId());
        if (scorm == null) {
            LOGGER.error("未找到 scorm，id：" + scormRecord.getScormId());
            return null;
        }
        Date now = new Date();
        // 解析JsonData数据
        String data = scormRecord.getData();
        JSONObject json = JSON.parseObject(data);
        String status = (String) json.get("cmi.core.lesson_status");
        String score = String.valueOf(json.get("cmi.core.score.raw"));
        String totalTime = String.valueOf(json.get("cmi.core.total_time"));
        String scormStartTimeT = json.get("scormStartTime").toString();
        // 把时间精度从毫秒到秒转换
        //long scormStartTime = Long.valueOf(scormStartTimeT.substring(0, scormStartTimeT.length() - 3));
        //Date scormStartTimeDate = new Date((scormStartTime * 1000));
        Date scormStartTimeDate = null;
        try{
            SimpleDateFormat timeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            LOGGER.info("scormStartTimeT={}",scormStartTimeT);
            //scormStartTimeDate = new Date(Long.valueOf(scormStartTimeT).longValue());
            scormStartTimeDate = timeFormat.parse(timeFormat.format( new Date(Long.valueOf(scormStartTimeT).longValue()) ));
            LOGGER.info("scormStartTimeDate={}", DateUtil.format(scormStartTimeDate,"yyyy-MM-dd-HH:mm:ss.SSS"));
        }catch (Exception e){
            LOGGER.info("转换日期参赛异常{}",e);
            scormStartTimeDate = new Date();
        }

        String terminalType = (String) json.get("terminal_type");
        String sessionTime = (String) json.get("cmi.core.session_time");
        // 是否以前也有记录
        EntityWrapper<ScormRecord> ew = QueryUtil.condition(new ScormRecord());
        ew.eq("scorm_id",scormRecord.getScormId());
        ew.eq("account_id",context.getAccountId() );
        ew.eq("start_study_time", scormStartTimeDate);
        ScormRecord oldScormRecord = scormRecordService.selectOne(ew);
        LOGGER.info("oldScormRecord={}",JSON.toJSONString(oldScormRecord));

        // scorm 完成状态
        boolean scormStatus = false;

        scormRecord.setFinished(0); // 默认状态设置成未完成
        if (!StringUtils.isEmpty(status)) {
            // 判断状态是否是完成
            if (org.apache.commons.lang3.StringUtils.indexOf(COMPLETED_STR, status.trim()) >= 0) {
                scormRecord.setFinished(1);
                scormStatus = true;
            }
        }
        // 设定学习分数
        if (!StringUtils.isEmpty(score) && !score.equals("null") && !score.trim().equals("") ) {
            scormRecord.setScore(Float.valueOf(score));
        }else {
            scormRecord.setScore(0f);
        }
        // 设置总学习时间
        if (!StringUtils.isEmpty(totalTime) && !totalTime.equals("null") && !totalTime.trim().equals("") ) {
            long seconds = countSeconds(totalTime);
            scormRecord.setTotalStudySeconds(seconds);
        }else {
            scormRecord.setTotalStudySeconds(0L);
        }
        // 设置本次学习时间
        if (!StringUtils.isEmpty(sessionTime) && !sessionTime.equals("null")  && !sessionTime.trim().equals("") ) {
            long seconds = countSeconds(sessionTime);
            scormRecord.setCurrentStudySeconds(seconds);
        }else {
            scormRecord.setCurrentStudySeconds(0L);
        }
        // 设定设备类型
        scormRecord.setTerminalType(terminalType);


        // 存在旧的数据就进行更新
        if (oldScormRecord != null) {
            LOGGER.info("存在旧的批处scorm学习原始记录");
            scormRecord.setId(oldScormRecord.getId());
            Integer oldFinished = oldScormRecord.getFinished();
            if(Integer.valueOf(1).equals(oldFinished)){
                scormRecord.setFinished(1);
                scormStatus = false;
            }
            Float oldScore = oldScormRecord.getScore();
            if(oldScore.compareTo(scormRecord.getScore()) ==1 ){ //如果指定的数小于参数返回 -1。如果指定的数与参数相等返回0。
                scormRecord.setScore(oldScore);
            }
            Long oldTotalStudySeconds = oldScormRecord.getTotalStudySeconds();
            if(oldTotalStudySeconds.compareTo(scormRecord.getTotalStudySeconds()) == 1){
                scormRecord.setTotalStudySeconds(oldTotalStudySeconds);
            }
            Long oldCurrentStudySeconds = oldScormRecord.getCurrentStudySeconds();
            if(oldTotalStudySeconds.compareTo(scormRecord.getCurrentStudySeconds()) == 1){
                scormRecord.setCurrentStudySeconds(oldCurrentStudySeconds);
            }

            scormRecord.setUpdateTime(now);
            scormRecord.setAccountId(oldScormRecord.getAccountId());
            scormRecord.setCompanyId(oldScormRecord.getCompanyId());
            scormRecord.setSiteId(oldScormRecord.getSiteId());

            scormRecordService.updateById(scormRecord);
        } else {
            LOGGER.info("不存在旧的批处scorm学习原始记录");
            // 插入新纪录(如果单次学习时长为0，就是要总学习时长填充
            if (scormRecord.getCurrentStudySeconds() == 0) {
                scormRecord.setCurrentStudySeconds(scormRecord.getTotalStudySeconds());
            }
            scormRecord.setId(idGenerator.generate());
            scormRecord.setAccountId(context.getAccountId());
            scormRecord.setCompanyId(context.getCompanyId());
            scormRecord.setSiteId(context.getSiteId());
            scormRecord.setCreateTime(now);
            scormRecord.setUpdateTime(now);
            scormRecord.setDeleted(0);
            scormRecord.setStartStudyTime(scormStartTimeDate);
            scormRecordService.insert(scormRecord);
        }
        // 查询学习记录表
        EntityWrapper<Recorde> ewR = QueryUtil.condition(new Recorde());
        ewR.eq("course_id",chapter.getCourseId());
        ewR.eq("account_id",context.getAccountId());
        ewR.eq("site_id", context.getSiteId());
        //ewR.eq("data_type",0);
        ewR.orderBy("id",false);
        List<Recorde> oldRecord = recordeService.selectList(ewR);
        // 初始化这次要生成的学习记录
        Recorde recorde = new Recorde();
        long successStatus = 0; // 完成状态
        recorde.setOvered(scormRecord.getFinished());// 完成状态默认根据请求过来状态更新
        // 如果本次状态是未完成状态 并且有旧的学习记录（那么旧的学习记录有完成状态，也表示学习完成
        /*if (org.apache.commons.lang3.StringUtils.indexOf(COMPLETED_STR, status.trim()) < 0 && !CollectionUtils.isEmpty(oldRecord)) {
            // 查看是否存在已完成的scorm课程学习记录
            Optional<Recorde> findRecorde = oldRecord.stream().filter(obj -> Integer.valueOf(1).equals(obj.getOvered())).findAny();
            if (findRecorde.isPresent()) {
                recorde.setOvered(1);
            }
        }*/
        if (CollectionUtils.isEmpty(oldRecord)) {
            Long tmpTotalStudySeconds = scormRecord.getTotalStudySeconds();
            // 入库学习记录
            recorde.setSiteId(context.getSiteId());
            recorde.setCourseId(chapter.getCourseId());
            recorde.setCompanyId(context.getCompanyId());
            recorde.setChapterId(chapter.getId());
            recorde.setAccountId(context.getAccountId());
            recorde.setEndTime(scormRecord.getCreateTime());
            recorde.setDuration(tmpTotalStudySeconds.intValue());
            recorde.setStartTime(scormRecord.getCreateTime());
            recorde.setId(idGenerator.generate());
            recorde.setSessionId(String.valueOf(context.getAccountId()));
            recorde.setExitstates(1);
            recorde.setOrgId(context.getOrgId());
            recorde.setCreateTime(scormRecord.getCreateTime());
            recorde.setValid(1);
            recorde.setTerminalDevice("scorm学习时间:"+ DateUtil.toMinute(scormRecord.getCreateTime()));
            recorde.setTerminalType("");
            recorde.setDataType(0);
            recorde.insert();
        } else {
            int size = oldRecord.size();
            Recorde recordeExits = oldRecord.get(0);
            Integer dur = scormRecord.getTotalStudySeconds().intValue();
            if(recordeExits.getDuration().compareTo(dur) == -1){
                recorde.setDuration(dur);
            }
            Long tmpTotalStudySeconds = scormRecord.getTotalStudySeconds();
            if (size > 1) {
/*                long sumDuration  = oldRecord.stream().mapToLong(Recorde::getDuration).sum();
                if(Objects.nonNull(tmpTotalStudySeconds)){
                    tmpTotalStudySeconds+=sumDuration;
                }*/
                recordeService.deleteBatchIds(oldRecord.stream().map(objKey -> objKey.getId()).collect(Collectors.toList()));
            }
            // 查看是否存在已完成的scorm课程学习记录
            Optional<Recorde> findRecorde = oldRecord.stream().filter(obj -> Integer.valueOf(1).equals(obj.getOvered())).findAny();
            if (findRecorde.isPresent()) {
                recorde.setOvered(1);
            }
            recorde.setEndTime(scormRecord.getCreateTime());
            recorde.setId(recordeExits.getId());
            recorde.setSiteId(recordeExits.getSiteId());
            recorde.setAccountId(recordeExits.getAccountId());
            recorde.setCourseId(recordeExits.getCourseId());
            recorde.setOrgId(recordeExits.getOrgId());
            recorde.setCompanyId(recordeExits.getCompanyId());
            //recorde.setOvered(scormRecord.getFinished());
            recordeService.updateById(recorde);
        }

        // 插入学习状态表，并标识课程是否已经学习完成
        // 查询课程学习状态
        EntityWrapper<CourseStudentStatus> studentStatusEw = QueryUtil.condition(new CourseStudentStatus());
        studentStatusEw.eq("course_id",recorde.getCourseId()).eq("account_id",recorde.getAccountId());
        studentStatusEw.orderBy("id",false);
        List<CourseStudentStatus> studentStatusList = studentStatusService.selectList(studentStatusEw);
        CourseStudentStatus studentStatus = null;

        // 计算完成状态
        Integer overedTem = recorde.getOvered();
        overedTem = overedTem == null ? 0 : overedTem;

        if (CollectionUtils.isEmpty(studentStatusList)) {
            studentStatus = new CourseStudentStatus();
            studentStatus.setId(idGenerator.generate());
            studentStatus.setCourseId(recorde.getCourseId());
            studentStatus.setAccountId(recorde.getAccountId());
            studentStatus.setCompanyId(recorde.getCompanyId());
            studentStatus.setSiteId(recorde.getSiteId());
            studentStatus.setOrgId(recorde.getOrgId());
            studentStatus.setStudentStatus(overedTem);
            studentStatus.setPercent(overedTem * 100);
        } else {
            studentStatus = studentStatusList.get(0);
            Integer oldStudentStatus = studentStatus.getStudentStatus();
            if(oldStudentStatus == null || oldStudentStatus.intValue()==0){
                studentStatus.setStudentStatus(overedTem);
                studentStatus.setPercent(overedTem * 100);
            }
            List<Long> removeIds = null;
            int size = studentStatusList.size();
            if ( size > 1) {
                removeIds = new ArrayList<>();
                for(int i=1;i<size;i++){
                    removeIds.add(studentStatusList.get(i).getId());
                }
            }
            if(!CollectionUtils.isEmpty(removeIds)){
                studentStatusService.deleteBatchIds(removeIds);
            }
        }
        // 保存课程学习状态
        studentStatus.setLastStudentTime(recorde.getEndTime());

        studentStatusService.insertOrUpdate(studentStatus);

        //Integer retFinished = scormRecord.getFinished();
        if (scormStatus) {
            // 发送课程学时
            Course course = courseService.selectById(chapter.getCourseId());
            Float totalDuration = course.getTotalDuration();
            if (Objects.nonNull(totalDuration) && Boolean.TRUE.equals(course.getTotalDurationGrant())) {
                CourseStudyHour hourEntiy = new CourseStudyHour();
                hourEntiy.setTotalDuration(course.getTotalDuration());
                hourEntiy.setAccountId(context.getAccountId());
                hourEntiy.setCompanyId(context.getCompanyId());
                hourEntiy.setCourseId(chapter.getCourseId());
                hourEntiy.setSiteId(context.getSiteId());
                hourEntiy.setOrgId(context.getOrgId());
                this.addCourseStudyHour(hourEntiy,"");
            }
            // 发送经验
            String mqSendStatu =  toPoint(chapter, scormRecord, EXPERIENCE_COURSE,context.getOrgId());
            //LOGGER.info("插入的学习记录对象Record={}",JSON.toJSONString(recorde));
            saveSendRQRecode("scorm课程按节完成发放经验"+"-"+mqSendStatu, chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
            // 发送积分
            Integer scoreTmp = chapter.getScore();
            if(scoreTmp != null && scoreTmp.intValue()>0){
                mqSendStatu =  toPoint(chapter, scormRecord, COURSE_MQ_SEND_SCORE,context.getOrgId());
                saveSendRQRecode("scorm课程按节完成发放积分"+"-"+mqSendStatu, chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
            }
        }

        boolean studyStatus = Integer.valueOf(1).equals(overedTem);
        // 发送培训消息
        try {
            cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE,
                    new EventWrapper<>(chapter.getCourseId(),
                            TrainingProjectEvent.getInstanceCourse(chapter.getCourseId(), context.getAccountId(),
                                    now, scormRecord.getTotalStudySeconds(), studyStatus, context.getSiteId())));
            saveSendRQRecode("scorm课程按节发送课程完成状态成功"+"-"+studyStatus, chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
        } catch (Exception e) {
            LOGGER.info("往培训项目发送消息发送异常");
            saveSendRQRecode("scorm课程按节发送课程完成状态异常，完成状态"+e.getMessage(), chapter.getCourseId(),scormRecord.getAccountId(), chapter.getId());
        }

        LOGGER.info("scorm课程保存处理请求完成时间={}", System.currentTimeMillis());
        return scormRecord;
    }

    /**
     * 保存发送MQ记录
     * @param envet 时间名称
     * @param courseId 课程id
     * @param accoutId 账户id
     * @param chapterId 章节id
     */
    private void saveSendRQRecode(String envet, Long courseId, Long accoutId, Long chapterId){
        Object isOpen = redisCache.get(CourseStudyController.IS_OPEN_COURSE_MQ_RECODE);
        if(isOpen != null && isOpen.equals("on")) {
            CourseMqRecord courseMqRecord = new CourseMqRecord();
            courseMqRecord.setId(idGenerator.generate());
            if (envet.length() > 250) {
                envet = envet.substring(0, 250);
            }
            courseMqRecord.setEnvetType(envet);
            courseMqRecord.setCourseId(courseId);
            courseMqRecord.setAccoutId(accoutId);
            courseMqRecord.setChapterId(chapterId);
            courseMqRecordService.insert(courseMqRecord);
        }
    }

    @Override
    public Integer getProjectsTotalDuration(ProjectsDurationParamVO paramVO) {
        RequestContext res = ContextHolder.get();
        Long duration = scormMapper.getProjectsTotalDuration(paramVO.getIsRealTime(),
                paramVO.getBizIds(), res.getAccountId());
        Long resultDuration = duration / 3600;
        Long remainder = duration % 3600;
        SimpleDateFormat smd = new SimpleDateFormat("");
        if (remainder > 0) {
            resultDuration++;
        }
        return Math.toIntExact(resultDuration);
    }

    @Override
    public Map<Long, Map<String, Object>> getCourseTotalDuration(List<Long> bizIds) {
        RequestContext res = ContextHolder.get();
        Map<Long, Map<String, Object>> resultMap = new HashMap<>(16);

        Map<Long, Map<String, Object>> durationMap = scormMapper.getCourseTotalDuration(bizIds, res.getAccountId());
        for (Long id : bizIds) {

            Map<String, Object> map = durationMap.get(id);

            Long duration = null;
            if (null != map) {
                String duration1 = String.valueOf(map.get("duration"));
                duration = Long.valueOf(duration1);
                String hourDuration = "";
                String minuteDuration = "";
                String secondDuration = "";
                if (null != duration) {
                    //小时
                    Long hour = duration / 3600;
                    if (hour > 0) {
                        hourDuration = hour + "小时";
                    }
                    //分
                    hour = duration % 3600;
                    hour = hour / 60;
                    if (hour > 0) {
                        minuteDuration = hour + "分";
                    }
                    //秒
                    hour = duration % 60;
                    if (hour > 0) {
                        secondDuration = hour + "秒";
                    }
                }
                String result = hourDuration + minuteDuration + secondDuration;
                Map<String, Object> map1 = new HashMap<>(2);
                map1.put("duration", result);
                Float score = (Float) map.get("score");
                map1.put("score", score);

                resultMap.put(id, map1);
            }

        }
        return resultMap;
    }

    /**
     * 发送课程学时
     *
     * @param entiy
     * @return
     */
    public Boolean addCourseStudyHour(CourseStudyHour entiy, String keyword) {
        Float totalDuration = entiy.getTotalDuration();

        if (Objects.nonNull(totalDuration)) {
            Float hour = null;
            if (totalDuration > 0f) {
                hour = studyHourService.studyHourSum(entiy.getSiteId(),
                        entiy.getAccountId(), entiy.getCourseId());
            }
            try {
                if (hour == null) {
                    entiy.setId(idGenerator.generate());
                    entiy.setCreateTime(new Date());
                    studyHourService.insert(entiy);
                } else if (totalDuration > hour) {
                    entiy.setId(idGenerator.generate());
                    entiy.setTotalDuration(totalDuration - hour);
                    entiy.setCreateTime(new Date());
                    studyHourService.insert(entiy);
                }
            } catch (Exception e) {
                LOGGER.info("课程插入失败，忽略插入{}", e);
            }

            // 插入学时明细
            CourseStudyHourDetailed entiyDetailed = new CourseStudyHourDetailed();
            BeanUtils.copyProperties(entiy, entiyDetailed, "id", "createTime");
            entiyDetailed.setId(idGenerator.generate());
            entiyDetailed.setCreateTime(new Date());
            entiyDetailed.setKeyWord(keyword);
            studyHourDetailedService.insert(entiyDetailed);
        }

        return true;
    }

    private static long countSeconds(String totalTime) {
        long seconds = 0L;
        if(totalTime == null || totalTime.trim().equals("")){
            return seconds;
        }
        String[] timeSplit = totalTime.split(":");
        String hours = timeSplit[0];
        if (!StringUtils.isEmpty(hours)) {
            seconds += Long.valueOf(timeSplit[0]) * 3600;
        }
        if (!StringUtils.isEmpty(timeSplit[1])) {
            seconds += Long.valueOf(timeSplit[1]) * 60;
        }
        if (!StringUtils.isEmpty(timeSplit[2])) {
            seconds += Double.valueOf(timeSplit[2]).longValue();
        }
        return seconds;
    }

    private String toPoint(Chapter chapter, ScormRecord scormRecord, String mqName, Long orgId) {

        PointParamVO pointParamVO = new PointParamVO();
        //事件名称
        pointParamVO.setEventName("pointCourse");
        //事件类型
        pointParamVO.setActivityType("课程学习完成积分发放-scorm课程积分发放");
        //学习活动来源
        pointParamVO.setActivitySource("课程学习");
        //学习活动名
        StringBuilder activityName = new StringBuilder();

        Course course = new Course();
        course.setId(chapter.getCourseId());
        course = course.selectById();
        if (course != null) {
            activityName.append(course.getName());
            activityName.append(":");
        }
        activityName.append(chapter.getName());
        pointParamVO.setActivityName(activityName.toString());
        //发放条件
        pointParamVO.setReleaseCondition("课程章节发放完成");
        //发放规则
        pointParamVO.setReleaseRules("按照章节获取取积分");
        //积分创建时间
        pointParamVO.setCreatePointTime(new Date());
        // 设置课程id
//            pointParamVO.setCourseId(chapter.getCourseId());
        //积分来源ID，（业务ID主键）
        pointParamVO.setSourceId(chapter.getId());

        pointParamVO.setCourseId(chapter.getCourseId());

        //积分数量，  正数 加积分，  负数减积分
        pointParamVO.setOperatingPoint(chapter.getScore());
        //学员ID
        pointParamVO.setAccountId(scormRecord.getAccountId());
        //学员名称
        pointParamVO.setAccountName("");
        pointParamVO.setSiteId(scormRecord.getSiteId());
        pointParamVO.setCompanyId(scormRecord.getCompanyId());
        pointParamVO.setOrgId(orgId);

        // 积分发放规则  1按课程  2 按章 3 按节
        int portRule = course.getPortRule();
        pointParamVO.setPortRule(portRule);
        pointParamVO.setSourceId(chapter.getCourseId());
        if (EXPERIENCE_COURSE.equals(mqName)) {
            // 积分发放规则  1按课程  2 按章 3 按节
            pointParamVO.setPortRule(1);
        }
        //调用积分接口添加积分
        String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
        if (null != sitePointId) {
            //发送积分获取消息
            //amqpTemplate.convertAndSend(mqName, sitePointId);
            return "ok";
        }
        return "sitePointId 为null";
    }
}
