package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.enums.ExceptionInfo;
import com.yizhi.course.application.mapper.ChapterMapper;
import com.yizhi.course.application.mapper.TestCaseMapper;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.vo.*;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 测验案件 服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2018-10-17
 */
@Service
public class TestCaseServiceImpl extends ServiceImpl<TestCaseMapper, TestCase> implements ITestCaseService {

    private Logger logger = LoggerFactory.getLogger(TestCaseServiceImpl .class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ITestService testService;

    @Autowired
    private ITestSubjectService testSubjectService;

    @Autowired
    private ITestSubjectOptionService testSubjectOptionService;

    @Autowired
    private ITestCaseInfoService testCaseInfoService;

    @Autowired
    private ChapterMapper chapterMapper;

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private ICourseService courseService;

    @Override
    public TestCaseRecordVO getRecordInfo(Long testCaseId){
        TestCaseRecordVO vo = new TestCaseRecordVO();

        // 查询答卷信息
        TestCase testCase = this.selectById(testCaseId);
        if(testCase == null){
            return null;
        }

        // 查询测验的信息
        TestInfo testInfo = testService.selectById(testCase.getTestId());
        BeanUtils.copyProperties(testInfo,vo,"id");
        vo.setTestId(testInfo.getId());

        vo.setId(testCase.getId());
        vo.setTestScore(testCase.getTestScore());

        // 查询答卷的答案记录信息
        TestCaseInfo testCaseInfoWh = new TestCaseInfo();
        testCaseInfoWh.setTestCaseId(testCase.getId());
        List<TestCaseInfo> caseInfos = testCaseInfoService.selectList(QueryUtil.condition(testCaseInfoWh));
        List<Long> subjetctIds = caseInfos.stream().map(obj->obj.getTestSubjectId()).collect(Collectors.toList());


        // 查询试卷的试题信息
        List<TestSubject> testSubjects = testSubjectService.selectBatchIds(subjetctIds);
        Map<Long,TestSubject> subjectMap = testSubjects.stream().collect(Collectors.toMap(subKey->subKey.getId(),subVal-> subVal));
        // 查询试题的答案
        EntityWrapper<TestSubjectOption> ewOption = QueryUtil.condition(new TestSubjectOption());
        ewOption.in("test_subject_id",subjetctIds);
        ewOption.orderBy("sort");
        List<TestSubjectOption> options = testSubjectOptionService.selectList(ewOption);
        Map<Long,List<TestSubjectOption>> optionMap = options.stream().collect(Collectors.groupingBy(option->option.getTestSubjectId()));
        vo.setIsPass(testCase.getStatus());// 是否通过

        // 循环测验试卷答题信息
        List<TestCaseSubRecordVO> testCaseSubRecordVOS = new ArrayList<>();
        for(TestCaseInfo testCaseInfo : caseInfos) {
            TestCaseSubRecordVO testCaseSubRecordVO = new TestCaseSubRecordVO();
            TestSubject testSubject = subjectMap.get(testCaseInfo.getTestSubjectId());
            BeanUtils.copyProperties(testSubject,testCaseSubRecordVO);
            testCaseSubRecordVO.setIsPass(true);
            // 组装试题的答案
            List<TestCaseRecordOpVO> opVOS = new ArrayList<>();


            List<TestSubjectOption> testSubjectOptionList = optionMap.get(testCaseInfo.getTestSubjectId());

            Map<Long,TestSubjectOption> optionMap1 = testSubjectOptionList.stream().collect(Collectors.toMap(key->key.getId(),val->val));
            String[] subOptions = testCaseInfo.getSubjectOptionIds().split(",");//试题选项的显示顺序
            String optionAnswerIds = testCaseInfo.getOptionAnswerIds();
            for (String os : subOptions){
                TestCaseRecordOpVO testCaseRecordOpVO = new TestCaseRecordOpVO();
                TestSubjectOption optionTemp = optionMap1.get(Long.valueOf(os));
                BeanUtils.copyProperties(optionTemp,testCaseRecordOpVO);
                if(optionAnswerIds.indexOf(optionTemp.getId().toString())>=0){
                    testCaseRecordOpVO.setSelectd(true);
                    if(!optionTemp.getAnswerResult()){
                        testCaseSubRecordVO.setIsPass(false);
                    }
                }else {
                    if(optionTemp.getAnswerResult()){
                        testCaseSubRecordVO.setIsPass(false);
                    }
                }
                opVOS.add(testCaseRecordOpVO);
            }

            testCaseSubRecordVO.setTestCaseRecordOpVOS(opVOS);
            testCaseSubRecordVOS.add(testCaseSubRecordVO);
        }

        vo.setTestCaseSubRecordVOS(testCaseSubRecordVOS);

        return vo;
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public TestCaseSubmitVO testSubmit(TestCaseVO vo) {
        TestCaseSubmitVO ret = null;

        checkField(vo);
        // 上下文
        RequestContext rc = ContextHolder.get();

        // 新增测验试题作答基本信息
        TestCase tc = new TestCase();
        BeanUtils.copyProperties(vo,tc);
        tc.setAccountId(rc.getAccountId());//账号id
        tc.setSiteId(rc.getSiteId());
        tc.setOrgId(rc.getOrgId());
        tc.setCompanyId(rc.getCompanyId());
        Long testCaseId = idGenerator.generate();
        tc.setId(testCaseId);
        tc.setCreateByName(rc.getAccountName());
        tc.setCreateTime(new Date());
        // 计算试题
        // 1 查询测验的基本信息
        TestInfo tf = new TestInfo();
        tf.setId(vo.getTestId());
        TestInfo tfData = testService.selectOne(QueryUtil.condition(tf));
        if(tfData == null){
            logger.info("根据测验id查询测验信息不能为空");
            throw new BizException(ExceptionInfo.NOT_EXIST_TEST.getCode(),ExceptionInfo.NOT_EXIST_TEST.getName());
        }
        // 2 查询测验的试题的列表
        TestSubject testSubject = new TestSubject();
        testSubject.setTestId(tfData.getId());
        List<TestSubject> testSubjects = testSubjectService.selectList(QueryUtil.condition(testSubject));
        Map<Long,TestSubject> testSubjectMap = testSubjects.stream().collect(Collectors.toMap(key->key.getId(), val->val));
        List<Long> testSubjectIds = new ArrayList(testSubjectMap.keySet());

        // 3.查询试题的答案
        EntityWrapper<TestSubjectOption> ewTestSubjectOption = QueryUtil.condition(new TestSubjectOption());
        ewTestSubjectOption.in("test_subject_id",testSubjectIds);
        ewTestSubjectOption.eq("answer_result",1);// 正确id
        List<TestSubjectOption> optionDatas = testSubjectOptionService.selectList(ewTestSubjectOption);
        Map<Long,List<TestSubjectOption>> optionDatasMap = optionDatas.stream().collect(Collectors.groupingBy(item->item.getTestSubjectId()));

        List<TestCaseInfoVO> testCaseInfoVOS = vo.getTestCaseInfoVOList();
        List<TestCaseInfo> saveObjs = new ArrayList<>();

        if(CollectionUtils.isNotEmpty(testCaseInfoVOS)) {
            try{
                ret = new TestCaseSubmitVO();
                ret.setTestName(tfData.getName());
                ret.setId(testCaseId);// 测验试卷id
                ret.setChapterId(vo.getChapterId());
                ret.setCourseId(vo.getCourseId());
                ret.setTestId(vo.getTestId());
                ret.setPassSource(tfData.getPassingScore());// 设定的通过分数

                List<Long> passIds = new ArrayList<>(); // 做正确的试题
                List<Long> errorIds = new ArrayList<>(); // 做错误的试题
                Integer source = 0; // 打分

                List<Long> caseSubjectId = testCaseInfoVOS.stream().map(item -> item.getTestSubjectId()).collect(Collectors.toList());

                String testType = tfData.getType();//测验类型


                for (TestCaseInfoVO item : testCaseInfoVOS) {
                    TestCaseInfo tci = new TestCaseInfo();
                    BeanUtils.copyProperties(item, tci);
                    // 获取试题信息
                    TestSubject subject = testSubjectMap.get(item.getTestSubjectId());
                    tci.setScore(subject.getScore()); //设置分数
                    tci.setResult(0);// 默认答错

                    if (subject != null) {
                        // 获取试题答案
                        List<TestSubjectOption> testSubjectOptions = optionDatasMap.get(subject.getId());
                        // 正确答案
                        List<Long> ans = testSubjectOptions.stream().map(item1 -> item1.getId()).collect(Collectors.toList());
                        tci.setCorrectAnswer(StringUtils.join(ans, ",")); // 设定正确答案
                        // 选择的答案
                        String optionAnswerIds = item.getOptionAnswerIds();
                        if (StringUtils.isNotBlank(optionAnswerIds)) {
                            List<Long> arr = getAnswerIds(optionAnswerIds);
                            if(ans.size() == arr.size()){
                                ans.removeAll(arr);
                            }
                            if (ans.size() > 0) {// 答错
                                errorIds.add(item.getTestSubjectId());
                            } else {//答对
                                if (TestServiceImpl.SUMMARY_TYPE.equals(testType)) {
                                    source = source.intValue() + subject.getScore();
                                }
                                passIds.add(item.getTestSubjectId());
                                tci.setResult(1);
                            }
                        }

                    }
                    tci.setTestCaseId(testCaseId);
                    tci.setTestId(vo.getTestId());
                    tci.setId(idGenerator.generate());
                    saveObjs.add(tci);

                    ret.setPassIds(passIds);
                    ret.setErrorIds(errorIds);

                }

                if (TestServiceImpl.PROCESS_TYPE.equals(testType)) {
                    List<Long> testSubjectIdExits = new ArrayList<>();// 测验试题是否全部答完
                    testSubjectIdExits.addAll(testSubjectIds);//把数据库查询的试题类别放入
                    //testSubjectIdExits.retainAll(caseSubjectId);//取出交集（共同出现的）
                    logger.info("caseSubjectId.size()={},testSubjectIds.size()={}", caseSubjectId.size(), testSubjectIds.size());
                    if (caseSubjectId.size() == testSubjectIds.size()) { // 答题个数必须和试题个数相等
                        testSubjectIdExits.removeAll(caseSubjectId);// 移除交集
                    }
                    logger.info("试题最后移除交集的大小={}", testSubjectIdExits.size());
                    ret.setIsPass(testSubjectIdExits.size() == 0);//过程测验全部答完就表示通过

                } else if (TestServiceImpl.SUMMARY_TYPE.equals(testType)) {
                    ret.setSource(source);
                    ret.setPassSource(tfData.getPassingScore());
                    ret.setIsPass(source.intValue() >= ret.getPassSource().intValue());
                    tc.setTestScore(source);
                }

                tc.setStatus(ret.getIsPass()?1:0);
            }catch (Exception e){
                logger.info(JSON.toJSONString(e));
                throw e;
            }
            testCaseInfoService.insertBatch(saveObjs);
        }else{
            logger.info("测验做题信息不能为空");
            throw new BizException(ExceptionInfo.TESE_CASE_INFO_EXITS.getCode(),ExceptionInfo.TESE_CASE_INFO_EXITS.getName());
        }

        // 如果试题处理完成，保存基本信息
        this.insert(tc);

        // 如果通过，则发送培训消息
        if (tc.getStatus().equals(1)) {
            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {
                    boolean isCourseFinished = courseService.isCourseFinished(tc.getCourseId(), tc.getAccountId(), false) == 1;
                    if (isCourseFinished) {
                        try {
                            cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, new EventWrapper<>(tc.getCourseId(),
                                            TrainingProjectEvent.getInstanceCourse(tc.getCourseId(), tc.getAccountId(), tc.getCreateTime(), 0L, true, tc.getSiteId())));
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
            });
        }
        return ret;

    }

    @Override
    @Transactional
    public TestCaseSimpleSubmitVO testCaseSimpleSubmit(TestCaseSimpleSubmitVO vo){
        //checkField(vo);
        // 上下文
        RequestContext rc = ContextHolder.get();
        Long testCaseId = vo.getId();
        TestCase testCase = null;
        if(testCaseId==null){
            testCase = new TestCase();
            testCaseId = idGenerator.generate();
            testCase.setId(testCaseId);
            testCase.setStatus(0);
            testCase.setCreateTime(new Date());
            testCase.setSiteId(rc.getSiteId());
            testCase.setAccountId(rc.getAccountId());
            testCase.setOrgId(rc.getOrgId());
            testCase.setCompanyId(rc.getCompanyId());
            testCase.setCourseId(vo.getCourseId());
            testCase.setTestId(vo.getTestId());
            testCase.setChapterId(vo.getChapterId());
            testCase.setCreateByName(rc.getAccountName());
            this.insert(testCase);
        }else {
            testCase = this.selectById(testCaseId);
        }

        vo.setId(testCaseId);
        vo.setChapterId(testCase.getChapterId());
        vo.setTestId(testCase.getTestId());
        vo.setCourseId(testCase.getCourseId());


        TestCaseInfo testCaseInfo = new TestCaseInfo();
        testCaseInfo.setTestCaseId(testCaseId);
        testCaseInfo.setTestId(testCase.getTestId());
        testCaseInfo.setTestSubjectId(vo.getTestSubjectId());
        TestCaseInfo testCaseInfoData = testCaseInfoService.selectOne(QueryUtil.condition(testCaseInfo));
        // 获取测验基本信息
        TestInfo testInfo = testService.selectById(testCase.getTestId());
        vo.setTestName(testInfo.getName());
        // 获取试题信息
        TestSubject subject = testSubjectService.selectById(vo.getTestSubjectId());
        vo.setAnalysis(subject.getExplain());
        // 获取试题答案选项(正确答案)
        TestSubjectOption subjectOption = new TestSubjectOption();
        subjectOption.setTestSubjectId(vo.getTestSubjectId());
        subjectOption.setAnswerResult(true);
        List<TestSubjectOption> options = testSubjectOptionService.selectList(QueryUtil.condition(subjectOption));

        String optionAnswerIds = vo.getOptionAnswerIds();
        boolean isPass = true;
        String correctAnswer = "";
        if(StringUtils.isNotBlank(optionAnswerIds)){
            optionAnswerIds = optionAnswerIds+",";
            for(TestSubjectOption obj:options){
                if(optionAnswerIds.indexOf(obj.getId().toString())<0){
                    isPass = false;
                }
                correctAnswer.concat(obj.getId().toString()).concat(",");
            }
        }else {
            isPass = false;
        }
        String testType = testInfo.getType();

        if(testCaseInfoData == null){
            testCaseInfoData = new TestCaseInfo();
            testCaseInfoData.setTestId(testCase.getTestId());
            testCaseInfoData.setTestCaseId(testCaseId);
            testCaseInfoData.setId(idGenerator.generate());
            testCaseInfoData.setTestSubjectId(vo.getTestSubjectId());
            testCaseInfoData.setOptionAnswerIds(vo.getOptionAnswerIds());
            testCaseInfoData.setSubjectOptionIds(vo.getSubjectOptionIds());
            testCaseInfoData.setCorrectAnswer(correctAnswer.substring(0,(correctAnswer.length()-2)));
            if(testType.equals(TestServiceImpl.SUMMARY_TYPE)){
                testCaseInfoData.setScore(subject.getScore());
            }
            testCaseInfoData.setResult(isPass?1:0);
        }else {
            if(testType.equals(TestServiceImpl.SUMMARY_TYPE)){
                testCaseInfoData.setScore(subject.getScore());
            }
            testCaseInfoData.setResult(isPass?1:0);
            testCaseInfoData.setOptionAnswerIds(vo.getOptionAnswerIds());
            testCaseInfoData.setSubjectOptionIds(vo.getSubjectOptionIds());
        }
        vo.setTestSubjectId(testCaseInfoData.getTestSubjectId());
        vo.setIsPass(isPass);

        if(vo.getIsLastTestSubject()){
            TestCaseInfo testCaseInfo1 = new TestCaseInfo();
            testCaseInfo1.setTestCaseId(testCaseId);
            List<TestCaseInfo> caseInfos = testCaseInfoService.selectList(QueryUtil.condition(testCaseInfo1));
            List<Long> errorIds = caseInfos.stream().filter(obj->obj.getResult().intValue()==0).map(val->val.getTestSubjectId()).collect(Collectors.toList());
            List<Long> passIds = caseInfos.stream().filter(obj->obj.getResult().intValue()==1).map(val->val.getTestSubjectId()).collect(Collectors.toList());
            if(errorIds == null){
                errorIds = new ArrayList<>();
            }
            if(passIds == null){
                passIds = new ArrayList<>();
            }
            if(testCaseInfoData == null){
                if(isPass){
                    passIds.add(vo.getTestSubjectId());
                }else {
                    errorIds.add(vo.getTestSubjectId());
                }
            }else {
                if(isPass){
                    errorIds.remove(errorIds.size()-1);
                    passIds.add(vo.getTestSubjectId());
                }
            }
            vo.setErrorIds(errorIds);
            vo.setPassIds(passIds);
        }

        return vo;
    }


    public static List<Long> getAnswerIds(String optionAnswerIds) {
        String[] ids = optionAnswerIds.split(",");
        List<Long> arr = new ArrayList<>();
        for(String id : ids){
            arr.add(Long.valueOf(id));
        }
        return arr;
    }

    /**
     * 检查测验案例传的测验id，课程id，章节id是否存在
     * @param vo
     */
    public boolean checkField(TestCaseVO vo){
        if(vo.getCourseId()==null){
            logger.info("课程id不能为空");
            throw new BizException(ExceptionInfo.TESE_CASE_FILED_COUER_ID.getCode(),ExceptionInfo.TESE_CASE_FILED_COUER_ID.getName());
        }
        if(vo.getChapterId()==null){
            logger.info("章节id不能为空");
            throw new BizException(ExceptionInfo.TESE_CASE_FILED_CHAPTER_ID.getCode(),ExceptionInfo.TESE_CASE_FILED_CHAPTER_ID.getName());
        }
        if(vo.getTestId()==null){
            logger.info("测验id不能为空");
            throw new BizException(ExceptionInfo.TESE_CASE_FILED_TEST_ID.getCode(),ExceptionInfo.TESE_CASE_FILED_TEST_ID.getName());
        }

        return true;
    }

    @Override
    public Integer selectMaxScore(Long courseId, Long chapterId,
                           Long testId,  Long accountId){
        return this.baseMapper.selectMaxScore(courseId,chapterId,testId,accountId);
    }

    @Override
    public List<TestCaseRecordVO> findMaxTestScore(Long courseId, Long accountId) {
        return this.baseMapper.selectTestMaxScore(courseId, accountId);
    }

    @Override
    public Boolean courseIsSucceed(Long courseId, Long accountId) {
        Chapter chapter = new Chapter();
        chapter.setSource(9);
        chapter.setCourseId(courseId);
        List<Chapter> chapters = chapterMapper.selectList(QueryUtil.condition(chapter));
        // 如果没有关联测验的课程直接返回true
        if(org.springframework.util.CollectionUtils.isEmpty(chapters)){
            return true;
        }
        List<Long> chapterIds = new ArrayList<>();
        //List<Long> testIds = new ArrayList<>();
        for(Chapter obj : chapters){
            chapterIds.add(obj.getId());
            //testIds.add(obj.getMaterialId());
        }
        // 关联测验课程，查询各个章节的测验是否通过
        EntityWrapper<TestCase> ew = QueryUtil.condition(new TestCase());
        ew.eq("course_id",courseId).eq("status",1).eq("account_id",accountId)
                .in("chapter_id",chapterIds);
        List<TestCase> testCaseList = this.baseMapper.selectList(ew);
        if(org.springframework.util.CollectionUtils.isEmpty(testCaseList)){
            return false;
        }

        Set<Long> retDataIds = testCaseList.stream().map(item->item.getChapterId()).collect(Collectors.toSet());
        logger.info("指定课程，查询的测验章节ids={},通过测验的章节ids={}",
                JSON.toJSONString(chapterIds),JSON.toJSONString(retDataIds));
        chapterIds.removeAll(retDataIds);
        return org.springframework.util.CollectionUtils.isEmpty(chapterIds)?true:false;
    }

    @Override
    public List<TestCaseRecordVO> chapterTestList(Long testId, Long accountId) {
        return this.baseMapper.chapterTestList(testId,accountId);
    }
}
