package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.enums.SqlLike;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.domain.TestInfo;
import com.yizhi.course.application.enums.ExceptionInfo;
import com.yizhi.course.application.mapper.TestMapper;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.ITestService;
import com.yizhi.course.application.service.ITestSubjectService;
import com.yizhi.course.application.vo.ChapterJoinTestVo;
import com.yizhi.course.application.vo.TestInfoListReqVO;
import com.yizhi.course.application.vo.TestInfoVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 测验表 服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2018-10-11
 */
@Service
public class TestServiceImpl extends ServiceImpl<TestMapper, TestInfo> implements ITestService {

    public static final String PROCESS_TYPE="01";//过程性测验
    public static final String SUMMARY_TYPE="02";//总结性测验

    private Logger logger = LoggerFactory.getLogger(TestServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ITestSubjectService testSubjectService;

    @Autowired
    private IChapterService chapterService;

    /**
     * 管理端查询测试信息列表
     *
     * @param reqVO     请求参数
     * @return
     */
    @Override
    public Page<TestInfoVO> listPage(TestInfoListReqVO reqVO) {
        // 上下文
        RequestContext requestContext = ContextHolder.get();
        Page<TestInfoVO> retPage = new Page<>();
        // 查询条件
        EntityWrapper<TestInfo> ew = QueryUtil.condition(new TestInfo());
        ew.eq("deleted",false);
        ew.eq("company_id",requestContext.getCompanyId());
        ew.eq("site_id",requestContext.getSiteId());

        String type = reqVO.getType();
        if(StringUtils.isNotBlank(type)){
            ew.eq("type",type);
        }
        Integer status = reqVO.getStatus();
        if(status != null){
            ew.eq("status",status);
        }else {
            boolean isRelevance = reqVO.getIsRelevance();
            if(isRelevance){
                ew.ne("status",0);
            }
        }
        String codeOrName = reqVO.getCodeOrName();
        if(StringUtils.isNotBlank(codeOrName)){
            ew.andNew().like("code",codeOrName, SqlLike.DEFAULT)
                    .or().like("name",codeOrName, SqlLike.DEFAULT)
                    .or().like("keywords", codeOrName);
        }

        // 分页信息

        Integer pageSize = reqVO.getPageSize();
        if(pageSize == null || pageSize == 0L ){
            pageSize = 10;
        }

        Integer pageNo =  reqVO.getPageNo();
        if(pageNo == null || pageNo == 0L){
            pageNo = 1;
        }
        Page<TestInfo> pageParam = new Page<>(pageNo,pageSize);

        // 排序
        ew.orderBy("id",false);
        Page<TestInfo> ret = this.selectPage(pageParam,ew);
        BeanUtils.copyProperties(ret,retPage,"records");

        List<TestInfoVO> data = new ArrayList<>();
        List<TestInfo> retData = ret.getRecords();
        /*if(CollectionUtils.isNotEmpty(retData)){
            Set<Long> testIds = retData.stream().parallel().map(obj->obj.getId()).collect(Collectors.toSet());
            EntityWrapper<Chapter>  ew1 = QueryUtil.condition(new Chapter());
            ew1.in("material_id",testIds);
            List<Chapter> chapters = chapterService.selectList(ew1);
            String testChapters = "";
            if(CollectionUtils.isNotEmpty(chapters)){
                testChapters = chapters.stream().map(item->item.getMaterialId().toString()).collect(Collectors.joining(",",",",","));
            }

            for(TestInfo obj:retData){
                TestInfoVO vo = new TestInfoVO();
                BeanUtils.copyProperties(obj,vo);
                Integer statusTemp = vo.getStatus();
                if(CollectionUtils.isNotEmpty(chapters) && statusTemp!=null){
                    if(testChapters.indexOf(vo.getId().toString()+",")>=0){
                        vo.setStatus(1);
                    }else if(statusTemp.intValue()==1){
                        vo.setStatus(2);
                    }
                }else if(statusTemp!=null && statusTemp.intValue()==1){
                    vo.setStatus(2);
                }
                data.add(vo);
            }

            retPage.setRecords(data);
        }*/
        if( CollectionUtils.isEmpty(retData) ){
            retPage.setRecords(data);
        }else {
            if( null == status ){
                List<Long> testIds = retData.stream().
                        map(obj->obj.getId()).collect(Collectors.toList());
                List<ChapterJoinTestVo> dataTem = chapterService.selectChapterTest(testIds);
                Map<Long,Integer> dataMap = dataTem.stream().collect(Collectors.toMap(
                        key->key.getMaterialId(),val->val.getCountNum()));

                for(TestInfo obj : retData){
                    TestInfoVO vo = new TestInfoVO();
                    BeanUtils.copyProperties(obj,vo);
                    Integer statusTem = vo.getStatus();
                    if(dataMap.containsKey(vo.getId())){
                        vo.setStatus(1);
                    }else if(Objects.nonNull(statusTem) && statusTem.intValue()!=0){
                        vo.setStatus(2);
                    }
                    data.add(vo);
                }

                retPage.setRecords(data);
            } else if (1 == status ) {
                List<Long> testIds = retData.stream().
                        map(obj -> obj.getId()).collect(Collectors.toList());
                List<ChapterJoinTestVo> dataTem = chapterService.selectChapterTest(testIds);
                Map<Long, Integer> dataMap = dataTem.stream().collect(Collectors.toMap(
                        key -> key.getMaterialId(), val -> val.getCountNum()));

                for (TestInfo obj : retData) {
                    TestInfoVO vo = new TestInfoVO();
                    BeanUtils.copyProperties(obj, vo);
                    if (dataMap.containsKey(vo.getId())) {
                        vo.setStatus(1);
                    }
                    data.add(vo);
                }

                retPage.setRecords(data);
            } else {
                List<Long> testIds = retData.stream().
                        map(obj -> obj.getId()).collect(Collectors.toList());
                List<ChapterJoinTestVo> dataTem = chapterService.selectChapterTest(testIds);
                Map<Long, Integer> dataMap = dataTem.stream().collect(Collectors.toMap(
                        key -> key.getMaterialId(), val -> val.getCountNum()));

                for (TestInfo obj : retData) {
                    TestInfoVO vo = new TestInfoVO();
                    BeanUtils.copyProperties(obj, vo);
                    Integer statusTem = vo.getStatus();
                    if (Objects.nonNull(statusTem) && statusTem.intValue() != 0) {
                        vo.setStatus(2);
                    }
                    data.add(vo);
                }

                retPage.setRecords(data);
            }
        }


        //logger.info("查询数据="+ JSON.toJSONString(retPage));
        return retPage;
    }

    /**
     * 根据测验id删除测验数据
     *
     * @param ids
     * @return 返回为空字符串为正常删除，否则返回异常信息
     */
    @Override
    @Transactional
    public Boolean deleteIds(List<Long> ids) {
        // 上下文
        RequestContext requestContext = ContextHolder.get();
        // 删除(逻辑删除)
        if(CollectionUtils.isNotEmpty(ids)){
            // 判断是否有使用中的的数据
           /* EntityWrapper<TestInfo> ewInfo = QueryUtil.condition(new TestInfo());
            ewInfo.in("id",ids);
            ewInfo.eq("deleted",false);
            // 使用中
            ewInfo.eq("status",1);
            int retCount = this.selectCount(ewInfo);*/
            //EntityWrapper<Chapter> ewStats = QueryUtil.condition(new Chapter());
            //ewStats.eq("source",9).in("material_id",ids);
            //List<Chapter> datas = chapterService.selectList(ewStats);
            List<ChapterJoinTestVo> datas = chapterService.selectChapterTest(ids);
            if(CollectionUtils.isNotEmpty(datas)){
                List<Long> existTest = datas.stream().filter(filterItem -> filterItem.getCountNum()>0).map(obj->obj.getMaterialId()).collect(Collectors.toList());
                if(CollectionUtils.isNotEmpty(existTest)){
                    List<TestInfo> tests = this.selectBatchIds(existTest);
                    String errmsg = tests.stream().map(obj->obj.getName()).collect(Collectors.joining(";","[","]"));
                    throw new BizException(ExceptionInfo.TEST_DELETE_FAIL.getCode(),errmsg+"被使用中，不能修改");
                }

            }

            ids.forEach(id -> {
                TestInfo upObj = new TestInfo();
                upObj.setId(id);
                upObj.setDeleted(true);
                upObj.setUpdateById(requestContext.getAccountId());
                upObj.setUpdateByName(requestContext.getAccountName());
                upObj.setUpdateTime(new Date());
                this.updateById(upObj);
            });
        }else {
            throw new BizException(ExceptionInfo.NOT_DELETE_TEST_DATA.getCode(),ExceptionInfo.NOT_DELETE_TEST_DATA.getName());
        }
        return true;
    }


    @Override
    @Transactional
    public TestInfoVO saveTest(TestInfoVO vo) throws BizException {
        // 上下文
        RequestContext requestContext = ContextHolder.get();
        // 保存的数据
        TestInfo saveObj = new TestInfo();
        BeanUtils.copyProperties(vo,saveObj);

        // 判断是否是新增
        Long id = vo.getId();
        if(id != null){

            if(StringUtils.isNotBlank(vo.getName())){
                String name = checkUpObj(vo, requestContext);
                if(!StringUtils.equals(name,vo.getName())){
                    checkName(vo, requestContext);
                }
                saveObj.setDeleted(null);
                saveObj.setCode(null);
                saveObj.setCompanyId(null);
                saveObj.setSiteId(null);
                saveObj.setOrgId(null);
                saveObj.setUpdateById(requestContext.getAccountId());
                saveObj.setUpdateByName(requestContext.getAccountName());
                saveObj.setUpdateTime(new Date());
            }
            Chapter chapter = new Chapter();
            chapter.setSource(9);
            chapter.setMaterialId(id);
            int size = chapterService.selectCount(QueryUtil.condition(chapter));
            if(size > 0){
                throw new BizException(ExceptionInfo.TEST_UPDATE_FAIL.getCode(),ExceptionInfo.TEST_UPDATE_FAIL.getName());
            }
            this.updateById(saveObj);
        }else{

            checkName(vo, requestContext);

            id = idGenerator.generate();
            saveObj.setId(id);
            saveObj.setDeleted(false);
            saveObj.setCode(UUID.randomUUID().toString().replace("-", "").toLowerCase());
            saveObj.setCompanyId(requestContext.getCompanyId());
            saveObj.setSiteId(requestContext.getSiteId());
            saveObj.setOrgId(requestContext.getOrgId());
            saveObj.setCreateById(requestContext.getAccountId());
            saveObj.setCreateByName(requestContext.getAccountName());
            saveObj.setCreateTime(new Date());
            this.insert(saveObj);
            BeanUtils.copyProperties(saveObj,vo);
        }
        return vo;
    }

    /**
     * 更新测验是否被使用中
     *
     * @param testId
     * @return
     */
    @Override
    @Transactional
    public Boolean updateStatus(Long testId,Long chapterId,Integer type) {

        if(type==4 && chapterId != null && testId.equals(chapterId)){
            return true;
        }

        TestInfo testInfoVO1 = this.baseMapper.selectForUp(testId);
        if(testInfoVO1 == null){
            return false;
        }
        TestInfo testInfoVO2 = null;
        if(type==4 && chapterId != null){
            testInfoVO2 = this.baseMapper.selectForUp(chapterId);
            //testInfoVO2.setId(chapterId);
        }


        Chapter chapter = new Chapter();
        chapter.setMaterialId(testId);
        EntityWrapper<Chapter> ew = QueryUtil.condition(new Chapter());
        ew.eq("source",9);
        ew.eq("material_id",testId);
        if(chapterId != null){
            ew.ne("id",chapterId);
        }

        int countNum = chapterService.selectCount(ew);


        if(type==1){
            testInfoVO1.setStatus(1);
        }
        if(type==2){
            // 从非测验更新成测验
            if(chapterId == null){
                testInfoVO1.setStatus(1);
            }else if(chapterId != null ){
                if(countNum==0){
                    testInfoVO1.setStatus(2);
                }else if(countNum > 0){
                    testInfoVO1.setStatus(1);
                }

            }
        }
        if(type==3 && chapterId != null){
             if(countNum ==0){
                testInfoVO1.setStatus(2);
            }else if(countNum > 0){
                 testInfoVO1.setStatus(1);
             }
        }

        if(type==4 && chapterId != null){
            if(countNum<=1){
                testInfoVO1.setStatus(2);
            }else if(countNum > 1){
                testInfoVO1.setStatus(1);
            }
            testInfoVO2.setStatus(1);
            this.updateById(testInfoVO2);
        }
        this.updateById(testInfoVO1);

        return true;
    }

    /**
     * 检查名字是否存在
     * @param vo
     * @param requestContext
     * @return
     */
    private void checkName(TestInfoVO vo, RequestContext requestContext) {
        EntityWrapper<TestInfo> ew = QueryUtil.condition(new TestInfo());
        ew.eq("company_id",requestContext.getCompanyId()).eq("site_id",requestContext.getSiteId());
        ew.eq("name",vo.getName()).eq("deleted",0);

        int extisCount = this.selectCount(ew);
        if(extisCount > 0){
            throw new BizException(ExceptionInfo.EXIST_TEST_NAME.getCode(),ExceptionInfo.EXIST_TEST_NAME.getName());
        }
    }

    /**
     * 检查更新的测验信息
     * @param vo
     * @param requestContext
     */
    private String checkUpObj(TestInfoVO vo, RequestContext requestContext) {

        EntityWrapper<TestInfo> ew = QueryUtil.condition(new TestInfo());
        ew.eq("company_id",requestContext.getCompanyId()).eq("site_id",requestContext.getSiteId());
        ew.eq("id",vo.getId());

        TestInfo extisObj = this.selectOne(ew);
        if(extisObj == null){
            logger.info("根据id查询的测验为空");
            throw new BizException(ExceptionInfo.NOT_EXIST_TEST.getCode(),ExceptionInfo.NOT_EXIST_TEST.getName());
        }

        // 过程测试更新到总结性测试
        if(StringUtils.equals(SUMMARY_TYPE,vo.getType())){
            logger.info("更新测验的类型为总结性测验");
            // 验证分数是否符合
            if(vo.getPassingScore()==null){
                logger.info("总结性测验的更新合格分数没有设定");
                throw new BizException(ExceptionInfo.PASSING_SCORE_EMPTY.getCode(),ExceptionInfo.PASSING_SCORE_EMPTY.getName());
            }
            Integer sumNum  = testSubjectService.sumScore(vo.getId());
            if(sumNum == null || sumNum == 0){
                logger.info("总结性测验的题目总设定分数为空或0");
                throw new BizException(ExceptionInfo.TEST_SUBJECT_EMPTY.getCode(),ExceptionInfo.TEST_SUBJECT_EMPTY.getName());
            }
            if(vo.getPassingScore().intValue()> sumNum){
                logger.info("总结性测验的更新合格分数大于题目总累计分数");
                throw new BizException(ExceptionInfo.PASSING_SCORE_EXCESS.getCode(),ExceptionInfo.PASSING_SCORE_EXCESS.getName());
            }

        }

        return extisObj.getName();
    }
}
