package com.yizhi.course.application.studyHourReportExport;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.course.application.service.ICourseStudyHourService;
import com.yizhi.course.application.vo.StudyHourReportVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className CertificateListExport
 * @description TODO
 * @date 2019/3/20
 **/

@Component
public class StudyHourDetailsExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(StudyHourDetailsExport.class);

    @Autowired
    private ICourseStudyHourService studyHourService;

    @Override
    protected String execute(Map<String, Object> map) {

        RequestContext res = (RequestContext) map.get("res");
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        String orgNameLike = (String) map.get("orgNameLike");
        String userName = (String) map.get("userName");
        Long companyId = res.getCompanyId();
        Long siteId = res.getSiteId();
        Long accountId = res.getAccountId();
        boolean isAdmin = res.isAdmin();

        Date submitTime = new Date();

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        List<StudyHourReportVO> resultList = new ArrayList<>();
        //获取下载数据
        try {
            resultList = studyHourService.studyHourDetailsDownload(companyId, siteId, isAdmin, accountId, userName, orgNameLike);
            if (CollectionUtils.isEmpty(resultList)) {
                success(taskContext, "未查找到学时明细记录");
            }
        } catch (Exception e) {
            fail(taskContext, "学时明细下载获取数据出现错误");
            LOGGER.error("学时明细下载获取数据出现错误", e);
        }

        try {

            //新建excel对象
            XSSFWorkbook workbook = new XSSFWorkbook();
            XSSFSheet sheet = workbook.createSheet("学时明细数据");
            XSSFRow row = sheet.createRow(0);

            CellStyle cellStyle = workbook.createCellStyle();
            XSSFFont font = workbook.createFont();
            font.setBold(true);
            cellStyle.setFont(font);

            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 8);
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);
            cell.setCellValue("学时明细数据");
            cell.setCellStyle(cellStyle);

            row = sheet.createRow(1);
            row.createCell(0).setCellValue("用户名");
            row.createCell(1).setCellValue("姓名");
            row.createCell(2).setCellValue("所在部门");
            row.createCell(3).setCellValue("用户当前状态");
            row.createCell(4).setCellValue("获得时间");
            row.createCell(5).setCellValue("课程编码");
            row.createCell(6).setCellValue("课程名称");
            row.createCell(7).setCellValue("学时数");
            row.createCell(8).setCellValue("所在组织架构");

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


            //数据写入excel
            if (CollectionUtils.isNotEmpty(resultList)) {
                for (int i = 0; i < resultList.size(); i++) {
                    StudyHourReportVO studyHourReportVO = resultList.get(i);
                    if (null != studyHourReportVO) {
                        row = sheet.createRow(i + 2);
                        row.createCell(0).setCellValue(null == studyHourReportVO.getUserName() ? "" : studyHourReportVO.getUserName());
                        row.createCell(1).setCellValue(null == studyHourReportVO.getFullName() ? "" : studyHourReportVO.getFullName());
                        row.createCell(2).setCellValue(null == studyHourReportVO.getOrgName() ? "" : studyHourReportVO.getOrgName());
                        row.createCell(3).setCellValue(null == studyHourReportVO.getAccountState() ? "停用" : studyHourReportVO.getAccountState());
                        row.createCell(4).setCellValue(sdf.format(studyHourReportVO.getCreateTime()));
                        row.createCell(5).setCellValue(studyHourReportVO.getCourseNo());
                        row.createCell(6).setCellValue(studyHourReportVO.getCourseName());
                        row.createCell(7).setCellValue(studyHourReportVO.getStudyHour());
                        row.createCell(8).setCellValue(studyHourReportVO.getOrgNames());
                    }
                }
            }

            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyyMMddHHmmss");
            String fileName = "学时明细数据" + sdf1.format(new Date()) + ".xls";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            //创建目标文件夹
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            String url = null;
            FileOutputStream os = null;
            File file = null;

            //创建文件并上传阿里云,删除本地文件
            try {
                os = new FileOutputStream(path);
                workbook.write(os);
                url = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", url);
            } catch (Exception e1) {
                fail(taskContext, "学时明细数据写入过程中出现错误");
                LOGGER.error("学时明细数据写入过程中出现错误", e1);
            } finally {
                if (null != os) {
                    os.close();
                }
                if (null != workbook) {
                    workbook.close();
                }
                if (null != file) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            fail(taskContext, "学时明细数据创建excel出现错误");
            LOGGER.error("学时明细数据创建excel出现错误", e);
        }
        return "ok";
    }
}
