package com.yizhi.course.application.task;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.vo.CourseListExportVO;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

@Component
public class CourseListExportAsync extends AbstractDefaultTask<String, Map<String,Object>> {

	private static final Logger logger = LoggerFactory.getLogger(CourseListExportAsync.class);
	
	@Autowired
	private ICourseService courseService;
	
	@Override
	protected String execute(Map<String, Object> arg0) {
		Long accountId=(Long) arg0.get("accountId");
		Long siteId=(Long) arg0.get("siteId");
		Long companyId=(Long) arg0.get("companyId");
		Long taskId=(Long) arg0.get("taskId");
		Date submitTime=(Date) arg0.get("submitTime");
		String serialNo=(String) arg0.get("serialNo");
		String taskName=(String) arg0.get("taskName");
		String name=(String) arg0.get("name");
		Long classifyId=(Long) arg0.get("classifyId");
		Integer shelves=(Integer) arg0.get("shelves");
		Integer type=(Integer) arg0.get("type");
		Integer courseType=(Integer) arg0.get("courseType");
		/**
		 * 走异步任务
		 */
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
		working(taskContext);		
		String upLoadUrl=null;
		String requestPath= FileConstant.SAVE_PATH;
		File fileDir=new File(requestPath);
		if(!fileDir.exists()) {
			fileDir.mkdir();
		}	
		//课程清单开始组装
		List<CourseListExportVO> list = courseService.listExport(name, classifyId, shelves, companyId, siteId, null, type,courseType);
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); 	
		SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss"); 	 	
		try {
			//excel生成过程: excel-->sheet-->row-->cell
			// 第一步，创建一个Excel文件
			XSSFWorkbook wb = new XSSFWorkbook();
			// 第二步，在webbook中添加一个sheet,对应Excel文件中的sheet
			XSSFSheet sheet = wb.createSheet("课程清单");
			XSSFFont font = wb.createFont();
			font.setFontName("宋体");
			font.setFontHeightInPoints((short) 11);
			font.setBold(true);
			XSSFCellStyle style = wb.createCellStyle();
			style.setFont(font);
			style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
			style.setVerticalAlignment(VerticalAlignment.CENTER);
			// 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
			XSSFRow row = sheet.createRow((int) 0);
			row.setHeight((short) (20*20));
			// 合并单元格  
			CellRangeAddress cra=new CellRangeAddress(0,0,0,8); // 起始行, 终止行, 起始列, 终止列  
			sheet.addMergedRegion(cra);
			XSSFCell cell = row.createCell((short) 0);
			cell.setCellValue("课程清单"+sf.format(submitTime));
			cell.setCellStyle(style);
			// 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
			XSSFRow row2 = sheet.createRow((int) 1);
			row2.setRowStyle(style);
			row2.setHeight((short) (14*20));
			// 第四步，创建单元格
			row2.createCell((short) 0).setCellValue("课程编码"); 
			row2.createCell((short) 1).setCellValue("课程名称"); 
			row2.createCell((short) 2).setCellValue("课程类型"); 
			row2.createCell((short) 3).setCellValue("课程积分"); 
			row2.createCell((short) 4).setCellValue("课程状态"); 
			row2.createCell((short) 5).setCellValue("创建时间"); 
			row2.createCell((short) 6).setCellValue("课程总学时");
			row2.createCell((short) 7).setCellValue("课程总学习时长");
			row2.createCell((short) 8).setCellValue("课程分类");
			CourseListExportVO course=null;
				for(int i=0;i<list.size();i++) {
					font.setBold(false);
					row = sheet.createRow(i + 2);
					row.setRowStyle(style);
					row.setHeight((short) (13*20));
					// 第四步，创建单元格，并设置值
					course=list.get(i);
					if(course!=null&&course.getCode()!=null) {
						row.createCell((short) 0).setCellValue(course.getCode());
					}
					if(course!=null&&course.getName()!=null) {
						row.createCell((short) 1).setCellValue(course.getName());
					}				
					if(course!=null&&course.getCourseType()!=null) {
						row.createCell((short) 2).setCellValue(course.getCourseType()==0?"一般课程":"scorm课程");
					}				
					if(course!=null&&course.getScore()!=null) {
						row.createCell((short) 3).setCellValue(course.getScore());
					}
					if(course!=null&&course.getShelves()!=null) {
						if (course.getShelves()==0) {
							row.createCell((short) 4).setCellValue("已下架");
						}else if (course.getShelves()==1) {
							row.createCell((short) 4).setCellValue("已上架");
						}else {
							row.createCell((short) 4).setCellValue("草稿");
						}

					}					
					if(course!=null&&course.getCreateTime()!=null) {
					    row.createCell((short) 5).setCellValue(df.format(course.getCreateTime()));
					}
					if(course!=null) {
						//课程总学时
						Float totalDuration = course.getTotalDuration();
						if (null == totalDuration){
							row.createCell((short) 6).setCellValue("0.0");
						}else {
							row.createCell((short) 6).setCellValue(String.valueOf(totalDuration));
						}
					}
					if(course!=null) {
						//课程总学习时长
						String totalHourString = course.getTotalHourString();
						if (StringUtils.isEmpty(totalHourString)){
							row.createCell((short) 7).setCellValue("00:00:00");
						}else {
							row.createCell((short) 7).setCellValue(totalHourString);
						}
					}
					if (course!=null&&course.getClassify()!=null) {
						String[] names = course.getClassify().split("/");
						if (names.length>0) {
							for (int j = names.length; j >0 ; j--) {
							    row.createCell((short) 7+j).setCellValue(names[j-1]);
							}
						}else {
						    row.createCell((short) 8).setCellValue(course.getClassify());
						}
					}
				}	
				
			// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
			StringBuffer fileNameSb = new StringBuffer().append("课程清单").append(sf.format(submitTime)).append(".xlsx");
			String fileName = fileNameSb.toString();
			String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
			FileOutputStream os=null;
			File file=null;
			try {
				os= new FileOutputStream(path);
				wb.write(os);
				//阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file=new File(path);
				success(taskContext,"成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入数据到Excel的过程中或者上传到阿里云中发生错误"+e1.getMessage());
				logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误"+e1.getMessage());	
			}
			finally {
				if(os!=null) {
				   os.close();
				}
				if(wb!=null) {
				   wb.close();
				}
				if(file!=null) {
				   file.delete();
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, "课程清单导出过程中发生错误，请查看日志"+e.getMessage());	
			logger.error("课程清单导出过程中发生错误，请查看日志"+e.getMessage());
		}
		return upLoadUrl;
	}

}
