package com.yizhi.course.application.task;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.RecordeListVo;
import com.yizhi.course.application.vo.RecordeVO;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.feign.PointUserFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 21:21 2018/11/14
 * @Modified By
 */
@Component
public class CoursePointInitAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    private static final Logger logger = LoggerFactory.getLogger(CoursePointInitAsync.class);

    @Autowired
    private IRecordeService recordeService;

    @Autowired
    private ICourseService courseService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    PointRedisFeignClients pointRedisFeignClients;

    @Autowired
    PointUserFeignClients pointUserFeignClients;

   /* @Autowired
    AmqpTemplate amqpTemplate;*/

    // 课程积分发放的事件名称
    private static final String EVENT_NAME = "pointCourse";

    // 课程积分发放MQ队列名称
    private static final String COURSE_MQ_SEND_SCORE = "course";

    @Override
    protected String execute(Map<String, Object> stringObjectMap) {
        logger.info("课程积分初始化处理开始");
        Long siteId = (Long)stringObjectMap.get("siteId");
        List<RecordeListVo> data = recordeService.findStuByAccountG(siteId);
        if(!CollectionUtils.isEmpty(data)){
            logger.info("查询学习记录表的数据条数="+data.size());
            for(RecordeListVo vo : data){

                try {
                    Long courseId = vo.getCourseId();
                    // 查询课程信息
                    Course course = courseService.selectById(courseId);
                    if(course == null){
                        logger.info("课程id={}，没有查询到课程信息",courseId);
                        continue;
                    }
                    // 查询章节信息
                    Chapter chapter = new Chapter();
                    chapter.setCourseId(courseId);
                    List<Chapter> chapterData = chapterService.selectList(QueryUtil.condition(chapter));
                    if(CollectionUtils.isEmpty(chapterData)){
                        logger.info("课程id={}，没有查询到课程的章节信息",courseId);
                        continue;
                    }
                    List<Chapter> dataChapterList =chapterData.stream().filter(obj -> Objects.nonNull(obj.getMaterialId()))
                            .collect(Collectors.toList());
                    if(CollectionUtils.isEmpty(dataChapterList)){
                        logger.info("课程id={}，没有查询到需要学习的课程章节信息",courseId);
                        continue;
                    }
                    int portRule = course.getPortRule();

                    // 如果存在单节的课程，不需要区分积分发放策略，
                    Chapter chapter1 = dataChapterList.get(0);
                    if(dataChapterList.size()==1 && Objects.nonNull(chapter1.getScore()) && chapter1.getScore().intValue()>0){
                        logger.info("课程id={}，只有一个学习章节时处理方式开始",courseId);
                        // 查询用户的学习记录，发放积分
                        Page<Recorde> page = new Page(1,1);
                        Recorde studyRecorde = new Recorde();
                        studyRecorde.setSiteId(vo.getSiteId());
                        studyRecorde.setAccountId(vo.getAccountId());
                        studyRecorde.setCourseId(courseId);
                        studyRecorde.setChapterId(chapter1.getId());
                        studyRecorde.setOvered(1);
                        EntityWrapper<Recorde> ew = QueryUtil.condition(studyRecorde);
                        ew.orderBy("end_time",false);
                        Page<Recorde> studyOne = recordeService.selectPage(page,ew);
                        List<Recorde> pageOneList = studyOne.getRecords();
                        if(!CollectionUtils.isEmpty(pageOneList)){
                            logger.info("课程id={}，只有一个学习章节时处理，查询有完成的课程章节记录",courseId);
                            // 发放积分
                            if(portRule == 2 && chapter1.getParentId() != null && chapter1.getParentId().longValue()>0){//按章发积分
                                Chapter chapterZ = chapterData.stream().filter(obj->obj.getId().equals(chapter1.getParentId())).findAny().get();
                                if(chapterZ != null){
                                    sendPoint(course,chapter1.getScore(),pageOneList.get(0),chapterZ,COURSE_MQ_SEND_SCORE);
                                }else {
                                    sendPoint(course,chapter1.getScore(),pageOneList.get(0),chapter1,COURSE_MQ_SEND_SCORE);
                                }

                            }else{
                                sendPoint(course,chapter1.getScore(),pageOneList.get(0),chapter1,COURSE_MQ_SEND_SCORE);
                            }
                        }else{
                            logger.info("课程id={}，只有一个学习章节时处理，没有查询出有完成的课程章节记录",courseId);
                        }
                    }else{
                        logger.info("课程id={}，多个学习章节时处理方式开始",courseId);
                        List<RecordeVO> stuData = recordeService.
                                findStuChapterByAccountG(vo.getSiteId(),vo.getCourseId(),vo.getAccountId());
                        switch (portRule){
                            case 1:
                                long okStatus = stuData.stream().filter(obj->Integer.valueOf(0).equals(obj.getOvered())).count();
                                if(okStatus==0){
                                    RecordeVO recordeVO = stuData.get(0);
                                    Recorde recorde = new Recorde();
                                    recorde.setCompanyId(recordeVO.getCompanyId());
                                    recorde.setSiteId(recordeVO.getSiteId());
                                    recorde.setOrgId(recordeVO.getOrgId());
                                    recorde.setAccountId(recordeVO.getAccountId());
                                    // 求和https://www.cnblogs.com/hahajava/p/9204500.html 和 https://blog.csdn.net/u011207553/article/details/80285386
                                    int courseSumScore = dataChapterList.stream().mapToInt(obj->obj.getScore()==null?0:obj.getScore().intValue()).sum();
                                    if(courseSumScore>0){
                                        sendPoint(course,courseSumScore,recorde,chapter1,COURSE_MQ_SEND_SCORE);
                                    }else{
                                        logger.info("课程id={}，多个学习章节时处理方式开始，按课程发放积分，课程设定积分为空",courseId);
                                    }
                                }else{
                                    logger.info("课程id={}，多个学习章节时处理方式开始，按课程发放积分，课程章节没有全部学习完成",courseId);
                                }
                                break;
                            case 2:
                                List<Chapter> listZ = chapterData.stream()
                                        .filter(obj -> Objects.isNull(obj.getParentId()) || obj.getParentId().longValue()==0)
                                        .collect(Collectors.toList());
                                for(Chapter zChapter : listZ){
                                    Long materialId = zChapter.getMaterialId();
                                    Integer scoreZ = zChapter.getScore();
                                    RecordeVO recordeVO = null;
                                    if(Objects.isNull(materialId)){
                                        logger.info("课程id={}，多个学习章节时处理方式开始，按章发放积分，章上没有直接挂内容的",courseId);

                                        List<Chapter> listJ = chapterData.stream().filter(obj->zChapter.getId().equals(obj.getParentId()))
                                                .collect(Collectors.toList());
                                        scoreZ = listJ.stream().mapToInt(obj->obj.getScore()==null?0:obj.getScore().intValue()).sum();
                                        List<Long> stuydIds = listJ.stream().map(obj->obj.getId()).collect(Collectors.toList());
                                        long coi = stuData.stream().filter(obj->(stuydIds.contains(obj.getChapterId())
                                                && Integer.valueOf(0).equals(obj.getOvered()))).count();
                                        if(coi>0){
                                            recordeVO = stuData.get(0);
                                        }else {
                                            logger.info("课程id={}，多个学习章节时处理方式开始，按章发放积分，章上直接挂内容的，章内容没有学习完成",courseId);
                                        }
                                    }else {
                                        logger.info("课程id={}，多个学习章节时处理方式开始，按章发放积分，章上没有直接挂内容的",courseId);
                                        Optional<RecordeVO> dataZ = stuData.stream().filter(obj->(zChapter.getId().equals(obj.getChapterId())
                                                && Integer.valueOf(1).equals(obj.getOvered()))).findAny();
                                        if(dataZ.isPresent()){
                                            recordeVO = dataZ.get();
                                        }else {
                                            logger.info("课程id={}，多个学习章节时处理方式开始，按章发放积分，章上没有直接挂内容的，章内容没有学习完成",courseId);
                                        }
                                    }
                                    if(recordeVO != null && scoreZ != null && scoreZ.intValue()>0){
                                        Recorde reco = new Recorde();
                                        reco.setCompanyId(recordeVO.getCompanyId());
                                        reco.setSiteId(recordeVO.getSiteId());
                                        reco.setOrgId(recordeVO.getOrgId());
                                        reco.setAccountId(recordeVO.getAccountId());
                                        sendPoint(course,scoreZ.intValue(),reco,zChapter,COURSE_MQ_SEND_SCORE);
                                    }else {
                                        logger.info("课程id={}，多个学习章节时处理方式开始，按章发放积分，学习完成或者章积分为空，不发送积分",courseId);
                                    }
                                }
                                break;
                            case 3:
                                for(Chapter itemChap : dataChapterList){
                                    Long itemId = itemChap.getId();
                                    Integer score = itemChap.getScore();
                                    if(score != null && score.intValue()>0){
                                        Optional<RecordeVO> recordeVOOpt = stuData.stream().filter(obj->(itemId.equals(obj.getChapterId())
                                                && Integer.valueOf(1).equals(obj.getOvered()))).findAny();

                                        if(recordeVOOpt.isPresent()){
                                            RecordeVO recordeVO = recordeVOOpt.get();
                                            Recorde reco = new Recorde();
                                            reco.setCompanyId(recordeVO.getCompanyId());
                                            reco.setSiteId(recordeVO.getSiteId());
                                            reco.setOrgId(recordeVO.getOrgId());
                                            reco.setAccountId(recordeVO.getAccountId());
                                            sendPoint(course,score.intValue(),reco,itemChap,COURSE_MQ_SEND_SCORE);
                                        }else {
                                            logger.info("课程id={}，多个学习章节时处理方式开始，按节发放积分，章没有学习记录为完成，不发送积分",courseId);
                                        }

                                    }
                                }
                                break;
                            default:
                        }
                    }
                } catch (Exception e) {
                    logger.info("初始化积分发放异常信息={}",e);
                    continue;
                }
            }
        }else {
            logger.info("没有查询到学习记录");
        }
        logger.info("课程积分初始化处理结束");
        return "ok";
    }



    /**
     * 发放积分
     * @param course 课程信息
     * @param score 发放的积分数
     * @param recorde 学习记录
     * @param chapter 章节信息
     */
    private void sendPoint(Course course, int score, Recorde recorde, Chapter chapter, String mqKey) {
        if(score == 0){
            logger.info("初始化积分操作准备发放积分时，发现积分为0，积分发放准备停止");
            return;
        }
        //学习活动名
        StringBuilder activityName = new StringBuilder();
        try {
            PointParamVO pointParamVO = new PointParamVO();
            pointParamVO.setPortRule(course.getPortRule());
            //事件名称
            pointParamVO.setEventName(EVENT_NAME);//PointEventEnum.POINTCOURSE
            //事件类型
            pointParamVO.setActivityType("课程学习完成积分发放");
            //学习活动来源
            pointParamVO.setActivitySource("课程学习");


            activityName.append(course.getName());

            //积分创建时间
            pointParamVO.setCreatePointTime(new Date());

            // 积分发放规则  1按课程  2 按章 3 按节
            int portRule = course.getPortRule();

            //积分来源ID，（业务ID主键）
            pointParamVO.setSourceId(course.getId());

            //发放条件
            pointParamVO.setActivityType("课程学习完成积分发放-按课程发放");
            //发放规则
            pointParamVO.setReleaseRules("按课程发放");
            if(portRule==3){
                pointParamVO.setReleaseRules("按节发放");
                //发放条件
                pointParamVO.setActivityType("课程学习完成积分发放-按节发放");

                activityName.append(":");
                activityName.append(chapter.getName());
                //积分来源ID，（业务ID主键）
                pointParamVO.setSourceId(chapter.getId());

            }else if(portRule==2){
                pointParamVO.setActivityType("课程学习完成积分发放-按章发放");
                pointParamVO.setReleaseRules("按章发放");

                activityName.append(":");
                activityName.append(chapter.getName());
                //积分来源ID，（业务ID主键）
                pointParamVO.setSourceId(chapter.getId());
            }

            pointParamVO.setActivityName(activityName.toString());

            //积分数量，  正数 加积分，  负数减积分
            pointParamVO.setOperatingPoint(score);

            //学员ID
            pointParamVO.setAccountId(recorde.getAccountId());
            //学员名称
            //pointParamVO.setAccountName(accountName);
            pointParamVO.setSiteId(recorde.getSiteId());
            pointParamVO.setCompanyId(recorde.getCompanyId());
            pointParamVO.setOrgId(recorde.getOrgId());

            pointParamVO.setPortRule(portRule);

            boolean isSend = pointUserFeignClients.isHasPoint(pointParamVO.getSourceId(),pointParamVO.getAccountId());
            if(isSend){
                logger.info("已经发生过积分，初始化不发生该数据的积分重发");
                return;
            }
            //调用积分接口添加积分
            String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
            logger.info("准备发送积分，设置{}对象到redis，sitePointId={}",pointParamVO,sitePointId);
            if (null != sitePointId) {
                //发送积分获取消息
               // amqpTemplate.convertAndSend(mqKey, sitePointId); //web端 定义 MQqueuesEnum.COURSE.
                // 配合小伟的经验发放
                /*String sitePointId1 = pointRedisFeignClients.addPointRedis(pointParamVO);
                LOGGER.info("积分发放完毕，准备发送经验值，设置{}对象到redis，sitePointId={}",pointParamVO,sitePointId);
                amqpTemplate.convertAndSend(EXPERIENCE_COURSE, sitePointId1);
                LOGGER.info("积分和经验消息队列发放成功");*/
            }

        } catch (Exception e) {
            logger.error("积分活动名为：" + activityName.toString() + "的积分消息发送失败！");
        }

    }
}
