package com.yizhi.course.application.task;

import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.controller.CourseStudyController;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.StudyLog;
import com.yizhi.studylog.application.feign.StudyLogClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 21:21 2018/11/14
 * @Modified By
 */
@Component
public class CourseStudyAnalysisAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    private static final Logger logger = LoggerFactory.getLogger(CourseStudyAnalysisAsync.class);

    @Autowired
    private IRecordeService recordeService;

    @Autowired
    private CourseStudyController courseStudyController;

    @Autowired
    private StudyLogClient studyLogClient;

    @Override
    protected String execute(Map<String, Object> stringObjectMap) {
        logger.info("开始分析系统的学习日志");
        List<StudyLog> studyLogList = (List<StudyLog>)stringObjectMap.get("courseStudyRecd");
        if( ! CollectionUtils.isEmpty(studyLogList)){
            if(studyLogList.size()>1){
                studyLogList = studyLogList.stream().sorted(Comparator.comparing(StudyLog::getTimestamp)).collect(Collectors.toList());
            }

            Recorde recorde = recordeService.saveRecorde(studyLogList);
            List<Long> delIds = studyLogList.stream().map(obj->obj.getId()).collect(Collectors.toList());
            courseStudyController.completionStatus(recorde);
            studyLogClient.deleteBatch(delIds);
        }
        return "ok";
    }
}
