package com.yizhi.course.application.task;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.domain.CourseStudentStatus;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.service.ICourseStudentStatusService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.RecordeListVo;
import com.yizhi.course.application.vo.RecordeVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 21:21 2018/11/14
 * @Modified By
 */
@Component
public class CourseStudyInitAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    private static final Logger logger = LoggerFactory.getLogger(CourseStudyInitAsync.class);

    @Autowired
    private IRecordeService recordeService;

    @Autowired
    private ICourseStudentStatusService studentStatusService;

    @Autowired
    IdGenerator idGenerator;

    @Override
    protected String execute(Map<String, Object> stringObjectMap) {
        try {
            Long siteId = (Long)stringObjectMap.get("siteId");
            List<RecordeListVo> data = recordeService.findStuByAccountG(siteId);
            logger.info("需要初始化课程的数据条数="+data.size());
            int index = 0;
            for(RecordeListVo item : data){
                logger.info("初始化课程状态"+index+"次");
                List<RecordeVO> stuData = recordeService.findStuChapterByAccountG(
                        item.getSiteId(),item.getCourseId(),item.getAccountId());
                if(!CollectionUtils.isEmpty(stuData)){
                    logger.info("要学习的课程章节数="+stuData.size()+"个");
                    RecordeVO lastStu = stuData.get(0);
                    Long accountId = lastStu.getAccountId();
                    if(accountId==null){
                        EntityWrapper<Recorde> ewRcd = QueryUtil.condition(new Recorde());
                        ewRcd.eq("account_id",item.getAccountId());
                        ewRcd.eq("course_id",item.getCourseId());
                        ewRcd.eq("site_id",item.getSiteId());
                        ewRcd.orderBy("create_time",false);
                        List<Recorde> dataTem1 = recordeService.selectList(ewRcd);
                        Recorde lastStuDimo = dataTem1.get(0);
                        BeanUtils.copyProperties(lastStuDimo,lastStu);
                        accountId = lastStuDimo.getAccountId();
                    }

                    long sumNum =  stuData.size();
                    long passCountNum = stuData.stream().filter(obj->Integer.valueOf(1).equals(obj.getOvered())).count();

                    // 标识课程是否已经学习完成
                    boolean studentStatusFlg = true;
                    // 查询课程学习状态
                    CourseStudentStatus studentStatusEw = new CourseStudentStatus();
                    studentStatusEw.setAccountId(accountId);
                    studentStatusEw.setSiteId(lastStu.getSiteId());
                    studentStatusEw.setCourseId(lastStu.getCourseId());
                    CourseStudentStatus studentStatus = studentStatusService.selectOne(QueryUtil.condition(studentStatusEw));
                    if(studentStatus == null){
                        studentStatus = new CourseStudentStatus();
                        studentStatus.setId(idGenerator.generate());
                        studentStatus.setCourseId(lastStu.getCourseId());
                        studentStatus.setAccountId(accountId);
                        studentStatus.setCompanyId(lastStu.getCompanyId());
                        studentStatus.setSiteId(lastStu.getSiteId());
                        studentStatus.setOrgId(lastStu.getOrgId());
                    }/*else{
                        int oldStudentStatus = studentStatus.getStudentStatus();
                        if(oldStudentStatus == 1){
                            studentStatusFlg = false;
                        }
                    }*/
                    studentStatus.setLastStudentTime(lastStu.getEndTime());
                    studentStatus.setStudentStatus(0);
                    studentStatus.setPercent(0);
                    if(sumNum==passCountNum){
                        studentStatus.setStudentStatus(1);
                        studentStatus.setPercent(100);
                    }else if(passCountNum>0){
                        double percent = BigDecimal.valueOf(passCountNum).
                                divide(BigDecimal.valueOf(sumNum),3,BigDecimal.ROUND_HALF_UP).doubleValue();
                        studentStatus.setPercent((int)(percent*100));
                    }
                    // 保存课程学习状态
                    if(studentStatusFlg){
                        studentStatusService.insertOrUpdate(studentStatus);
                    }
                }
                index++;
            }
        } catch (Exception e) {
            logger.info("异常信息={}",e);
            //return false;
        }
        logger.info("******************************************************************课程初始化处理完成");
        return "ok";
    }
}
