package com.yizhi.course.application.task;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.course.application.controller.ReportMkCourseController;
import com.yizhi.course.application.vo.MkCourseExportRepVO;
import com.yizhi.course.application.vo.ReportMarketingCourseAccountVo;
import com.yizhi.course.application.vo.ReportMarketingCourseVo;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.vo.AccountOrgsRepVO;
import com.yizhi.system.application.vo.AccountOrgsVO;
import com.yizhi.util.application.constant.MarketingCourseReportType;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 16:05 2018/8/27
 * @Modified By
 */
@Component
public class MarketingCourseExportAsync extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger logger = LoggerFactory.getLogger(MarketingCourseExportAsync.class);

    @Autowired
    private ReportMkCourseController reportMkCourseController;

    @Autowired
    private OrganizationClient organizationClient;

    @Autowired
    private AccountClient accountClient;


    @Override
    protected String execute(Map<String, Object> param) {

        RequestContext requestContext = (RequestContext) param.get("requestContext");
        ContextHolder.set(requestContext);
        logger.info("########################################报表统计导出开始,上下文信息={}", JSON.toJSONString(requestContext));
        String business = (String) param.get("business");// 获取业务标识
        String startDate = (String) param.get("startDate");
        String endDate = (String) param.get("endDate");
        List<Object[]> exportBodyData = new ArrayList<>();//导出的主体数据
        String[] titles = null; // 标题字段列表
        String exportName = (String) param.get("exportName"); // 表头名称
        int cloumSize = 0;
        String exportDesc = "统计周期：" + LocalDate.parse(startDate, DateTimeFormatter.ISO_DATE).
                format(DateTimeFormatter.ofPattern("yyyy-MM-dd")) + "～" +
                LocalDate.parse(endDate, DateTimeFormatter.ISO_DATE).
                        format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));// 报表统计说明

        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
        Date submitTime = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());//任务提交时间
        Long taskId = submitTime.getTime();//任务标识
        //String serialNo = UUID.randomUUID().toString();//任务编号
        String serialNo = (String) param.get("serialNo");
        Long submitterId = requestContext.getAccountId();//任务提交人id
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();//公司id
        String taskName = null;//任务名称
        TaskContext taskContext = null;
        logger.info("########################################报表统计导出准备进入业务系统");
        if (StringUtils.equals(MarketingCourseReportType.BY_COURSE_EXPORT, business)) {//按课程下载导出
            logger.info("########################################按课程导出");
            //exportName = "学习营销课程打开转发统计";
            //taskName = exportName+"_"+now.format(formatter);
            taskName = exportName + "_" + serialNo;
            // 获取部门编码或名称关联的orgId
            taskContext = new TaskContext(taskId, serialNo, taskName, submitterId, submitTime, siteId, companyId);
            working(taskContext);
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
                fail(taskContext, "导出报表的查询时间有误，请重新选择日期进行导出");
            }
            titles = new String[]{"课程编码", "课程名称", "打开人数", "有效转发人数", "阅读次数", "阅读总时长", "平均阅读时长"};
            cloumSize = titles.length;
            // 获取业务参数
            String kwd = (String) param.get("kwd");
            MkCourseExportRepVO vo = new MkCourseExportRepVO();
            vo.setStartDate(startDate);
            vo.setEndDate(endDate);
            vo.setKwd(kwd);
            vo.setRequestContext(requestContext);
            Page<ReportMarketingCourseVo> result =
                    reportMkCourseController.marketingCourseGroupExp(vo);
            List<ReportMarketingCourseVo> data = result.getRecords();

            for (ReportMarketingCourseVo item : data) {
                List<Object> itemData = new ArrayList<>();
                itemData.add(item.getCode());
                itemData.add(item.getName());
                itemData.add(item.getOpenCount());
                itemData.add(item.getTotalSend());
                Integer readCount = item.getTotalReadCount();
                itemData.add(readCount);
                itemData.add(getString(item.getReadTimeSum(), false));
                boolean isRound = false;
                if (Objects.nonNull(readCount) && readCount.intValue() > 1) {
                    isRound = true;
                }
                itemData.add(getString(item.getReadTimeAvg(), isRound));
                //Object[] itemData = new Object[]{item.getCode(),item.getName(),item.getTotalSend(),item.getTotalReadCount()};
                exportBodyData.add(itemData.toArray());
            }
            logger.info("########################################学习营销课程打开转发统计完毕");
        } else {
            logger.info("########################################按用户导出数据查询开始");
            //exportName = "学习营销用户转发统计";
            // 获取部门编码或名称关联的orgId
            titles = new String[]{"用户名", "姓名", "用户状态", "转发课程数", "阅读次数", "所在部门"};
            if (StringUtils.equals(MarketingCourseReportType.BY_USER_EXPORT, business)) {// 明细下载
                //exportName = "学习营销用户转发明细统计";
                // 获取部门编码或名称关联的orgId
                //titles = new String[]{"用户名","姓名","用户状态","转发课程数","阅读次数","课程名称","课程编号","所在部门"};
                titles = new String[]{"用户名", "姓名", "用户状态", "打开课程数", "有效转发课程数", "阅读次数", "所在部门", "所在组织架构"};
            }
            cloumSize = titles.length;
            //taskName = exportName+"_"+now.format(formatter);
            taskName = exportName + "_" + serialNo;
            taskContext = new TaskContext(taskId, serialNo, taskName, submitterId, submitTime, siteId, companyId);
            //taskContext = new TaskContext(taskId,taskName,submitterId,submitTime,siteId,companyId);
            working(taskContext);
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
                fail(taskContext, exportName + "导出报表的查询时间有误，请重新选择日期进行导出");
            }

            String orgKwd = (String) param.get("orgKwd");
            if (StringUtils.isNotBlank(orgKwd)) {
                List<Long> orgIds = organizationClient.findOrgIdByNameOrCode(orgKwd, companyId);
                if (CollectionUtils.isEmpty(orgIds)) {
                    orgKwd = "-1";
                } else {
                    orgKwd = orgIds.stream().map(Object::toString).collect(Collectors.joining(","));
                }
            }
            String accountKwd = (String) param.get("accountKwd");
            // 获取用户关键字对应的用户id集合
            if (StringUtils.isNotBlank(accountKwd)) {
                List<Long> accountIds = accountClient.findAIdsByKeyword(accountKwd, companyId);
                if (CollectionUtils.isEmpty(accountIds)) {
                    accountKwd = "-1";
                } else {
                    accountKwd = accountIds.stream().map(Object::toString).collect(Collectors.joining(","));
                }
            }
            MkCourseExportRepVO vo = new MkCourseExportRepVO();
            vo.setStartDate(startDate);
            vo.setEndDate(endDate);
            vo.setOrgKwd(orgKwd);
            vo.setAccountKwd(accountKwd);
            vo.setRequestContext(requestContext);
            Page<ReportMarketingCourseAccountVo> result =
                    reportMkCourseController.marketingCourseAccountGroupExp(vo);
            List<ReportMarketingCourseAccountVo> data = result.getRecords();
            List<Long> accountIds = data.stream().map(item -> item.getAccountId()).collect(Collectors.toList());
            Map<Long, AccountOrgsVO> datamap = new HashMap<>();
            if (!CollectionUtils.isEmpty(accountIds)) {
                AccountOrgsRepVO findvo = new AccountOrgsRepVO();
                findvo.setIds(accountIds);
                findvo.setIdsType(1);
                findvo.setIsFindParent(true);
//                findvo.setIsFindParent(false);
                List<AccountOrgsVO> retData = accountClient.findCcountOrgs(findvo);
                datamap = retData.stream().parallel().collect(Collectors.toMap(key -> key.getId(), val -> val));

            }
            for (ReportMarketingCourseAccountVo item : data) {
                List<Object> listData = new ArrayList<>();
                String name = item.getAccountName();
                String fullName = item.getAccountFullName();
                String status = "";
                //List<String> orgNames = new ArrayList<>();
                String orgName = "";
                String orgNames = "";
                if (datamap.containsKey(item.getAccountId())) {
                    AccountOrgsVO accountOrgsVO = datamap.get(item.getAccountId());
                    name = accountOrgsVO.getName();
                    fullName = accountOrgsVO.getFullName();
                    status = accountOrgsVO.getStatus();
                    //orgNames = accountOrgsVO.getOrgNames();
                    orgName = accountOrgsVO.getOrgName();
                    if (!CollectionUtils.isEmpty(accountOrgsVO.getOrgNames())){
                        for (String s : accountOrgsVO.getOrgNames()){
                            if (StringUtils.isNotBlank(orgNames)){
                                orgNames = orgNames + "/" + s;
                            }else {
                                orgNames = orgNames + s;
                            }
                        }
                    }else {
                        orgNames = "--";
                    }
                }
                listData.add(name);
                listData.add(fullName);
                listData.add(status);
                //listData.add(item.getTotalSendCount()); //转发次数
                listData.add(item.getOpenCount());
                listData.add(item.getTotalSend());
                listData.add(item.getTotalReadCount());
                /*if(StringUtils.equals(MarketingCourseReportType.BY_USER_DETAIL_EXPORT,business)) {// 明细下载
                    listData.add(item.getCourseName());
                    listData.add(item.getCourseCode());
                }*/
                //listData.addAll(orgNames);
                listData.add(orgName);
                listData.add(orgNames);

                exportBodyData.add(listData.toArray());
            }
            logger.info("########################################按用户导出数据查询结束");
        }
        /*logger.info("########################################按用户导出数据查询数据={}条;任务上下文={}",
                exportBodyData.size(),JSON.toJSONString(taskContext));*/
        // 生成excel
        String fileName = taskName + ".xls";

        String upLoadUrl = createExcel(exportBodyData, titles, exportName, cloumSize, exportDesc, fileName);

        /*if(excelData == null){
            fail(taskContext, exportName+"导出报表生成excel发送错误,请重新导出报表操作");
        }

        //阿里云返回url
        String upLoadUrl = OssUpload.upload(excelData, fileName);
        if(StringUtils.isBlank(upLoadUrl)){
            fail(taskContext, exportName+"导出报表上传到oss服务器失败,请重新导出报表操作");
        }*/
        success(taskContext, "成功", upLoadUrl);
        logger.info("########################################报表统计导出完成,下载url=" + upLoadUrl);
        return "ok";
    }

    /*public static void main(String[] args) {
        System.out.println(getString(3665500L,true));
    }
*/
    // mysql窃取字符串UPDATE scorm set resource_href=substring(resource_href,LOCATE('/',resource_href,8)) WHERE resource_href LIKE '%http:%'

    /**
     * 根据毫秒算出时分秒字符串
     *
     * @param readSum
     * @return
     */
    private static String getString(Long readSum, boolean isRound) {
        if (readSum == null || readSum < 1000L) {
            return "";
        }
        long hour = readSum / 3600000;
        Long remnant = readSum % 3600000;
        long minute = remnant / 60000;
        remnant = remnant % 60000;
        long second = 0;
        if (isRound) {
            // 第一种 df2.format(doube_var)
            DecimalFormat df2 = new DecimalFormat("###");//"###.00"
            // 第二种
            BigDecimal b1 = new BigDecimal(remnant);
            int scale = 0;//scale表示表示需要精确到小数点以后几位。
            second = b1.divide(new BigDecimal(1000), scale, BigDecimal.ROUND_HALF_UP).longValue();
            //System.out.println(df2.format(rslt));
            //second = Long.valueOf(df2.format(rslt));
        } else {
            second = remnant / 1000;
        }

        String str = "";
        if (hour > 0) {
            str += hour + "时";
        }
        if (minute > 0) {
            str += minute + "分";
        }
        if (second > 0) {
            str += second + "秒";
        }
        return str;
    }

    /**
     * 生成excel
     *
     * @param exportBodyData 导出的主体数据
     * @param titles         标题头
     * @param exportName     导出顶上大标题
     * @param cloumSize      列大小
     * @param exportDesc     导出的小标题说明
     */
    public static String createExcel(List<Object[]> exportBodyData, String[] titles,
                                     String exportName, int cloumSize, String exportDesc,
                                     String fileName) {

        String requestPath = FileConstant.SAVE_PATH;
        File fileDir = new File(requestPath);
        if (!fileDir.exists()) {
            fileDir.mkdir();
        }
        //String fileDir= "D:\\data";
        String filePath = fileDir + File.separator + fileName;
        String upLoadUrl = "";
        File file = new File(filePath);
        try (HSSFWorkbook workbook = new HSSFWorkbook();
             FileOutputStream os = new FileOutputStream(filePath)) {
            // 创建第一个sheet页
            Sheet sheet = workbook.createSheet(exportName);
            //sheet.setDefaultColumnWidth(20);//设置sheet页默认列宽度设置成100个字符宽度

            int index = 0;
            if (StringUtils.isNotBlank(exportName)) {
                // 设置单元格的高度，其实就是设置行的高度
                // HeightInPoints的单位是点，而Height的单位是1/20个点，所以Height的值永远是HeightInPoints的20倍
                Row row1 = sheet.createRow(index);
                //row.setHeight(Short.valueOf("400"));//20*200
                Cell exportNameCell = row1.createCell(0);
                exportNameCell.setCellValue(exportName);
                HSSFCellStyle exportNameStye = workbook.createCellStyle();
                HSSFFont headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 18);
                // //粗体显示
                headfont.setBold(true);
                exportNameStye.setFont(headfont);
                exportNameStye.setAlignment(HorizontalAlignment.CENTER);//居中
                exportNameStye.setVerticalAlignment(VerticalAlignment.CENTER);//设置垂直居中
                exportNameCell.setCellStyle(exportNameStye);

                //sheet.createRow(index).setRowStyle();
                //合并列
                CellRangeAddress region = new CellRangeAddress(index, index, 0, cloumSize - 1);
                sheet.addMergedRegion(region);
                index++;
            }
            if (StringUtils.isNotBlank(exportDesc)) {
                Cell exportDescCell = sheet.createRow(index).createCell(0);
                exportDescCell.setCellValue(exportDesc);
                HSSFCellStyle exportDescStye = workbook.createCellStyle();
                HSSFFont headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 14);
                // //粗体显示
                headfont.setBold(true);
                exportDescStye.setFont(headfont);
                exportDescStye.setAlignment(HorizontalAlignment.LEFT);
                exportDescStye.setVerticalAlignment(VerticalAlignment.CENTER);//设置垂直居中
                exportDescCell.setCellStyle(exportDescStye);
                //sheet.createRow(index).setRowStyle();
                //合并列
                CellRangeAddress region = new CellRangeAddress(index, index, 0, cloumSize - 1);
                sheet.addMergedRegion(region);
                index++;
            }
            if (titles != null) {
                Row titleRow = sheet.createRow(index);
                HSSFCellStyle titlesStye = workbook.createCellStyle();
                HSSFFont headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 12);
                // //粗体显示
                headfont.setBold(true);
                titlesStye.setFont(headfont);
                titlesStye.setAlignment(HorizontalAlignment.CENTER);
                titlesStye.setVerticalAlignment(VerticalAlignment.CENTER);
                //titleRow.setRowStyle(titlesStye);
                for (int k = 0; k < titles.length; k++) {
                    //sheet.autoSizeColumn(k, true);// 列自适应宽度
                    Cell titleCell = titleRow.createCell(k);
                    titleCell.setCellStyle(titlesStye);
                    titleCell.setCellValue((String) titles[k]);
                    sheet.setColumnWidth(k, 20 * 256);
                }

                index++;
            }
            for (int rownum = 0; rownum < exportBodyData.size(); rownum++) {
                Row row = sheet.createRow(index);
                Object[] itemData = exportBodyData.get(rownum);
                int cellnumSum = itemData.length;


                for (int cellnum = 0; cellnum < cellnumSum; cellnum++) {
                    //sheet.setColumnWidth(cellnum,100 * 256);
                    Cell cell = row.createCell(cellnum);
                    Object valObj = itemData[cellnum];
                    String val = valObj == null ? " " : valObj.toString();
                    cell.setCellValue(val);
                }
                index++;
            }
            //ByteArrayOutputStream os = new ByteArrayOutputStream();
            workbook.write(os);
            //阿里云返回url
            upLoadUrl = OssUpload.upload(filePath, fileName);
            file.delete();
            //success(taskContext,"成功", upLoadUrl);
            //data = os.toByteArray();
        } catch (IOException e) {

            logger.info("生成excel发送异常{}", e);
            return null;
        }
        return upLoadUrl;
    }

}
