package com.yizhi.course.application.task;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.course.application.service.IReportMkCourseService;
import com.yizhi.course.application.vo.MarketingCourseViewVo;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 16:05 2018/8/27
 * @Modified By
 */
@Component
public class MarketingCourseViewExportAsync extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger logger = LoggerFactory.getLogger(MarketingCourseViewExportAsync.class);

    @Autowired
    private IReportMkCourseService reportMkCourseService;

    @Override
    protected String execute(Map<String, Object> param) {

        Long courseId = (Long) param.get("courseId");
        Long siteId = (Long) param.get("siteId");

        Long accountId = (Long) param.get("accountId");
        Long companyId = (Long) param.get("companyId");
        Long taskId = (Long) param.get("taskId");

        String courseName = param.get("companyId").toString();

        List<Object[]> exportBodyData = new ArrayList<>();//导出的主体数据
        String[] titles = null; // 标题字段列表

        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
        Date submitTime = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());//任务提交时间
        //Long taskId = submitTime.getTime();//任务标识
        String serialNo = UUID.randomUUID().toString();//任务编号

        String taskName = courseName+"-浏览数据_"+serialNo;//任务名称

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);

        working(taskContext);
        titles = new String[]{"昵称", "手机号", "浏览次数"};
        logger.info("获取 学习营销课程打开转发统计 的数据开始");
        Page<MarketingCourseViewVo> pageData = reportMkCourseService.marketingCourseViewList(courseId,null,null,null);

        if(pageData == null || pageData.getRecords() == null){
            logger.info("获取 学习营销课程打开转发统计 的数据，没有数据");
            fail(taskContext, "没有数据");
        }


        List<MarketingCourseViewVo> data = pageData.getRecords();
        logger.info("获取 学习营销课程打开转发统计 的数据，总共数据获取到= "+data.size());

        for (MarketingCourseViewVo item : data) {
            List<Object> itemData = new ArrayList<>();
            itemData.add(item.getAccountName());
            itemData.add(item.getTel());
            itemData.add(item.getVeiwCount());
            exportBodyData.add(itemData.toArray());
        }

        // 生成excel
        String fileName = taskName + ".xls";

        logger.info("获取 学习营销课程打开转发统计 的数据,准备生成excel = "+fileName);
        String upLoadUrl = createExcel(exportBodyData, titles, "浏览数据", titles.length, null, fileName);

        success(taskContext, "成功", upLoadUrl);
        logger.info("########################################报表统计导出完成,下载url=" + upLoadUrl);
        return "ok";
    }

    /**
     * 生成excel
     *
     * @param exportBodyData 导出的主体数据
     * @param titles         标题头
     * @param exportName     导出顶上大标题
     * @param cloumSize      列大小
     * @param exportDesc     导出的小标题说明
     */
    public static String createExcel(List<Object[]> exportBodyData, String[] titles,
                                     String exportName, int cloumSize, String exportDesc,
                                     String fileName) {

        String requestPath = FileConstant.SAVE_PATH;
        File fileDir = new File(requestPath);
        if (!fileDir.exists()) {
            fileDir.mkdir();
        }
        //String fileDir= "D:\\data";
        String filePath = fileDir + File.separator + fileName;
        String upLoadUrl = "";
        File file = new File(filePath);
        try (HSSFWorkbook workbook = new HSSFWorkbook();
             FileOutputStream os = new FileOutputStream(filePath)) {
            // 创建第一个sheet页
            Sheet sheet = workbook.createSheet(exportName);
            //sheet.setDefaultColumnWidth(20);//设置sheet页默认列宽度设置成100个字符宽度

            int index = 0;

            if (titles != null) {
                Row titleRow = sheet.createRow(index);
                HSSFCellStyle titlesStye = workbook.createCellStyle();
                HSSFFont headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 12);
                // //粗体显示
                headfont.setBold(true);
                titlesStye.setFont(headfont);
                titlesStye.setAlignment(HorizontalAlignment.CENTER);
                titlesStye.setVerticalAlignment(VerticalAlignment.CENTER);
                //titleRow.setRowStyle(titlesStye);
                for (int k = 0; k < titles.length; k++) {
                    //sheet.autoSizeColumn(k, true);// 列自适应宽度
                    Cell titleCell = titleRow.createCell(k);
                    titleCell.setCellStyle(titlesStye);
                    titleCell.setCellValue((String) titles[k]);
                    sheet.setColumnWidth(k, 80 * 256);
                }

                index++;
            }
            for (int rownum = 0; rownum < exportBodyData.size(); rownum++) {
                Row row = sheet.createRow(index);
                Object[] itemData = exportBodyData.get(rownum);
                int cellnumSum = itemData.length;


                for (int cellnum = 0; cellnum < cellnumSum; cellnum++) {
                    //sheet.setColumnWidth(cellnum,100 * 256);
                    Cell cell = row.createCell(cellnum);
                    Object valObj = itemData[cellnum];
                    String val = valObj == null ? " " : valObj.toString();
                    cell.setCellValue(val);
                }
                index++;
            }
            //ByteArrayOutputStream os = new ByteArrayOutputStream();
            workbook.write(os);
            //阿里云返回url
            upLoadUrl = OssUpload.upload(filePath, fileName);
            file.delete();
            //success(taskContext,"成功", upLoadUrl);
            //data = os.toByteArray();
        } catch (IOException e) {

            logger.info("生成excel发送异常{}", e);
            return null;
        }
        return upLoadUrl;
    }

}
